@@ -1,16 +1,18 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id$ */
+
 /* Alpha-buffering memory devices */
 #include "memory_.h"
 #include "gx.h"
@@ -95,6 +97,28 @@ mem_alpha_copy_alpha(gx_device * dev, const byte * data, int data_x,
            int raster, gx_bitmap_id id, int x, int y, int width, int height,
                      gx_color_index color, int depth)
 {				/* Just use copy_color. */
+    if (depth == 8) {
+        /* We don't support depth=8 in this function, but this doesn't
+         * matter, because:
+         * 1) This code is only called for dTextAlphaBits > 0, when
+         *    DisableFAPI=true. And we don't support DisableFAPI.
+         * 2) Even if we did support DisableFAPI, this can never actually
+         *    be called because gx_compute_text_oversampling arranges that
+         *    log2_scale.{x,y} sum to <= alpha_bits, and this code is only
+         *    called if it sums to MORE than alpha_bits.
+         * 3) Even if copy_alpha DID somehow manage to be called, the
+         *    only place that uses depth==8 is the imagemask interpolation
+         *    code, and that can never hit this code. (Type 3 fonts might
+         *    include Imagemasks, but those don't go through FAPI).
+         *
+         * If in the future we ever rearrange the conditions under which
+         * this code is called (so that it CAN be called with depth == 8)
+         * then this will probably be best implemented by decimating the
+         * input alpha values to either 2 or 4 bits as appropriate and
+         * then recursively calling us back.
+         */
+        return_error(gs_error_unknownerror);
+    }
     return (color == 0 ?
             (*dev_proc(dev, fill_rectangle)) (dev, x, y, width, height,
                                               color) :
@@ -143,13 +167,14 @@ static dev_proc_close_device(mem_abuf_close);
 static dev_proc_copy_mono(mem_abuf_copy_mono);
 static dev_proc_fill_rectangle(mem_abuf_fill_rectangle);
 static dev_proc_get_clipping_box(mem_abuf_get_clipping_box);
+static dev_proc_fill_rectangle_hl_color(mem_abuf_fill_rectangle_hl_color);
 
 /* The device descriptor. */
 static const gx_device_memory mem_alpha_buffer_device =
-mem_device("image(alpha buffer)", 0, 1,
+mem_device_hl("image(alpha buffer)", 0, 1,
            gx_forward_map_rgb_color, gx_forward_map_color_rgb,
-         mem_abuf_copy_mono, gx_default_copy_color, mem_abuf_fill_rectangle,
-           gx_no_strip_copy_rop);
+           mem_abuf_copy_mono, gx_default_copy_color, mem_abuf_fill_rectangle,
+           gx_no_strip_copy_rop, mem_abuf_fill_rectangle_hl_color);
 
 /* Make an alpha-buffer memory device. */
 /* We use abuf instead of alpha_buffer because */
@@ -157,7 +182,7 @@ mem_device("image(alpha buffer)", 0, 1,
 void
 gs_make_mem_abuf_device(gx_device_memory * adev, gs_memory_t * mem,
                      gx_device * target, const gs_log2_scale_point * pscale,
-                        int alpha_bits, int mapped_x)
+                        int alpha_bits, int mapped_x, bool devn)
 {
     gs_make_mem_device(adev, &mem_alpha_buffer_device, mem, 0, target);
     adev->max_fill_band = 1 << pscale->y;
@@ -166,6 +191,10 @@ gs_make_mem_abuf_device(gx_device_memory * adev, gs_memory_t * mem,
     adev->mapped_x = mapped_x;
     set_dev_proc(adev, close_device, mem_abuf_close);
     set_dev_proc(adev, get_clipping_box, mem_abuf_get_clipping_box);
+    if (!devn) 
+        adev->save_hl_color = NULL; /* This is the test for when we flush the
+                                       the buffer as to what copy_alpha type
+                                       use */
     adev->color_info.anti_alias.text_bits =
       adev->color_info.anti_alias.graphics_bits =
         alpha_bits;
@@ -199,15 +228,15 @@ abuf_flush_block(gx_device_memory * adev, int y)
     if (buffer_y >= adev->height)
         buffer_y -= adev->height;
     bits = scan_line_base(adev, buffer_y);
-    {				/*
-                                 * Many bits are typically zero.  Save time by computing
-                                 * an accurate X bounding box before compressing.
-                                 * Unfortunately, in order to deal with alpha nibble swapping
-                                 * (see gsbitops.c), we can't expand the box only to pixel
-                                 * boundaries:
-                                 int alpha_mask = -1 << adev->log2_alpha_bits;
-                                 * Instead, we must expand it to byte boundaries,
-                                 */
+    {/*
+      * Many bits are typically zero.  Save time by computing
+      * an accurate X bounding box before compressing.
+      * Unfortunately, in order to deal with alpha nibble swapping
+      * (see gsbitops.c), we can't expand the box only to pixel
+      * boundaries:
+          int alpha_mask = -1 << adev->log2_alpha_bits;
+      * Instead, we must expand it to byte boundaries, 
+      */
         int alpha_mask = ~7;
         gs_int_rect bbox;
         int width;
@@ -219,13 +248,24 @@ abuf_flush_block(gx_device_memory * adev, int y)
         bits_compress_scaled(bits, bbox.p.x, width, block_height,
                              adev->raster, bits, draster, &adev->log2_scale,
                              adev->log2_alpha_bits);
-        return (*dev_proc(target, copy_alpha)) (target,
-                                          bits, 0, draster, gx_no_bitmap_id,
-                                              (adev->mapped_x + bbox.p.x) >>
-                                                adev->log2_scale.x,
-                                                y >> adev->log2_scale.y,
-                                             width >> adev->log2_scale.x, 1,
-                                              adev->save_color, alpha_bits);
+        /* Set up with NULL when adev initialized */
+        if (adev->save_hl_color == NULL) { 
+            return (*dev_proc(target, copy_alpha)) (target,
+                                              bits, 0, draster, gx_no_bitmap_id,
+                                                  (adev->mapped_x + bbox.p.x) >>
+                                                    adev->log2_scale.x,
+                                                    y >> adev->log2_scale.y,
+                                                 width >> adev->log2_scale.x, 1,
+                                                  adev->save_color, alpha_bits);
+        } else {
+            return (*dev_proc(target, copy_alpha_hl_color)) (target,
+                                              bits, 0, draster, gx_no_bitmap_id,
+                                                  (adev->mapped_x + bbox.p.x) >>
+                                                    adev->log2_scale.x,
+                                                    y >> adev->log2_scale.y,
+                                                 width >> adev->log2_scale.x, 1,
+                                                  adev->save_hl_color, alpha_bits);
+        }
     }
 }
 /* Flush the entire buffer. */
@@ -321,9 +361,9 @@ y_transfer_next(y_transfer * pyt, gx_device * dev)
         tby -= mdev->height;
         tbh = ms + mh - dev->height - tby;
     }
-    if_debug7('V',
-              "[V]abuf: my=%d, mh=%d, ms=%d, ty=%d, th=%d, tby=%d, tbh=%d\n",
-              my, mh, ms, ty, th, tby, tbh);
+    if_debug7m('V', mdev->memory,
+               "[V]abuf: my=%d, mh=%d, ms=%d, ty=%d, th=%d, tby=%d, tbh=%d\n",
+               my, mh, ms, ty, th, tby, tbh);
     if (tbh > th)
         tbh = th;
     pyt->height_left = th - tbh;
@@ -389,6 +429,38 @@ mem_abuf_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
     return 0;
 }
 
+/* Fill a rectangle. */
+static int
+mem_abuf_fill_rectangle_hl_color(gx_device * dev, const gs_fixed_rect *rect,
+                                 const gs_imager_state *pis,
+                                 const gx_drawing_color *pdcolor,
+                                 const gx_clip_path *pcpath)
+{
+    gx_device_memory * const mdev = (gx_device_memory *)dev;
+    y_transfer yt;
+    int x = fixed2int(rect->p.x);
+    int y = fixed2int(rect->p.y);
+    int w = fixed2int(rect->q.x) - x;
+    int h = fixed2int(rect->q.y) - y;
+
+    x -= mdev->mapped_x;
+    fit_fill_xy(dev, x, y, w, h);
+    fit_fill_w(dev, x, w);	/* don't limit h */
+    /* or check w <= 0, h <= 0 */
+    mdev->save_hl_color = pdcolor;
+    y_transfer_init(&yt, dev, y, h);
+    while (yt.height_left > 0) {
+        int code = y_transfer_next(&yt, dev);
+
+        if (code < 0)
+            return code;
+        (*dev_proc(&mem_mono_device, fill_rectangle)) (dev,
+                                    x, yt.transfer_y, w, yt.transfer_height,
+                                                       (gx_color_index) 1);
+    }
+    return 0;
+}
+
 /* Get the clipping box.  We must scale this up by the number of alpha bits. */
 static void
 mem_abuf_get_clipping_box(gx_device * dev, gs_fixed_rect * pbox)
