@@ -1,16 +1,18 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id$ */
+
 /* Default and device-independent RasterOp algorithms */
 #include "memory_.h"
 #include "gx.h"
@@ -48,33 +50,33 @@ trace_copy_rop(const char *cname, gx_device * dev,
                int x, int y, int width, int height,
                int phase_x, int phase_y, gs_logical_operation_t lop)
 {
-    dlprintf4("%s: dev=0x%lx(%s) depth=%d\n",
-              cname, (ulong) dev, dev->dname, dev->color_info.depth);
-    dlprintf4("  source data=0x%lx x=%d raster=%u id=%lu colors=",
-              (ulong) sdata, sourcex, sraster, (ulong) id);
+    dmlprintf4(dev->memory, "%s: dev=0x%lx(%s) depth=%d\n",
+               cname, (ulong) dev, dev->dname, dev->color_info.depth);
+    dmlprintf4(dev->memory, "  source data=0x%lx x=%d raster=%u id=%lu colors=",
+               (ulong) sdata, sourcex, sraster, (ulong) id);
     if (scolors)
-        dprintf2("(%lu,%lu);\n", scolors[0], scolors[1]);
+        dmprintf2(dev->memory, "(%lu,%lu);\n", scolors[0], scolors[1]);
     else
-        dputs("none;\n");
+        dmputs(dev->memory, "none;\n");
     if (textures)
-        dlprintf8("  textures=0x%lx size=%dx%d(%dx%d) raster=%u shift=%d(%d)",
+        dmlprintf8(dev->memory, "  textures=0x%lx size=%dx%d(%dx%d) raster=%u shift=%d(%d)",
                   (ulong) textures, textures->size.x, textures->size.y,
                   textures->rep_width, textures->rep_height,
                   textures->raster, textures->shift, textures->rep_shift);
     else
-        dlputs("  textures=none");
+        dmlputs(dev->memory, "  textures=none");
     if (tcolors)
-        dprintf2(" colors=(%lu,%lu)\n", tcolors[0], tcolors[1]);
+        dmprintf2(dev->memory, " colors=(%lu,%lu)\n", tcolors[0], tcolors[1]);
     else
-        dputs(" colors=none\n");
-    dlprintf7("  rect=(%d,%d),(%d,%d) phase=(%d,%d) op=0x%x\n",
+        dmputs(dev->memory, " colors=none\n");
+    dmlprintf7(dev->memory, "  rect=(%d,%d),(%d,%d) phase=(%d,%d) op=0x%x\n",
               x, y, x + width, y + height, phase_x, phase_y,
               (uint) lop);
     if (gs_debug_c('B')) {
         if (sdata)
-            debug_dump_bitmap(sdata, sraster, height, "source bits");
+            debug_dump_bitmap(dev->memory, sdata, sraster, height, "source bits");
         if (textures && textures->data)
-            debug_dump_bitmap(textures->data, textures->raster,
+            debug_dump_bitmap(dev->memory, textures->data, textures->raster,
                               textures->size.y, "textures bits");
     }
 }
@@ -129,6 +131,7 @@ gx_default_strip_copy_rop2(gx_device * dev,
     int code;
     int py;
     int is_planar = 0;
+    int plane_depth;
 
 #ifdef DEBUG
     if (gs_debug_c('b'))
@@ -156,16 +159,16 @@ gx_default_strip_copy_rop2(gx_device * dev,
     pmdev->height = block_height;
     pmdev->bitmap_memory = mem;
     pmdev->color_info = dev->color_info;
-    if (dev_proc(dev, dev_spec_op)(dev, gxdso_is_native_planar, NULL, 0))
+    if (dev->is_planar)
     {
         gx_render_plane_t planes[GX_DEVICE_COLOR_MAX_COMPONENTS];
         int num_comp = dev->color_info.num_components;
-        int depth = dev->color_info.depth/num_comp;
         int i;
+        plane_depth = dev->color_info.depth / num_comp;
         for (i = 0; i < num_comp; i++)
         {
-            planes[i].shift = depth * (num_comp - 1 - i);
-            planes[i].depth = depth;
+            planes[i].shift = plane_depth * (num_comp - 1 - i);
+            planes[i].depth = plane_depth;
             planes[i].index = i;
         }
         /* RJW: This code, like most of ghostscripts planar support,
@@ -477,7 +480,7 @@ pack_planar_from_standard(gx_device_memory * dev, int y, int destx,
         return;
     }
 
-    for (plane = 0; plane < dev->num_planes; plane++) {
+    for (plane = 0; plane < dev->color_info.num_components; plane++) {
         byte *dest = scan_line_base(dev, y + plane * dev->height);
         dp[plane] = dest + (bit_x >> 3);
         buf[plane] = (shift == 8 ? 0 : *dp[plane] & (0xff00 >> shift));
@@ -547,22 +550,22 @@ pack_planar_from_standard(gx_device_memory * dev, int y, int destx,
                 /* We have pdepth*num_planes bits in 'pixel'. We need to copy
                  * them (topmost bits first) into the buffer, packing them at
                  * shift position. */
-                int pshift = pdepth*(dev->num_planes-1);
+                int pshift = pdepth*(dev->color_info.num_components-1);
 #endif
                 /* Can we fit another pdepth bits into our buffer? */
                 shift -= pdepth;
                 if (shift < 0) {
                     /* No, so flush the buffer to the planes. */
-                    for (plane = 0; plane < dev->num_planes; plane++)
+                    for (plane = 0; plane < dev->color_info.num_components; plane++)
                         *dp[plane]++ = buf[plane];
                     shift += 8;
                 }
                 /* Copy the next pdepth bits into each planes buffer */
 #ifdef ORIGINAL_CODE_KEPT_FOR_REFERENCE
-                for (plane = 0; plane < dev->num_planes; pshift+=8,plane++)
+                for (plane = 0; plane < dev->color_info.num_components; pshift+=8,plane++)
                     buf[plane] += (byte)(((pixel>>pshift) & pmask)<<shift);
 #else
-                for (plane = 0; plane < dev->num_planes; pshift-=pdepth,plane++)
+                for (plane = 0; plane < dev->color_info.num_components; pshift-=pdepth,plane++)
                     buf[plane] += (byte)(((pixel>>pshift) & pmask)<<shift);
 #endif
                 break;
@@ -571,11 +574,11 @@ pack_planar_from_standard(gx_device_memory * dev, int y, int destx,
     }
     if (width > 0 && depth <= 8) {
         if (shift == 0)
-            for (plane = 0; plane < dev->num_planes; plane++)
+            for (plane = 0; plane < dev->color_info.num_components; plane++)
                 *dp[plane] = buf[plane];
         else {
             int mask = (1<<shift)-1;
-            for (plane = 0; plane < dev->num_planes; plane++)
+            for (plane = 0; plane < dev->color_info.num_components; plane++)
                 *dp[plane] = (*dp[plane] & mask) + buf[plane];
         }
     }
@@ -600,8 +603,8 @@ mem_default_strip_copy_rop2(gx_device * dev,
                             gs_logical_operation_t lop,
                             uint planar_height)
 {
-    dlprintf("mem_default_strip_copy_rop2 should never be called!\n");
-    return gs_error_Fatal;
+    dmlprintf(dev->memory, "mem_default_strip_copy_rop2 should never be called!\n");
+    return_error(gs_error_Fatal);
 }
 
 int
@@ -689,7 +692,7 @@ mem_default_strip_copy_rop(gx_device * dev,
 
     /* We know the device is a memory device, so we can store the
      * result directly into its scan lines, unless it is planar. */
-    if (tdev->num_planes <= 1) {
+    if (!tdev->is_planar || tdev->color_info.num_components <= 1) {
         if ((rop_depth == 24) && (dev_proc(dev, dev_spec_op)(dev,
                                       gxdso_is_std_cmyk_1bit, NULL, 0) > 0)) {
             pack = pack_cmyk_1bit_from_standard;
