@@ -14,6 +14,9 @@
 */
 
 
+/* prevent gp.h from defining fopen */
+#define fopen fopen
+
 
 #ifndef __CARBON__
 //#include "MacHeaders"
@@ -54,6 +57,7 @@
 #include <console.h>
 
 #include "gx.h"
+
 #include "gp.h"
 #include "gpmisc.h"
 #include "gxdevice.h"
@@ -135,13 +139,6 @@ int fake_stdin = 0;
 
 /* Do platform-dependent initialization */
 
-void
-setenv(const char * env, char *p) {
-//	if ( strcmp(env,"outfile") == 0) {
-//	   sprintf((char *)&g_fcout[0],"%s",p);
-//	}
-}
-
 char *
 getenv(const char * env) {
 
@@ -162,7 +159,7 @@ getenv(const char * env) {
                 convertSpecToPath(&pFile, fpath, 256);
 //		sprintf(fpath,"%s",fpath);
                 p = (char*)malloc((size_t) ( 4*strlen(fpath) + 40));
-                sprintf(p,"%s,%sGhostscript:lib,%sGhostscript:fonts",
+                gs_sprintf(p,"%s,%sGhostscript:lib,%sGhostscript:fonts",
                                                 (char *)&fpath[0],(char *)&fpath[0],
                                                 (char *)&fpath[0] );
 
@@ -402,9 +399,9 @@ gp_open_scratch_file (const gs_memory_t *mem,
                 }
                 FSMakeFSSpec(foundVRefNum, foundDirID,thepfname, &fSpec);
                 convertSpecToPath(&fSpec, thefname, sizeof(thefname) - 1);
-                sprintf(fname,"%s",thefname);
+                gs_sprintf(fname,"%s",thefname);
    } else {
-       sprintf((char*)&thefname[0],"%s\0",fname);
+       gs_sprintf((char*)&thefname[0],"%s\0",fname);
        memmove((char*)&thepfname[1],(char *)&thefname[0],strlen(thefname));
            thepfname[0]=strlen(thefname);
    }
@@ -535,9 +532,9 @@ gp_fopen (const char * fname, const char * mode) {
 //(*pgsdll_callback) (GSDLL_STDOUT, thefname, strlen(fname));
    if ( strrchr(fname,':') == NULL )
 //      sprintf((char *)&thefname[0],"%s%s\0",g_homeDir,fname);
-      sprintf((char *)&thefname[0],"%s%s\0","",fname);
+      gs_sprintf((char *)&thefname[0],"%s%s\0","",fname);
    else
-       sprintf((char*)&thefname[0],"%s\0",fname);
+       gs_sprintf((char*)&thefname[0],"%s\0",fname);
 
    fid = fopen(thefname,mode);
 
@@ -545,6 +542,39 @@ gp_fopen (const char * fname, const char * mode) {
 
 }
 
+int gp_stat(const char *path, struct stat *buf)
+{
+    return stat(path, buf);
+}
+
+int gp_can_share_fdesc(void)
+{
+    return 0;
+}
+
+FILE *gp_open_scratch_file_rm(const gs_memory_t *mem,
+                              const char        *prefix,
+                                    char         fname[gp_file_name_sizeof],
+                              const char        *mode)
+{
+    return NULL;
+}
+
+FILE *gp_fdup(FILE *f, const char *mode)
+{
+    return NULL;
+}
+
+int gp_fpread(char *buf, uint count, int64_t offset, FILE *f)
+{
+    return -1;
+}
+
+int gp_fpwrite(char *buf, uint count, int64_t offset, FILE *f)
+{
+    return -1;
+}
+
 FILE *
 popen (const char * fname, const char * mode ) {
         return gp_fopen (fname,  mode);
@@ -658,7 +688,7 @@ const char *gp_file_name_current(void)
 {   return ":";
 }
 
-bool gp_file_name_is_partent_allowed(void)
+bool gp_file_name_is_parent_allowed(void)
 {   return true;
 }
 
