@@ -29,9 +29,14 @@
 /* DLL version must now be used under MS-Windows */
 /* Russell Lang 16 March 1996 */
 
+/* prevent gp.h from defining fopen */
+#define fopen fopen
+
+#include "windows_.h"
 #include "stdio_.h"
 #include "string_.h"
 #include "memory_.h"
+#include "stat_.h"
 #include "pipe_.h"
 #include <stdlib.h>
 #include <stdarg.h>
@@ -41,6 +46,7 @@
 #include <fcntl.h>
 #include <signal.h>
 #include "gx.h"
+
 #include "gp.h"
 #include "gpcheck.h"
 #include "gpmisc.h"
@@ -48,11 +54,10 @@
 #include "gsexit.h"
 #include "scommon.h"
 
-#include "windows_.h"
-#include <stdarg.h>
 #include <shellapi.h>
 #include <winspool.h>
 #include "gp_mswin.h"
+#include "winrtsup.h"
 
 /* Library routines not declared in a standard header */
 extern char *getenv(const char *);
@@ -262,7 +267,7 @@ gp_printfile(const char *filename, const char *pmport)
 
         /* WinNT stores default printer in registry and win.ini */
         /* Win95 stores default printer in win.ini */
-#ifdef WINDOWS_NO_UNICODE
+#ifdef GS_NO_UTF8
         GetProfileString("windows", "device", "", buf, sizeof(buf));
 #else
         wchar_t wbuf[512];
@@ -271,7 +276,7 @@ gp_printfile(const char *filename, const char *pmport)
         GetProfileStringW(L"windows", L"device", L"",  wbuf, sizeof(wbuf));
         l = wchar_to_utf8(NULL, wbuf);
         if (l < 0 || l > sizeof(buf))
-            return gs_error_undefinedfilename;
+            return_error(gs_error_undefinedfilename);
         wchar_to_utf8(buf, wbuf);
 #endif
         if ((p = strchr(buf, ',')) != NULL)
@@ -283,7 +288,7 @@ gp_printfile(const char *filename, const char *pmport)
         else
             return gp_printfile_win32(filename, (char *)NULL);
     } else
-        return gs_error_undefinedfilename;
+        return_error(gs_error_undefinedfilename);
 }
 
 #define PRINT_BUF_SIZE 16384u
@@ -316,7 +321,7 @@ get_queues(void)
         char buf[256];
 
         free(enumbuffer);
-        sprintf(buf, "EnumPrinters() failed, error code = %d", GetLastError());
+        gs_sprintf(buf, "EnumPrinters() failed, error code = %d", GetLastError());
         MessageBox((HWND) NULL, buf, szAppName, MB_OK | MB_ICONSTOP);
         return NULL;
     }
@@ -381,7 +386,7 @@ BOOL gp_OpenPrinter(char *port, LPHANDLE printer)
 #ifdef METRO
     return FALSE;
 #else
-#ifdef WINDOWS_NO_UNICODE
+#ifdef GS_NO_UTF8
     return OpenPrinter(port, printer, NULL);
 #else
     BOOL opened;
@@ -423,7 +428,7 @@ gp_printfile_win32(const char *filename, char *port)
     if (!gp_OpenPrinter(port, &printer)) {
         char buf[256];
 
-        sprintf(buf, "OpenPrinter() failed for \042%s\042, error code = %d", port, GetLastError());
+        gs_sprintf(buf, "OpenPrinter() failed for \042%s\042, error code = %d", port, GetLastError());
         MessageBox((HWND) NULL, buf, szAppName, MB_OK | MB_ICONSTOP);
         free(buffer);
         return FALSE;
@@ -436,7 +441,7 @@ gp_printfile_win32(const char *filename, char *port)
     if (!StartDocPrinter(printer, 1, (LPBYTE) & di)) {
         char buf[256];
 
-        sprintf(buf, "StartDocPrinter() failed, error code = %d", GetLastError());
+        gs_sprintf(buf, "StartDocPrinter() failed, error code = %d", GetLastError());
         MessageBox((HWND) NULL, buf, szAppName, MB_OK | MB_ICONSTOP);
         AbortPrinter(printer);
         free(buffer);
@@ -457,7 +462,7 @@ gp_printfile_win32(const char *filename, char *port)
     if (!EndDocPrinter(printer)) {
         char buf[256];
 
-        sprintf(buf, "EndDocPrinter() failed, error code = %d", GetLastError());
+        gs_sprintf(buf, "EndDocPrinter() failed, error code = %d", GetLastError());
         MessageBox((HWND) NULL, buf, szAppName, MB_OK | MB_ICONSTOP);
         AbortPrinter(printer);
         return FALSE;
@@ -465,7 +470,7 @@ gp_printfile_win32(const char *filename, char *port)
     if (!ClosePrinter(printer)) {
         char buf[256];
 
-        sprintf(buf, "ClosePrinter() failed, error code = %d", GetLastError());
+        gs_sprintf(buf, "ClosePrinter() failed, error code = %d", GetLastError());
         MessageBox((HWND) NULL, buf, szAppName, MB_OK | MB_ICONSTOP);
         return FALSE;
     }
@@ -480,7 +485,7 @@ gp_printfile_win32(const char *filename, char *port)
 FILE *mswin_popen(const char *cmd, const char *mode)
 {
     SECURITY_ATTRIBUTES saAttr;
-#ifdef WINDOWS_NO_UNICODE
+#ifdef GS_NO_UTF8
     STARTUPINFO siStartInfo;
 #else
     STARTUPINFOW siStartInfo;
@@ -493,7 +498,7 @@ FILE *mswin_popen(const char *cmd, const char *mode)
     HANDLE hChildStderrWr = INVALID_HANDLE_VALUE;
     HANDLE hProcess = GetCurrentProcess();
     int handle = 0;
-#ifdef WINDOWS_NO_UNICODE
+#ifdef GS_NO_UTF8
     char *command = NULL;
 #else
     wchar_t *command = NULL;
@@ -543,7 +548,7 @@ FILE *mswin_popen(const char *cmd, const char *mode)
     siStartInfo.hStdError = hChildStderrWr;
 
     if (handle == 0) {
-#ifdef WINDOWS_NO_UNICODE
+#ifdef GS_NO_UTF8
         command = (char *)malloc(strlen(cmd)+1);
         if (command)
             strcpy(command, cmd);
@@ -557,7 +562,7 @@ FILE *mswin_popen(const char *cmd, const char *mode)
     }
 
     if (handle == 0)
-#ifdef WINDOWS_NO_UNICODE
+#ifdef GS_NO_UTF8
         if (!CreateProcess(NULL,
 #else
         if (!CreateProcessW(NULL,
@@ -606,11 +611,12 @@ FILE *mswin_popen(const char *cmd, const char *mode)
 
 /* Create and open a scratch file with a given name prefix. */
 /* Write the actual file name at fname. */
-FILE *
-gp_open_scratch_file(const gs_memory_t *mem,
-                     const char        *prefix,
-                           char        *fname,
-                     const char        *mode)
+static FILE *
+gp_open_scratch_file_generic(const gs_memory_t *mem,
+                             const char        *prefix,
+                                   char        *fname,
+                             const char        *mode,
+                                   int          remove)
 {
     UINT n;
     DWORD l;
@@ -619,17 +625,35 @@ gp_open_scratch_file(const gs_memory_t *mem,
     FILE *f = NULL;
     char sTempDir[_MAX_PATH];
     char sTempFileName[_MAX_PATH];
+#if defined(METRO) || !defined(GS_NO_UTF8)
+    wchar_t wTempDir[_MAX_PATH];
+    wchar_t wTempFileName[_MAX_PATH];
+    wchar_t wPrefix[_MAX_PATH];
+#endif
 
     memset(fname, 0, gp_file_name_sizeof);
     if (!gp_file_name_is_absolute(prefix, strlen(prefix))) {
-        int plen = sizeof(sTempDir);
+        int plen = _MAX_PATH;
 
-        if (gp_gettmpdir(sTempDir, &plen) != 0)
-            l = GetTempPath(sizeof(sTempDir), sTempDir);
-        else
+        /* gp_gettmpdir will always return a UTF8 encoded string unless
+         * GS_NO_UTF8 is defined, due to the windows specific version of
+         * gp_getenv being called (in gp_wgetv.c, not gp_getnv.c) */
+        if (gp_gettmpdir(sTempDir, &plen) != 0) {
+#ifdef METRO
+            /* METRO always uses UTF8 for 'ascii' - there is no concept of
+             * local encoding, so GS_NO_UTF8 makes no difference here. */
+            l = GetTempPathWRT(_MAX_PATH, wTempDir);
+            l = wchar_to_utf8(sTempDir, wTempDir);
+#elif defined(GS_NO_UTF8)
+            l = GetTempPathA(_MAX_PATH, sTempDir);
+#else
+            GetTempPathW(_MAX_PATH, wTempDir);
+            l = wchar_to_utf8(sTempDir, wTempDir);
+#endif
+        } else
             l = strlen(sTempDir);
     } else {
-        strncpy(sTempDir, prefix, sizeof(sTempDir));
+        strncpy(sTempDir, prefix, _MAX_PATH);
         prefix = "";
         l = strlen(sTempDir);
     }
@@ -637,8 +661,20 @@ gp_open_scratch_file(const gs_memory_t *mem,
     if (sTempDir[l-1] == '/')
         sTempDir[l-1] = '\\';		/* What Windoze prefers */
 
-    if (l <= sizeof(sTempDir)) {
-        n = GetTempFileName(sTempDir, prefix, 0, sTempFileName);
+    if (l <= _MAX_PATH) {
+#ifdef METRO
+        utf8_to_wchar(wTempDir, sTempDir);
+        utf8_to_wchar(wPrefix, prefix);
+        n = GetTempFileNameWRT(wTempDir, wPrefix, wTempFileName);
+        n = wchar_to_utf8(sTempFileName, wTempFileName);
+#elif defined(GS_NO_UTF8)
+        n = GetTempFileNameA(sTempDir, prefix, 0, sTempFileName);
+#else
+        utf8_to_wchar(wTempDir, sTempDir);
+        utf8_to_wchar(wPrefix, prefix);
+        GetTempFileNameW(wTempDir, wPrefix, 0, wTempFileName);
+        n = wchar_to_utf8(sTempFileName, wTempFileName);
+#endif
         if (n == 0) {
             /* If 'prefix' is not a directory, it is a path prefix. */
             int l = strlen(sTempDir), i;
@@ -652,16 +688,27 @@ gp_open_scratch_file(const gs_memory_t *mem,
                     break;
                 }
             }
-            if (i > 0)
-                n = GetTempFileName(sTempDir, sTempDir + i, 0, sTempFileName);
+            if (i > 0) {
+#ifdef METRO
+                utf8_to_wchar(wPrefix, sTempDir + i);
+                GetTempFileNameWRT(wTempDir, wPrefix, wTempFileName);
+                n = wchar_to_utf8(sTempFileName, wTempFileName);
+#elif defined(GS_NO_UTF8)
+                n = GetTempFileNameA(sTempDir, sTempDir + i, 0, sTempFileName);
+#else
+                utf8_to_wchar(wPrefix, sTempDir + i);
+                GetTempFileNameW(wTempDir, wPrefix, 0, wTempFileName);
+                n = wchar_to_utf8(sTempFileName, wTempFileName);
+#endif
+            }
         }
         if (n != 0) {
-#ifdef WINDOWS_NO_UNICODE
+#ifdef GS_NO_UTF8
             hfile = CreateFile(sTempFileName,
                                GENERIC_READ | GENERIC_WRITE | DELETE,
                                FILE_SHARE_READ | FILE_SHARE_WRITE,
                                NULL, CREATE_ALWAYS,
-                               FILE_ATTRIBUTE_NORMAL /* | FILE_FLAG_DELETE_ON_CLOSE */,
+                               FILE_ATTRIBUTE_NORMAL | (remove ? FILE_FLAG_DELETE_ON_CLOSE : 0),
                                NULL);
 #else
             int len = utf8_to_wchar(NULL, sTempFileName);
@@ -670,22 +717,23 @@ gp_open_scratch_file(const gs_memory_t *mem,
                 hfile = INVALID_HANDLE_VALUE;
             else {
                 utf8_to_wchar(uni, sTempFileName);
+#ifdef METRO
+                hfile = CreateFile2(uni,
+                                    GENERIC_READ | GENERIC_WRITE | DELETE,
+                                    FILE_SHARE_READ | FILE_SHARE_WRITE,
+                                    CREATE_ALWAYS | (remove ? FILE_FLAG_DELETE_ON_CLOSE : 0),
+                                    NULL);
+#else
                 hfile = CreateFileW(uni,
                                     GENERIC_READ | GENERIC_WRITE | DELETE,
                                     FILE_SHARE_READ | FILE_SHARE_WRITE,
                                     NULL, CREATE_ALWAYS,
-                                    FILE_ATTRIBUTE_NORMAL /* | FILE_FLAG_DELETE_ON_CLOSE */,
+                                    FILE_ATTRIBUTE_NORMAL | (remove ? FILE_FLAG_DELETE_ON_CLOSE : 0),
                                     NULL);
+#endif
                 free(uni);
             }
 #endif
-            /*
-             * Can't apply FILE_FLAG_DELETE_ON_CLOSE due to
-             * the logics of clist_fclose. Also note that
-             * gdev_prn_render_pages requires multiple temporary files
-             * to exist simultaneousely, so that keeping all them opened
-             * may exceed available CRTL file handles.
-             */
         }
     }
     if (hfile != INVALID_HANDLE_VALUE) {
@@ -714,11 +762,29 @@ gp_open_scratch_file(const gs_memory_t *mem,
     return f;
 }
 
+FILE *
+gp_open_scratch_file(const gs_memory_t *mem,
+                     const char        *prefix,
+                           char        *fname,
+                     const char        *mode)
+{
+    return gp_open_scratch_file_generic(mem, prefix, fname, mode, 0);
+}
+
+FILE *
+gp_open_scratch_file_rm(const gs_memory_t *mem,
+                        const char        *prefix,
+                              char        *fname,
+                        const char        *mode)
+{
+    return gp_open_scratch_file_generic(mem, prefix, fname, mode, 1);
+}
+
 /* Open a file with the given name, as a stream of uninterpreted bytes. */
 FILE *
 gp_fopen(const char *fname, const char *mode)
 {
-#ifdef WINDOWS_NO_UNICODE
+#ifdef GS_NO_UTF8
     return fopen(fname, mode);
 #else
     int len = utf8_to_wchar(NULL, fname);
@@ -739,6 +805,87 @@ gp_fopen(const char *fname, const char *mode)
 #endif
 }
 
+int gp_stat(const char *path, struct _stat *buf)
+{
+#ifdef GS_NO_UTF8
+    return stat(path, buf);
+#else
+    int len = utf8_to_wchar(NULL, path);
+    wchar_t *uni;
+    int ret;
+
+    if (len <= 0)
+        return -1;
+    uni = malloc(len*sizeof(wchar_t));
+    if (uni == NULL)
+        return -1;
+    utf8_to_wchar(uni, path);
+    ret = _wstat(uni, buf);
+    free(uni);
+    return ret;
+#endif
+}
+
+/* test whether gp_fdup is supported on this platform  */
+int gp_can_share_fdesc(void)
+{
+    return 1;
+}
+
+/* Create a second open FILE on the basis of a given one */
+FILE *gp_fdup(FILE *f, const char *mode)
+{
+    int fd = fileno(f);
+    if (fd < 0)
+        return NULL;
+
+    fd = dup(fd);
+    if (fd < 0)
+        return NULL;
+
+    return fdopen(fd, mode);
+}
+
+/* Read from a specified offset within a FILE into a buffer */
+int gp_fpread(char *buf, uint count, int64_t offset, FILE *f)
+{
+    OVERLAPPED overlapped;
+    DWORD ret;
+    HANDLE hnd = (HANDLE)_get_osfhandle(fileno(f));
+
+    if (hnd == INVALID_HANDLE_VALUE)
+        return -1;
+
+    memset(&overlapped, 0, sizeof(OVERLAPPED));
+    overlapped.Offset = (DWORD)offset;
+    overlapped.OffsetHigh = (DWORD)(offset >> 32);
+
+    if (!ReadFile((HANDLE)hnd, buf, count, &ret, &overlapped))
+        return -1;
+
+    return ret;
+}
+
+/* Write to a specified offset within a FILE from a buffer */
+int gp_fpwrite(char *buf, uint count, int64_t offset, FILE *f)
+{
+    OVERLAPPED overlapped;
+    DWORD ret;
+    HANDLE hnd = (HANDLE)_get_osfhandle(fileno(f));
+
+    if (hnd == INVALID_HANDLE_VALUE)
+        return -1;
+
+    memset(&overlapped, 0, sizeof(OVERLAPPED));
+    overlapped.Offset = (DWORD)offset;
+    overlapped.OffsetHigh = (DWORD)(offset >> 32);
+
+    if (!WriteFile((HANDLE)hnd, buf, count, &ret, &overlapped))
+        return -1;
+
+    return ret;
+}
+
 /* ------ Font enumeration ------ */
 
  /* This is used to query the native os for a list of font names and
@@ -825,8 +972,26 @@ int gp_fseek_64(FILE *strm, gs_offset_t offset, int origin)
 #endif
 }
 
+bool gp_fseekable (FILE *f)
+{
+    struct __stat64 s;
+    int fno;
+    
+    fno = fileno(f);
+    if (fno < 0)
+        return(false);
+    
+    if (_fstat64(fno, &s) < 0)
+        return(false);
+
+    return((bool)S_ISREG(s.st_mode));
+}
+
 /* -------------------------  _snprintf -----------------------------*/
 
+#if defined(_MSC_VER) && _MSC_VER>=1900 /* VS 2014 and later have (finally) snprintf */
+#  define STDC99
+#else
 /* Microsoft Visual C++ 2005  doesn't properly define snprintf,
    which is defined in the C standard ISO/IEC 9899:1999 (E) */
 
@@ -844,3 +1009,4 @@ int snprintf(char *buffer, size_t count, const char *format, ...)
     } else
         return 0;
 }
+#endif
