@@ -16,10 +16,14 @@
 
 /* "Unix-like" file system platform routines for Ghostscript */
 
+/* prevent gp.h from defining fopen */
+#define fopen fopen
+
 #include "stdio_.h"             /* for FILENAME_MAX */
 #include "memory_.h"
 #include "string_.h"
 #include "gx.h"
+
 #include "gp.h"
 #include "gpmisc.h"
 #include "gsstruct.h"
@@ -147,6 +151,89 @@ gp_fopen(const char *fname, const char *mode)
     return fopen(fname, mode);
 }
 
+int gp_stat(const char *path, struct stat *buf)
+{
+    return stat(path, buf);
+}
+
+int gp_can_share_fdesc(void)
+{
+#if defined(HAVE_PREAD_PWRITE) && HAVE_PREAD_PWRITE == 1
+    return 1;
+#else
+    return 0;	/* can't share FILE * descriptors w/o pread due to seek..read..seek */
+#endif
+}
+
+FILE *gp_open_scratch_file_rm(const gs_memory_t *mem,
+                              const char        *prefix,
+                                    char         fname[gp_file_name_sizeof],
+                              const char        *mode)
+{
+    FILE *f = gp_open_scratch_file_generic(mem, prefix, fname, mode, false);
+    /* Unlink file immediately to avoid it being left around if the program
+     * is killed. On this platform readers access temp files by cloning the
+     * FILE pointer and without accessing the file by name */
+    if (f)
+        unlink(fname);
+    return f;
+}
+
+FILE *gp_fdup(FILE *f, const char *mode)
+{
+    int fd = fileno(f);
+    if (fd < 0)
+        return NULL;
+    fd = dup(fd);
+    if (fd < 0)
+        return NULL;
+    return fdopen(fd, mode);
+}
+
+int gp_fpread(char *buf, uint count, int64_t offset, FILE *f)
+{
+#if defined(HAVE_PREAD_PWRITE) && HAVE_PREAD_PWRITE == 1
+    return pread(fileno(f), buf, count, offset);
+#else
+    int c;
+    int64_t os, curroff = gp_ftell_64(f);
+    if (curroff < 0) return curroff;
+    
+    os = gp_fseek_64(f, offset, 0);
+    if (os < 0) return os;
+    
+    c = fread(buf, 1, count, f);
+    if (c < 0) return c;
+    
+    os = gp_fseek_64(f, curroff, 0);
+    if (os < 0) return os;
+    
+    return c;
+#endif
+}
+
+int gp_fpwrite(char *buf, uint count, int64_t offset, FILE *f)
+{
+#if defined(HAVE_PREAD_PWRITE) && HAVE_PREAD_PWRITE == 1
+    return pwrite(fileno(f), buf, count, offset);
+#else
+    int c;
+    int64_t os, curroff = gp_ftell_64(f);
+    if (curroff < 0) return curroff;
+    
+    os = gp_fseek_64(f, offset, 0);
+    if (os < 0) return os;
+    
+    c = fwrite(buf, 1, count, f);
+    if (c < 0) return c;
+    
+    os = gp_fseek_64(f, curroff, 0);
+    if (os < 0) return os;
+    
+    return c;
+#endif
+}
+
 /* Set a file into binary or text mode. */
 int
 gp_setmode_binary(FILE * pfile, bool mode)
@@ -269,6 +356,11 @@ gp_enumerate_files_init(const char *pat, uint patlen, gs_memory_t * mem)
     /* pattern and work could be allocated as strings, */
     /* but it's simpler for GC and freeing to allocate them as bytes. */
 
+    pfen->memory = mem;
+    pfen->dstack = 0;
+    pfen->first_time = true;
+    pfen->patlen = patlen;
+    pfen->work = 0;
     pfen->pattern =
         (char *)gs_alloc_bytes(mem, patlen + 1,
                                "gp_enumerate_files(pattern)");
@@ -313,10 +405,6 @@ gp_enumerate_files_init(const char *pat, uint patlen, gs_memory_t * mem)
         pfen->worklen = p - work;
     }
 
-    pfen->memory = mem;
-    pfen->dstack = 0;
-    pfen->first_time = true;
-    pfen->patlen = patlen;
     return pfen;
 }
 
@@ -356,11 +444,13 @@ gp_enumerate_files_next(file_enum * pfen, char *ptr, uint maxlen)
             worklen = p - work;
         } else
             worklen = 0;
-        p = rchr(pattern, '/', pathead);
-        if (p != 0)
-            pathead = p - pattern;
-        else
-            pathead = 0;
+        if (pathead != pfen->patlen) {
+            p = rchr(pattern, '/', pathead);
+            if (p != 0)
+                pathead = p - pattern;
+            else
+                pathead = 0;
+        }
 
         if (popdir(pfen)) {     /* Back up the directory tree. */
             if_debug1m('e', pfen->memory, "[e]file_enum:Dir popped '%s'\n", work);
@@ -394,7 +484,7 @@ gp_enumerate_files_next(file_enum * pfen, char *ptr, uint maxlen)
         goto top;
 
     /* Perhaps descend into subdirectories */
-    if (pathead < pfen->patlen) {
+    if (pathead < maxlen) {
         DIR *dp;
 
         if (((stat(work, &stbuf) >= 0)
@@ -404,7 +494,7 @@ gp_enumerate_files_next(file_enum * pfen, char *ptr, uint maxlen)
          * we'll be able to list it anyway.
          * If it isn't or we can't, no harm done. */
              : 0))
-            goto top;
+            goto winner;
 
         if (pfen->patlen == pathead + 1) {      /* Listing "foo/?/" -- return this entry */
             /* if it's a directory. */
@@ -457,7 +547,7 @@ gp_enumerate_files_next(file_enum * pfen, char *ptr, uint maxlen)
                 d->entry = pfen->dirp;
                 pfen->dstack = d;
             } else
-                DO_NOTHING;     /* >>> e_VMerror!!! */
+                DO_NOTHING;     /* >>> gs_error_VMerror!!! */
 
             if_debug1m('e', pfen->memory, "[e]file_enum:Dir pushed '%s'\n",
                        work);
@@ -470,7 +560,8 @@ gp_enumerate_files_next(file_enum * pfen, char *ptr, uint maxlen)
     /* We have a winner! */
     pfen->worklen = worklen;
     pfen->pathead = pathead;
-    memcpy(ptr, work, len);
+    memcpy(ptr, work, len > maxlen ? maxlen : len);
+
     return len;
 }
 
@@ -539,3 +630,18 @@ int gp_fseek_64(FILE *strm, int64_t offset, int origin)
     return fseeko(strm, offset1, origin);
 #endif
 }
+
+bool gp_fseekable (FILE *f)
+{
+    struct stat s;
+    int fno;
+    
+    fno = fileno(f);
+    if (fno < 0)
+        return(false);
+    
+    if (fstat(fno, &s) < 0)
+        return(false);
+
+    return((bool)S_ISREG(s.st_mode));
+}
