@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2008 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gp_unix_cache.c 11089 2010-04-20 18:52:36Z robin $ */
+
 /* Generic POSIX persistent-cache implementation for Ghostscript */
 
 #include "stdio_.h"
@@ -20,7 +22,7 @@
 #include <stdlib.h> /* should use gs_malloc() instead */
 #include "gconfigd.h"
 #include "gp.h"
-#include "md5.h"
+#include "gsmd5.h"
 
 /* ------ Persistent data cache types ------*/
 
@@ -79,7 +81,7 @@ static char *gp_cache_prefix(void)
     if (plen >= 1 && prefix[0] == '~') {
         char *home, *path;
         int hlen = 0;
-	unsigned int pathlen = 0;
+        unsigned int pathlen = 0;
         gp_file_name_combine_result result;
 
         if (gp_getenv("HOME", (char *)NULL, &hlen) < 0) {
@@ -102,6 +104,7 @@ static char *gp_cache_prefix(void)
                 prefix = path;
             } else {
                 dlprintf1("file_name_combine failed with code %d\n", result);
+                free(path);
             }
             free(home);
         }
@@ -213,15 +216,28 @@ static int gp_cache_loaditem(FILE *file, gp_cache_entry *item, gp_cache_alloc al
     unsigned char version;
     unsigned char *filekey;
     int len, keylen;
+    size_t read;
 
-    (void)fread(&version, 1, 1, file);
+    read = fread(&version, 1, 1, file);
+    if (read != 1) {
+#ifdef DEBUG_CACHE
+        dlprintf("pcache file read failed (1)\n");
+#endif
+        return -1;
+    }
     if (version != GP_CACHE_VERSION) {
 #ifdef DEBUG_CACHE
         dlprintf2("pcache file version mismatch (%d vs expected %d)\n", version, GP_CACHE_VERSION);
 #endif
         return -1;
     }
-    (void)fread(&keylen, 1, sizeof(keylen), file);
+    read = fread(&keylen, 1, sizeof(keylen), file);
+    if (read != sizeof(keylen)) {
+#ifdef DEBUG_CACHE
+        dlprintf("pcache file read failed (2)\n");
+#endif
+        return -1;
+    }
     if (keylen != item->keylen) {
 #ifdef DEBUG_CACHE
         dlprintf2("pcache file has correct hash but wrong key length (%d vs %d)\n",
@@ -234,7 +250,13 @@ static int gp_cache_loaditem(FILE *file, gp_cache_entry *item, gp_cache_alloc al
         dprintf("pcache: couldn't allocate file key!\n");
         return -1;
     }
-    (void)fread(filekey, 1, keylen, file);
+    read = fread(filekey, 1, keylen, file);
+    if (read != keylen) {
+#ifdef DEBUG_CACHE
+        dlprintf("pcache file read failed (3)\n");
+#endif
+        return -1;
+    }
     if (memcmp(filekey, item->key, keylen)) {
 #ifdef DEBUG_CACHE
         dlprintf("pcache file has correct hash but doesn't match the full key\n");
@@ -246,7 +268,13 @@ static int gp_cache_loaditem(FILE *file, gp_cache_entry *item, gp_cache_alloc al
     }
     free(filekey);
 
-    (void)fread(&len, 1, sizeof(len), file);
+    read = fread(&len, 1, sizeof(len), file);
+    if (read != sizeof(len)) {
+#ifdef DEBUG_CACHE
+        dlprintf("pcache file read failed (4)\n");
+#endif
+        return -1;
+    }
 #ifdef DEBUG_CACHE
     dlprintf2("key matches file with version %d, data length %d\n", version, len);
 #endif
@@ -322,7 +350,6 @@ gp_cache_write_entry(FILE *file, gp_cache_entry *item)
     return 0;
 }
 
-
 /* insert a buffer under a (type, key) pair */
 int gp_cache_insert(int type, byte *key, int keylen, void *buffer, int buflen)
 {
@@ -343,7 +370,7 @@ int gp_cache_insert(int type, byte *key, int keylen, void *buffer, int buflen)
         outfn[len-1] = '\0';
     }
 
-    in = fopen(infn, "r");
+    in = gp_fopen(infn, "r");
     if (in == NULL) {
         dlprintf1("pcache: unable to open '%s'\n", infn);
         free(prefix);
@@ -351,7 +378,7 @@ int gp_cache_insert(int type, byte *key, int keylen, void *buffer, int buflen)
         free(outfn);
         return -1;
     }
-    out = fopen(outfn, "w");
+    out = gp_fopen(outfn, "w");
     if (out == NULL) {
         dlprintf1("pcache: unable to open '%s'\n", outfn);
         fclose(in);
@@ -377,7 +404,8 @@ int gp_cache_insert(int type, byte *key, int keylen, void *buffer, int buflen)
 
     /* save it to disk */
     path = gp_cache_itempath(prefix, &item);
-    file = fopen(path, "wb");
+    file = gp_fopen(path, "wb");
+    free(path);
     if (file != NULL) {
         gp_cache_saveitem(file, &item);
         fclose(file);
@@ -435,7 +463,7 @@ int gp_cache_query(int type, byte* key, int keylen, void **buffer,
         outfn[len-1] = '\0';
     }
 
-    in = fopen(infn, "r");
+    in = gp_fopen(infn, "r");
     if (in == NULL) {
         dlprintf1("pcache: unable to open '%s'\n", infn);
         free(prefix);
@@ -443,7 +471,7 @@ int gp_cache_query(int type, byte* key, int keylen, void **buffer,
         free(outfn);
         return -1;
     }
-    out = fopen(outfn, "w");
+    out = gp_fopen(outfn, "w");
     if (out == NULL) {
         dlprintf1("pcache: unable to open '%s'\n", outfn);
         fclose(in);
@@ -466,7 +494,8 @@ int gp_cache_query(int type, byte* key, int keylen, void **buffer,
 
     /* look for it on the disk */
     path = gp_cache_itempath(prefix, &item);
-    file = fopen(path, "rb");
+    file = gp_fopen(path, "rb");
+    free(path);
     if (file != NULL) {
         hit = gp_cache_loaditem(file, &item, alloc, userdata);
         fclose(file);
