@@ -33,6 +33,8 @@
 #include "gsovrc.h"
 #include "stream.h"
 #include "gsicc_cache.h"
+#include "gxdevice.h"
+#include "gxcie.h"
 
 /* ---------------- Color space ---------------- */
 
@@ -60,7 +62,7 @@ const gs_color_space_type gs_color_space_type_Separation = {
     gx_set_overprint_Separation,
     gx_final_Separation, gx_no_adjust_color_count,
     gx_serialize_Separation,
-    gx_cspace_is_linear_default
+    gx_cspace_is_linear_default, gx_polarity_subtractive
 };
 
 /* GC procedures */
@@ -132,9 +134,11 @@ gx_install_Separation(gs_color_space * pcs, gs_state * pgs)
          * Give the device an opportunity to capture equivalent colors for any
          * spot colors which might be present in the color space.
          */
-        if (code >= 0)
-            code = dev_proc(pgs->device, update_spot_equivalent_colors)
+        if (code >= 0) {
+            if (dev_proc(pgs->device, update_spot_equivalent_colors))
+               code = dev_proc(pgs->device, update_spot_equivalent_colors)
                                                             (pgs->device, pgs);
+        }
     }
     return code;
 }
@@ -149,20 +153,23 @@ gx_set_overprint_Separation(const gs_color_space * pcs, gs_state * pgs)
     
     dev_proc(dev, get_profile)(dev, &(dev_profile));
     if (pcmap->use_alt_cspace)
-        if (dev_profile->sim_overprint)
+        if (dev_profile->sim_overprint && 
+            dev->color_info.polarity == GX_CINFO_POLARITY_SUBTRACTIVE &&
+            !gx_device_must_halftone(dev))
             return gx_simulated_set_overprint(pcs->base_space, pgs);
         else
             return gx_spot_colors_set_overprint(pcs->base_space, pgs);
     else {
         gs_overprint_params_t   params;
 
+        /* We should not have to blend if we don't need the alternate tint transform */
+        params.blendspot = false;
         params.retain_any_comps = pgs->overprint &&
                                   pcs->params.separation.sep_type != SEP_ALL;
         if (params.retain_any_comps) {
             params.retain_spot_comps = false;
             params.drawn_comps = 0;
             params.k_value = 0;
-            params.blendspot = false;
             if (pcs->params.separation.sep_type != SEP_NONE) {
                 int     mcomp = pcmap->color_map[0];
 
@@ -170,7 +177,6 @@ gx_set_overprint_Separation(const gs_color_space * pcs, gs_state * pgs)
                     gs_overprint_set_drawn_comp( params.drawn_comps, mcomp);
             }
         }
-
         pgs->effective_overprint_mode = 0;
         return gs_state_update_overprint(pgs, &params);
     }
@@ -297,10 +303,21 @@ gx_remap_Separation(const gs_client_color * pcc, const gs_color_space * pcs,
                        gs_color_select_t select)
 {
     int code = 0;
+    bool mapped = false;
 
-    if (pcs->params.separation.sep_type != SEP_NONE)
-        code = gx_default_remap_color(pcc, pcs, pdc, pis, dev, select);
-    else {
+    if (pcs->params.separation.sep_type != SEP_NONE) {
+        /* First see if we want to do a direct remap with a named color table.
+           Note that this call occurs regardless of the alt color space boolean
+           value.  A decision can be made in gsicc_transform_named_color to
+           return false if you don't want those named colors to be mapped */
+        if (pcs->params.separation.sep_type == SEP_OTHER &&
+            pis->icc_manager->device_named != NULL) {
+            /* Try to apply the direct replacement */
+            mapped = gx_remap_named_color(pcc, pcs, pdc, pis, dev, select);
+        }
+        if (!mapped)
+            code = gx_default_remap_color(pcc, pcs, pdc, pis, dev, select);
+    } else {
         color_set_null(pdc);
     }
     /* Save original color space and color info into dev color */
@@ -317,46 +334,10 @@ gx_concretize_Separation(const gs_client_color *pc, const gs_color_space *pcs,
     gs_client_color cc;
     gs_color_space *pacs = pcs->base_space;
     bool is_lab;
-    int k;
-    int num_des_comps = dev->color_info.num_components;
-    gsicc_namedcolor_t named_color;
 
     if (pcs->params.separation.sep_type == SEP_OTHER &&
         pcs->params.separation.use_alt_cspace) {
         gs_device_n_map *map = pcs->params.separation.map;
-        /* First see if we have a named color object that we can use to try
-           to map from the spot color into device values.  */
-        if (pis->icc_manager->device_named != NULL) {
-            /* There is a table present.  If we have the colorant name
-               then get the device values */
-            gx_color_value device_values[GX_DEVICE_COLOR_MAX_COMPONENTS];
-            const gs_separation_name name = pcs->params.separation.sep_name;
-            byte *pname;
-            uint name_size;
-            gsicc_rendering_param_t rendering_params;
-
-            /* Define the rendering intents. */
-            rendering_params.black_point_comp = pis->blackptcomp;
-            rendering_params.graphics_type_tag = dev->graphics_type_tag;
-            rendering_params.override_icc = false;
-            rendering_params.preserve_black = gsBKPRESNOTSPECIFIED;
-            rendering_params.rendering_intent = pis->renderingintent;
-            rendering_params.cmm = gsCMM_DEFAULT;
-            pcs->params.separation.get_colorname_string(pis->memory, name,
-                                                &pname, &name_size);
-            /* Make the name structure and initialized it */
-            named_color.colorant_name = (char*) pname;
-            named_color.name_size = name_size;
-            code = gsicc_transform_named_color(pc->paint.values, &named_color,
-                                               1, device_values, pis, dev, NULL,
-                                               &rendering_params);
-            if (code == 0) {
-                for (k = 0; k < num_des_comps; k++){
-                    pconc[k] = float2frac(((float) device_values[k])/65535.0);
-                }
-                return(0);
-            }
-        }
         /* Check the 1-element cache first. */
         if (map->cache_valid && map->tint[0] == pc->paint.values[0]) {
             int i, num_out = gs_color_space_num_components(pacs);
@@ -373,7 +354,9 @@ gx_concretize_Separation(const gs_client_color *pc, const gs_color_space *pcs,
         (*pacs->type->restrict_color)(&cc, pacs);
         /* First check if this was PS based. */
         if (gs_color_space_is_PSCIE(pacs)) {
-            /* If we have not yet create the profile do that now */
+            /* We may have to rescale data to 0 to 1 range */
+            rescale_cie_colors(pacs, &cc);
+            /* If we have not yet created the profile do that now */
             if (pacs->icc_equivalent == NULL) {
                 gs_colorspace_set_icc_equivalent(pacs, &(is_lab), pis->memory);
             }
@@ -388,8 +371,7 @@ gx_concretize_Separation(const gs_client_color *pc, const gs_color_space *pcs,
             cc.paint.values[2] = (cc.paint.values[2]+128)/255.0;
         }
         return cs_concretize_color(&cc, pacs, pconc, pis, dev);
-    }
-    else {
+    } else {
         pconc[0] = gx_unit_frac(pc->paint.values[0]);
     }
     return 0;
@@ -413,8 +395,8 @@ gx_remap_concrete_Separation(const frac * pconc,  const gs_color_space * pcs,
 
         return (*pacs->type->remap_concrete_color)
                                 (pconc, pacs, pdc, pis, dev, select);
-    }
-    else {
+    } else {
+        /* We need to determine if we did a direct color replacement */
         gx_remap_concrete_separation(pconc[0], pdc, pis, dev, select);
         return 0;
     }
