@@ -152,7 +152,7 @@ update_Separation_spot_equivalent_cmyk_colors(gx_device * pdev,
 
 /* This is used for getting the equivalent CMYK values for the spots that
    may exist in a DeviceN output ICC profile */
-static void
+static int
 update_ICC_spot_equivalent_cmyk_colors(gx_device * pdev,
                     const gs_state * pgs, const gs_color_space * pcs,
                     gs_devn_params * pdevn_params,
@@ -165,6 +165,8 @@ update_ICC_spot_equivalent_cmyk_colors(gx_device * pdev,
 
 
     code = dev_proc(pdev, get_profile)(pdev, &dev_profile);
+    if (code < 0)
+        return code;
     /*
      * Check if the ICC spot names matche any of the
      * separations for which we need an equivalent CMYK color.
@@ -197,9 +199,9 @@ update_ICC_spot_equivalent_cmyk_colors(gx_device * pdev,
             }
         }
     }
+    return 0;
 }
 
-
 static void
 update_DeviceN_spot_equivalent_cmyk_colors(gx_device * pdev,
                     const gs_state * pgs, const gs_color_space * pcs,
@@ -282,7 +284,7 @@ static bool check_all_colors_known(int num_spot,
 }
 
 /* If possible, update the equivalent CMYK color for a spot color */
-void
+int
 update_spot_equivalent_cmyk_colors(gx_device * pdev, const gs_state * pgs,
     gs_devn_params * pdevn_params, equivalent_cmyk_color_params * pparams)
 {
@@ -291,14 +293,17 @@ update_spot_equivalent_cmyk_colors(gx_device * pdev, const gs_state * pgs,
     int code;
 
     code = dev_proc(pdev, get_profile)(pdev, &dev_profile);
+    if (code < 0)
+        return code;
+
     /* If all of the color_info is valid then there is nothing to do. */
     if (pparams->all_color_info_valid)
-        return;
+        return 0;
 
     /* Verify that we really have some separations. */
     if (pdevn_params->separations.num_separations == 0) {
         pparams->all_color_info_valid = true;
-        return;
+        return 0;
     }
     /*
      * Verify that the given color space is a Separation or a DeviceN color
@@ -322,17 +327,21 @@ update_spot_equivalent_cmyk_colors(gx_device * pdev, const gs_state * pgs,
             dev_profile->spotnames != NULL) {
             /* In this case, we are trying to set up the equivalent colors
                for the spots in the output ICC profile */
-            update_ICC_spot_equivalent_cmyk_colors(pdev, pgs, pcs, pdevn_params, 
-                                                   pparams);
+            code = update_ICC_spot_equivalent_cmyk_colors(pdev, pgs, pcs,
+                                                          pdevn_params,
+                                                          pparams);
+            if (code < 0)
+                return code;
             pparams->all_color_info_valid = check_all_colors_known
                     (pdevn_params->separations.num_separations, pparams);
         }
     }
+    return 0;
 }
 
 static void
 save_spot_equivalent_cmyk_color(int sep_num,
-                equivalent_cmyk_color_params * pparams, frac cmyk[4])
+                equivalent_cmyk_color_params * pparams, const frac cmyk[4])
 {
     pparams->color[sep_num].c = cmyk[0];
     pparams->color[sep_num].m = cmyk[1];
@@ -433,11 +442,19 @@ cmap_separation_capture_cmyk_color(frac all, gx_device_color * pdc,
     dmprintf(pis->memory, "cmap_separation_capture_cmyk_color - this routine should not be executed\n");
 }
 
+/* The call to this is actually going to occur if we happen to be using a 
+   named color profile and doing a replacement.  Since the destination profile
+   will have been CMYK based during the swap out to find the equivalent color, we can
+   go ahead and just grab the cmyk portion */
 static void
 cmap_devicen_capture_cmyk_color(const frac * pcc, gx_device_color * pdc,
      const gs_imager_state * pis, gx_device * dev, gs_color_select_t select)
 {
-    dmprintf(pis->memory, "cmap_devicen_capture_cmyk_color - this routine should not be executed\n");
+    equivalent_cmyk_color_params * pparams =
+        ((color_capture_device *)dev)->pequiv_cmyk_colors;
+    int sep_num = ((color_capture_device *)dev)->sep_num;
+
+    save_spot_equivalent_cmyk_color(sep_num, pparams, pcc);
 }
 
 /*
@@ -452,13 +469,22 @@ capture_spot_equivalent_cmyk_colors(gx_device * pdev, const gs_state * pgs,
     gs_imager_state temp_state = *((const gs_imager_state *)pgs);
     color_capture_device temp_device = { 0 };
     gx_device_color dev_color;
-    gsicc_rendering_param_t render_cond;   
-    int code;
+    gsicc_rendering_param_t render_cond;
     cmm_dev_profile_t *dev_profile;
     cmm_profile_t *curr_output_profile;
-    cmm_dev_profile_t temp_profile;
+    cmm_dev_profile_t temp_profile = {	/* Initialize to 0's/NULL's */
+                          { 0 } /* device_profile[] */, 0 /* proof_profile */,
+                          0 /* link_profile */, 0 /* oi_profile */, 
+                          0 /* postren_profile */,
+                          { {0} } /* rendercond[] */, 0 /* devicegraytok */, 
+                          0 /* graydection */, 0 /* pageneutralcolor */, 
+                          0 /* usefastcolor */, 0 /* supports_devn */,
+                          0 /* sim_overprint */, 0 /* spotnames */, 
+                          0 /* prebandthreshold */, 0 /* memory */, 
+                          { 0 } /* rc_header */
+                          };
 
-    code = dev_proc(pdev, get_profile)(pdev, &dev_profile);
+    dev_proc(pdev, get_profile)(pdev, &dev_profile);
     gsicc_extract_profile(pdev->graphics_type_tag,
                           dev_profile, &(curr_output_profile), &render_cond);
     /*
@@ -470,18 +496,15 @@ capture_spot_equivalent_cmyk_colors(gx_device * pdev, const gs_state * pgs,
     temp_device.color_info = pdev->color_info;
     temp_device.sep_num = sep_num;
     temp_device.pequiv_cmyk_colors = pparams;
-    temp_device.memory = pgs->memory; 
+    temp_device.memory = pgs->memory;
 
     temp_profile.usefastcolor = false;  /* This avoids a few headaches */
     temp_profile.prebandthreshold = true;
     temp_profile.supports_devn = false;
-    temp_profile.device_profile[1] = NULL;
-    temp_profile.device_profile[2] = NULL;
-    temp_profile.device_profile[2] = NULL;
-    temp_profile.link_profile = NULL;
-    temp_profile.proof_profile = NULL;
-    temp_profile.spotnames = NULL;
-    temp_profile.oi_profile = NULL;
+    temp_profile.rendercond[0] = render_cond;
+    temp_profile.rendercond[1] = render_cond;
+    temp_profile.rendercond[2] = render_cond;
+    temp_profile.rendercond[3] = render_cond;
     temp_device.icc_struct = &temp_profile;
 
     /* The equivalent CMYK colors are used for
@@ -512,6 +535,14 @@ capture_spot_equivalent_cmyk_colors(gx_device * pdev, const gs_state * pgs,
     temp_state.color_component_map.use_alt_cspace = true;
 
     /* Now capture the color */
-    pcs->type->remap_color (pcc, pcs, &dev_color, &temp_state, 
+    pcs->type->remap_color (pcc, pcs, &dev_color, &temp_state,
                             (gx_device *)&temp_device, gs_color_select_texture);
 }
+
+/* Used for detecting if we are trying to find the equivalant color during
+   named color replacement */
+bool
+named_color_equivalent_cmyk_colors(const gs_imager_state * pis)
+{
+    return pis->cmap_procs == &cmap_capture_cmyk_color;
+}
