@@ -77,6 +77,7 @@ public_st_cmap_lookup_range_element();
  */
 
 #ifndef GS_THREADSAFE
+#ifdef DEBUG
 static void
 print_msg_str_in_range(const byte *str,
                        const byte *key_lo, const byte *key_hi,
@@ -90,6 +91,7 @@ print_msg_str_in_range(const byte *str,
     dlprintf("\n");
 }
 #endif
+#endif
 
 static int
 gs_cmap_get_shortest_chr(const gx_code_map_t * pcmap, uint *pfidx)
@@ -373,7 +375,6 @@ gs_cmap_adobe1_decode_next(const gs_cmap_t * pcmap_in,
 
     uint pm_index;
     uint pm_fidx;
-    gs_char pm_chr;
 
     /* For first, check defined map */
     if_debug0('J', "[J]GCDN() check def CMap\n");
@@ -388,7 +389,6 @@ gs_cmap_adobe1_decode_next(const gs_cmap_t * pcmap_in,
     /* save partially matched results */
     pm_index = *pindex;
     pm_fidx = *pfidx;
-    pm_chr = *pchr;
 
     /* check notdef map. */
     if_debug0('J', "[J]GCDN() check notdef CMap\n");
@@ -425,7 +425,6 @@ gs_cmap_adobe1_decode_next(const gs_cmap_t * pcmap_in,
          * at the end of Fonts chapter.
          */
 
-        const byte *str = pstr->data + save_index;
         uint ssize = pstr->size - save_index;
         int chr_size_shortest =
                 gs_cmap_get_shortest_chr(&pcmap->def, pfidx);
@@ -437,6 +436,7 @@ gs_cmap_adobe1_decode_next(const gs_cmap_t * pcmap_in,
 #ifndef GS_THREADSAFE
 #ifdef DEBUG
             if (gs_debug_c('J')) {
+                const byte *str = pstr->data + save_index;
                 dlprintf1("[J]GCDN() no partial match, skip %d byte (",
                                                chr_size_shortest);
                 debug_print_string_hex_nomem(str, chr_size_shortest);
