@@ -219,7 +219,7 @@ gs_font_dir_alloc2(gs_memory_t * struct_mem, gs_memory_t * bits_mem)
 {
     gs_font_dir *pdir = 0;
 
-#if !arch_small_memory
+#if !ARCH_SMALL_MEMORY
 #  ifdef DEBUG
     if (!gs_debug_c('.'))
 #  endif
@@ -251,6 +251,7 @@ gs_font_dir_alloc2_limits(gs_memory_t * struct_mem, gs_memory_t * bits_mem,
 
     if (pdir == 0)
         return 0;
+    memset(pdir, 0, sizeof(*pdir));
     code = gx_char_cache_alloc(struct_mem, bits_mem, pdir,
                                bmax, mmax, cmax, upper);
     if (code < 0) {
@@ -263,7 +264,7 @@ gs_font_dir_alloc2_limits(gs_memory_t * struct_mem, gs_memory_t * bits_mem,
     pdir->smax = smax;
     pdir->align_to_pixels = false;
     pdir->glyph_to_unicode_table = NULL;
-    pdir->grid_fit_tt = 2;
+    pdir->grid_fit_tt = 1;
     pdir->memory = struct_mem;
     pdir->tti = 0;
     pdir->ttm = 0;
@@ -441,7 +442,7 @@ gs_font_find_similar(const gs_font_dir * pdir, const gs_font **ppfont,
 
 /* scalefont */
 int
-gs_scalefont(gs_font_dir * pdir, const gs_font * pfont, floatp scale,
+gs_scalefont(gs_font_dir * pdir, const gs_font * pfont, double scale,
              gs_font ** ppfont)
 {
     gs_matrix mat;
@@ -858,6 +859,7 @@ gs_default_font_info(gs_font *font, const gs_point *pscale, int members,
              ) {
             gs_glyph_info_t glyph_info;
 
+            memset(&glyph_info, 0x00, sizeof(gs_glyph_info_t));
             code = font->procs.glyph_info(font, glyph, pmat,
                                           (GLYPH_INFO_WIDTH0 << wmode),
                                           &glyph_info);
