@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gsht.c 12212 2011-02-27 23:26:10Z mvrhel $ */
+
 /* setscreen operator for Ghostscript library */
 #include "memory_.h"
 #include "string_.h"
@@ -24,44 +26,11 @@
 #include "gzstate.h"
 #include "gxdevice.h"           /* for gzht.h */
 #include "gzht.h"
-#include "gswts.h"
 #include "gxfmap.h"             /* For effective transfer usage in threshold */
 
-#define TRANSFER_INVERSE_SIZE 1024
-#define TRANSFER_IN_THRESHOLDS 0
-
-/* Used in threshold from tiles construction */
-static const uint32_t bit_order[32]={
-#if arch_is_big_endian
-        0x80000000, 0x40000000, 0x20000000, 0x10000000, 0x08000000, 0x04000000, 0x02000000, 0x01000000,
-        0x00800000, 0x00400000, 0x00200000, 0x00100000, 0x00080000, 0x00040000, 0x00020000, 0x00010000,
-        0x00008000, 0x00004000, 0x00002000, 0x00001000, 0x00000800, 0x00000400, 0x00000200, 0x00000100,
-        0x00000080, 0x00000040, 0x00000020, 0x00000010, 0x00000008, 0x00000004, 0x00000002, 0x00000001
-#else
-        0x00000080, 0x00000040, 0x00000020, 0x00000010, 0x00000008, 0x00000004, 0x00000002, 0x00000001,
-        0x00008000, 0x00004000, 0x00002000, 0x00001000, 0x00000800, 0x00000400, 0x00000200, 0x00000100,
-        0x00800000, 0x00400000, 0x00200000, 0x00100000, 0x00080000, 0x00040000, 0x00020000, 0x00010000,
-        0x80000000, 0x40000000, 0x20000000, 0x10000000, 0x08000000, 0x04000000, 0x02000000, 0x01000000
-#endif
-    };
-
 /* Forward declarations */
 void gx_set_effective_transfer(gs_state *);
 
-/*
- * *HACK ALERT*
- *
- * Value stored in the width field of a well-tempered screen halftone
- * order, to indicate that the wts field of this order points to the
- * same structure as an earlier order. This is used to suppress
- * multiple realeases of shared wts_screen_t orders.
- *
- * The width field is available for this purpose at it is nominally
- * unused in a well-tempered screening halftone.
- */
-static const ushort    ht_wts_suppress_release = (ushort)(-1);
-
-
 /* Structure types */
 public_st_ht_order();
 private_st_ht_order_component();
@@ -214,7 +183,7 @@ gx_imager_setscreenphase(gs_imager_state * pis, int x, int y,
         for (i = 0; i < gs_color_select_count; ++i)
             gx_imager_setscreenphase(pis, x, y, (gs_color_select_t) i);
         return 0;
-    } else if (select < 0 || select >= gs_color_select_count)
+    } else if ((int)select < 0 || (int)select >= gs_color_select_count)
         return_error(gs_error_rangecheck);
     pis->screen_phase[select].x = x;
     pis->screen_phase[select].y = y;
@@ -242,7 +211,7 @@ int
 gs_currentscreenphase_pis(const gs_imager_state * pis, gs_int_point * pphase,
                       gs_color_select_t select)
 {
-    if (select < 0 || select >= gs_color_select_count)
+    if ((int)select < 0 || (int)select >= gs_color_select_count)
         return_error(gs_error_rangecheck);
     *pphase = pis->screen_phase[select];
     return 0;
@@ -286,17 +255,12 @@ gx_ht_process_screen_memory(gs_screen_enum * penum, gs_state * pgs,
  * Internal procedure to allocate and initialize either an internally
  * generated or a client-defined halftone order.  For spot halftones,
  * the client is responsible for calling gx_compute_cell_values.
- *
- * Note: this function is used for old-style halftones only. WTS
- * halftones are allocated in gs_sethalftone_try_wts().
  */
 int
 gx_ht_alloc_ht_order(gx_ht_order * porder, uint width, uint height,
                      uint num_levels, uint num_bits, uint strip_shift,
                      const gx_ht_order_procs_t *procs, gs_memory_t * mem)
 {
-    porder->wse = NULL;
-    porder->wts = NULL;
     porder->threshold = NULL;
     porder->width = width;
     porder->height = height;
@@ -357,8 +321,6 @@ gx_ht_copy_ht_order(gx_ht_order * pdest, gx_ht_order * psrc, gs_memory_t * mem)
     if (pdest->bit_data != 0)
         memcpy(pdest->bit_data, psrc->bit_data,
                 psrc->num_bits * psrc->procs->bit_data_elt_size);
-    pdest->wse = psrc->wse;
-    pdest->wts = psrc->wts;
     pdest->transfer = psrc->transfer;
     rc_increment(pdest->transfer);
     return 0;
@@ -374,8 +336,6 @@ gx_ht_move_ht_order(gx_ht_order * pdest, gx_ht_order * psrc)
     uint    width = psrc->width, height = psrc->height, shift = psrc->shift;
 
     pdest->params = psrc->params;
-    pdest->wse = psrc->wse;
-    pdest->wts = 0;
     pdest->width = width;
     pdest->height = height;
     pdest->raster = bitmap_raster(width);
@@ -393,7 +353,6 @@ gx_ht_move_ht_order(gx_ht_order * pdest, gx_ht_order * psrc)
     pdest->transfer = psrc->transfer;
 }
 
-
 /* Allocate and initialize the contents of a halftone order. */
 /* The client must have set the defining values in porder->params. */
 int
@@ -486,6 +445,7 @@ gx_sort_ht_order(gx_ht_bit * recs, uint N)
     for (i = 0; i < N; i++)
         recs[i].offset = i;
     qsort((void *)recs, N, sizeof(*recs), compare_samples);
+#ifndef GS_THREADSAFE
 #ifdef DEBUG
     if (gs_debug_c('H')) {
         uint i;
@@ -496,6 +456,7 @@ gx_sort_ht_order(gx_ht_bit * recs, uint N)
                       i, recs[i].offset, recs[i].mask);
     }
 #endif
+#endif
 }
 
 /*
@@ -585,11 +546,11 @@ gx_ht_construct_bits(gx_ht_order * porder)
         gx_ht_construct_bit(phb, porder->width, phb->offset);
 #ifdef DEBUG
     if (gs_debug_c('H')) {
-        dlprintf1("[H]Halftone order bits 0x%lx:\n", (ulong)porder->bit_data);
+        dmlprintf1(porder->data_memory, "[H]Halftone order bits 0x%lx:\n", (ulong)porder->bit_data);
         for (i = 0, phb = (gx_ht_bit *)porder->bit_data;
              i < porder->num_bits;
              i++, phb++)
-            dlprintf3("%4d: %u:0x%lx\n", i, phb->offset,
+            dmlprintf3(porder->data_memory, "%4d: %u:0x%lx\n", i, phb->offset,
                       (ulong) phb->mask);
     }
 #endif
@@ -604,13 +565,8 @@ gx_ht_order_release(gx_ht_order * porder, gs_memory_t * mem, bool free_cache)
     if (free_cache) {
         if (porder->cache != 0)
             gx_ht_free_cache(mem, porder->cache);
-        else if (porder->wse != 0)
-            gs_wts_free_enum(porder->wse);
     }
     porder->cache = 0;
-    if (porder->wts != 0 && porder->width != ht_wts_suppress_release)
-        gs_wts_free_screen(porder->wts);
-    porder->wts = 0;
     rc_decrement(porder->transfer, "gx_ht_order_release(transfer)");
     porder->transfer = 0;
     if (porder->data_memory != 0) {
@@ -623,6 +579,7 @@ gx_ht_order_release(gx_ht_order * porder, gs_memory_t * mem, bool free_cache)
         gs_free_object(porder->data_memory->non_gc_memory, porder->threshold,
                        "gx_ht_order_release(threshold)");
     }
+    porder->threshold = 0;
     porder->levels = 0;
     porder->bit_data = 0;
 }
@@ -869,18 +826,6 @@ gs_cname_to_colorant_number(gs_state * pgs, byte * pname, uint name_size,
  *                  operand device halftone; it is not set in the device
  *                  halftone in the imager state.
  *
- *  wse            Points to an "enumerator" instance, used to construct
- *                 a well-tempered screen. This is only required while
- *                 the well-tempered screen is being constructed. This
- *                 field is always a null pointer in the device halftone
- *                 in the imager state.
- *
- *  wts            Points to the "constructed" form of a well-tempered
- *                 screen. The "construction" operation occurs as part
- *                 of the installation process. Hence, this should
- *                 always be a null pointer in the operand device
- *                 halftone.
- *
  *  orig_height,   The height and shift values of the halftone cell,
  *  orig_shift     prior to any replication. These fields are currently
  *                 unused, and will always be the same as the height
@@ -998,9 +943,9 @@ gx_imager_dev_ht_install(
     int                     i, code = 0;
     bool                    used_default = false;
     int                     lcm_width = 1, lcm_height = 1;
-    gs_wts_screen_enum_t *  wse0 = pdht->order.wse;
-    wts_screen_t *          wts0 = 0;
     bool                    mem_diff = pdht->rc.memory != pis->memory;
+    uint w, h;
+    int dw, dh;
 
     /* construct the new device halftone structure */
     memset(&dht.order, 0, sizeof(dht.order));
@@ -1065,37 +1010,10 @@ gx_imager_dev_ht_install(
 
     /*
      * Copy the default order to any remaining components.
-     *
-     * For well-tempered screens, generate the wts_screen_t structure
-     * for each component that corresponds to the sample information
-     * that has been gathered.
-     *
-     * Some caution is necessary here, as multiple component orders may
-     * have wse fields pointing to the same gs_wts_creeen_enum_t
-     * structure. This structure should only be released once. If
-     * multiple components have such a wse value, it will be the same as
-     * pdht->order.wse pointer, so we can just release that pointer once
-     * when done.
-     *
-     * If serveral component orders have the same wse value, this code
-     * will create just one wts_screen_t structure. In a somewhat ugly
-     * hack, the width field (which is otherwise unused) will be set to
-     * 0xffff for all components other than the first component that
-     * makes use of a give wts_screen_t structure. gx_ht_order_release
-     * will check this field to see if it should release the structure
-     * pointed to by the wts field of a component order.
-     *
-     * Components that are not well-tempered screens require a cache.
-     * In practice, either all or non of the components will be well-
-     * tempered screens, but we ignore that fact here.
-     *
-     * While engaged in all of these other activities, also calculate
-     * the lcm_width and lcm_heigth values (only for non-well-tempered
-     * components).
      */
+
     for (i = 0; i < num_comps && code >= 0; i++) {
-        gx_ht_order *           porder = &dht.components[i].corder;
-        gs_wts_screen_enum_t *  wse;
+        gx_ht_order *porder = &dht.components[i].corder;
 
         if (dht.components[i].comp_number != i) {
             if (used_default || mem_diff)
@@ -1106,61 +1024,44 @@ gx_imager_dev_ht_install(
             }
             dht.components[i].comp_number = i;
         }
-        if ((wse = porder->wse) != 0) {
-            wts_screen_t *  wts = 0;
 
-            porder->width = 0;
-            porder->wse = 0;
-            if (wse != wse0)
-                wts = wts_screen_from_enum(wse);
-            else {
-                if (wts0 == 0)
-                    wts0 = wts_screen_from_enum(wse);
-                else
-                    porder->width = ht_wts_suppress_release;
-                wts = wts0;
-            }
-            if (wts == 0)
+        w = porder->width;
+        h = porder->full_height;
+        dw = igcd(lcm_width, w);
+        dh = igcd(lcm_height, h);
+
+        lcm_width /= dw;
+        lcm_height /= dh;
+        lcm_width = (w > max_int / lcm_width ? max_int : lcm_width * w);
+        lcm_height = (h > max_int / lcm_height ? max_int : lcm_height * h);
+
+        if (porder->cache == 0) {
+            uint            tile_bytes, num_tiles, slots_wanted, rep_raster, rep_count;
+            gx_ht_cache *   pcache;
+
+            tile_bytes = porder->raster
+                          * (porder->num_bits / porder->width);
+            num_tiles = 1 + gx_ht_cache_default_bits_size() / tile_bytes;
+            /*
+             * Limit num_tiles to a reasonable number allowing for width repition.
+             * The most we need is one cache slot per bit.
+             * This prevents allocations of large cache bits that will never
+             * be used. See rep_count limit in gxht.c
+             */
+            slots_wanted = 1 + ( porder->width * porder->height );
+            rep_raster = ((num_tiles*tile_bytes) / porder->height /
+                            slots_wanted) & ~(align_bitmap_mod - 1);
+            rep_count = rep_raster * 8 / porder->width;
+            if (rep_count > sizeof(ulong) * 8 && (num_tiles >
+                    1 + ((num_tiles * 8 * sizeof(ulong)) / rep_count) ))
+                num_tiles = 1 + ((num_tiles * 8 * sizeof(ulong)) / rep_count);
+            pcache = gx_ht_alloc_cache( pis->memory, num_tiles,
+                                        tile_bytes * num_tiles );
+            if (pcache == NULL)
                 code = gs_error_VMerror;
-            else
-                porder->wts = wts;
-        } else if (porder->wts == 0) {
-            uint   w = porder->width, h = porder->full_height;
-            int    dw = igcd(lcm_width, w), dh = igcd(lcm_height, h);
-
-            lcm_width /= dw;
-            lcm_height /= dh;
-            lcm_width = (w > max_int / lcm_width ? max_int : lcm_width * w);
-            lcm_height = (h > max_int / lcm_height ? max_int : lcm_height * h);
-
-            if (porder->cache == 0) {
-                uint            tile_bytes, num_tiles, slots_wanted, rep_raster, rep_count;
-                gx_ht_cache *   pcache;
-
-                tile_bytes = porder->raster
-                              * (porder->num_bits / porder->width);
-                num_tiles = 1 + gx_ht_cache_default_bits_size() / tile_bytes;
-                /*
-                 * Limit num_tiles to a reasonable number allowing for width repition.
-                 * The most we need is one cache slot per bit.
-                 * This prevents allocations of large cache bits that will never
-                 * be used. See rep_count limit in gxht.c
-                 */
-                slots_wanted = 1 + ( porder->width * porder->height );
-                rep_raster = ((num_tiles*tile_bytes) / porder->height /
-                                slots_wanted) & ~(align_bitmap_mod - 1);
-                rep_count = rep_raster * 8 / porder->width;
-                if (rep_count > sizeof(ulong) * 8 && (num_tiles >
-                        1 + ((num_tiles * 8 * sizeof(ulong)) / rep_count) ))
-                    num_tiles = 1 + ((num_tiles * 8 * sizeof(ulong)) / rep_count);
-                pcache = gx_ht_alloc_cache( pis->memory, num_tiles,
-                                            tile_bytes * num_tiles );
-                if (pcache == NULL)
-                    code = gs_error_VMerror;
-                else {
-                    porder->cache = pcache;
-                    gx_ht_init_cache(pis->memory, pcache, porder);
-                }
+            else {
+                porder->cache = pcache;
+                gx_ht_init_cache(pis->memory, pcache, porder);
             }
         }
     }
@@ -1205,8 +1106,7 @@ gx_imager_dev_ht_install(
         /*
          * Everything worked. "Assume ownership" of the appropriate
          * portions of the source device halftone by clearing the
-         * associated references. This includes explicitly releasing
-         * any gs_wts_screen_enum_t structures. Since we might have
+         * associated references.  Since we might have
          * pdht == pis->dev_ht, this must done before updating pis->dev_ht.
          *
          * If the default order has been used for a device component, and
@@ -1226,8 +1126,6 @@ gx_imager_dev_ht_install(
 
                 if ( comp_num >= 0                            &&
                      comp_num < GX_DEVICE_COLOR_MAX_COMPONENTS  ) {
-                    if (p_s_order->wse != 0)
-                        gs_wts_free_enum(p_s_order->wse);
                     memset(p_s_order, 0, sizeof(*p_s_order));
                 } else if ( comp_num == GX_DEVICE_COLOR_MAX_COMPONENTS &&
                             used_default                                 )
@@ -1235,8 +1133,6 @@ gx_imager_dev_ht_install(
             }
         }
         if (used_default) {
-            if (wse0 != 0)
-                gs_wts_free_enum(wse0);
             memset(&pdht->order, 0, sizeof(pdht->order));
         }
 
@@ -1328,8 +1224,9 @@ gx_ht_install(gs_state * pgs, const gs_halftone * pht,
 void
 gx_imager_set_effective_xfer(gs_imager_state * pis)
 {
-    const gx_device_halftone *pdht = pis->dev_ht;
+    gx_device_halftone *pdht = pis->dev_ht;
     gx_transfer_map *pmap;
+    gx_ht_order *porder;
     int i, component_num;
 
     for (i = 0; i < GX_DEVICE_COLOR_MAX_COMPONENTS; i++)
@@ -1355,11 +1252,26 @@ gx_imager_set_effective_xfer(gs_imager_state * pis)
     if (pdht == NULL)
         return;                 /* not initialized yet */
 
+    /* Since the transfer function is pickled into the threshold array (if any)*/
+    /*  we need to free it so it can be reconstructed with the current transfer */
+    porder = &(pdht->order);
+    if (porder->threshold != NULL) {
+        gs_free_object(porder->data_memory->non_gc_memory, porder->threshold,
+                       "set_effective_transfer(threshold)");
+        porder->threshold = 0;
+    }
     for (i = 0; i < pdht->num_comp; i++) {
         pmap = pdht->components[i].corder.transfer;
         if (pmap != NULL)
             pis->effective_transfer[i] = pmap;
+        porder = &(pdht->components[i].corder);
+        if (porder->threshold != NULL) {
+            gs_free_object(porder->data_memory->non_gc_memory, porder->threshold,
+                           "set_effective_transfer(threshold)");
+            porder->threshold = 0;
+        }
     }
+
 }
 
 void
@@ -1367,211 +1279,73 @@ gx_set_effective_transfer(gs_state * pgs)
 {
     gx_imager_set_effective_xfer((gs_imager_state *) pgs);
 }
-#if TRANSFER_IN_THRESHOLDS
-#define round(x)    (((x) < 0.0) ? (ceil ((x) - 0.5)) : (floor ((x) + 0.5)))
-/* This creates a 256 entry LUT to invert the effective transfer function so
-   that it is built into the threshold values.  This only works correctly
-   for monotonic curves. */
-static int
-gx_ht_construct_transfer_inverse(gs_memory_t *memory, byte *transfer_inverse, 
-                                 gs_imager_state * pis, int  plane_index,
-                                 bool *is_inverting) 
+
+/* Check if the transfer function for a component is monotonic.	*/
+/* Used to determine if we can do fast halftoning		*/
+bool
+gx_transfer_is_monotonic(const gs_imager_state *pis, int plane_index)
 {
-    frac frac_val;
-    float float_val;
-    frac *transfer_sampled;
-    int k, min_pos, max_pos, mid_pos, max_pos_start, min_pos_start;
-    bool done, hit;
-    frac min_val, max_val;
-    frac zero_val, one_val;
-    int offset;
-    frac prev_value;
-    bool is_monotonic = true;
+    if (pis->effective_transfer[plane_index]->proc != gs_identity_transfer) {
+        bool threshold_inverted;
+        int t_level;
+        frac mapped, prev;
 
-    is_monotonic = true;
-    /* First construct a representation of the forward curve */
-    transfer_sampled = (frac *)gs_malloc(memory, TRANSFER_INVERSE_SIZE, 
-                                         sizeof(frac), 
-                                         "gx_ht_construct_transfer_inverse");
-    if (transfer_sampled == NULL) {
-        return -1 ;         /* error if allocation failed   */
-    }
-    zero_val = gx_map_color_frac(pis, 0, effective_transfer[plane_index]);
-    one_val = gx_map_color_frac(pis, frac_1, effective_transfer[plane_index]);
-    if (zero_val > one_val) {
-        *is_inverting = true;
-    } else {
-        *is_inverting = false;
-    }
-    if (*is_inverting) {
-        min_pos_start = 0;
-        max_pos_start = TRANSFER_INVERSE_SIZE - 1;
-        offset = TRANSFER_INVERSE_SIZE - 1;
-        prev_value  = gx_map_color_frac(pis, frac_1, effective_transfer[plane_index]);
-        for (k = 0; k < TRANSFER_INVERSE_SIZE; k++) {
-            float_val = (float) (offset - k)/(float) (TRANSFER_INVERSE_SIZE - 1);
-            frac_val = float2frac(float_val);
-            transfer_sampled[k] = gx_map_color_frac(pis, frac_val, 
-                                            effective_transfer[plane_index]);
-            if (prev_value > transfer_sampled[k]) {
-                is_monotonic = false;            
-            }
-            prev_value = transfer_sampled[k];
-            /* Take note of any leading zeros and ending ones for the inversion */
-            if (transfer_sampled[k] == 0) min_pos_start = k;
-            if (transfer_sampled[k] == frac_1 && k < max_pos_start) max_pos_start = k;
-        }
-    } else {
-        min_pos_start = 0;
-        max_pos_start = TRANSFER_INVERSE_SIZE - 1;
-        prev_value  = gx_map_color_frac(pis, 0, effective_transfer[plane_index]);
-        for (k = 0; k < TRANSFER_INVERSE_SIZE; k++) {
-            float_val = (float) k / (float) (TRANSFER_INVERSE_SIZE - 1);
-            frac_val = float2frac(float_val);
-            transfer_sampled[k] = gx_map_color_frac(pis, frac_val, 
-                                            effective_transfer[plane_index]);
-            if (prev_value > transfer_sampled[k]) {
-                is_monotonic = false;            
-            }
-            prev_value = transfer_sampled[k];
-            /* Take note of any leading zeros and ending ones for the inversion */
-            if (transfer_sampled[k] == 0) min_pos_start = k;
-            if (transfer_sampled[k] == frac_1 && k < max_pos_start) max_pos_start = k;
+        prev = gx_map_color_frac(pis, frac_0, effective_transfer[plane_index]);
+        threshold_inverted = prev >
+                             gx_map_color_frac(pis, frac_1, effective_transfer[plane_index]);
+        for (t_level = 1; t_level < 255; t_level++) {
+            mapped = gx_map_color_frac(pis, byte2frac(t_level), effective_transfer[plane_index]);
+            if ((threshold_inverted && mapped > prev) ||
+                (!threshold_inverted && mapped < prev))
+                return false;
+            prev = mapped;
         }
     }
-    if (!is_monotonic) {
-        gs_free_object(memory, transfer_sampled, "gx_ht_construct_transfer_inverse");
-        return -1;
-    }
-    /* Now run over our invertible range from min_pos to max_pos doing
-       a binary search with interpolation for the inversion */
-    for (k = 0; k < 256; k++) {
-        float_val = (float) k / 256.0;
-        frac_val = float2frac(float_val);
-        /* Find where this value occurs */
-        done = false;
-        hit = false;
-        max_pos = max_pos_start;
-        min_pos = min_pos_start;
-        while (!done) {
-            mid_pos = min_pos + (max_pos - min_pos) / 2;
-            if (frac_val > transfer_sampled[mid_pos]) {
-                min_pos = mid_pos;
-            } else if (frac_val < transfer_sampled[mid_pos]) {
-                max_pos = mid_pos;
-            } else {
-                done = true;
-                hit = true;
-            }
-            if (max_pos - min_pos == 1) {
-                done = true;
-            }
-        }
-        /* OK found the point go ahead and interpolate if needed */
-        if (*is_inverting) {
-            if (hit) {
-                transfer_inverse[255 - k] = 
-                    round((float) mid_pos * 255.0 / (float) (TRANSFER_INVERSE_SIZE - 1));            
-            } else {
-                /* Interpolate */
-                min_val = transfer_sampled[min_pos];
-                max_val = transfer_sampled[max_pos];
-                float_val = (float) min_pos + 
-                    (float) (max_pos - min_pos) * (float) (frac_val - min_val) /                         
-                    (float) (max_val - min_val);
-                transfer_inverse[255 - k] = 
-                    round(float_val * 255.0 / (float) (TRANSFER_INVERSE_SIZE - 1));            
-            }
-        } else {
-            if (hit) {
-                transfer_inverse[k] = 
-                    round((float) mid_pos * 255.0 / (float) (TRANSFER_INVERSE_SIZE - 1));            
-            } else {
-                /* Interpolate */
-                min_val = transfer_sampled[min_pos];
-                max_val = transfer_sampled[max_pos];
-                float_val = (float) min_pos + 
-                    (float) (max_pos - min_pos) * (float) (frac_val - min_val) /                         
-                    (float) (max_val - min_val);
-                transfer_inverse[k] = 
-                    round(float_val * 255.0 / (float) (TRANSFER_INVERSE_SIZE - 1));            
-            }
-        }
-    }
-    /* Make sure the end points are set */
-    if (*is_inverting) {
-        transfer_inverse[255] = 0;
-        transfer_inverse[0] = 255;
-    } else {
-        transfer_inverse[255] = 255;
-        transfer_inverse[0] = 0;
-    }
-    gs_free_object(memory, transfer_sampled, "gx_ht_construct_transfer_inverse");
-    return 0;
+    return true;
 }
-#undef round
-#endif
 
-/* Lifted from threshold_from_order in gdevtsep.c.  This creates a threshold
-   array from the tiles.  Threshold is allocated in non-gc memory and is
-   not known to the gc */
+/* This creates a threshold array from the tiles.  Threshold is allocated in
+   non-gc memory and is not known to the GC. The algorithm cycles through the
+   threshold values, computing the shade the same way as gx_render_device_DeviceN
+   so that the threshold matches the non-threshold halftoning.
+*/
 int
-gx_ht_construct_threshold( gx_ht_order *d_order, gx_device *dev, 
+gx_ht_construct_threshold( gx_ht_order *d_order, gx_device *dev,
                            const gs_imager_state * pis, int plane_index)
 {
-    int i, j, l, prev_l;
+    int i, j;
     unsigned char *thresh;
     gs_memory_t *memory = d_order->data_memory->non_gc_memory;
     uint max_value;
     unsigned long hsize, nshades;
-    int t_level, t_level_adjust;
+    int t_level;
     int row, col;
-    int delta, delta_sum = 0;
-    bool have_transfer = false;
-    byte *transfer_inverse = NULL;
     int code;
-    byte init_value = 255;
-    bool is_inverting = false;
+    int num_repeat, shift, num_levels = d_order->num_levels;
+    int row_kk, col_kk, kk;
+    frac t_level_frac_color;
+    int shade, base_shade = 0;
+    bool have_transfer = false, threshold_inverted = false;
+
+    /* We can have simple or complete orders.  Simple ones tile the threshold
+       with shifts.   To handle those we simply loop over the number of
+       repeats making sure to shift columns when we set our threshold values */
+    num_repeat = d_order->full_height / d_order->height;
+    shift = d_order->shift;
 
     if (d_order == NULL) return -1;
     if (d_order->threshold != NULL) return 0;
-    d_order->threshold_inverts = is_inverting;
-    thresh = (byte *)gs_malloc(memory, d_order->num_bits, 1,
+    thresh = (byte *)gs_malloc(memory, d_order->width * d_order->full_height, 1,
                               "gx_ht_construct_threshold");
     if (thresh == NULL) {
         return -1 ;         /* error if allocation failed   */
     }
-    d_order->threshold_inverts = false;
-#if TRANSFER_IN_THRESHOLDS
     /* Check if we need to apply a transfer function to the values */
     if (pis->effective_transfer[plane_index]->proc != gs_identity_transfer) {
-        transfer_inverse = (byte *)gs_malloc(memory, 256, 1, 
-                                    "gx_ht_construct_threshold");
-        if (transfer_inverse == NULL) {
-            return -1 ;         /* error if allocation failed   */
-        }
-        /* Create an inverse of the transfer.  Hopefully it is invertible.  If
-           not, we will get some strange results... */
-        code = gx_ht_construct_transfer_inverse(memory, transfer_inverse, pis, 
-                                        plane_index, &is_inverting);
-        d_order->threshold_inverts = is_inverting;
-        if (code < 0) {
-            /* If this failed then we will go and use the non-threshold code */
-#ifdef DEBUG
-            gs_warn("Transfer function inversion for threshold matrix failed!");
-#endif
-            if (transfer_inverse != NULL) {
-               gs_free_object(memory, transfer_inverse, "gx_ht_construct_threshold");
-            }
-            return -1;
-        } else {
-            have_transfer = true;
-            init_value = transfer_inverse[255];
-        }
-    } else {
-        d_order->threshold_inverts = false;
+        have_transfer = true;
+        threshold_inverted = gx_map_color_frac(pis, frac_0, effective_transfer[plane_index]) >
+                                gx_map_color_frac(pis, frac_1, effective_transfer[plane_index]);
     }
-#endif
     /* Adjustments to ensure that we properly map our 256 levels into
       the number of shades that we have in our halftone screen.  For example
       if we have a 16x16 screen, we have 257 shadings that we can represent
@@ -1581,70 +1355,70 @@ gx_ht_construct_threshold( gx_ht_order *d_order, gx_device *dev,
     max_value = (dev->color_info.gray_index == plane_index) ?
          dev->color_info.dither_grays - 1 :
          dev->color_info.dither_colors - 1;
-    hsize = d_order->num_levels;
+    hsize = num_levels;
     nshades = hsize * max_value + 1;
 
-    if (d_order == NULL) return -1;
-    if (d_order->threshold != NULL) return 0;
-    thresh = (byte *)gs_malloc(memory, d_order->num_bits, 1,
-                               "gx_ht_construct_threshold");
-    if( thresh == NULL ) {
-        return -1 ;         /* error if allocation failed   */
+    /* search upwards to find the correct value for the last threshold value */
+    /* Use this to initialize the threshold array (transition to all white) */
+    shade = 0;
+    t_level = 0;
+    do {
+        t_level++;
+        t_level_frac_color = byte2frac(threshold_inverted ? 255 - t_level : t_level);
+        if (have_transfer)
+            t_level_frac_color = gx_map_color_frac(pis, t_level_frac_color, effective_transfer[plane_index]);
+        shade = t_level_frac_color * nshades / (frac_1_long + 1);
+    } while (shade < num_levels && t_level < 255);
+    /* Initialize the thresholds to the lowest level that will be all white */
+    for( i = 0; i < d_order->width * d_order->full_height; i++ ) {
+        thresh[i] = t_level;
     }
-    for( i = 0; i < d_order->num_bits; i++ ) {
-        thresh[i] = init_value;
-    }
-    prev_l = 0;
-    l = 1;
-    while (l < d_order->num_levels) {
-        /* If we have some dots to turn on then proceed */
-        if (d_order->levels[l] > d_order->levels[prev_l]) {
-            t_level = (256 * l) / d_order->num_levels;
-            t_level_adjust = byte2frac(t_level) * nshades / (frac_1_long + 1);
-            delta = t_level_adjust - t_level;
-            if (delta > delta_sum) {
-                /* We had a change in our value.  We need to do some adjustments.
-                   This particular level stays were it is and subsequent ones
-                   will be offset by delta_sum. */
-                t_level -= delta_sum;
-                delta_sum += delta;
-            } else {
-                t_level -= delta_sum;
+    for (t_level = 1; t_level < 256; t_level++) {
+        t_level_frac_color = byte2frac(threshold_inverted ? 255 - t_level : t_level);
+        if (have_transfer)
+            t_level_frac_color = gx_map_color_frac(pis, t_level_frac_color, effective_transfer[plane_index]);
+        shade = t_level_frac_color * nshades / (frac_1_long + 1);
+        if (shade < num_levels && shade > base_shade) {
+            if (d_order->levels[shade] > d_order->levels[base_shade]) {
+                /* Loop over the number of dots that we have to set in going
+                   to this new shade from the old shade */
+                for (j = d_order->levels[base_shade]; j < d_order->levels[shade]; j++) {
+                    gs_int_point ppt;
+                    code = d_order->procs->bit_index(d_order, j, &ppt);
+                    if (code < 0)
+                        return code;
+                    row = ppt.y;
+                    col = ppt.x;
+                    if( col < (int)d_order->width ) {
+                        for (kk = 0; kk < num_repeat; kk++) {
+                            row_kk = row + kk * d_order->height;
+                            col_kk = col + kk * shift;
+                            col_kk = col_kk % d_order->width;
+                            *(thresh + col_kk + (row_kk * d_order->width)) = t_level;
+                        }
+                    }
+                }
             }
-            /* If needed, map through the inverse of the transfer function */
-            if (have_transfer) {
-                t_level = transfer_inverse[t_level];
-            }
-            /* Loop over the number of dots that we have to set in going
-               to this new level from the old level */
-            for (j = d_order->levels[prev_l]; j < d_order->levels[l]; j++) {
-                gs_int_point ppt;
-                code = d_order->procs->bit_index(d_order, j, &ppt);
-                if (code < 0)
-                    return code;
-                row = ppt.y;
-                col = ppt.x;
-                if (col < (int)d_order->width)
-                    *(thresh + col + (row * d_order->width)) = t_level;
-            }
-            prev_l = l;
+            base_shade = shade;
         }
-        l++;
     }
     d_order->threshold = thresh;
+    d_order->threshold_inverted = threshold_inverted;
+    if (dev->color_info.polarity == GX_CINFO_POLARITY_SUBTRACTIVE) {
+        for(i = 0; i < (int)d_order->height; i++ ) {
+            for( j=(int)d_order->width-1; j>=0; j-- )
+                *(thresh+j+(i*d_order->width)) = 255 - *(thresh+j+(i*d_order->width));
+        }
+    }
 #ifdef DEBUG
-   if ( gs_debug_c('h') ) {
-      for( i=0; i<(int)d_order->height; i++ ) {
-         dprintf1("threshold array row %3d= ", i);
-         for( j=(int)d_order->width-1; j>=0; j-- )
-            dprintf1("%3d ", *(thresh+j+(i*d_order->width)) );
-         dprintf("\n");
-      }
+    if ( gs_debug_c('h') ) {
+         for( i=0; i<(int)d_order->height; i++ ) {
+            dmprintf1(memory, "threshold array row %3d= ", i);
+            for( j=0; j<(int)(d_order->width); j++ )
+                dmprintf1(memory, "%3d ", *(thresh+j+(i*d_order->width)) );
+            dmprintf(memory, "\n");
+        }
    }
 #endif
-   if (transfer_inverse != NULL) {
-       gs_free_object(memory, transfer_inverse, "gx_ht_construct_threshold");
-   }
     return 0;
 }
-
