@@ -1,19 +1,20 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsmisc.c 11956 2010-12-17 16:48:02Z till $ */
-/* Miscellaneous utilities for Ghostscript library */
 
+/* Miscellaneous utilities for Ghostscript library */
 
 /*
  * In order to capture the original definition of sqrt, which might be
@@ -41,7 +42,6 @@ orig_sqrt(double x)
 #include "string_.h"
 #include "gx.h"
 #include "gpcheck.h"            /* for gs_return_check_interrupt */
-#include "gserror.h"            /* for prototype */
 #include "gserrors.h"
 #include "gxfarith.h"
 #include "gxfixed.h"
@@ -73,6 +73,7 @@ int outprintf(const gs_memory_t *mem, const char *fmt, ...)
     return count;
 }
 
+#ifndef GS_THREADSAFE
 int errprintf_nomem(const char *fmt, ...)
 {
     int count;
@@ -90,6 +91,7 @@ int errprintf_nomem(const char *fmt, ...)
     va_end(args);
     return count;
 }
+#endif
 
 int errprintf(const gs_memory_t *mem, const char *fmt, ...)
 {
@@ -119,12 +121,21 @@ int errprintf(const gs_memory_t *mem, const char *fmt, ...)
 char gs_debug[128];
 
 /* Test whether a given debugging option is selected. */
-/* Upper-case letters automatically include their lower-case counterpart. */
+/* Some letters automatically imply others. */
 bool
 gs_debug_c(int c)
 {
-    return
-        (c >= 'a' && c <= 'z' ? gs_debug[c] | gs_debug[c ^ 32] : gs_debug[c]);
+    bool ret = gs_debug[c];
+
+    /* Unroll the first one, to minimise the speed hit */
+    c = gs_debug_flag_implied_by[c];
+    if (c == 0)
+        return ret;
+
+    do {
+        ret |= gs_debug[c];
+    } while ((c = gs_debug_flag_implied_by[c]) != 0);
+    return ret;
 }
 
 /* Define the formats for debugging printout. */
@@ -137,11 +148,6 @@ const char *const dprintf_file_only_format = "%10s(unkn): ";
  * out/errprintf, not fprintf nor fput[cs], because of the way that
  * stdout/stderr are implemented on DLL/shared library builds.
  */
-void
-dflush(void)
-{
-    errflush_nomem();
-}
 static const char *
 dprintf_file_tail(const char *file)
 {
@@ -153,6 +159,12 @@ dprintf_file_tail(const char *file)
         --tail;
     return tail;
 }
+#ifndef GS_THREADSAFE
+void
+dflush(void)
+{
+    errflush_nomem();
+}
 #if __LINE__                    /* compiler provides it */
 void
 dprintf_file_and_line(const char *file, int line)
@@ -161,6 +173,11 @@ dprintf_file_and_line(const char *file, int line)
         dpf(dprintf_file_and_line_format,
                 dprintf_file_tail(file), line);
 }
+void
+lprintf_file_and_line(const char *file, int line)
+{
+    epf("%s(%d): ", file, line);
+}
 #else
 void
 dprintf_file_only(const char *file)
@@ -168,18 +185,12 @@ dprintf_file_only(const char *file)
     if (gs_debug['/'])
         dpf(dprintf_file_only_format, dprintf_file_tail(file));
 }
-#endif
 void
-printf_program_ident(const gs_memory_t *mem, const char *program_name, long revision_number)
+lprintf_file_only(FILE * f, const char *file)
 {
-    if (program_name)
-        outprintf(mem, (revision_number ? "%s " : "%s"), program_name);
-    if (revision_number) {
-        int fpart = revision_number % 100;
-
-        outprintf(mem, "%d.%02d", (int)(revision_number / 100), fpart);
-    }
+    epf("%s(?): ", file);
 }
+#endif
 void
 eprintf_program_ident(const char *program_name,
                       long revision_number)
@@ -194,6 +205,34 @@ eprintf_program_ident(const char *program_name,
         epf(": ");
     }
 }
+#endif
+#if __LINE__                    /* compiler provides it */
+void
+dmprintf_file_and_line(const gs_memory_t *mem,const char *file, int line)
+{
+    if (gs_debug['/'])
+        dpfm(mem, dprintf_file_and_line_format,
+                dprintf_file_tail(file), line);
+}
+#else
+void
+dmprintf_file_only(const gs_memory_t *mem,const char *file)
+{
+    if (gs_debug['/'])
+        dpfm(mem, dprintf_file_only_format, dprintf_file_tail(file));
+}
+#endif
+void
+printf_program_ident(const gs_memory_t *mem, const char *program_name, long revision_number)
+{
+    if (program_name)
+        outprintf(mem, (revision_number ? "%s " : "%s"), program_name);
+    if (revision_number) {
+        int fpart = revision_number % 100;
+
+        outprintf(mem, "%d.%02d", (int)(revision_number / 100), fpart);
+    }
+}
 void
 emprintf_program_ident(const gs_memory_t *mem,
                        const char *program_name,
@@ -211,15 +250,15 @@ emprintf_program_ident(const gs_memory_t *mem,
 }
 #if __LINE__                    /* compiler provides it */
 void
-lprintf_file_and_line(const char *file, int line)
+mlprintf_file_and_line(const gs_memory_t *mem, const char *file, int line)
 {
-    epf("%s(%d): ", file, line);
+    epfm(mem, "%s(%d): ", file, line);
 }
 #else
 void
-lprintf_file_only(FILE * f, const char *file)
+mlprintf_file_only(const gs_memory_t *mem, FILE * f, const char *file)
 {
-    epf("%s(?): ", file);
+    epfm(mem, "%s(?): ", file);
 }
 #endif
 
@@ -253,7 +292,7 @@ gs_return_check_interrupt(const gs_memory_t *mem, int code)
     }
 }
 
-
+#ifndef GS_THREADSAFE
 int gs_throw_imp(const char *func, const char *file, int line, int op, int code, const char *fmt, ...)
 {
     char msg[1024];
@@ -272,7 +311,6 @@ int gs_throw_imp(const char *func, const char *file, int line, int op, int code,
            */
     }
 
-
     /* throw */
     if (op == 0)
         errprintf_nomem("+ %s:%d: %s(): %s\n", file, line, func, msg);
@@ -291,6 +329,7 @@ int gs_throw_imp(const char *func, const char *file, int line, int op, int code,
 
     return code;
 }
+#endif
 
 const char *gs_errstr(int code)
 {
@@ -316,7 +355,6 @@ const char *gs_errstr(int code)
     }
 }
 
-
 /* ------ Substitutes for missing C library functions ------ */
 
 #ifdef MEMORY__NEED_MEMMOVE     /* see memory_.h */
@@ -489,49 +527,10 @@ gs_realloc(void *old_ptr, size_t old_size, size_t new_size)
 
 /* ------ Debugging support ------ */
 
-/* Dump a region of memory. */
-void
-debug_dump_bytes(const byte * from, const byte * to, const char *msg)
-{
-    const byte *p = from;
-
-    if (from < to && msg)
-        dprintf1("%s:\n", msg);
-    while (p != to) {
-        const byte *q = min(p + 16, to);
-
-        dprintf1("0x%lx:", (ulong) p);
-        while (p != q)
-            dprintf1(" %02x", *p++);
-        dputc('\n');
-    }
-}
-
-/* Dump a bitmap. */
-void
-debug_dump_bitmap(const byte * bits, uint raster, uint height, const char *msg)
-{
-    uint y;
-    const byte *data = bits;
-
-    for (y = 0; y < height; ++y, data += raster)
-        debug_dump_bytes(data, data + raster, (y == 0 ? msg : NULL));
-}
-
-/* Print a string. */
-void
-debug_print_string(const byte * chrs, uint len)
-{
-    uint i;
-
-    for (i = 0; i < len; i++)
-        dputc(chrs[i]);
-    dflush();
-}
-
+#ifndef GS_THREADSAFE
 /* Print a string in hexdump format. */
 void
-debug_print_string_hex(const byte * chrs, uint len)
+debug_print_string_hex_nomem(const byte * chrs, uint len)
 {
     uint i;
 
@@ -539,6 +538,58 @@ debug_print_string_hex(const byte * chrs, uint len)
         dprintf1("%02x", chrs[i]);
     dflush();
 }
+#endif
+
+/* Dump a region of memory. */
+void
+debug_dump_bytes(const gs_memory_t *mem, const byte * from, const byte * to, const char *msg)
+{
+    const byte *p = from;
+
+    if (from < to && msg)
+        dmprintf1(mem, "%s:\n", msg);
+    while (p != to) {
+        const byte *q = min(p + 16, to);
+
+        dmprintf1(mem, "0x%lx:", (ulong) p);
+        while (p != q)
+            dmprintf1(mem, " %02x", *p++);
+        dmputc(mem, '\n');
+    }
+}
+
+/* Dump a bitmap. */
+void
+debug_dump_bitmap(const gs_memory_t *mem, const byte * bits, uint raster, uint height, const char *msg)
+{
+    uint y;
+    const byte *data = bits;
+
+    for (y = 0; y < height; ++y, data += raster)
+        debug_dump_bytes(mem, data, data + raster, (y == 0 ? msg : NULL));
+}
+
+/* Print a string. */
+void
+debug_print_string(const gs_memory_t *mem, const byte * chrs, uint len)
+{
+    uint i;
+
+    for (i = 0; i < len; i++)
+        dmputc(mem, chrs[i]);
+    dmflush(mem);
+}
+
+/* Print a string in hexdump format. */
+void
+debug_print_string_hex(const gs_memory_t *mem, const byte * chrs, uint len)
+{
+    uint i;
+
+    for (i = 0; i < len; i++)
+        dmprintf1(mem, "%02x", chrs[i]);
+    dmflush(mem);
+}
 
 /*
  * The following code prints a hex stack backtrace on Linux/Intel systems.
@@ -549,12 +600,14 @@ debug_print_string_hex(const byte * chrs, uint len)
  * first_arg is the first argument of the procedure into which this code
  * is patched.
  */
+#ifndef GS_THREADSAFE
 #define BACKTRACE(first_arg)\
   BEGIN\
     ulong *fp_ = (ulong *)&first_arg - 2;\
     for (; fp_ && (fp_[1] & 0xff000000) == 0x08000000; fp_ = (ulong *)*fp_)\
         dprintf2("  fp=0x%lx ip=0x%lx\n", (ulong)fp_, fp_[1]);\
   END
+#endif
 
 /* ------ Arithmetic ------ */
 
@@ -867,10 +920,12 @@ fixed_mult_quo(fixed signed_A, fixed B, fixed C)
 double
 gs_sqrt(double x, const char *file, int line)
 {
+#ifndef GS_THREADSAFE
     if (gs_debug_c('~')) {
         dprintf3("[~]sqrt(%g) at %s:%d\n", x, (const char *)file, line);
         dflush();
     }
+#endif
     return orig_sqrt(x);
 }
 
@@ -1065,3 +1120,120 @@ gx_intersect_small_bars(fixed q0x, fixed q0y, fixed q1x, fixed q1y, fixed q2x, f
     }
     return false;
 }
+
+/* gs_debug_flags handling code */
+
+const gs_debug_flag_details gs_debug_flags[] =
+{
+#define FLAG(a,b,c,d) {1, # a ,d}
+#define UNUSED(a) { 0, "", "" },
+#include "gdbflags.h"
+#undef FLAG
+#undef UNUSED
+};
+
+const byte gs_debug_flag_implied_by[] =
+{
+#define FLAG(a,b,c,d) c
+#define UNUSED(a) 0,
+#include "gdbflags.h"
+#undef FLAG
+#undef UNUSED
+};
+
+int gs_debug_flags_parse(gs_memory_t *heap, const char *arg)
+{
+#ifdef DEBUG
+    const char *p;
+    int i, j;
+    int result = 0;
+
+    while (*arg != 0) {
+        /* Skip over any commas */
+        byte value = 0xff;
+        while (*arg == ',')
+            arg++;
+        if (*arg == '-') {
+            value = 0;
+            arg++;
+        }
+        if (*arg == ',') {
+            arg++;
+            continue;
+        }
+        if (*arg == 0)
+            return result;
+        for (i=0; i < gs_debug_flags_max; i++) {
+            char c, d;
+            if (!gs_debug_flags[i].used)
+                continue;
+            p = arg;
+            for (j=0; j < sizeof(gs_debug_flags[i].short_desc); j++) {
+                d = gs_debug_flags[i].short_desc[j];
+                c = *p++;
+                if (d == '_') d = '-';
+                if (c == ',')
+                    c = 0;
+                if ((c != d) || (c == 0))
+                    break;
+            }
+            if ((c == 0) && (d == 0))
+                break;
+        }
+        if (i < gs_debug_flags_max)
+            gs_debug[i] = value;
+        else {
+            outprintf(heap, "Unknown debug flag: ");
+            p = arg;
+            do {
+                char c = *p++;
+                if ((c == 0) || (c == ','))
+                    break;
+                outprintf(heap, "%c", c);
+            } while (1);
+            outprintf(heap, "\n");
+            result = gs_error_Fatal;
+        }
+        arg = p-1;
+    }
+    return result;
+#else
+    outprintf(heap, "Warning: debug flags ignored in release builds\n");
+    return 0;
+#endif
+}
+
+void
+gs_debug_flags_list(gs_memory_t *heap)
+{
+#ifdef DEBUG
+    int i, j;
+
+    outprintf(heap, "Debugging flags are as follows:\n\n-Z  --debug=            Description\n");
+    for (i=0; i < gs_debug_flags_max; i++) {
+        if (!gs_debug_flags[i].used)
+            continue;
+        outprintf(heap, " %c  ", (i < 32 || i > 126 ? ' ' : i));
+        for (j=0; j < sizeof(gs_debug_flags[i].short_desc); j++) {
+            char c = gs_debug_flags[i].short_desc[j];
+            if (c == 0)
+                break;
+            if (c == '_')
+                c = '-';
+            outprintf(heap, "%c", c);
+        }
+        for (; j < sizeof(gs_debug_flags[i].short_desc); j++) {
+            outprintf(heap, " ");
+        }
+        outprintf(heap, "%s\n", gs_debug_flags[i].long_desc);
+    }
+    outprintf(heap,
+              "\nDebugging may be enabled by using -Zx (where x is one of the 1 character\n"
+              "codes given above), or by using --debug=xxxxx. Multiple flags may be specified\n"
+              "at once, using -Zxyz or --debug=xxxxx,yyyyy,zzzzz. -Z=-x or --debug=-xxxxx\n"
+              "disables a flag once set.\n");
+#else
+    outprintf(heap, "No debug flags supported in release builds\n");
+#endif
+}
+
