@@ -63,7 +63,7 @@ write_color_index(gx_color_index cindex, byte * data, uint * psize)
         ;
     if (num_bytes > *psize) {
         *psize = num_bytes;
-        return gs_error_rangecheck;
+        return_error(gs_error_rangecheck);
     }
     ctmp = cindex;
     *psize = num_bytes;
@@ -125,6 +125,7 @@ c_overprint_equal(const gs_composite_t * pct0, const gs_composite_t * pct1)
  */
 #define OVERPRINT_ANY_COMPS     1
 #define OVERPRINT_SPOT_COMPS    2
+#define OVERPRINT_BLEND         4
 
 /*
  * Convert an overprint compositor to string form for use by the command
@@ -140,15 +141,15 @@ c_overprint_write(const gs_composite_t * pct, byte * data, uint * psize, gx_devi
     /* encoded the booleans in a single byte */
     if (pparams->retain_any_comps) {
         flags |= OVERPRINT_ANY_COMPS;
-
-        /* write out the component bits only if necessary (and possible) */
-        if (pparams->retain_spot_comps && !pparams->blendspot)
+        if (pparams->blendspot)
+            flags |= OVERPRINT_BLEND;
+        if (pparams->retain_spot_comps)
             flags |= OVERPRINT_SPOT_COMPS;
-        else {
-            uint    tmp_size = (avail > 0 ? avail - 1 : 0);
-            int     code = write_color_index( pparams->drawn_comps,
-                                              data + 1,
-                                              &tmp_size );
+        /* write out the component bits only if necessary (and possible) */
+        if (!pparams->retain_spot_comps || pparams->blendspot) {
+            uint tmp_size = (avail > 0 ? avail - 1 : 0);
+            int code = write_color_index(pparams->drawn_comps, data + 1, 
+                                             &tmp_size);
             /* It would be nice to do have an If RGB OP case, then write out 
                K value, but on the reader side, there is no way to find this
                out so we will always write it out if we are writing the
@@ -199,11 +200,11 @@ c_overprint_read(
     params.retain_any_comps = (flags & OVERPRINT_ANY_COMPS) != 0;
     params.retain_spot_comps = (flags & OVERPRINT_SPOT_COMPS) != 0;
     params.idle = 0;
-    params.blendspot = false;
+    params.blendspot = (flags & OVERPRINT_BLEND) != 0;
     params.k_value = 0;
 
     /* check if the drawn_comps array is present */
-    if (params.retain_any_comps && !params.retain_spot_comps) {
+    if (params.retain_any_comps && (!params.retain_spot_comps || params.blendspot)) {
         code = read_color_index(&params.drawn_comps, data + 1, size - 1);
         if (code < 0)
             return code;
@@ -220,15 +221,14 @@ c_overprint_read(
 /*
  * Check for closing compositor.
  */
-static int
+static gs_compositor_closing_state
 c_overprint_is_closing(const gs_composite_t *this, gs_composite_t **ppcte, gx_device *dev)
 {
     if (*ppcte != NULL && (*ppcte)->type->comp_id != GX_COMPOSITOR_OVERPRINT)
-        return 0;
-    return 3;
+        return COMP_ENQUEUE;
+    return COMP_REPLACE_PREV;
 }
 
-static composite_create_default_compositor_proc(c_overprint_create_default_compositor);
 static composite_create_default_compositor_proc(c_overprint_create_default_compositor);
 static composite_equal_proc(c_overprint_equal);
 static composite_write_proc(c_overprint_write);
@@ -362,7 +362,7 @@ gs_private_st_suffix_add0_final( st_overprint_device_t,
                                  overprint_device_t_enum_ptrs,
                                  overprint_device_t_reloc_ptrs,
                                  gx_device_finalize,
-                                 st_device_forward );
+                                 st_device_forward);
 
 /*
  * In the default (overprint false) case, the overprint device is almost
@@ -659,7 +659,7 @@ const overprint_device_t    gs_overprint_device = {
  * takes advantage of the fact that depths that are > 8 must be a multiple
  * of 8 and <= 64
  */
-#if !arch_is_big_endian
+#if !ARCH_IS_BIG_ENDIAN
 
 static gx_color_index
 swap_color_index(int depth, gx_color_index color)
@@ -697,7 +697,7 @@ swap_color_index(int depth, gx_color_index color)
     return color;
 }
 
-#endif  /* !arch_is_big_endian */
+#endif  /* !ARCH_IS_BIG_ENDIAN */
 
 /*
  * Update the retain_mask field to reflect the information in the
@@ -709,7 +709,7 @@ set_retain_mask(overprint_device_t * opdev)
 {
     int             i, ncomps = opdev->color_info.num_components;
     gx_color_index  drawn_comps = opdev->drawn_comps, retain_mask = 0;
-#if !arch_is_big_endian
+#if !ARCH_IS_BIG_ENDIAN
     int             depth = opdev->color_info.depth;
 #endif
 
@@ -717,7 +717,7 @@ set_retain_mask(overprint_device_t * opdev)
         if ((drawn_comps & 0x1) == 0)
             retain_mask |= opdev->color_info.comp_mask[i];
     }
-#if !arch_is_big_endian
+#if !ARCH_IS_BIG_ENDIAN
     if (depth > 8)
         retain_mask = swap_color_index(depth, retain_mask);
 #endif
@@ -785,29 +785,30 @@ update_overprint_params(
         gx_color_index                  drawn_comps = 0;
         static const frac               frac_13 = float2frac(1.0 / 3.0);
 
-        if ((pprocs = dev_proc(opdev, get_color_mapping_procs)(dev)) == 0 ||
+        pprocs = get_color_mapping_procs_subclass(dev);
+        if (pprocs == 0 ||
             pprocs->map_gray == 0                                         ||
             pprocs->map_rgb == 0                                          ||
             pprocs->map_cmyk == 0                                           )
             return_error(gs_error_unknownerror);
 
-        pprocs->map_gray(dev, frac_13, cvals);
+        map_gray_subclass(pprocs, dev, frac_13, cvals);
         drawn_comps |= check_drawn_comps(ncomps, cvals);
 
-        pprocs->map_rgb(dev, 0, frac_13, frac_0, frac_0, cvals);
+        map_rgb_subclass(pprocs, dev, 0, frac_13, frac_0, frac_0, cvals);
         drawn_comps |= check_drawn_comps(ncomps, cvals);
-        pprocs->map_rgb(dev, 0, frac_0, frac_13, frac_0, cvals);
+        map_rgb_subclass(pprocs, dev, 0, frac_0, frac_13, frac_0, cvals);
         drawn_comps |= check_drawn_comps(ncomps, cvals);
-        pprocs->map_rgb(dev, 0, frac_0, frac_0, frac_13, cvals);
+        map_rgb_subclass(pprocs, dev, 0, frac_0, frac_0, frac_13, cvals);
         drawn_comps |= check_drawn_comps(ncomps, cvals);
 
-        pprocs->map_cmyk(dev, frac_13, frac_0, frac_0, frac_0, cvals);
+        map_cmyk_subclass(pprocs, dev, frac_13, frac_0, frac_0, frac_0, cvals);
         drawn_comps |= check_drawn_comps(ncomps, cvals);
-        pprocs->map_cmyk(dev, frac_0, frac_13, frac_0, frac_0, cvals);
+        map_cmyk_subclass(pprocs, dev, frac_0, frac_13, frac_0, frac_0, cvals);
         drawn_comps |= check_drawn_comps(ncomps, cvals);
-        pprocs->map_cmyk(dev, frac_0, frac_0, frac_13, frac_0, cvals);
+        map_cmyk_subclass(pprocs, dev, frac_0, frac_0, frac_13, frac_0, cvals);
         drawn_comps |= check_drawn_comps(ncomps, cvals);
-        pprocs->map_cmyk(dev, frac_0, frac_0, frac_0, frac_13, cvals);
+        map_cmyk_subclass(pprocs, dev, frac_0, frac_0, frac_0, frac_13, cvals);
         drawn_comps |= check_drawn_comps(ncomps, cvals);
 
         opdev->drawn_comps = drawn_comps;
@@ -1015,8 +1016,6 @@ overprint_copy_planes(gx_device * dev, const byte * data, int data_x, int raster
     int                     k,j;
     gs_memory_t *           mem = dev->memory;
     gx_color_index          comps = opdev->drawn_comps;
-    gx_color_index          mask;
-    int                     shift;
     byte                    *curr_data = (byte *) data + data_x;
     int                     row, offset;
 
@@ -1032,8 +1031,6 @@ overprint_copy_planes(gx_device * dev, const byte * data, int data_x, int raster
 
         fit_fill(tdev, x, y, w, h);
         byte_depth = depth / num_comps;
-        mask = ((gx_color_index)1 << byte_depth) - 1;
-        shift = 16 - byte_depth;
 
         /* allocate a buffer for the returned data */
         raster = bitmap_raster(w * byte_depth);
@@ -1122,7 +1119,6 @@ overprint_fill_rectangle_hl_color(gx_device *dev,
     gx_color_index          comps = opdev->drawn_comps;
     gx_color_index          mask;
     int                     shift;
-    bool                    blendspot = opdev->blendspot;
 
     if (tdev == 0)
         return 0;
@@ -1180,29 +1176,16 @@ overprint_fill_rectangle_hl_color(gx_device *dev,
                                "overprint_fill_rectangle_hl_color" );
                 return code;
             }
-            if (blendspot) {
-                /* Skip the plane if this component is not to be drawn.  We have
-                   to do a get bits for each plane due to the fact that we have
-                   to do a copy_planes at the end.  If we had a copy_plane 
-                   operation we would just get the ones need and set those. */
-                if ((comps & 0x01) == 1) {
-                    /* Not sure if a loop or a memset is better here */
-                    memset(gb_params.data[k], 
-                           ((pdcolor->colors.devn.values[k]) >> shift & mask), w);
-                }
-                comps >>= 1;
-            } else {
-                /* Skip the plane if this component is not to be drawn.  We have
-                   to do a get bits for each plane due to the fact that we have
-                   to do a copy_planes at the end.  If we had a copy_plane 
-                   operation we would just get the ones need and set those. */
-                if ((comps & 0x01) == 1) {
-                    /* Not sure if a loop or a memset is better here */
-                    memset(gb_params.data[k], 
-                           ((pdcolor->colors.devn.values[k]) >> shift & mask), w);
-                }
-                comps >>= 1;
+            /* Skip the plane if this component is not to be drawn.  We have
+                to do a get bits for each plane due to the fact that we have
+                to do a copy_planes at the end.  If we had a copy_plane 
+                operation we would just get the ones needed and set those. */
+            if ((comps & 0x01) == 1) {
+                /* Not sure if a loop or a memset is better here */
+                memset(gb_params.data[k], 
+                        ((pdcolor->colors.devn.values[k]) >> shift & mask), w);
             }
+            comps >>= 1;
         }
         code = dev_proc(tdev, copy_planes)(tdev, gb_buff, 0, raster, 
                                            gs_no_bitmap_id, x, y - 1, w, 1, 1);
@@ -1234,7 +1217,7 @@ overprint_sep_fill_rectangle(
          * bitmap. This is required only for littl-endian processors, and
          * then only if the depth > 8.
          */
-#if !arch_is_big_endian
+#if !ARCH_IS_BIG_ENDIAN
         if (depth > 8)
             color = swap_color_index(depth, color);
 #endif
@@ -1359,6 +1342,9 @@ c_overprint_create_default_compositor(
 
     gx_device_copy_params((gx_device *)opdev, tdev);
     gx_device_set_target((gx_device_forward *)opdev, tdev);
+    opdev->pad = tdev->pad;
+    opdev->log2_align_mod = tdev->log2_align_mod;
+    opdev->is_planar = tdev->is_planar;
 
     params = ovrpct->params;
     params.idle = ovrpct->idle;
