@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gspaint.c 12299 2011-03-17 09:54:53Z chrisl $ */
+
 /* Painting procedures for Ghostscript library */
 #include "math_.h"		/* for fabs */
 #include "gx.h"
@@ -30,11 +32,13 @@
 #include "gzcpath.h"
 #include "gxhldevc.h"
 #include "gsutil.h"
+#include "gxdevsop.h"
+#include "gsicc_cms.h"
 
 /* Define the nominal size for alpha buffers. */
 #define abuf_nominal_SMALL 500
 #define abuf_nominal_LARGE 2000
-#if arch_small_memory
+#if ARCH_SMALL_MEMORY
 #  define abuf_nominal abuf_nominal_SMALL
 #else
 #ifdef DEBUG
@@ -56,10 +60,10 @@ gs_erasepage(gs_state * pgs)
     int code;
 
     if ((code = gs_gsave(pgs)) < 0)
-	return code;
+        return code;
     if ((code = gs_setgray(pgs, 1.0)) >= 0) {
-	/* Fill the page directly, ignoring clipping. */
-	code = gs_fillpage(pgs);
+        /* Fill the page directly, ignoring clipping. */
+        code = gs_fillpage(pgs);
     }
     gs_grestore(pgs);
     return code;
@@ -71,24 +75,38 @@ gs_fillpage(gs_state * pgs)
 {
     gx_device *dev = gs_currentdevice(pgs);
     int code;
+    gx_cm_color_map_procs *   pprocs;
 
+    pprocs = get_color_mapping_procs_subclass(dev);
     /* If we get here without a valid get_color_mapping_procs, fail */
-    if (dev_proc(dev, get_color_mapping_procs) == NULL || 
-        dev_proc(dev, get_color_mapping_procs) == gx_error_get_color_mapping_procs) {
-	emprintf1(dev->memory,
+    if (pprocs == NULL ||
+        /* Deliberately use the terminal device here */
+        dev_proc(dev, get_color_mapping_procs) ==  gx_error_get_color_mapping_procs) {
+        emprintf1(dev->memory,
                   "\n   *** Error: No get_color_mapping_procs for device: %s\n",
                   dev->dname);
-	return_error(gs_error_Fatal);
+        return_error(gs_error_Fatal);
     }
     /* Processing a fill object operation */
-    gs_set_object_tag((gs_imager_state*) pgs, GS_PATH_TAG);
+    dev_proc(pgs->device, set_graphics_type_tag)(pgs->device, GS_PATH_TAG);
 
-    gx_set_dev_color(pgs);
+    code = gx_set_dev_color(pgs);
+    if (code != 0)
+        return code;
 
     code = (*dev_proc(dev, fillpage))(dev, (gs_imager_state *)pgs,
-				      gs_currentdevicecolor_inline(pgs));
+                                      gs_currentdevicecolor_inline(pgs));
     if (code < 0)
-	return code;
+        return code;
+
+    /* If GrayDetection is set, make sure monitoring is enabled. */
+    if (dev->icc_struct != NULL &&
+            dev->icc_struct->graydetection && !dev->icc_struct->pageneutralcolor) {
+        dev->icc_struct->pageneutralcolor = true;	/* start detecting again */
+        code = gsicc_mcm_begin_monitor(pgs->icc_link_cache, dev);
+    }
+    if (code < 0)
+        return code;
     return (*dev_proc(dev, sync_output)) (dev);
 }
 /*
@@ -113,62 +131,63 @@ scale_paths(gs_state * pgs, int log2_scale_x, int log2_scale_y, bool do_path)
      * aliased.
      */
     const gx_path_segments *seg_clip =
-	(pgs->clip_path->path_valid ? pgs->clip_path->path.segments : 0);
+        (pgs->clip_path->path_valid ? pgs->clip_path->path.segments : 0);
     const gx_clip_rect_list *list_clip = pgs->clip_path->rect_list;
     const gx_path_segments *seg_view_clip;
     const gx_clip_rect_list *list_view_clip;
     const gx_path_segments *seg_effective_clip =
-	(pgs->effective_clip_path->path_valid ?
-	 pgs->effective_clip_path->path.segments : 0);
+        (pgs->effective_clip_path->path_valid ?
+         pgs->effective_clip_path->path.segments : 0);
     const gx_clip_rect_list *list_effective_clip =
-	pgs->effective_clip_path->rect_list;
+        pgs->effective_clip_path->rect_list;
 
     gx_cpath_scale_exp2_shared(pgs->clip_path, log2_scale_x, log2_scale_y,
-			       false, false);
+                               false, false);
     if (pgs->view_clip != 0 && pgs->view_clip != pgs->clip_path) {
-	seg_view_clip =
-	    (pgs->view_clip->path_valid ? pgs->view_clip->path.segments : 0);
-	list_view_clip = pgs->view_clip->rect_list;
-	gx_cpath_scale_exp2_shared(pgs->view_clip, log2_scale_x, log2_scale_y,
-				   list_view_clip == list_clip,
-				   seg_view_clip && seg_view_clip == seg_clip);
+        seg_view_clip =
+            (pgs->view_clip->path_valid ? pgs->view_clip->path.segments : 0);
+        list_view_clip = pgs->view_clip->rect_list;
+        gx_cpath_scale_exp2_shared(pgs->view_clip, log2_scale_x, log2_scale_y,
+                                   list_view_clip == list_clip,
+                                   seg_view_clip && seg_view_clip == seg_clip);
     } else
-	seg_view_clip = 0, list_view_clip = 0;
+        seg_view_clip = 0, list_view_clip = 0;
     if (pgs->effective_clip_path != pgs->clip_path &&
-	pgs->effective_clip_path != pgs->view_clip
-	)
-	gx_cpath_scale_exp2_shared(pgs->effective_clip_path, log2_scale_x,
-				   log2_scale_y,
-				   list_effective_clip == list_clip ||
-				   list_effective_clip == list_view_clip,
-				   seg_effective_clip &&
-				   (seg_effective_clip == seg_clip ||
-				    seg_effective_clip == seg_view_clip));
+        pgs->effective_clip_path != pgs->view_clip
+        )
+        gx_cpath_scale_exp2_shared(pgs->effective_clip_path, log2_scale_x,
+                                   log2_scale_y,
+                                   list_effective_clip == list_clip ||
+                                   list_effective_clip == list_view_clip,
+                                   seg_effective_clip &&
+                                   (seg_effective_clip == seg_clip ||
+                                    seg_effective_clip == seg_view_clip));
     if (do_path) {
-	const gx_path_segments *seg_path = pgs->path->segments;
+        const gx_path_segments *seg_path = pgs->path->segments;
 
-	gx_path_scale_exp2_shared(pgs->path, log2_scale_x, log2_scale_y,
-				  seg_path == seg_clip ||
-				  seg_path == seg_view_clip ||
-				  seg_path == seg_effective_clip);
+        gx_path_scale_exp2_shared(pgs->path, log2_scale_x, log2_scale_y,
+                                  seg_path == seg_clip ||
+                                  seg_path == seg_view_clip ||
+                                  seg_path == seg_effective_clip);
     }
     return 0;
 }
 static void
-scale_dash_pattern(gs_state * pgs, floatp scale)
+scale_dash_pattern(gs_state * pgs, double scale)
 {
     int i;
 
     for (i = 0; i < pgs->line_params.dash.pattern_size; ++i)
-	pgs->line_params.dash.pattern[i] *= scale;
+        pgs->line_params.dash.pattern[i] *= scale;
     pgs->line_params.dash.offset *= scale;
     pgs->line_params.dash.pattern_length *= scale;
     pgs->line_params.dash.init_dist_left *= scale;
     if (pgs->line_params.dot_length_absolute)
-	pgs->line_params.dot_length *= scale;
+        pgs->line_params.dot_length *= scale;
 }
 static int
-alpha_buffer_init(gs_state * pgs, fixed extra_x, fixed extra_y, int alpha_bits)
+alpha_buffer_init(gs_state * pgs, fixed extra_x, fixed extra_y, int alpha_bits, 
+                  bool devn)
 {
     gx_device *dev = gs_currentdevice_inline(pgs);
     int log2_alpha_bits = ilog2(alpha_bits);
@@ -191,21 +210,26 @@ alpha_buffer_init(gs_state * pgs, fixed extra_x, fixed extra_y, int alpha_bits)
     band_space = raster << log2_scale.y;
     height = (abuf_nominal / band_space) << log2_scale.y;
     if (height == 0)
-	height = 1 << log2_scale.y;
+        height = 1 << log2_scale.y;
     mem = pgs->memory;
     mdev = gs_alloc_struct(mem, gx_device_memory, &st_device_memory,
-			   "alpha_buffer_init");
+                           "alpha_buffer_init");
     if (mdev == 0)
-	return 0;		/* if no room, don't buffer */
+        return 0;		/* if no room, don't buffer */
+    /* We may have to update the marking parameters if we have a pdf14 device
+       as our target.  Need to do while dev is still active in pgs */
+    if (dev_proc(dev, dev_spec_op)(dev, gxdso_is_pdf14_device, NULL, 0) > 0) {
+        gs_update_trans_marking_params(pgs);
+    }
     gs_make_mem_abuf_device(mdev, mem, dev, &log2_scale,
-			    alpha_bits, ibox.p.x << log2_scale.x);
+                            alpha_bits, ibox.p.x << log2_scale.x, devn);
     mdev->width = width;
     mdev->height = height;
     mdev->bitmap_memory = mem;
     if ((*dev_proc(mdev, open_device)) ((gx_device *) mdev) < 0) {
-	/* No room for bits, punt. */
-	gs_free_object(mem, mdev, "alpha_buffer_init");
-	return 0;
+        /* No room for bits, punt. */
+        gs_free_object(mem, mdev, "alpha_buffer_init");
+        return 0;
     }
     gx_set_device_only(pgs, (gx_device *) mdev);
     scale_paths(pgs, log2_scale.x, log2_scale.y, true);
@@ -217,12 +241,12 @@ static int
 alpha_buffer_release(gs_state * pgs, bool newpath)
 {
     gx_device_memory *mdev =
-	(gx_device_memory *) gs_currentdevice_inline(pgs);
+        (gx_device_memory *) gs_currentdevice_inline(pgs);
     int code = (*dev_proc(mdev, close_device)) ((gx_device *) mdev);
 
     if (code >= 0)
-	scale_paths(pgs, -mdev->log2_scale.x, -mdev->log2_scale.y,
-		!(newpath && !gx_path_is_shared(pgs->path)));
+        scale_paths(pgs, -mdev->log2_scale.x, -mdev->log2_scale.y,
+                !(newpath && !gx_path_is_shared(pgs->path)));
     /* Reference counting will free mdev. */
     gx_set_device_only(pgs, mdev->target);
     return code;
@@ -231,13 +255,14 @@ alpha_buffer_release(gs_state * pgs, bool newpath)
 static int do_fill(gs_state *pgs, int rule)
 {
     int code, abits, acode, rcode = 0;
+    bool devn;
 
     /* Here we need to distinguish text from vectors to compute the object tag.
        Actually we need to know whether this function is called to rasterize a character,
        or to rasterize a vector graphics to the output device.
        Currently we assume it works for the bitrgbtags device only,
        which is a low level device with a 4-component color model.
-       We use the fact that with printers a character is usually being rendered 
+       We use the fact that with printers a character is usually being rendered
        to a 1bpp cache device rather than to the output device.
        Therefore we hackly look whether the target device
        "has a color" : either it's a multicomponent color model,
@@ -246,45 +271,50 @@ static int do_fill(gs_state *pgs, int rule)
        This check has several limitations :
        1. It doesn't work with -dNOCACHE.
        2. It doesn't work with large characters,
-	  which cannot fit into a cache cell and thus they
-	  render directly to the output device.
+          which cannot fit into a cache cell and thus they
+          render directly to the output device.
        3. It doesn't work for TextAlphaBits=2 or 4.
-	  We don't care of this case because
-	  text antialiasing usually usn't applied to printers.
+          We don't care of this case because
+          text antialiasing usually usn't applied to printers.
        4. It doesn't work for things like with "(xyz) true charpath stroke".
-	  That's unfortunate, we'd like to improve someday.
+          That's unfortunate, we'd like to improve someday.
        5. It doesn't work for high level devices when a Type 3 character is being constructed.
-	  This case is not important for low level devices
-	  (which a printer is), because low level device doesn't accept
-	  Type 3 charproc streams immediately.
+          This case is not important for low level devices
+          (which a printer is), because low level device doesn't accept
+          Type 3 charproc streams immediately.
        6. It doesn't work properly while an insiding testing,
-	  which sets gs_hit_device, which is uncolored.
+          which sets gs_hit_device, which is uncolored.
      */
     if (gx_device_has_color(gs_currentdevice(pgs))) {
-	gs_set_object_tag((gs_imager_state*) pgs, GS_PATH_TAG);
+        dev_proc(pgs->device, set_graphics_type_tag)(pgs->device, GS_PATH_TAG);
     }
     else {
-	gs_set_object_tag((gs_imager_state*) pgs, GS_TEXT_TAG);
+        dev_proc(pgs->device, set_graphics_type_tag)(pgs->device, GS_TEXT_TAG);
     }
-    gx_set_dev_color(pgs);
+    code = gx_set_dev_color(pgs);
+    if (code != 0)
+        return code;
     code = gs_state_color_load(pgs);
     if (code < 0)
-	return code;
-    abits = alpha_buffer_bits(pgs);
-    if (!color_is_pure(gs_currentdevicecolor_inline(pgs))) {
-        abits = 0;
+        return code;
+    abits = 0;
+    {
+        gx_device_color *col = gs_currentdevicecolor_inline(pgs);
+        devn = color_is_devn(col);
+        if (color_is_pure(col) || devn)
+            abits = alpha_buffer_bits(pgs);
     }
     if (abits > 1) {
-	acode = alpha_buffer_init(pgs, pgs->fill_adjust.x,
-				  pgs->fill_adjust.y, abits);
-	if (acode < 0)
-	    return acode;
+        acode = alpha_buffer_init(pgs, pgs->fill_adjust.x,
+                                  pgs->fill_adjust.y, abits, devn);
+        if (acode < 0)
+            return acode;
     } else
-	acode = 0;
+        acode = 0;
     code = gx_fill_path(pgs->path, gs_currentdevicecolor_inline(pgs), pgs, rule,
-			pgs->fill_adjust.x, pgs->fill_adjust.y);
+                        pgs->fill_adjust.x, pgs->fill_adjust.y);
     if (acode > 0)
-	rcode = alpha_buffer_release(pgs, code >= 0);
+        rcode = alpha_buffer_release(pgs, code >= 0);
     if (code >= 0 && rcode < 0)
         code = rcode;
 
@@ -300,8 +330,8 @@ fill_with_rule(gs_state * pgs, int rule)
     /* If we're inside a charpath, just merge the current path */
     /* into the parent's path. */
     if (pgs->in_charpath)
-	code = gx_path_add_char_path(pgs->show_gstate->path, pgs->path,
-				     pgs->in_charpath);
+        code = gx_path_add_char_path(pgs->show_gstate->path, pgs->path,
+                                     pgs->in_charpath);
             /* If we're rendering a glyph cached, the show machinery decides
              * whether to actually image it on the output or not, but uncached
              * will render directly to the output, so for text rendering
@@ -311,13 +341,13 @@ fill_with_rule(gs_state * pgs, int rule)
     else if (gs_is_null_device(pgs->device)
             || (pgs->show_gstate && pgs->text_rendering_mode == 3
             && pgs->in_cachedevice == CACHE_DEVICE_NOT_CACHING)) {
-	/* Handle separately to prevent gs_state_color_load - bug 688308. */
-	gs_newpath(pgs);
-	code = 0;
+        /* Handle separately to prevent gs_state_color_load - bug 688308. */
+        gs_newpath(pgs);
+        code = 0;
     } else {
         code = do_fill(pgs, rule);
-	if (code >= 0)
-	    gs_newpath(pgs);
+        if (code >= 0)
+            gs_newpath(pgs);
     }
     return code;
 }
@@ -340,21 +370,14 @@ static int
 do_stroke(gs_state * pgs)
 {
     int code, abits, acode, rcode = 0;
-
-    /* to distinguish text from vectors we hackly look at the
-       target device 1 bit per component is a cache and this is
-       text else it is a path */
-    if (gx_device_has_color(gs_currentdevice(pgs)))
-        gs_set_object_tag((gs_imager_state*) pgs, GS_PATH_TAG);
-    else
-        gs_set_object_tag((gs_imager_state*) pgs, GS_TEXT_TAG);
+    bool devn;
 
     /* Here we need to distinguish text from vectors to compute the object tag.
        Actually we need to know whether this function is called to rasterize a character,
        or to rasterize a vector graphics to the output device.
        Currently we assume it works for the bitrgbtags device only,
        which is a low level device with a 4-component color model.
-       We use the fact that with printers a character is usually being rendered 
+       We use the fact that with printers a character is usually being rendered
        to a 1bpp cache device rather than to the output device.
        Therefore we hackly look whether the target device
        "has a color" : either it's a multicomponent color model,
@@ -376,69 +399,73 @@ do_stroke(gs_state * pgs)
           Type 3 charproc streams immediately.
      */
     if (gx_device_has_color(gs_currentdevice(pgs))) {
-	gs_set_object_tag((gs_imager_state*) pgs, GS_PATH_TAG);
+        dev_proc(pgs->device, set_graphics_type_tag)(pgs->device, GS_PATH_TAG);
     }
     else {
-	gs_set_object_tag((gs_imager_state*) pgs, GS_TEXT_TAG);
+        dev_proc(pgs->device, set_graphics_type_tag)(pgs->device, GS_TEXT_TAG);
     }
-    /* Evil: The following call is a macro that might return! */
-    gx_set_dev_color(pgs);
+    code = gx_set_dev_color(pgs);
+    if (code != 0)
+        return code;
     code = gs_state_color_load(pgs);
     if (code < 0)
-	return code;
-    abits = alpha_buffer_bits(pgs);
-    if (!color_is_pure(gs_currentdevicecolor_inline(pgs))) {
-        abits = 0;
+        return code;
+    abits = 0;
+    {
+        gx_device_color *col = gs_currentdevicecolor_inline(pgs);
+        devn = color_is_devn(col);
+        if (color_is_pure(col) || devn)
+            abits = alpha_buffer_bits(pgs);
     }
     if (abits > 1) {
-	/*
-	 * Expand the bounding box by the line width.
-	 * This is expensive to compute, so we only do it
-	 * if we know we're going to buffer.
-	 */
-	float xxyy = fabs(pgs->ctm.xx) + fabs(pgs->ctm.yy);
-	float xyyx = fabs(pgs->ctm.xy) + fabs(pgs->ctm.yx);
-	float scale = (float)(1 << (abits / 2));
-	float orig_width = gs_currentlinewidth(pgs);
-	float new_width = orig_width * scale;
-	fixed extra_adjust =
-		float2fixed(max(xxyy, xyyx) * new_width / 2);
-	float orig_flatness = gs_currentflat(pgs);
-	gx_path spath;
+        /*
+         * Expand the bounding box by the line width.
+         * This is expensive to compute, so we only do it
+         * if we know we're going to buffer.
+         */
+        float xxyy = fabs(pgs->ctm.xx) + fabs(pgs->ctm.yy);
+        float xyyx = fabs(pgs->ctm.xy) + fabs(pgs->ctm.yx);
+        float scale = (float)(1 << (abits / 2));
+        float orig_width = gs_currentlinewidth(pgs);
+        float new_width = orig_width * scale;
+        fixed extra_adjust =
+                float2fixed(max(xxyy, xyyx) * new_width / 2);
+        float orig_flatness = gs_currentflat(pgs);
+        gx_path spath;
 
-	/* Scale up the line width, dash pattern, and flatness. */
-	if (extra_adjust < fixed_1)
-	    extra_adjust = fixed_1;
-	acode = alpha_buffer_init(pgs,
-				  pgs->fill_adjust.x + extra_adjust,
-				  pgs->fill_adjust.y + extra_adjust,
-				  abits);
-	if (acode < 0)
-	    return acode;
-	gs_setlinewidth(pgs, new_width);
-	scale_dash_pattern(pgs, scale);
-	gs_setflat(pgs, orig_flatness * scale);
-	/*
-	 * The alpha-buffer device requires that we fill the
-	 * entire path as a single unit.
-	 */
-	gx_path_init_local(&spath, pgs->memory);
-	code = gx_stroke_add(pgs->path, &spath, pgs, false);
-	gs_setlinewidth(pgs, orig_width);
-	scale_dash_pattern(pgs, 1.0 / scale);
-	if (code >= 0)
-	    code = gx_fill_path(&spath, gs_currentdevicecolor_inline(pgs), pgs,
-				gx_rule_winding_number,
-				pgs->fill_adjust.x,
-				pgs->fill_adjust.y);
-	gs_setflat(pgs, orig_flatness);
-	gx_path_free(&spath, "gs_stroke");
-	if (acode > 0)
-	    rcode = alpha_buffer_release(pgs, code >= 0);
+        /* Scale up the line width, dash pattern, and flatness. */
+        if (extra_adjust < fixed_1)
+            extra_adjust = fixed_1;
+        acode = alpha_buffer_init(pgs,
+                                  pgs->fill_adjust.x + extra_adjust,
+                                  pgs->fill_adjust.y + extra_adjust,
+                                  abits, devn);
+        if (acode < 0)
+            return acode;
+        gs_setlinewidth(pgs, new_width);
+        scale_dash_pattern(pgs, scale);
+        gs_setflat(pgs, orig_flatness * scale);
+        /*
+         * The alpha-buffer device requires that we fill the
+         * entire path as a single unit.
+         */
+        gx_path_init_local(&spath, pgs->memory);
+        code = gx_stroke_add(pgs->path, &spath, pgs, false);
+        gs_setlinewidth(pgs, orig_width);
+        scale_dash_pattern(pgs, 1.0 / scale);
+        if (code >= 0)
+            code = gx_fill_path(&spath, gs_currentdevicecolor_inline(pgs), pgs,
+                                gx_rule_winding_number,
+                                pgs->fill_adjust.x,
+                                pgs->fill_adjust.y);
+        gs_setflat(pgs, orig_flatness);
+        gx_path_free(&spath, "gs_stroke");
+        if (acode > 0)
+            rcode = alpha_buffer_release(pgs, code >= 0);
     } else
-	code = gx_stroke_fill(pgs->path, pgs);
+        code = gx_stroke_fill(pgs->path, pgs);
     if (code >= 0 && rcode < 0)
-	code = rcode;
+        code = rcode;
     return code;
 }
 
@@ -453,26 +480,26 @@ gs_stroke(gs_state * pgs)
      * into the parent's path.
      */
     if (pgs->in_charpath) {
-	if (pgs->in_charpath == cpm_true_charpath) {
-	    /*
-	     * A stroke inside a true charpath should do the
-	     * equivalent of strokepath.
-	     */
-	    code = gs_strokepath(pgs);
-	    if (code < 0)
-		return code;
-	}
-	code = gx_path_add_char_path(pgs->show_gstate->path, pgs->path,
-				     pgs->in_charpath);
+        if (pgs->in_charpath == cpm_true_charpath) {
+            /*
+             * A stroke inside a true charpath should do the
+             * equivalent of strokepath.
+             */
+            code = gs_strokepath(pgs);
+            if (code < 0)
+                return code;
+        }
+        code = gx_path_add_char_path(pgs->show_gstate->path, pgs->path,
+                                     pgs->in_charpath);
     }
     if (gs_is_null_device(pgs->device)) {
-	/* Handle separately to prevent gs_state_color_load. */
-	gs_newpath(pgs);
-	code = 0;
+        /* Handle separately to prevent gs_state_color_load. */
+        gs_newpath(pgs);
+        code = 0;
     } else {
         code = do_stroke(pgs);
-	if (code >= 0)
-	    gs_newpath(pgs);
+        if (code >= 0)
+            gs_newpath(pgs);
     }
     return code;
 }
@@ -487,15 +514,15 @@ gs_strokepath_aux(gs_state * pgs, bool traditional)
     gx_path_init_local(&spath, pgs->path->memory);
     code = gx_stroke_add(pgs->path, &spath, pgs, traditional);
     if (code < 0) {
-	gx_path_free(&spath, "gs_strokepath");
-	return code;
+        gx_path_free(&spath, "gs_strokepath");
+        return code;
     }
     pgs->device->sgr.stroke_stored = false;
     code = gx_path_assign_free(pgs->path, &spath);
     if (code < 0)
-	return code;
+        return code;
     /* NB: needs testing with PCL */
-    if (gs_currentcpsimode(pgs->memory) && gx_path_is_void(pgs->path))
+    if (gx_path_is_void(pgs->path))
         pgs->current_point_valid = false;
     else
         gx_setcurrentpoint(pgs, fixed2float(spath.position.x), fixed2float(spath.position.y));
