@@ -158,7 +158,9 @@ gs_pattern1_make_pattern(gs_client_color * pcc,
             gs_set_logical_op(saved, lop_default);
             break;
         case 2:         /* uncolored */
-            gx_set_device_color_1(saved);
+            code = gx_set_device_color_1(saved);
+            if (code < 0)
+                goto fsaved;
             break;
         default:
             code = gs_note_error(gs_error_rangecheck);
@@ -405,11 +407,22 @@ clamp_pattern_bbox(gs_pattern1_instance_t * pinst, gs_rect * pbbox,
     gs_point dev_pat_origin, dev_step;
     int code;
 
+    double xepsilon = FLT_EPSILON * width;
+    double yepsilon = FLT_EPSILON * height;
+
     /*
      * Scan across the page.  We determine the region to be scanned
      * by working in the pattern coordinate space.  This is logically
      * simpler since XStep and YStep are on axis in the pattern space.
      */
+    /* But, since we are starting below bottom left, and 'incrementing' by
+     * xstep and ystep, make sure they are not negative, or we will be in
+     * a very long loop indeed.
+     */
+    if (xstep < 0)
+        xstep *= -1;
+    if (ystep < 0)
+        ystep *= -1;
     /*
      * Convert the page dimensions from device coordinates into the
      * pattern coordinate frame.
@@ -450,7 +463,21 @@ clamp_pattern_bbox(gs_pattern1_instance_t * pinst, gs_rect * pbbox,
             xupper = (xdev + pbbox->q.x < width) ? pbbox->q.x : -xdev + width;
             ylower = (ydev + pbbox->p.y > 0) ? pbbox->p.y : -ydev;
             yupper = (ydev + pbbox->q.y < height) ? pbbox->q.y : -ydev + height;
-            if (xlower < xupper && ylower < yupper) {
+
+            /* The use of floating point in these calculations causes us
+             * problems. Values which go through the calculation without ever
+             * being 'large' retain more accuracy in the lower bits than ones
+             * which momentarily become large. This is seen in bug 694528
+             * where a y value of 0.00017... becomes either 0 when 8000 is
+             * first added to it, then subtracted. This can lead to yupper
+             * and ylower being different.
+             *
+             * The "fix" implemented here is to amend the following test to
+             * ensure that the region found is larger that 'epsilon'. The
+             * epsilon values are calculated to reflect the floating point
+             * innacuracies at the appropriate range.
+             */
+            if (xlower + xepsilon < xupper && ylower + yepsilon < yupper) {
                 /*
                  * The pattern intersects the page.  Expand required area if
                  * needed.
@@ -855,6 +882,143 @@ bitmap_paint(gs_image_enum * pen, gs_data_image_t * pim,
     return code;
 }
 
+int pixmap_high_level_pattern(gs_state * pgs)
+{
+    gs_matrix m;
+    gs_rect bbox;
+    gs_fixed_rect clip_box;
+    int code;
+    gx_device_color *pdc = gs_currentdevicecolor_inline(pgs);
+    const gs_client_pattern *ppat = gs_getpattern(&pdc->ccolor);
+    gs_color_space *pcs;
+    gs_pattern1_instance_t *pinst =
+        (gs_pattern1_instance_t *)gs_currentcolor(pgs)->pattern;
+    const pixmap_info *ppmap = ppat->client_data;
+
+    code = gx_pattern_cache_add_dummy_entry((gs_imager_state *)pgs,
+        pinst, pgs->device->color_info.depth);
+    if (code < 0)
+        return code;
+
+    code = gs_gsave(pgs);
+    if (code < 0)
+        return code;
+
+    dev_proc(pgs->device, get_initial_matrix)(pgs->device, &m);
+    gs_setmatrix(pgs, &m);
+    code = gs_bbox_transform(&ppat->BBox, &ctm_only(pgs), &bbox);
+    if (code < 0) {
+        gs_grestore(pgs);
+            return code;
+    }
+    clip_box.p.x = float2fixed(bbox.p.x);
+    clip_box.p.y = float2fixed(bbox.p.y);
+    clip_box.q.x = float2fixed(bbox.q.x);
+    clip_box.q.y = float2fixed(bbox.q.y);
+    code = gx_clip_to_rectangle(pgs, &clip_box);
+    if (code < 0) {
+        gs_grestore(pgs);
+        return code;
+    }
+
+    {
+        pattern_accum_param_s param;
+        param.pinst = (void *)pinst;
+        param.graphics_state = (void *)pgs;
+        param.pinst_id = pinst->id;
+
+        code = dev_proc(pgs->device, dev_spec_op)(pgs->device,
+                                gxdso_pattern_start_accum, &param, sizeof(pattern_accum_param_s));
+    }
+
+    if (code < 0) {
+        gs_grestore(pgs);
+        return code;
+    }
+
+    if (ppmap->pcspace != 0)
+        code = image_PaintProc(&pdc->ccolor, pgs);
+    else {
+        pcs = gs_cspace_new_DeviceGray(pgs->memory);
+        gs_setcolorspace(pgs, pcs);
+        code = mask_PaintProc(&pdc->ccolor, pgs);
+    }
+    if (code < 0)
+        return code;
+
+    code = gs_grestore(pgs);
+    if (code < 0)
+        return code;
+
+    {
+        pattern_accum_param_s param;
+        param.pinst = (void *)pinst;
+        param.graphics_state = (void *)pgs;
+        param.pinst_id = pinst->id;
+
+        code = dev_proc(pgs->device, dev_spec_op)(pgs->device,
+                          gxdso_pattern_finish_accum, &param, sizeof(pattern_accum_param_s));
+    }
+
+    return code;
+}
+
+static int pixmap_remap_mask_pattern(const gs_client_color *pcc, gs_state *pgs)
+{
+    const gs_client_pattern *ppat = gs_getpattern(pcc);
+    int code = 0;
+
+    /* pgs->device is the newly created pattern accumulator, but we want to test the device
+     * that is 'behind' that, the actual output device, so we use the one from
+     * the saved graphics state.
+     */
+    if (pgs->have_pattern_streams)
+        code = dev_proc(pcc->pattern->saved->device, dev_spec_op)(pcc->pattern->saved->device,
+                                gxdso_pattern_can_accum, (void *)ppat, ppat->uid.id);
+
+    if (code == 1) {
+        /* Device handles high-level patterns, so return 'remap'.
+         * This closes the internal accumulator device, as we no longer need
+         * it, and the error trickles back up to the PDL client. The client
+         * must then take action to start the device's accumulator, draw the
+         * pattern, close the device's accumulator and generate a cache entry.
+         * See px_high_level_pattern above.
+         */
+        return_error(gs_error_Remap_Color);
+    } else {
+        mask_PaintProc(pcc, pgs);
+        return 0;
+    }
+}
+
+static int pixmap_remap_image_pattern(const gs_client_color *pcc, gs_state *pgs)
+{
+    const gs_client_pattern *ppat = gs_getpattern(pcc);
+    int code = 0;
+
+    /* pgs->device is the newly created pattern accumulator, but we want to test the device
+     * that is 'behind' that, the actual output device, so we use the one from
+     * the saved graphics state.
+     */
+    if (pgs->have_pattern_streams)
+        code = dev_proc(pcc->pattern->saved->device, dev_spec_op)(pcc->pattern->saved->device,
+                                gxdso_pattern_can_accum, (void *)ppat, ppat->uid.id);
+
+    if (code == 1) {
+        /* Device handles high-level patterns, so return 'remap'.
+         * This closes the internal accumulator device, as we no longer need
+         * it, and the error trickles back up to the PDL client. The client
+         * must then take action to start the device's accumulator, draw the
+         * pattern, close the device's accumulator and generate a cache entry.
+         * See px_high_level_pattern above.
+         */
+        return_error(gs_error_Remap_Color);
+    } else {
+        image_PaintProc(pcc, pgs);
+        return 0;
+    }
+}
+
 /*
  * Make a pattern from a bitmap or pixmap. The pattern may be colored or
  * uncolored, as determined by the mask operand. This code is intended
@@ -917,7 +1081,7 @@ gs_makepixmappattern(
     pat.BBox.q.y = pbitmap->size.y;
     pat.XStep = (float)pbitmap->size.x;
     pat.YStep = (float)pbitmap->size.y;
-    pat.PaintProc = (mask ? mask_PaintProc : image_PaintProc);
+    pat.PaintProc = (mask ? pixmap_remap_mask_pattern : pixmap_remap_image_pattern);
     pat.client_data = ppmap;
 
     /* set the ctm to be the identity */
@@ -1160,10 +1324,12 @@ static RELOC_PTRS_WITH(dc_colored_masked_reloc_ptrs, gx_device_color *cptr)
 RELOC_PTRS_END
 static ENUM_PTRS_WITH(dc_devn_masked_enum_ptrs, gx_device_color *cptr)
 ENUM_SUPER(gx_device_color, st_client_color, ccolor, 0);
+(void)cptr; /* Avoid unused warning */
 ENUM_PTRS_END
 static RELOC_PTRS_WITH(dc_devn_masked_reloc_ptrs, gx_device_color *cptr)
 {
     RELOC_SUPER(gx_device_color, st_client_color, ccolor);
+    (void)cptr; /* Avoid unused warning */
 }
 RELOC_PTRS_END
 static ENUM_PTRS_BEGIN(dc_binary_masked_enum_ptrs)
@@ -1415,7 +1581,6 @@ typedef struct tile_trans_clist_info_s {
     int n_chan; /* number of pixel planes including alpha */
     int width;
     int height;
-    gs_blend_mode_t blend_mode;
 } tile_trans_clist_info_t;
 
 typedef struct gx_dc_serialized_tile_s {
@@ -1425,6 +1590,7 @@ typedef struct gx_dc_serialized_tile_s {
     gs_matrix step_matrix;
     gs_rect bbox;
     int flags;
+    gs_blend_mode_t blending_mode;	/* in case tile has transparency */
 } gx_dc_serialized_tile_t;
 
 enum {
@@ -1565,6 +1731,7 @@ gx_dc_pattern_trans_write_raster(gx_color_tile *ptile, int64_t offset, byte *dat
                   | (ptile->has_overlap ? TILE_HAS_OVERLAP : 0);
         buf.step_matrix = ptile->step_matrix;
         buf.bbox = ptile->bbox;
+        buf.blending_mode = ptile->blending_mode;
         if (sizeof(buf) > left) {
             /* For a while we require the client to provide enough buffer size. */
             return_error(gs_error_unregistered); /* Must not happen. */
@@ -1575,7 +1742,6 @@ gx_dc_pattern_trans_write_raster(gx_color_tile *ptile, int64_t offset, byte *dat
         offset1 += sizeof(buf);
 
         /* Do the transparency information now */
-        trans_info.blend_mode = ptile->ttrans->blending_mode;
         trans_info.height = ptile->ttrans->height;
         trans_info.n_chan = ptile->ttrans->n_chan;
         trans_info.planestride = ptile->ttrans->planestride;
@@ -1643,8 +1809,10 @@ gx_dc_pattern_write(
         /* A special case for writing a known pattern :
            Just write the tile id. */
         gs_id id = ptile->id; /* Ensure sizeof(gs_id). */
-
-        memcpy(dp, &ptile->id, sizeof(id));
+		if_debug2m('?', dev->memory,
+			"[v*] Writing trans tile ID into clist, uid = %ld id = %ld \n",
+			ptile->uid.id, ptile->id);
+		memcpy(dp, &ptile->id, sizeof(id));
         *psize = sizeof(gs_id);
         return 0;
     }
@@ -1652,9 +1820,12 @@ gx_dc_pattern_write(
     /* Check if pattern has transparency object
        If so then that is what we will stuff in
        the clist */
-
-    if (ptile->ttrans != NULL)
-        return gx_dc_pattern_trans_write_raster(ptile, offset, data, psize);
+	if (ptile->ttrans != NULL) {
+		if_debug2m('?', dev->memory,
+			"[v*] Writing trans tile into clist, uid = %ld id = %ld \n",
+			ptile->uid.id, ptile->id);
+		return gx_dc_pattern_trans_write_raster(ptile, offset, data, psize);
+	}
 
     if (ptile->cdev == NULL)
         return gx_dc_pattern_write_raster(ptile, offset, data, psize, dev);
@@ -1685,6 +1856,7 @@ gx_dc_pattern_write(
                   | (ptile->is_simple ? TILE_IS_SIMPLE : 0)
                   | (ptile->has_overlap ? TILE_HAS_OVERLAP : 0)
                   | (ptile->cdev->common.page_uses_transparency ? TILE_USES_TRANSP : 0);
+        buf.blending_mode = ptile->blending_mode;    /* in case tile has transparency */
         if (sizeof(buf) > left) {
             /* For a while we require the client to provide enough buffer size. */
             return_error(gs_error_unregistered); /* Must not happen. */
@@ -1840,7 +2012,7 @@ gx_dc_pattern_read(
     int64_t offset1 = offset;
     gx_color_tile *ptile;
     int code, l;
-    tile_trans_clist_info_t trans_info;
+    tile_trans_clist_info_t trans_info = { { { 0 } } };
     int cache_space_needed;
 
     if (offset == 0) {
@@ -1908,6 +2080,7 @@ gx_dc_pattern_read(
         ptile->tiling_type = (buf.flags & TILE_TYPE_MASK)>>TILE_TYPE_SHIFT;
         ptile->is_simple = !!(buf.flags & TILE_IS_SIMPLE);
         ptile->has_overlap = !!(buf.flags & TILE_HAS_OVERLAP);
+        ptile->blending_mode = buf.blending_mode;
         ptile->is_dummy = 0;
 
         if (!(buf.flags & TILE_IS_CLIST)) {
@@ -1919,7 +2092,6 @@ gx_dc_pattern_read(
                 ptile->ttrans = new_pattern_trans_buff(mem);
                 /* trans_info was loaded above */
 
-                ptile->ttrans->blending_mode = trans_info.blend_mode;
                 ptile->ttrans->height = trans_info.height;
                 ptile->ttrans->n_chan = trans_info.n_chan;
                 ptile->ttrans->pdev14 = NULL;
@@ -1931,6 +2103,9 @@ gx_dc_pattern_read(
                 ptile->ttrans->rowstride = trans_info.rowstride;
                 ptile->ttrans->width = trans_info.width;
                 pdevc->type = &gx_dc_pattern_trans;
+				if_debug2m('?', pis->memory,
+					"[v*] Reading trans tile from clist into cache, uid = %ld id = %ld \n",
+					ptile->uid.id, ptile->id);
 
                 code = gx_dc_pattern_read_trans_buff(ptile, offset1, dp, left, mem);
                 if (code < 0)
@@ -1968,8 +2143,6 @@ gx_dc_pattern_read(
                                &inst, "gx_dc_pattern_read");
             if (ptile->cdev == NULL)
                 return_error(gs_error_VMerror);
-            ptile->cdev->common.band_params.page_uses_transparency =
-                                                         !!(buf.flags & TILE_USES_TRANSP);
             ptile->cdev->common.page_uses_transparency = !!(buf.flags & TILE_USES_TRANSP);
             code = dev_proc(&ptile->cdev->writer, open_device)((gx_device *)&ptile->cdev->writer);
             if (code < 0)
