@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsropt.h 12277 2011-03-11 05:45:19Z alexcher $ */
+
 /* RasterOp / transparency type definitions */
 
 #ifndef gsropt_INCLUDED
@@ -92,14 +94,21 @@ typedef enum {
  * RasterOp operand is called texture, not pattern.
  */
 
+/* We have to define rop3_{T,S,D} as enum values to shut compilers
+ * up that do switch checking. We also have to #define them to allow
+ * the preprocessor templating to work. */
+
 /* 3-input RasterOp */
 typedef enum {
     rop3_0 = 0,
-#define rop3_T 0xf0              /* texture */
+    rop3_T = 0xf0,              /* texture */
+#define rop3_T 0xf0
 #define rop3_T_shift 4
-#define rop3_S 0xcc              /* source */
+    rop3_S = 0xcc,              /* source */
+#define rop3_S 0xcc
 #define rop3_S_shift 2
     rop3_D = 0xaa,              /* destination */
+#define rop3_D 0xaa
 #define rop3_D_shift 1
     rop3_1 = 0xff,
     rop3_default = rop3_T | rop3_S
@@ -203,12 +212,15 @@ typedef enum {
  *
  * In addition, we define a "pdf14" flag which indicates that PDF
  * transparency is in effect. This doesn't change rendering in any way,
- * but does force the lop to be considered non-idempotent.
+ * but does force the lop to be considered non-idempotent. The lop_pdf14
+ * bit is used for fill/stroke ops but even if this is clear, we still
+ * may be painting in a transparency buffer in an idempotent mode.
  */
 #define lop_rop(lop) ((gs_rop3_t)((lop) & 0xff))        /* must be low-order bits */
 #define lop_S_transparent 0x100
 #define lop_T_transparent 0x200
 #define lop_pdf14 0x400
+
 typedef uint gs_logical_operation_t;
 
 #define lop_default\
@@ -300,7 +312,7 @@ struct rop_run_op_s {
     const byte *scolors;
     const byte *tcolors;
     void (*release)(rop_run_op *);
-    void *opaque;    
+    void *opaque;
 };
 
 /* Flags for passing into rop_get_run_op */
@@ -377,19 +389,15 @@ void rop_release_run_op(rop_run_op *op);
 #  define ENDIAN_SWAP_INT _byteswap_ulong
 #endif
 
-#elif defined(__GNUC__) /* Are we using GCC? */
+#elif defined(HAVE_BSWAP32)
 
-#ifdef __i386__ /* Are we on an x86? */
-#define GCC_VERSION (__GNUC__ * 10000 + __GNUC_MINOR__ * 100 + __GNUC_PATCHLEVEL__)
-#if GCC_VERSION >= 40300 /* Modern enough to have byteswap intrinsics? */
+#define ENDIAN_SWAP_INT __builtin_bswap32
+
+#elif defined(HAVE_BYTESWAP_H)
 
 #include <byteswap.h>
-
 #define ENDIAN_SWAP_INT bswap_32
 
-#endif /* GCC >= 4.3 */
-#endif /* x86 or later */
-
 #endif
 
 #endif /* gsropt_INCLUDED */
