@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Definitions for shading */
 
 #ifndef gsshade_INCLUDED
@@ -162,10 +164,6 @@ typedef struct gs_shading_mesh_s {
     gs_shading_mesh_params_t params;
 } gs_shading_mesh_t;
 
-#define private_st_shading_mesh()	/* in gsshade.c */\
-  gs_private_st_composite(st_shading_mesh, gs_shading_mesh_t,\
-    "gs_shading_mesh_t", shading_mesh_enum_ptrs, shading_mesh_reloc_ptrs)
-
 /* Define Free-form Gouraud triangle mesh shading. */
 typedef struct gs_shading_FfGt_params_s {
     gs_shading_mesh_params_common;
