@@ -158,7 +158,6 @@ typedef struct gs_state_parts_s {
 /* GC descriptors */
 extern_st(st_imager_state);
 public_st_gs_state();
-
 /* GC procedures for gs_state */
 static ENUM_PTRS_WITH(gs_state_enum_ptrs, gs_state *gsvptr)
 ENUM_PREFIX(st_imager_state, gs_state_num_ptrs + 2);
@@ -223,8 +222,19 @@ gs_state_alloc(gs_memory_t * mem)
 
     if (pgs == 0)
         return 0;
-    pgs->saved = 0;
     *(gs_imager_state *)pgs = gstate_initial;   /* this sets is_gstate == true */
+    /* Need to set up at least enough to make gs_state_free happy */
+    pgs->saved = 0;
+    pgs->path = NULL;
+    pgs->clip_path = NULL;
+    pgs->clip_stack = NULL;
+    pgs->view_clip = NULL;
+    pgs->effective_clip_path = NULL;
+    pgs->font = NULL;
+    pgs->root_font = NULL;
+    pgs->show_gstate = NULL;
+    pgs->device = NULL;
+    pgs->dfilter_stack = NULL;
 
     /*
      * Just enough of the state is initialized at this point
@@ -247,6 +257,8 @@ gs_state_alloc(gs_memory_t * mem)
     pgs->clip_path = gx_cpath_alloc(mem, "gs_state_alloc(clip_path)");
     pgs->clip_stack = 0;
     pgs->view_clip = gx_cpath_alloc(mem, "gs_state_alloc(view_clip)");
+    if (pgs->view_clip == NULL)
+        goto fail;
     pgs->view_clip->rule = 0;   /* no clipping */
     pgs->effective_clip_id = pgs->clip_path->id;
     pgs->effective_view_clip_id = gs_no_id;
@@ -254,14 +266,24 @@ gs_state_alloc(gs_memory_t * mem)
     pgs->effective_clip_shared = true;
     /* Initialize things so that gx_remap_color won't crash. */
     pgs->color[0].color_space = gs_cspace_new_DeviceGray(pgs->memory);
+    if (pgs->color[0].color_space == NULL)
+        goto fail;
     pgs->color[1].color_space = gs_cspace_new_DeviceGray(pgs->memory);
+    if (pgs->color[1].color_space == NULL)
+        goto fail;
     pgs->in_cachedevice = 0;
     gs_swapcolors_quick(pgs); /* To color 1 */
-    gx_set_device_color_1(pgs); /* sets colorspace and client color */
+    code = gx_set_device_color_1(pgs); /* sets colorspace and client color */
+    if (code < 0)
+        goto fail;
     gs_swapcolors_quick(pgs); /* To color 0 */
-    gx_set_device_color_1(pgs); /* sets colorspace and client color */
+    code = gx_set_device_color_1(pgs); /* sets colorspace and client color */
+    if (code < 0)
+        goto fail;
     pgs->device = 0;            /* setting device adjusts refcts */
-    gs_nulldevice(pgs);
+    code = gs_nulldevice(pgs);
+    if (code < 0)
+        goto fail;
     gs_setalpha(pgs, 1.0);
     gs_settransfer(pgs, gs_identity_transfer);
     gs_setflat(pgs, 1.0);
@@ -621,15 +643,19 @@ gs_state_update_overprint(gs_state * pgs, const gs_overprint_params_t * pparams)
     gx_device *         dev = pgs->device;
     gx_device *         ovptdev;
 
-    if ( (code = gs_create_overprint(&pct, pparams, pgs->memory)) >= 0 &&
-         (code = dev_proc(dev, create_compositor)( dev,
+    code = gs_create_overprint(&pct, pparams, pgs->memory);
+    if (code >= 0) {
+        code = dev_proc(dev, create_compositor)( dev,
                                                    &ovptdev,
                                                    pct,
                                                    pis,
                                                    pgs->memory,
-                                                   NULL)) >= 0   ) {
-        if (ovptdev != dev)
-            gx_set_device_only(pgs, ovptdev);
+                                                   NULL);
+        if (code >= 0 || code == gs_error_handled){
+            if (ovptdev != dev)
+                gx_set_device_only(pgs, ovptdev);
+            code = 0;
+        }
     }
     if (pct != 0)
         gs_free_object(pgs->memory, pct, "gs_state_update_overprint");
@@ -799,7 +825,7 @@ gs_initgraphics(gs_state * pgs)
 
 /* setfilladjust */
 int
-gs_setfilladjust(gs_state * pgs, floatp adjust_x, floatp adjust_y)
+gs_setfilladjust(gs_state * pgs, double adjust_x, double adjust_y)
 {
 #define CLAMP_TO_HALF(v)\
     ((v) <= 0 ? fixed_0 : (v) >= 0.5 ? fixed_half : float2fixed(v));
@@ -875,7 +901,7 @@ gstate_free_parts(const gs_state * parts, gs_memory_t * mem, client_name_t cname
         gx_cpath_free(parts->effective_clip_path, cname);
     gx_cpath_free(parts->clip_path, cname);
     if (parts->path)
-	gx_path_free(parts->path, cname);
+        gx_path_free(parts->path, cname);
 }
 
 /* Allocate the privately allocated parts of a gstate. */
@@ -990,6 +1016,7 @@ gstate_clone(gs_state * pfrom, gs_memory_t * mem, client_name_t cname,
     }
     gs_imager_state_copied((gs_imager_state *)pgs);
     /* Don't do anything to clip_stack. */
+
     rc_increment(pgs->device);
     *parts.color[0].ccolor    = *pfrom->color[0].ccolor;
     *parts.color[0].dev_color = *pfrom->color[0].dev_color;
@@ -1212,4 +1239,4 @@ int gs_swapcolors(gs_state *pgs)
         return gs_do_set_overprint(pgs);
     }
     return 0;
-}
\ No newline at end of file
+}
