@@ -195,6 +195,25 @@ gs_text_enum_init(gs_text_enum_t *pte, const gs_text_enum_procs_t *procs,
     return code;
 }
 
+gs_text_enum_t *
+gs_text_enum_alloc(gs_memory_t * mem, gs_imager_state * pis, client_name_t cname)
+{
+    gs_text_enum_t *penum;
+
+    rc_alloc_struct_1(penum, gs_text_enum_t, &st_gs_text_enum, mem,
+                      return 0, cname);
+    penum->rc.free = rc_free_text_enum;
+
+    /* Initialize pointers for GC */
+    penum->text.operation = 0;  /* no pointers relevant */
+    penum->dev = 0;
+    penum->pis = pis;
+    penum->fapi_log2_scale.x = penum->fapi_log2_scale.y = -1;
+    penum->fapi_glyph_shift.x = penum->fapi_glyph_shift.y = 0;
+    penum->fstack.depth = -1;
+    return penum;
+}
+
 /*
  * Copy the dynamically changing elements from one enumerator to another.
  * This is useful primarily for enumerators that sometimes pass the
@@ -251,7 +270,6 @@ gs_text_begin(gs_state * pgs, const gs_text_params_t * text,
         return_error(gs_error_undefinedresult); /* sic! : CPSI compatibility */
     if (text->operation & TEXT_DO_DRAW) {
         code = gx_effective_clip_path(pgs, &pcpath);
-        dev_proc(pgs->device, set_graphics_type_tag)(pgs->device, GS_TEXT_TAG);
         if (code < 0)
             return code;
     }
@@ -320,7 +338,7 @@ gs_show_begin(gs_state * pgs, const byte * str, uint size,
     return gs_text_begin(pgs, &text, mem, ppte);
 }
 int
-gs_ashow_begin(gs_state * pgs, floatp ax, floatp ay, const byte * str, uint size,
+gs_ashow_begin(gs_state * pgs, double ax, double ay, const byte * str, uint size,
                gs_memory_t * mem, gs_text_enum_t ** ppte)
 {
     gs_text_params_t text;
@@ -333,7 +351,7 @@ gs_ashow_begin(gs_state * pgs, floatp ax, floatp ay, const byte * str, uint size
     return gs_text_begin(pgs, &text, mem, ppte);
 }
 int
-gs_widthshow_begin(gs_state * pgs, floatp cx, floatp cy, gs_char chr,
+gs_widthshow_begin(gs_state * pgs, double cx, double cy, gs_char chr,
                    const byte * str, uint size,
                    gs_memory_t * mem, gs_text_enum_t ** ppte)
 {
@@ -348,8 +366,8 @@ gs_widthshow_begin(gs_state * pgs, floatp cx, floatp cy, gs_char chr,
     return gs_text_begin(pgs, &text, mem, ppte);
 }
 int
-gs_awidthshow_begin(gs_state * pgs, floatp cx, floatp cy, gs_char chr,
-                    floatp ax, floatp ay, const byte * str, uint size,
+gs_awidthshow_begin(gs_state * pgs, double cx, double cy, gs_char chr,
+                    double ax, double ay, const byte * str, uint size,
                     gs_memory_t * mem, gs_text_enum_t ** ppte)
 {
     gs_text_params_t text;
