@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gstparam.h 12038 2011-01-19 18:00:59Z robin $ */
+
 /* Transparency parameter definitions */
 
 #ifndef gstparam_INCLUDED
@@ -20,6 +22,7 @@
 #include "gsccolor.h"
 #include "gsrefct.h"
 #include "gscspace.h"
+#include "stdint_.h"
 
 /* Define the names of the known blend modes. */
 typedef enum {
@@ -50,26 +53,6 @@ typedef enum {
   "HardLight", "Overlay", "SoftLight", "Luminosity", "Hue",\
   "Saturation", "Color", "Compatible"
 
-/* Define the common part for a transparency stack state. */
-typedef enum {
-    TRANSPARENCY_STATE_Group = 1,	/* must not be 0 */
-    TRANSPARENCY_STATE_Mask
-} gs_transparency_state_type_t;
-#define GS_TRANSPARENCY_STATE_COMMON\
-    gs_transparency_state_t *saved;\
-    gs_transparency_state_type_t type
-typedef struct gs_transparency_state_s gs_transparency_state_t;
-struct gs_transparency_state_s {
-    GS_TRANSPARENCY_STATE_COMMON;
-};
-
-/* Define the common part for a cached transparency mask. */
-#define GS_TRANSPARENCY_MASK_COMMON\
-    rc_header rc
-typedef struct gs_transparency_mask_s {
-    GS_TRANSPARENCY_MASK_COMMON;
-} gs_transparency_mask_t;
-
 /* Define the parameter structure for a transparency group. */
 #ifndef gs_color_space_DEFINED
 #  define gs_color_space_DEFINED
@@ -91,17 +74,18 @@ typedef struct gs_transparency_group_params_s {
     int group_color_numcomps;
     gs_transparency_color_t group_color;
     int64_t icc_hashcode;                    /* Needed when we are doing clist reading */
-    cmm_profile_t *iccprofile;               /* The profile  */    
+    cmm_profile_t *iccprofile;               /* The profile  */
 } gs_transparency_group_params_t;
 
 /* Define the parameter structure for a transparency mask. */
 typedef enum {
     TRANSPARENCY_MASK_Alpha,
-    TRANSPARENCY_MASK_Luminosity
+    TRANSPARENCY_MASK_Luminosity,
+    TRANSPARENCY_MASK_None          /* special case for removing a SMask from the tos */
 } gs_transparency_mask_subtype_t;
 
 #define GS_TRANSPARENCY_MASK_SUBTYPE_NAMES\
-  "Alpha", "Luminosity"
+  "Alpha", "Luminosity", "None"
 
 /* See the gx_transparency_mask_params_t type below */
 /* (Update gs_trans_mask_params_init if these change.) */
@@ -111,11 +95,11 @@ typedef struct gs_transparency_mask_params_s {
     int Background_components;
     float Background[GS_CLIENT_COLOR_MAX_COMPONENTS];
     float GrayBackground;
-    int (*TransferFunction)(floatp in, float *out, void *proc_data);
+    int (*TransferFunction)(double in, float *out, void *proc_data);
     gs_function_t *TransferFunction_data;
     bool replacing;
     int64_t icc_hashcode;                    /* Needed when we are doing clist reading */
-    cmm_profile_t *iccprofile;               /* The profile  */    
+    cmm_profile_t *iccprofile;               /* The profile  */
 } gs_transparency_mask_params_t;
 
 #define MASK_TRANSFER_FUNCTION_SIZE 256
@@ -134,7 +118,7 @@ typedef struct gx_transparency_mask_params_s {
     uint mask_id;
     byte transfer_fn[MASK_TRANSFER_FUNCTION_SIZE];
     int64_t icc_hashcode;                    /* Needed when we are doing clist reading */
-    cmm_profile_t *iccprofile;               /* The profile  */    
+    cmm_profile_t *iccprofile;               /* The profile  */
 } gx_transparency_mask_params_t;
 
 /*
@@ -148,14 +132,14 @@ typedef struct gx_transparency_mask_params_s {
              sizeof(((gs_pdf14trans_params_t *)0)->subtype) + \
              sizeof(((gs_pdf14trans_params_t *)0)->group_color_numcomps) + \
              4 /* group color, replacing, function_is_identity, Background_components */ + \
-	     sizeof(((gs_pdf14trans_params_t *)0)->bbox) + \
-	     sizeof(((gs_pdf14trans_params_t *)0)->mask_id) + \
-	     sizeof(((gs_pdf14trans_params_t *)0)->Background) + \
+             sizeof(((gs_pdf14trans_params_t *)0)->bbox) + \
+             sizeof(((gs_pdf14trans_params_t *)0)->mask_id) + \
+             sizeof(((gs_pdf14trans_params_t *)0)->Background) + \
              sizeof(float)*4 + /* If cmyk background */ \
-	     sizeof(((gs_pdf14trans_params_t *)0)->GrayBackground) + \
+             sizeof(((gs_pdf14trans_params_t *)0)->GrayBackground) + \
              sizeof(int64_t)) /* ICC band information */
 #define MAX_CLIST_TRANSPARENCY_COMPOSITOR_SIZE (MAX_CLIST_TRANSPARENCY_BUFFER_SIZE + \
-	     sizeof(((gs_pdf14trans_params_t *)0)->transfer_fn))
+             sizeof(((gs_pdf14trans_params_t *)0)->transfer_fn))
 #define MAX_CLIST_COMPOSITOR_SIZE MAX_CLIST_TRANSPARENCY_COMPOSITOR_SIZE
 
 /* Select the opacity or shape parameters. */
