@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Structure definitions for standard allocator */
 /* Requires gsmemory.h, gsstruct.h */
 
@@ -77,7 +79,7 @@ typedef struct gs_ref_memory_s gs_ref_memory_t;
  */
 typedef uint string_mark_unit;
 
-#define log2_sizeof_string_mark_unit arch_log2_sizeof_int
+#define log2_sizeof_string_mark_unit ARCH_LOG2_SIZEOF_INT
 /*
  * Define the quantum of relocation for strings, which determines
  * the quantum for reserving space.  This value must be a power of 2,
@@ -86,7 +88,7 @@ typedef uint string_mark_unit;
  */
 typedef uint string_reloc_offset;
 
-#define log2_string_data_quantum (arch_log2_sizeof_int + 4)
+#define log2_string_data_quantum (ARCH_LOG2_SIZEOF_INT + 4)
 #define string_data_quantum (1 << log2_string_data_quantum)
 /*
  * Define the quantum for reserving string space, including data,
@@ -152,6 +154,7 @@ struct chunk_s {
     uint inner_count;		/* number of chunks of which this is */
                                 /*   the outer chunk, if any */
     bool has_refs;		/* true if any refs in chunk */
+    bool c_alone;               /* this chunk is for a single allocation */
     /*
      * Free lists for single bytes in blocks of 1 to 2*N-1 bytes, one per
      * 256 bytes in [csbase..climit), where N is sizeof(uint). The chain
@@ -276,14 +279,14 @@ void alloc_free_chunk(chunk_t *, gs_ref_memory_t *);
 /* define the list of variables being printed as a macro. */
 #define dprintf_chunk_format\
   "%s 0x%lx (0x%lx..0x%lx, 0x%lx..0x%lx..0x%lx)\n"
-#define dprintf_chunk(msg, cp)\
-  dprintf7(dprintf_chunk_format,\
-           msg, (ulong)(cp), (ulong)(cp)->cbase, (ulong)(cp)->cbot,\
-           (ulong)(cp)->ctop, (ulong)(cp)->climit, (ulong)(cp)->cend)
-#define if_debug_chunk(c, msg, cp)\
-  if_debug7(c, dprintf_chunk_format,\
+#define dmprintf_chunk(mem, msg, cp)\
+  dmprintf7(mem, dprintf_chunk_format,\
             msg, (ulong)(cp), (ulong)(cp)->cbase, (ulong)(cp)->cbot,\
             (ulong)(cp)->ctop, (ulong)(cp)->climit, (ulong)(cp)->cend)
+#define if_debug_chunk(c, mem, msg, cp)\
+  if_debug7m(c, mem,dprintf_chunk_format,\
+             msg, (ulong)(cp), (ulong)(cp)->cbase, (ulong)(cp)->cbot,\
+             (ulong)(cp)->ctop, (ulong)(cp)->climit, (ulong)(cp)->cend)
 
 /* ================ Allocator state ================ */
 
