@@ -35,10 +35,6 @@
 #include "gzpath.h"
 #include "gxfcid.h"
 
-/* Define the maximum size of a full temporary bitmap when rasterizing, */
-/* in bits (not bytes). */
-static const uint MAX_TEMP_BITMAP_BITS = 80000;
-
 /* Define whether the show operation uses the character outline data, */
 /* as opposed to just needing the width (or nothing). */
 #define SHOW_USES_OUTLINE(penum)\
@@ -180,10 +176,8 @@ gx_default_text_begin(gx_device * dev, gs_imager_state * pis,
     penum->show_gstate =
         (propagate_charpath && (pgs->in_charpath != 0) ?
          pgs->show_gstate : pgs);
-    if((operation &
-         (TEXT_DO_NONE | TEXT_RETURN_WIDTH | TEXT_RENDER_MODE_3)) ==
-         (TEXT_DO_NONE | TEXT_RETURN_WIDTH)) {
-        /* This is stringwidth. */
+    if (!(~operation & (TEXT_DO_NONE | TEXT_RETURN_WIDTH))) {
+        /* This is stringwidth (or a PDF with text in rendering mode 3) . */
         gx_device_null *dev_null =
             gs_alloc_struct(mem, gx_device_null, &st_device_null,
                             "stringwidth(dev_null)");
@@ -281,7 +275,7 @@ gx_default_text_restore_state(gs_text_enum_t *pte)
 
 static int
     set_cache_device(gs_show_enum *penum, gs_state *pgs,
-                     floatp llx, floatp lly, floatp urx, floatp ury);
+                     double llx, double lly, double urx, double ury);
 
 /* This is the default implementation of text enumerator set_cache. */
 static int
@@ -366,7 +360,7 @@ gx_show_text_set_cache(gs_text_enum_t *pte, const double *pw,
 /* Note that this returns 1 if the current show operation is */
 /* non-displaying (stringwidth or cshow). */
 int
-set_char_width(gs_show_enum *penum, gs_state *pgs, floatp wx, floatp wy)
+set_char_width(gs_show_enum *penum, gs_state *pgs, double wx, double wy)
 {
     int code;
 
@@ -489,8 +483,8 @@ compute_glyph_raster_params(gs_show_enum *penum, bool in_setcachedevice, int *al
 /* Return 1 if we just set up a cache device. */
 /* Used by setcachedevice and setcachedevice2. */
 static int
-set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
-                 floatp urx, floatp ury)
+set_cache_device(gs_show_enum * penum, gs_state * pgs, double llx, double lly,
+                 double urx, double ury)
 {
     gs_glyph glyph;
 
@@ -525,7 +519,7 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
         static const fixed max_cdim[3] =
         {
 #define max_cd(n)\
-            (fixed_1 << (arch_sizeof_short * 8 - n)) - (fixed_1 >> n) * 3
+            (fixed_1 << (ARCH_SIZEOF_SHORT * 8 - n)) - (fixed_1 >> n) * 3
             max_cd(0), max_cd(1), max_cd(2)
 #undef max_cd
         };
@@ -611,7 +605,7 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
          * full oversampling with compression at the end.
          */
         code = gx_alloc_char_bits(dir, penum->dev_cache,
-                                (iwidth > MAX_TEMP_BITMAP_BITS / iheight &&
+                                (iwidth > MAX_CCACHE_TEMP_BITMAP_BITS / iheight &&
                                  log2_scale.x + log2_scale.y > alpha_bits ?
                                  penum->dev_cache2 : NULL),
                                 iwidth, iheight, &log2_scale, depth, &cc);
@@ -689,9 +683,13 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
         clip_box.q.y = int2fixed(iheight);
         if ((code = gx_clip_to_rectangle(pgs, &clip_box)) < 0)
             return code;
-        gx_set_device_color_1(pgs);     /* write 1's */
+        code = gx_set_device_color_1(pgs);     /* write 1's */
+        if (code < 0)
+            return code;
         gs_swapcolors_quick(pgs);
-        gx_set_device_color_1(pgs);     /* write 1's */
+        code = gx_set_device_color_1(pgs);     /* write 1's */
+        if (code < 0)
+            return code;
         gs_swapcolors_quick(pgs);
         pgs->in_cachedevice = CACHE_DEVICE_CACHING;
     }
@@ -921,7 +919,7 @@ show_move(gs_show_enum * penum)
          */
         if (SHOW_IS_ADD_TO_SPACE(penum)
             && (!penum->single_byte_space
-            || penum->fstack.depth <= 0)) {
+            || penum->bytes_decoded == 1)) {
             gs_char chr = gx_current_char((const gs_text_enum_t *)penum);
 
             if (chr == penum->text.space.s_char) {
@@ -984,7 +982,7 @@ show_proceed(gs_show_enum * penum)
   (++(penum->xy_index), next_char_glyph(pte, pchr, pglyph))
     gs_char chr;
     gs_glyph glyph;
-    int code;
+    int code, start;
     cached_char *cc;
     gs_log2_scale_point log2_scale;
 
@@ -1002,6 +1000,7 @@ show_proceed(gs_show_enum * penum)
     if (penum->can_cache >= 0) {
         /* Loop with cache */
         for (;;) {
+            start = penum->index;
             switch ((code = get_next_char_glyph((gs_text_enum_t *)penum,
                                                 &chr, &glyph))
                     ) {
@@ -1035,6 +1034,7 @@ show_proceed(gs_show_enum * penum)
                         SET_CURRENT_GLYPH(penum, glyph);
                     } else
                         SET_CURRENT_GLYPH(penum, glyph);
+                    penum->bytes_decoded = penum->index - start;
                     penum->is_pure_color = gs_color_writes_pure(penum->pgs); /* Save
                                  this data for compute_glyph_raster_params to work
                                  independently on the color change in BuildChar.
@@ -1299,6 +1299,7 @@ static int
 show_finish(gs_show_enum * penum)
 {
     gs_state *pgs = penum->pgs;
+    int code = 0, rcode;
 
     if ((penum->text.operation & TEXT_DO_FALSE_CHARPATH) ||
         (penum->text.operation & TEXT_DO_TRUE_CHARPATH)) {
@@ -1308,17 +1309,16 @@ show_finish(gs_show_enum * penum)
     if (penum->auto_release)
         penum->procs->release((gs_text_enum_t *)penum, "show_finish");
 
-    if((penum->text.operation &
-         (TEXT_DO_NONE | TEXT_RETURN_WIDTH | TEXT_RENDER_MODE_3)) ==
-         (TEXT_DO_NONE | TEXT_RETURN_WIDTH)) {
-        /* Save the accumulated width before returning, */
-        /* and undo the extra gsave. */
-        int code = gs_currentpoint(pgs, &penum->returned.total_width);
-        int rcode = gs_grestore(pgs);
+    if (!SHOW_IS_STRINGWIDTH(penum))
+       return 0;
 
-        return (code < 0 ? code : rcode);
-    }
-    return 0;
+    /* Save the accumulated width before returning, if we are not in PDF text rendering mode 3, */
+    /* and undo the extra gsave. */
+    if (!(penum->text.operation & TEXT_RENDER_MODE_3))
+        code = gs_currentpoint(pgs, &penum->returned.total_width);
+    rcode = gs_grestore(pgs);
+
+    return (code < 0 ? code : rcode);
 }
 
 /* Release the structure. */
@@ -1466,7 +1466,7 @@ show_state_setup(gs_show_enum * penum)
             double fdx = pgs->char_tm.tx - pgs->ctm.tx;
             double fdy = pgs->char_tm.ty - pgs->ctm.ty;
 
-#define int_bits (arch_sizeof_int * 8 - 1)
+#define int_bits (ARCH_SIZEOF_INT * 8 - 1)
             if (!(f_fits_in_bits(fdx, int_bits) &&
                   f_fits_in_bits(fdy, int_bits))
                 )
@@ -1488,9 +1488,18 @@ show_set_scale(const gs_show_enum * penum, gs_log2_scale_point *log2_scale)
      * Decide whether to oversample.
      * We have to decide this each time setcachedevice is called.
      */
-    const gs_state *pgs = penum->pgs;
+    const gs_state *pgs = NULL;
 
-    if ((penum->charpath_flag == cpm_show ||
+    if (gs_object_type(penum->pis->memory, penum) == &st_gs_show_enum) {
+        pgs = penum->pgs;
+    } else {
+        if (penum->pis->is_gstate)
+            pgs = (gs_state *)penum->pis;
+        else
+            outprintf(penum->memory, "Warning: cannot calculate text oversample (TextAlphaBits)\n");
+    }
+
+    if (pgs != NULL && (penum->charpath_flag == cpm_show ||
          penum->charpath_flag == cpm_charwidth) &&
         SHOW_USES_OUTLINE(penum)
         /* && gx_path_is_void_inline(pgs->path) */
