@@ -38,7 +38,6 @@ public_st_clip_list();
 public_st_clip_path();
 private_st_clip_rect_list();
 public_st_device_clip();
-private_st_cpath_enum();
 private_st_cpath_path_list();
 
 /* GC procedures for gx_clip_path */
@@ -564,18 +563,24 @@ gx_cpath_reset(gx_clip_path * pcpath)
     return gx_cpath_from_rectangle(pcpath, &null_rect);
 }
 
-/* If a clipping path is a rectangle, return the rectangle. */
-const gs_fixed_rect *
-cpath_is_rectangle(const gx_clip_path * pcpath)
+/* If a clipping path is a rectangle, return the rectangle.
+ * If its a rectangular path, also return the rectangle.
+ */
+gx_path_rectangular_type cpath_is_rectangle(const gx_clip_path * pcpath, gs_fixed_rect *rect)
 {
-    if (pcpath->path_valid)
-        return NULL;
+    if (pcpath->path_valid) {
+        return gx_path_is_rectangle((const gx_path *)&pcpath->path, rect);
+    }
     if (pcpath->inner_box.p.x != pcpath->path.bbox.p.x ||
         pcpath->inner_box.p.y != pcpath->path.bbox.p.y ||
         pcpath->inner_box.q.x != pcpath->path.bbox.q.x ||
         pcpath->inner_box.q.y != pcpath->path.bbox.q.y)
-        return NULL;
-    return &pcpath->inner_box;
+        return prt_none;
+    rect->p.x = pcpath->inner_box.p.x;
+    rect->p.y = pcpath->inner_box.p.y;
+    rect->q.x = pcpath->inner_box.q.x;
+    rect->q.y = pcpath->inner_box.q.y;
+    return prt_closed;
 }
 
 /* Intersect a new clipping path with an old one. */
@@ -624,11 +629,8 @@ gx_cpath_intersect_with_params(gx_clip_path *pcpath, /*const*/ gx_path *ppath_or
             new_box.q = new_box.p;
             changed = 1;
         } else {
-            if (params != NULL) {
-                /* Called from gx_default_fill_path for converting
-                   a filling path into a clipping path.
-                   Apply same adjustment as for filling the path. */
-                gs_fixed_point adjust = params->adjust;
+            {   /* Apply same adjustment as for filling the path. */
+                gs_fixed_point adjust = params != NULL ? params->adjust : pis->fill_adjust;
                 fixed adjust_xl, adjust_xu, adjust_yl, adjust_yu;
 
                 if (adjust.x == -1)
