@@ -28,16 +28,13 @@
 #include "gspath.h"
 #include "gzstate.h"
 #include "gxfcid.h"
-#include "gxchar.h"             /* for st_gs_show_enum */
+#include "gxchar.h"             /* for st_gs_show_enum and MAX_CCACHE_TEMP_BITMAP_BITS */
 #include "gdebug.h"
 #include "gsimage.h"
 #include "gsbittab.h"
 
 #include "gxfapi.h"
 
-/* lifted from gxchar.c */
-static const uint MAX_TEMP_BITMAP_BITS = 80000;
-
 extern_gs_get_fapi_server_inits();
 
 /* FIXME */
@@ -348,16 +345,18 @@ gs_fapi_prepare_font(gs_font *pfont, gs_fapi_server *I, int subfont, const char
     gs_font_base *pbfont = (gs_font_base *)pfont;
     int code, bbox_set = 0;
     int BBox[4], scale;
-    double size, size1;
+    int units[2];
+    double size;
     gs_fapi_font_scale font_scale =
         { {1, 0, 0, 1, 0, 0}, {0, 0}, {1, 1}, true };
 
     scale = 1 << I->frac_shift;
-    size1 = size = 1 / hypot(pbfont->FontMatrix.xx, pbfont->FontMatrix.xy);
+    size = 1 / hypot(pbfont->FontMatrix.xx, pbfont->FontMatrix.xy);
+    /* I believe this is just to ensure minimal rounding problems with scalers that
+       scale the FontBBox values with the font scale.
+     */
     if (size < 1000)
         size = 1000;
-    if (size1 > 100)
-        size1 = (int)(size1 + 0.5);
 
     font_scale.matrix[0] = font_scale.matrix[3] = (int)(size * scale + 0.5);
 
@@ -402,15 +401,15 @@ gs_fapi_prepare_font(gs_font *pfont, gs_fapi_server *I, int subfont, const char
         if ((code =
              gs_fapi_renderer_retcode(mem, I,
                                       I->get_font_bbox(I, &I->ff,
-                                                       BBox))) < 0) {
+                                                       BBox, units))) < 0) {
             gs_fapi_release_typeface(I, &pbfont->FAPI_font_data);
             return code;
         }
         /* Refine FontBBox : */
-        pbfont->FontBBox.p.x = (double)BBox[0] * size1 / size;
-        pbfont->FontBBox.p.y = (double)BBox[1] * size1 / size;
-        pbfont->FontBBox.q.x = (double)BBox[2] * size1 / size;
-        pbfont->FontBBox.q.y = (double)BBox[3] * size1 / size;
+        pbfont->FontBBox.p.x = ((double)BBox[0] / units[0]);
+        pbfont->FontBBox.p.y = ((double)BBox[1] / units[1]);
+        pbfont->FontBBox.q.x = ((double)BBox[2] / units[0]);
+        pbfont->FontBBox.q.y = ((double)BBox[3] / units[1]);
 
         bbox_set = 1;
     }
@@ -438,6 +437,7 @@ gs_fapi_prepare_font(gs_font *pfont, gs_fapi_server *I, int subfont, const char
         for (i = 0; i < n; i++) {
             gs_font_type1 *pbfont1 = FDArray[i];
             int BBox_temp[4];
+            int units_temp[2];
 
             pbfont1->FontBBox = pbfont->FontBBox;       /* Inherit FontBBox from the type 9 font. */
 
@@ -465,7 +465,7 @@ gs_fapi_prepare_font(gs_font *pfont, gs_fapi_server *I, int subfont, const char
             if ((code =
                  gs_fapi_renderer_retcode(mem, I,
                                           I->get_font_bbox(I, &I->ff,
-                                                           BBox_temp))) < 0) {
+                                                           BBox_temp, units_temp))) < 0) {
                 break;
             }
         }
@@ -644,7 +644,7 @@ static void
 bits_merge(byte *dest, const byte *src, uint nbytes)
 {       long *dp = (long *)dest;
         const long *sp = (const long *)src;
-        uint n = (nbytes + sizeof(long) - 1) >> arch_log2_sizeof_long;
+        uint n = (nbytes + sizeof(long) - 1) >> ARCH_LOG2_SIZEOF_LONG;
 
         for ( ; n >= 4; sp += 4, dp += 4, n -= 4 )
           dp[0] |= sp[0], dp[1] |= sp[1], dp[2] |= sp[2], dp[3] |= sp[3];
@@ -697,7 +697,7 @@ on:           switch ( (dbyte = sbyte = *++sp) ) {
                     break;
                   *dp++ = 0xff;
                   bits_on += 8 -
-                    byte_count_bits[(*zp & (zmask - 1)) + (zp[1] & -zmask)];
+                    byte_count_bits[(*zp & (zmask - 1)) + (zp[1] & -(int)zmask)];
                   ++zp;
                   i += 8;
                   goto on;
@@ -883,7 +883,7 @@ fapi_image_uncached_glyph(gs_font *pfont, gs_state *pgs, gs_show_enum *penum,
         /* Make a matrix that will place the image */
         /* at (x,y) with no transformation. */
         gs_image_t_init_mask(&image, true);
-        gs_make_translation((floatp) - x, (floatp) (- y + ascent), &image.ImageMatrix);
+        gs_make_translation((double) - x, (double) (- y + ascent), &image.ImageMatrix);
         gs_matrix_multiply(&ctm_only(penum_pgs), &image.ImageMatrix, &image.ImageMatrix);
         image.Width = w + bold;
         image.Height = h + bold;
@@ -1014,6 +1014,11 @@ gs_fapi_finish_render(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, gs_f
      * non-cacheing, non-marking cases, we must not draw the glyph.
      */
     if (pgs->in_charpath && !SHOW_IS(penum, TEXT_DO_NONE)) {
+        gs_point pt;
+
+        if ((code = gs_currentpoint(penum_pgs, &pt)) < 0)
+            return code;
+
         if ((code =
              outline_char(mem, I, import_shift_v, penum_s, penum_pgs->path,
                           !pbfont->PaintType)) < 0) {
@@ -1029,15 +1034,26 @@ gs_fapi_finish_render(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, gs_f
 
     }
     else {
-        int code = I->get_char_raster(I, &rast);
+        int code;
+        memset(&rast, 0x00, sizeof(rast));
+
+        if ((code = I->get_char_raster(I, &rast)) < 0 && code != gs_error_unregistered)
+            return code;
 
         if (!SHOW_IS(penum, TEXT_DO_NONE) && I->use_outline) {
             /* The server provides an outline instead the raster. */
             gs_imager_state *pis = (gs_imager_state *) penum_pgs->show_gstate;
             gs_point pt;
 
+            /* This mimics code which is used below in the case where I->Use_outline is set.
+             * This is usually caused by setting -dTextAlphaBits, and will result in 'undefinedresult'
+             * errors. We want the code to work the same regardless off the setting of -dTextAlphaBits
+             * and it seems the simplest way to provoke the same error is to do the same steps....
+             * Note that we do not actually ever use the returned value.
+             */
             if ((code = gs_currentpoint(penum_pgs, &pt)) < 0)
                 return code;
+
             if ((code =
                  outline_char(mem, I, import_shift_v, penum_s,
                               penum_pgs->path, !pbfont->PaintType)) < 0)
@@ -1063,10 +1079,12 @@ gs_fapi_finish_render(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, gs_f
 
                 penum_pgs->fill_adjust.x = penum_pgs->fill_adjust.y = 0;
 
-                if ((code = gs_fill(penum_pgs)) < 0)
-                    return code;
+                code = gs_fill(penum_pgs);
 
                 penum_pgs->in_cachedevice = in_cachedevice;
+
+                if (code < 0)
+                    return code;
             }
             if ((code = gs_moveto(penum_pgs, pt.x, pt.y)) < 0)
                 return code;
@@ -1074,6 +1092,16 @@ gs_fapi_finish_render(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, gs_f
         else {
             int rast_orig_x = rast.orig_x;
             int rast_orig_y = -rast.orig_y;
+            gs_point pt;
+
+            /* This mimics code which is used above in the case where I->Use_outline is set.
+             * This is usually caused by setting -dTextAlphaBits, and will result in 'undefinedresult'
+             * errors. We want the code to work the same regardless off the setting of -dTextAlphaBits
+             * and it seems the simplest way to provoke the same error is to do the same steps....
+             * Note that we do not actually ever use the returned value.
+             */
+            if ((code = gs_currentpoint(penum_pgs, &pt)) < 0)
+                return code;
 
             if (penum_pgs->in_cachedevice == CACHE_DEVICE_CACHING) {    /* Using GS cache */
                 /*  GS and renderer may transform coordinates few differently.
@@ -1155,8 +1183,7 @@ gs_fapi_finish_render(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, gs_f
 #define GET_S16_MSB(p) (int)((GET_U16_MSB(p) ^ 0x8000) - 0x8000)
 
 #define MTX_EQ(mtx1,mtx2) (mtx1->xx == mtx2->xx && mtx1->xy == mtx2->xy && \
-                           mtx1->yx == mtx2->yx && mtx1->yy == mtx2->yy && \
-                           mtx1->tx == mtx2->tx && mtx1->ty == mtx2->ty)
+                           mtx1->yx == mtx2->yx && mtx1->yy == mtx2->yy)
 
 int
 gs_fapi_do_char(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, char *font_file_path,
@@ -1192,7 +1219,6 @@ gs_fapi_do_char(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, char *font
     int code;
     bool imagenow = false;
     bool align_to_pixels = gs_currentaligntopixels(pbfont->dir);
-    gs_fapi_descendant_code fdc;
     gs_memory_t *mem = pfont->memory;
     enum
     {
@@ -1223,10 +1249,12 @@ gs_fapi_do_char(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, char *font
          */
         I->max_bitmap =
             pbfont->dir->ccache.upper + (pbfont->dir->ccache.upper >> 1) <
-            MAX_TEMP_BITMAP_BITS ? pbfont->dir->ccache.upper +
-            (pbfont->dir->ccache.upper >> 1) : MAX_TEMP_BITMAP_BITS;
+            MAX_CCACHE_TEMP_BITMAP_BITS ? pbfont->dir->ccache.upper +
+            (pbfont->dir->ccache.upper >> 1) : MAX_CCACHE_TEMP_BITMAP_BITS;
     }
 
+    I->grid_fit = pbfont->dir->grid_fit_tt;
+
     /* Compute the scale : */
     if (!SHOW_IS(penum, TEXT_DO_NONE) && !I->use_outline) {
         gs_currentcharmatrix(pgs, NULL, 1);     /* make char_tm valid */
@@ -1244,6 +1272,7 @@ gs_fapi_do_char(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, char *font
      */
 
     I->ff.memory = pbfont->memory;
+    I->ff.subfont = subfont;
     I->ff.font_file_path = font_file_path;
     I->ff.client_font_data = pbfont;
     I->ff.client_font_data2 = pbfont;
@@ -1251,6 +1280,7 @@ gs_fapi_do_char(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, char *font
     I->ff.is_type1 = bIsType1GlyphData;
     I->ff.is_cid = bCID;
     I->ff.is_outline_font = pbfont->PaintType != 0;
+    I->ff.metrics_only = fapi_gs_char_show_width_only(penum);
 
     if (!I->ff.is_mtx_skipped)
         I->ff.is_mtx_skipped = (gs_fapi_get_metrics_count(&I->ff) != 0);
@@ -1294,13 +1324,20 @@ gs_fapi_do_char(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, char *font
         scale_ctm.xx = dev->HWResolution[0] / 72;
         scale_ctm.yy = dev->HWResolution[1] / 72;
 
-        code = gs_matrix_invert((const gs_matrix *)&scale_ctm, &scale_ctm);
+        if ((code = gs_matrix_invert((const gs_matrix *)&scale_ctm, &scale_ctm)) < 0)
+            return code;
 
-        code = gs_matrix_multiply(ctm, &scale_ctm, &scale_mat); /* scale_mat ==  CTM - resolution scaling */
+        if ((code = gs_matrix_multiply(ctm, &scale_ctm, &scale_mat)) < 0)  /* scale_mat ==  CTM - resolution scaling */
+            return code;
 
-        code = I->get_fontmatrix(I, &scale_ctm);
-        code = gs_matrix_invert((const gs_matrix *)&scale_ctm, &scale_ctm);
-        code = gs_matrix_multiply(&scale_mat, &scale_ctm, &scale_mat);  /* scale_mat ==  CTM - resolution scaling - FontMatrix scaling */
+        if ((code = I->get_fontmatrix(I, &scale_ctm)) < 0)
+            return code;
+
+        if ((code = gs_matrix_invert((const gs_matrix *)&scale_ctm, &scale_ctm)) < 0)
+            return code;
+
+        if ((code = gs_matrix_multiply(&scale_mat, &scale_ctm, &scale_mat)) < 0)  /* scale_mat ==  CTM - resolution scaling - FontMatrix scaling */
+            return code;
 
         font_scale.matrix[0] =
             (fracint) (scale_mat.xx * FontMatrix_div * scale + 0.5);
@@ -1336,25 +1373,17 @@ gs_fapi_do_char(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, char *font
                 font_scale.matrix[3] = 1;
         }
 
-        if (!bCID
-            || (pbfont->FontType != ft_encrypted
-                && pbfont->FontType != ft_encrypted2)) {
-            fdc = gs_fapi_toplevel_prepared;
-        }
-        else {
-            fdc = gs_fapi_toplevel_prepared;
-        }
         if ((code =
              gs_fapi_renderer_retcode(mem, I,
                                       I->get_scaled_font(I, &I->ff,
                                                          &font_scale, NULL,
-                                                         fdc))) < 0) {
+                                                         gs_fapi_toplevel_prepared))) < 0) {
             return code;
         }
     }
 
     cr.char_codes_count = 1;
-    cr.char_codes[0] = (index == GS_NO_GLYPH) ? 0 : index;
+    cr.char_codes[0] = index;
     cr.client_char_code = chr;
     cr.is_glyph_index = true;
     enc_char_name_string.data = NULL;
@@ -1378,6 +1407,9 @@ gs_fapi_do_char(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, char *font
             int l = I->ff.get_glyphdirectory_data(&(I->ff), cr.char_codes[0],
                                                   &data_ptr);
 
+            /* We do not need to apply the unitsPerEm scale here because
+             * these values are read directly from the glyph data.
+             */
             if (MetricsCount == 2 && l >= 4) {
                 if (!bVertical0) {
                     cr.sb_x = GET_S16_MSB(data_ptr + 2) * scale;
@@ -1394,6 +1426,15 @@ gs_fapi_do_char(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, char *font
             }
         }
     }
+    /* Metrics in GS are scaled to a 1.0x1.0 square, as that's what Postscript
+     * fonts expect. But for Truetype we need the "native" units,
+     */
+    em_scale_x = 1.0;
+    if (pfont->FontType == ft_TrueType) {
+        gs_font_type42 *pfont42 = (gs_font_type42 *) pfont;
+        em_scale_x = pfont42->data.unitsPerEm;
+    }
+
     if (cr.metrics_type != gs_fapi_metrics_replace && bVertical) {
         double pwv[4];
 
@@ -1404,10 +1445,10 @@ gs_fapi_do_char(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, char *font
             return code;
         if (code == 0 /* metricsNone */ ) {
             if (bCID && (!bIsType1GlyphData && font_file_path)) {
-                cr.sb_x = fapi_round(sbw[2] / 2 * scale);
-                cr.sb_y = fapi_round(pbfont->FontBBox.q.y * scale);
-                cr.aw_y = fapi_round(-pbfont->FontBBox.q.x * scale);    /* Sic ! */
-                cr.metrics_scale = (bIsType1GlyphData ? 1000 : 1);
+                cr.sb_x = (fracint)(fapi_round( (sbw[2] / 2)         * scale) * em_scale_x);
+                cr.sb_y = (fracint)(fapi_round( pbfont->FontBBox.q.y * scale) * em_scale_x);
+                cr.aw_y = (fracint)(fapi_round(-pbfont->FontBBox.q.x * scale) * em_scale_x);    /* Sic ! */
+                cr.metrics_scale = 1;
                 cr.metrics_type = gs_fapi_metrics_replace;
                 sbw[0] = sbw[2] / 2;
                 sbw[1] = pbfont->FontBBox.q.y;
@@ -1420,10 +1461,10 @@ gs_fapi_do_char(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, char *font
             }
         }
         else {
-            cr.sb_x = fapi_round(pwv[2] * scale);
-            cr.sb_y = fapi_round(pwv[3] * scale);
-            cr.aw_x = fapi_round(pwv[0] * scale);
-            cr.aw_y = fapi_round(pwv[1] * scale);
+            cr.sb_x = (fracint)(fapi_round(pwv[2] * scale) * em_scale_x);
+            cr.sb_y = (fracint)(fapi_round(pwv[3] * scale) * em_scale_x);
+            cr.aw_x = (fracint)(fapi_round(pwv[0] * scale) * em_scale_x);
+            cr.aw_y = (fracint)(fapi_round(pwv[1] * scale) * em_scale_x);
             cr.metrics_scale = (bIsType1GlyphData ? 1000 : 1);
             cr.metrics_type = (code == 2 /* metricsSideBearingAndWidth */ ? gs_fapi_metrics_replace
                                : gs_fapi_metrics_replace_width);
@@ -1453,10 +1494,10 @@ gs_fapi_do_char(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, char *font
             }
         }
         else {
-            cr.sb_x = fapi_round(sbw[0] * scale);
-            cr.sb_y = fapi_round(sbw[1] * scale);
-            cr.aw_x = fapi_round(sbw[2] * scale);
-            cr.aw_y = fapi_round(sbw[3] * scale);
+            cr.sb_x = fapi_round(sbw[0] * scale * em_scale_x);
+            cr.sb_y = fapi_round(sbw[1] * scale * em_scale_x);
+            cr.aw_x = fapi_round(sbw[2] * scale * em_scale_x);
+            cr.aw_y = fapi_round(sbw[3] * scale * em_scale_x);
             cr.metrics_scale = (bIsType1GlyphData ? 1000 : 1);
             cr.metrics_type = (code == 2 /* metricsSideBearingAndWidth */ ? gs_fapi_metrics_replace
                                : gs_fapi_metrics_replace_width);
@@ -1465,20 +1506,10 @@ gs_fapi_do_char(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, char *font
     }
     memset(&metrics, 0x00, sizeof(metrics));
     /* Take metrics from font : */
-    if (fapi_gs_char_show_width_only(penum)) {
-        code = I->get_char_width(I, &I->ff, &cr, &metrics);
-        /* A VMerror could be a real out of memory, or the glyph being too big for a bitmap
-         * so it's worth retrying as an outline glyph
-         */
-        if (code == gs_error_VMerror && I->use_outline == false) {
-            I->max_bitmap = 0;
-            I->use_outline = true;
-            goto retry_oversampling;
-        }
-
+    if (I->ff.metrics_only) {
+        code = I->get_char_outline_metrics(I, &I->ff, &cr, &metrics);
     }
     else if (I->use_outline) {
-
         code = I->get_char_outline_metrics(I, &I->ff, &cr, &metrics);
     }
     else {
@@ -1497,20 +1528,10 @@ gs_fapi_do_char(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, char *font
                 I->release_char_data(I);
                 goto retry_oversampling;
             }
-            if ((code =
-                 gs_fapi_renderer_retcode(mem, I,
-                                          I->get_char_outline_metrics(I,
-                                                                      &I->ff,
-                                                                      &cr,
-                                                                      &metrics)))
-                < 0)
-                return code;
+            code = I->get_char_outline_metrics(I, &I->ff, &cr, &metrics);
         }
     }
 
-    if (!bVertical)
-        metrics.v_escapement = 0;
-
     /* This handles the situation where a charstring has been replaced with a PS procedure.
      * against the rules, but not *that* rare.
      * It's also something that GS does internally to simulate font styles.
@@ -1633,7 +1654,7 @@ gs_fapi_do_char(gs_font *pfont, gs_state *pgs, gs_text_enum_t *penum, char *font
      */
 
     /* Don't allow caching if we're only returning the metrics */
-    if (fapi_gs_char_show_width_only(penum)) {
+    if (I->ff.metrics_only) {
         pgs->in_cachedevice = CACHE_DEVICE_NOT_CACHING;
     }
 
@@ -1723,6 +1744,9 @@ gs_fapi_passfont(gs_font *pfont, int subfont, char *font_file_path,
     
     list = gs_fapi_get_server_list(mem);
     
+    if (!list) /* Should never happen */
+      return_error(gs_error_unregistered);
+
     (*fapi_id) = NULL;
 
     pbfont = (gs_font_base *) pfont;
@@ -1880,13 +1904,10 @@ gs_fapi_find_server(gs_memory_t *mem, const char *name, gs_fapi_server **server,
                                     &server_param, &server_param_size);
         }
 
-        if ((code =
-             gs_fapi_renderer_retcode(mem, (*servs),
-                                      (*servs)->ensure_open((*servs),
-                                                            server_param,
-                                                            server_param_size)))
-            < 0) {
-        }
+        code = gs_fapi_renderer_retcode(mem, (*servs),
+                                 (*servs)->ensure_open((*servs),
+                                                       server_param,
+                                                       server_param_size));
 
         if (free_params) {
             gs_free_object(mem->non_gc_memory, server_param,
