@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxfill.c 11497 2010-07-08 20:01:59Z robin $ */
+
 /* A topological spot decomposition algorithm with dropout prevention. */
 /*
    This is a dramaticly reorganized and improved revision of the
@@ -51,11 +53,13 @@
 #include "stdint_.h"
 #include "vdtrace.h"
 #include "gsstate.h"            /* for gs_currentcpsimode */
+#include "gxdevsop.h"
 /*
 #include "gxfilltr.h" - Do not remove this comment. "gxfilltr.h" is included below.
 #include "gxfillsl.h" - Do not remove this comment. "gxfillsl.h" is included below.
 #include "gxfillts.h" - Do not remove this comment. "gxfillts.h" is included below.
 */
+#include "assert_.h"
 
 /* #define ENABLE_TRAP_AMALGAMATION */
 
@@ -79,7 +83,7 @@
 #define TRY_TO_EXTEND_TRAP 0
 #endif
 
-#ifdef DEBUG
+#if defined(DEBUG) && !defined(GS_THREADSAFE)
 /* Define the statistics structure instance. */
 stats_fill_t stats_fill;
 #endif
@@ -91,10 +95,8 @@ stats_fill_t stats_fill;
 /* we are inside if the winding number is odd. */
 #define INSIDE_PATH_P(inside, rule) ((inside & rule) != 0)
 
-
 /* ---------------- Active line management ---------------- */
 
-
 /*
  * Define the ordering criterion for active lines that overlap in Y.
  * Return -1, 0, or 1 if lp1 precedes, coincides with, or follows lp2.
@@ -111,7 +113,7 @@ x_order(const active_line *lp1, const active_line *lp2)
     bool s1;
 
     INCR(order);
-    if (lp1->x_current < lp2->x_current)
+    if (!lp1 || !lp2 || lp1->x_current < lp2->x_current)
         return -1;
     else if (lp1->x_current > lp2->x_current)
         return 1;
@@ -154,42 +156,42 @@ gs_private_st_simple(st_active_line, active_line, "active_line");
 #ifdef DEBUG
 /* Internal procedures for printing and checking active lines. */
 static void
-print_active_line(const char *label, const active_line * alp)
+print_active_line(const gs_memory_t *mem, const char *label, const active_line * alp)
 {
-    dlprintf5("[f]%s 0x%lx(%d): x_current=%f x_next=%f\n",
-              label, (ulong) alp, alp->direction,
-              fixed2float(alp->x_current), fixed2float(alp->x_next));
-    dlprintf5("    start=(%f,%f) pt_end=0x%lx(%f,%f)\n",
-              fixed2float(alp->start.x), fixed2float(alp->start.y),
-              (ulong) alp->pseg,
-              fixed2float(alp->end.x), fixed2float(alp->end.y));
-    dlprintf2("    prev=0x%lx next=0x%lx\n",
-              (ulong) alp->prev, (ulong) alp->next);
+    dmlprintf5(mem, "[f]%s 0x%lx(%d): x_current=%f x_next=%f\n",
+               label, (ulong) alp, alp->direction,
+               fixed2float(alp->x_current), fixed2float(alp->x_next));
+    dmlprintf5(mem, "    start=(%f,%f) pt_end=0x%lx(%f,%f)\n",
+               fixed2float(alp->start.x), fixed2float(alp->start.y),
+               (ulong) alp->pseg,
+               fixed2float(alp->end.x), fixed2float(alp->end.y));
+    dmlprintf2(mem, "    prev=0x%lx next=0x%lx\n",
+               (ulong) alp->prev, (ulong) alp->next);
 }
 static void
-print_line_list(const active_line * flp)
+print_line_list(const gs_memory_t *mem, const active_line * flp)
 {
     const active_line *lp;
 
     for (lp = flp; lp != 0; lp = lp->next) {
         fixed xc = lp->x_current, xn = lp->x_next;
 
-        dlprintf3("[f]0x%lx(%d): x_current/next=%g",
+        dmlprintf3(mem, "[f]0x%lx(%d): x_current/next=%g",
                   (ulong) lp, lp->direction,
                   fixed2float(xc));
         if (xn != xc)
-            dprintf1("/%g", fixed2float(xn));
-        dputc('\n');
+            dmprintf1(mem, "/%g", fixed2float(xn));
+        dmputc(mem, '\n');
     }
 }
 static inline void
-print_al(const char *label, const active_line * alp)
+print_al(const gs_memory_t *mem, const char *label, const active_line * alp)
 {
     if (gs_debug_c('F'))
-        print_active_line(label, alp);
+        print_active_line(mem, label, alp);
 }
 #else
-#define print_al(label,alp) DO_NOTHING
+#define print_al(mem,label,alp) DO_NOTHING
 #endif
 
 static inline bool
@@ -212,7 +214,6 @@ static void insert_x_new(active_line *, line_list *);
 static int  end_x_line(active_line *, const line_list *, bool);
 static int step_al(active_line *alp, bool move_iterator);
 
-
 #define FILL_LOOP_PROC(proc) int proc(line_list *, fixed band_mask)
 static FILL_LOOP_PROC(spot_into_scan_lines);
 static FILL_LOOP_PROC(spot_into_trapezoids);
@@ -252,11 +253,18 @@ init_line_list(line_list *ll, gs_memory_t * mem)
     ll->local_margin_alloc_count = 0;
     ll->margin_set0.sect = ll->local_section0;
     ll->margin_set1.sect = ll->local_section1;
+
+    ll->x_head.prev = NULL;
+    /* Bug 695234: Initialise the following to pacify valgrind */
+    ll->x_head.start.x = 0;
+    ll->x_head.start.y = 0;
+    ll->x_head.end.x = 0;
+    ll->x_head.end.y = 0;
+
     /* Do not initialize ll->bbox_left, ll->bbox_width - they were set in advance. */
     INCR(fill);
 }
 
-
 /* Unlink any line_close segments added temporarily. */
 static inline void
 unclose_path(gx_path * ppath, int count)
@@ -338,10 +346,10 @@ gx_general_fill_path(gx_device * pdev, const gs_imager_state * pis,
             vd_bar(x0, k, x1, k, 1, RGB(128, 128, 128));
     }
     /* Check the bounding boxes. */
-    if_debug6('f', "[f]adjust=%g,%g bbox=(%g,%g),(%g,%g)\n",
-              fixed2float(adjust.x), fixed2float(adjust.y),
-              fixed2float(ibox.p.x), fixed2float(ibox.p.y),
-              fixed2float(ibox.q.x), fixed2float(ibox.q.y));
+    if_debug6m('f', pdev->memory, "[f]adjust=%g,%g bbox=(%g,%g),(%g,%g)\n",
+               fixed2float(adjust.x), fixed2float(adjust.y),
+               fixed2float(ibox.p.x), fixed2float(ibox.p.y),
+               fixed2float(ibox.q.x), fixed2float(ibox.q.y));
     if (pcpath)
         gx_cpath_inner_box(pcpath, &bbox);
     else
@@ -353,9 +361,9 @@ gx_general_fill_path(gx_device * pdev, const gs_imager_state * pis,
          */
         if (pcpath)
             gx_cpath_outer_box(pcpath, &bbox);
-        if_debug4('f', "   outer_box=(%g,%g),(%g,%g)\n",
-                  fixed2float(bbox.p.x), fixed2float(bbox.p.y),
-                  fixed2float(bbox.q.x), fixed2float(bbox.q.y));
+        if_debug4m('f', pdev->memory, "   outer_box=(%g,%g),(%g,%g)\n",
+                   fixed2float(bbox.p.x), fixed2float(bbox.p.y),
+                   fixed2float(bbox.q.x), fixed2float(bbox.q.y));
         rect_intersect(ibox, bbox);
         if (ibox.p.x - adjust.x >= ibox.q.x + adjust.x ||
             ibox.p.y - adjust.y >= ibox.q.y + adjust.y
@@ -457,11 +465,11 @@ gx_general_fill_path(gx_device * pdev, const gs_imager_state * pis,
         pfpath = ppath;
     else if (is_spotan_device(dev))
         pfpath = ppath;
-    else if (!big_path && gx_path__check_curves(ppath, pco_small_curves, fo.fixed_flat))
+    else if (!big_path && !pis->accurate_curves && gx_path__check_curves(ppath, pco_small_curves, fo.fixed_flat))
         pfpath = ppath;
     else {
         code = gx_path_copy_reducing(ppath, &ffpath, fo.fixed_flat, NULL,
-                            pco_small_curves);
+                                     pco_small_curves | (pis->accurate_curves ? pco_accurate : 0));
         if (code < 0)
             return code;
         pfpath = &ffpath;
@@ -486,7 +494,7 @@ gx_general_fill_path(gx_device * pdev, const gs_imager_state * pis,
             /*
              * Note that execution pass here only for character size
              * grater that MAX_LOCAL_SECTION and lesser than
-             * SMALL_CHARACTER. Therefore with !arch_small_memory
+             * SMALL_CHARACTER. Therefore with !ARCH_SMALL_MEMORY
              * the dynamic allocation only happens for characters
              * wider than 100 pixels.
              */
@@ -521,25 +529,27 @@ gx_general_fill_path(gx_device * pdev, const gs_imager_state * pis,
     free_line_list(&lst);
     if (pfpath != ppath)        /* had to flatten */
         gx_path_free(pfpath, "gx_general_fill_path");
-#ifdef DEBUG
+#if defined(DEBUG) && !defined(GS_THREADSAFE)
     if (gs_debug_c('f')) {
-        dlputs("[f]  # alloc    up  down horiz step slowx  iter  find  band bstep bfill\n");
-        dlprintf5(" %5ld %5ld %5ld %5ld %5ld",
-                  stats_fill.fill, stats_fill.fill_alloc,
-                  stats_fill.y_up, stats_fill.y_down,
-                  stats_fill.horiz);
-        dlprintf4(" %5ld %5ld %5ld %5ld",
-                  stats_fill.x_step, stats_fill.slow_x,
-                  stats_fill.iter, stats_fill.find_y);
-        dlprintf3(" %5ld %5ld %5ld\n",
-                  stats_fill.band, stats_fill.band_step,
-                  stats_fill.band_fill);
-        dlputs("[f]    afill slant shall sfill mqcrs order slowo\n");
-        dlprintf7("       %5ld %5ld %5ld %5ld %5ld %5ld %5ld\n",
-                  stats_fill.afill, stats_fill.slant,
-                  stats_fill.slant_shallow, stats_fill.sfill,
-                  stats_fill.mq_cross, stats_fill.order,
-                  stats_fill.slow_order);
+        dmlputs(ppath->memory,
+                "[f]  # alloc    up  down horiz step slowx  iter  find  band bstep bfill\n");
+        dmlprintf5(ppath->memory, " %5ld %5ld %5ld %5ld %5ld",
+                   stats_fill.fill, stats_fill.fill_alloc,
+                   stats_fill.y_up, stats_fill.y_down,
+                   stats_fill.horiz);
+        dmlprintf4(ppath->memory, " %5ld %5ld %5ld %5ld",
+                   stats_fill.x_step, stats_fill.slow_x,
+                   stats_fill.iter, stats_fill.find_y);
+        dmlprintf3(ppath->memory, " %5ld %5ld %5ld\n",
+                   stats_fill.band, stats_fill.band_step,
+                   stats_fill.band_fill);
+        dmlputs(ppath->memory,
+                "[f]    afill slant shall sfill mqcrs order slowo\n");
+        dmlprintf7(ppath->memory, "       %5ld %5ld %5ld %5ld %5ld %5ld %5ld\n",
+                   stats_fill.afill, stats_fill.slant,
+                   stats_fill.slant_shallow, stats_fill.sfill,
+                   stats_fill.mq_cross, stats_fill.order,
+                   stats_fill.slow_order);
     }
 #endif
     return code;
@@ -593,7 +603,7 @@ gx_default_fill_path(gx_device * pdev, const gs_imager_state * pis,
         gs_imager_state *pis_noconst = (gs_imager_state *)pis; /* Break const. */
 
         if (ppath != NULL) {
-            code = gx_cpath_init_local_shared(&cpath_intersection, pcpath, pdev->memory);
+            code = gx_cpath_init_local_shared_nested(&cpath_intersection, pcpath, pdev->memory, 1);
             if (code < 0)
                 return code;
             if (pcpath == NULL) {
@@ -625,7 +635,8 @@ gx_default_fill_path(gx_device * pdev, const gs_imager_state * pis,
             cb.q.x = fixed2int_pixround(clip_box.q.x);
             cb.q.y = fixed2int_pixround(clip_box.q.y);
             if (gx_dc_is_pattern2_color(pdevc) &&
-                    (*dev_proc(pdev, pattern_manage))(pdev, gs_no_id, NULL, pattern_manage__handles_clip_path) > 0) {
+                    (*dev_proc(pdev, dev_spec_op))(pdev,
+                         gxdso_pattern_handles_clip_path, NULL, 0) > 0) {
                 /* A special interaction with clist writer device :
                    pass the intersected clipping path. It uses an unusual call to
                    fill_path with NULL device color. */
@@ -634,8 +645,8 @@ gx_default_fill_path(gx_device * pdev, const gs_imager_state * pis,
             } else {
                 gx_make_clip_device_on_stack(&cdev, pcpath1, pdev);
                 dev = (gx_device *)&cdev;
-                if ((*dev_proc(pdev, pattern_manage))(pdev,
-                        gs_no_id, NULL, pattern_manage__shading_area) > 0)
+                if ((*dev_proc(pdev, dev_spec_op))(pdev,
+                        gxdso_pattern_shading_area, NULL, 0) > 0)
                     set_dev_proc(&cdev, fill_path, pass_shading_area_through_clip_path_device);
                 code = 0;
             }
@@ -649,6 +660,7 @@ gx_default_fill_path(gx_device * pdev, const gs_imager_state * pis,
         if (pcpath1 != pcpath2)
             gx_cpath_free(&cpath_with_shading_bbox, "shading_fill_cpath_intersection");
     } else {
+#ifndef GS_THREADSAFE
         bool got_dc = false;
         vd_save;
         if (vd_allowed('F') || vd_allowed('f')) {
@@ -664,10 +676,15 @@ gx_default_fill_path(gx_device * pdev, const gs_imager_state * pis,
             }
         } else
             vd_disable;
+#endif
+
         code = gx_general_fill_path(pdev, pis, ppath, params, pdevc, pcpath);
+
+#ifndef GS_THREADSAFE
         if (got_dc)
             vd_release_dc;
         vd_restore;
+#endif
     }
     return code;
 }
@@ -747,12 +764,12 @@ insert_y_line(line_list *ll, active_line *alp)
     }
     ll->y_line = alp;
     vd_bar(alp->start.x, alp->start.y, alp->end.x, alp->end.y, 1, RGB(0, 255, 0));
-    print_al("add ", alp);
+    print_al(ll->memory, "add ", alp);
 }
 
 typedef struct contour_cursor_s {
     segment *prev, *pseg, *pfirst, *plast;
-    gx_flattened_iterator *fi;
+    gx_flattened_iterator fi;
     bool more_flattened;
     bool first_flattened;
     int dir;
@@ -765,9 +782,9 @@ static inline int
 compute_dir(const fill_options *fo, fixed y0, fixed y1)
 {
     if (max(y0, y1) < fo->ymin)
-        return 2;
+        return DIR_OUT_OF_Y_RANGE;
     if (min(y0, y1) > fo->ymax)
-        return 2;
+        return DIR_OUT_OF_Y_RANGE;
     return (y0 < y1 ? DIR_UP :
             y0 > y1 ? DIR_DOWN : DIR_HORIZONTAL);
 }
@@ -820,14 +837,14 @@ start_al_pair(line_list *ll, contour_cursor *q, contour_cursor *p)
     if (q->monotonic_y)
         code = add_y_line(q->prev, q->pseg, DIR_DOWN, ll);
     else
-        code = add_y_curve_part(ll, q->prev, q->pseg, DIR_DOWN, q->fi,
+        code = add_y_curve_part(ll, q->prev, q->pseg, DIR_DOWN, &q->fi,
                             !q->first_flattened, false, q->monotonic_x);
     if (code < 0)
         return code;
     if (p->monotonic_y)
         code = add_y_line(p->prev, p->pseg, DIR_UP, ll);
     else
-        code = add_y_curve_part(ll, p->prev, p->pseg, DIR_UP, p->fi,
+        code = add_y_curve_part(ll, p->prev, p->pseg, DIR_UP, &p->fi,
                             p->more_flattened, false, p->monotonic_x);
     return code;
 }
@@ -841,41 +858,41 @@ start_al_pair_from_min(line_list *ll, contour_cursor *q)
 
     /* q stands at the first segment, which isn't last. */
     do {
-        code = gx_flattened_iterator__next(q->fi);
+        code = gx_flattened_iterator__next(&q->fi);
         if (code < 0)
             return code;
         q->more_flattened = code;
-        dir = compute_dir(fo, q->fi->ly0, q->fi->ly1);
-        if (q->fi->ly0 > fo->ymax && ll->y_break > q->fi->y0)
-            ll->y_break = q->fi->ly0;
-        if (q->fi->ly1 > fo->ymax && ll->y_break > q->fi->ly1)
-            ll->y_break = q->fi->ly1;
-        if (dir == DIR_UP && ll->main_dir == DIR_DOWN && q->fi->ly0 >= fo->ymin) {
-            code = add_y_curve_part(ll, q->prev, q->pseg, DIR_DOWN, q->fi,
-                            true, true, q->monotonic_x);
-            if (code < 0)
-                return code;
-            code = add_y_curve_part(ll, q->prev, q->pseg, DIR_UP, q->fi,
-                            q->more_flattened, false, q->monotonic_x);
-            if (code < 0)
-                return code;
-        } else if (q->fi->ly0 < fo->ymin && q->fi->ly1 >= fo->ymin) {
-            code = add_y_curve_part(ll, q->prev, q->pseg, DIR_UP, q->fi,
-                            q->more_flattened, false, q->monotonic_x);
-            if (code < 0)
-                return code;
-        } else if (q->fi->ly0 >= fo->ymin && q->fi->ly1 < fo->ymin) {
-            code = add_y_curve_part(ll, q->prev, q->pseg, DIR_DOWN, q->fi,
-                            true, false, q->monotonic_x);
+        dir = compute_dir(fo, q->fi.ly0, q->fi.ly1);
+        if (q->fi.ly0 > fo->ymax && ll->y_break > q->fi.y0)
+            ll->y_break = q->fi.ly0;
+        if (q->fi.ly1 > fo->ymax && ll->y_break > q->fi.ly1)
+            ll->y_break = q->fi.ly1;
+        if (q->fi.ly0 >= fo->ymin) {
+            if (dir == DIR_UP && ll->main_dir == DIR_DOWN) {
+                code = add_y_curve_part(ll, q->prev, q->pseg, DIR_DOWN, &q->fi,
+                                        true, true, q->monotonic_x);
+                if (code < 0)
+                    return code;
+                code = add_y_curve_part(ll, q->prev, q->pseg, DIR_UP, &q->fi,
+                                        q->more_flattened, false, q->monotonic_x);
+                if (code < 0)
+                    return code;
+            } else if (q->fi.ly1 < fo->ymin) {
+                code = add_y_curve_part(ll, q->prev, q->pseg, DIR_DOWN, &q->fi,
+                                        true, false, q->monotonic_x);
+                if (code < 0)
+                    return code;
+            }
+        } else if (q->fi.ly1 >= fo->ymin) {
+            code = add_y_curve_part(ll, q->prev, q->pseg, DIR_UP, &q->fi,
+                                    q->more_flattened, false, q->monotonic_x);
             if (code < 0)
                 return code;
         }
         q->first_flattened = false;
         q->dir = dir;
-        ll->main_dir = (dir == DIR_DOWN ? DIR_DOWN :
-                        dir == DIR_UP ? DIR_UP : ll->main_dir);
-        if (!q->more_flattened)
-            break;
+        if (dir == DIR_DOWN || dir == DIR_UP)
+            ll->main_dir = dir;
     } while(q->more_flattened);
     /* q stands at the last segment. */
     return 0;
@@ -884,10 +901,8 @@ start_al_pair_from_min(line_list *ll, contour_cursor *q)
 }
 
 static inline int
-init_contour_cursor(line_list *ll, contour_cursor *q)
+init_contour_cursor(const fill_options * const fo, contour_cursor *q)
 {
-    const fill_options * const fo = ll->fo;
-
     if (q->pseg->type == s_curve) {
         curve_segment *s = (curve_segment *)q->pseg;
         fixed ymin = min(min(q->prev->pt.y, s->p1.y), min(s->p2.y, s->pt.y));
@@ -907,11 +922,11 @@ init_contour_cursor(line_list *ll, contour_cursor *q)
         curve_segment *s = (curve_segment *)q->pseg;
         int k = gx_curve_log2_samples(q->prev->pt.x, q->prev->pt.y, s, fo->fixed_flat);
 
-        if (!gx_flattened_iterator__init(q->fi, q->prev->pt.x, q->prev->pt.y, s, k))
+        if (!gx_flattened_iterator__init(&q->fi, q->prev->pt.x, q->prev->pt.y, s, k))
             return_error(gs_error_rangecheck);
     } else {
         q->dir = compute_dir(fo, q->prev->pt.y, q->pseg->pt.y);
-        gx_flattened_iterator__init_line(q->fi,
+        gx_flattened_iterator__init_line(&q->fi,
             q->prev->pt.x, q->prev->pt.y, q->pseg->pt.x, q->pseg->pt.y); /* fake for curves. */
         vd_bar(q->prev->pt.x, q->prev->pt.y, q->pseg->pt.x, q->pseg->pt.y, 1, RGB(0, 0, 255));
     }
@@ -920,140 +935,192 @@ init_contour_cursor(line_list *ll, contour_cursor *q)
 }
 
 static int
-scan_contour(line_list *ll, contour_cursor *q)
+scan_contour(line_list *ll, segment *pfirst, segment *plast, segment *prev)
 {
-    contour_cursor p;
-    gx_flattened_iterator fi, save_fi;
-    segment *pseg;
+    contour_cursor p, q;
     int code;
     bool only_horizontal = true, saved = false;
     const fill_options * const fo = ll->fo;
     contour_cursor save_q;
 
+    q.prev = prev;
+    q.pseg = plast;
+    q.pfirst = pfirst;
+    q.plast = plast;
+
     memset(&save_q, 0, sizeof(save_q)); /* Quiet gcc warning. */
     p.monotonic_x = false; /* Quiet gcc warning. */
-    p.fi = &fi;
-    save_q.dir = 2;
+    q.monotonic_x = false; /* Quiet gcc warning. */
+
+    /* The first thing we do is to walk BACKWARDS along the contour (aka the subpath or
+     * set of curve segments). We stop either when we reach the beginning again,
+     * or when we hit a non-horizontal segment. If we hit the beginning, then we
+     * remember that the segment is 'only_horizontal'. Otherwise, we remember the
+     * direction of this contour (UP or DOWN). */
+    save_q.dir = DIR_OUT_OF_Y_RANGE;
     ll->main_dir = DIR_HORIZONTAL;
-    for (; ; q->pseg = q->prev, q->prev = q->prev->prev) {
-        code = init_contour_cursor(ll, q);
+    for (; ; q.pseg = q.prev, q.prev = q.prev->prev) {
+        /* Prepare to walk FORWARDS along the single curve segment in 'q'. */
+        code = init_contour_cursor(fo, &q);
         if (code < 0)
             return code;
         for (;;) {
-            code = gx_flattened_iterator__next(q->fi);
+            code = gx_flattened_iterator__next(&q.fi);
+            assert(code >= 0); /* Indicates an internal error */
             if (code < 0)
                 return code;
+            q.dir = compute_dir(fo, q.fi.ly0, q.fi.ly1);
+            if (q.dir == DIR_DOWN || q.dir == DIR_UP)
+                ll->main_dir = q.dir;
+            /* Exit the loop when we hit the end of the single curve segment */
             if (!code)
                 break;
-            q->first_flattened = false;
-            q->dir = compute_dir(fo, q->fi->ly0, q->fi->ly1);
-            ll->main_dir = (q->dir == DIR_DOWN ? DIR_DOWN :
-                            q->dir == DIR_UP ? DIR_UP : ll->main_dir);
+            q.first_flattened = false;
         }
-        q->dir = compute_dir(fo, q->fi->ly0, q->fi->ly1);
-        q->more_flattened = false;
-        ll->main_dir = (q->dir == DIR_DOWN ? DIR_DOWN :
-                        q->dir == DIR_UP ? DIR_UP : ll->main_dir);
+        /* Thus first_flattened == true, iff the curve needed no subdivisions */
+        q.more_flattened = false;
+        /* ll->maindir == DIR_HORIZONTAL, iff the curve is entirely horizontal
+         * (or entirely out of y range). It will contain whatever the last UP or DOWN
+         * section was otherwise. */
         if (ll->main_dir != DIR_HORIZONTAL) {
             only_horizontal = false;
+            /* Now we know we're not entirely horizontal, we can abort this run. */
             break;
         }
-        if (!saved && q->dir != 2) {
-            save_q = *q;
-            save_fi = *q->fi;
+        /* If we haven't saved one yet, and we are not out of range, save this one.
+         * i.e. save_q and save_fi are the first 'in range' section. We can save
+         * time in subsequent passes through by starting here. */
+        if (!saved && q.dir != DIR_OUT_OF_Y_RANGE) {
+            save_q = q;
             saved = true;
         }
-        if (q->prev == q->pfirst)
-            break;
+        if (q.prev == q.pfirst)
+            break; /* Exit if we're back at the start of the contour */
     }
+
+    /* Second run through; this is where we actually do the hard work. */
+    /* Restore q if we saved it above */
     if (saved) {
-        *q = save_q;
-        *q->fi = save_fi;
+        q = save_q;
     }
-    for (pseg = q->pfirst; pseg != q->plast; pseg = pseg->next) {
-        p.prev = pseg;
-        p.pseg = pseg->next;
-        if (!fo->pseudo_rasterization || only_horizontal
-                || p.prev->pt.x != p.pseg->pt.x || p.prev->pt.y != p.pseg->pt.y
-                || p.pseg->type == s_curve) {
-            code = init_contour_cursor(ll, &p);
-            if (code < 0)
-                return code;
+    /* q is now at the start of a run that we know will head in an
+     * ll->main_dir direction. Start p at the next place and walk
+     * forwards. */
+    for (p.prev = q.pfirst; p.prev != q.plast; p.prev = p.pseg) {
+        bool added;
+        p.pseg = p.prev->next;
+        /* Can we ignore this segment entirely? */
+        if (fo->pseudo_rasterization && !only_horizontal && p.pseg->type != s_curve &&
+            p.prev->pt.x == p.pseg->pt.x && p.prev->pt.y == p.pseg->pt.y)
+            continue;
+
+        /* Prepare to walk down 'p' */
+        code = init_contour_cursor(fo, &p);
+        if (code < 0)
+            return code;
+
+        do {
+            /* Find the next flattened section that is within range */
             do {
-                code = gx_flattened_iterator__next(p.fi);
+                code = gx_flattened_iterator__next(&p.fi);
+                assert(code >= 0); /* Indicates an internal error */
                 if (code < 0)
                     return code;
                 p.more_flattened = code;
-                p.dir = compute_dir(fo, p.fi->ly0, p.fi->ly1);
-            } while (p.more_flattened && p.dir == 2);
-            if (p.fi->ly0 > fo->ymax && ll->y_break > p.fi->ly0)
-                ll->y_break = p.fi->ly0;
-            if (p.fi->ly1 > fo->ymax && ll->y_break > p.fi->ly1)
-                ll->y_break = p.fi->ly1;
-            if (p.monotonic_y && p.dir == DIR_HORIZONTAL &&
-                    !fo->pseudo_rasterization &&
+                p.dir = compute_dir(fo, p.fi.ly0, p.fi.ly1);
+            } while (p.more_flattened && p.dir == DIR_OUT_OF_Y_RANGE);
+
+            /* So either we're in range, or we've reached the end of the segment (or both) */
+            /* FIXME: Can we break here if p.dir == DIR_OUT_OF_Y_RANGE? */
+
+            /* Set ll->y_break to be the smallest line endpoint we find > ymax */
+            if (p.fi.ly0 > fo->ymax && ll->y_break > p.fi.ly0)
+                ll->y_break = p.fi.ly0;
+            if (p.fi.ly1 > fo->ymax && ll->y_break > p.fi.ly1)
+                ll->y_break = p.fi.ly1;
+
+            added = 0;
+            if (p.dir == DIR_HORIZONTAL) {
+                if (p.monotonic_y) {
+                    if (!fo->pseudo_rasterization) {
+                        if (
 #ifdef FILL_ZERO_WIDTH
-                    (fo->adjust_below | fo->adjust_above) != 0) {
+                            (fo->adjust_below | fo->adjust_above) != 0) {
 #else
-                    fixed2int_pixround(p.pseg->pt.y - fo->adjust_below) <
-                    fixed2int_pixround(p.pseg->pt.y + fo->adjust_above)) {
+                            (fo->adjust_below + fo->adjust_above >= (fixed_1 - fixed_epsilon) ||
+                             fixed2int_pixround(p.pseg->pt.y - fo->adjust_below) <
+                             fixed2int_pixround(p.pseg->pt.y + fo->adjust_above))) {
 #endif
-                /* Add it here to avoid double processing in process_h_segments. */
-                code = add_y_line(p.prev, p.pseg, DIR_HORIZONTAL, ll);
-                if (code < 0)
-                    return code;
+                            /* Add it here to avoid double processing in process_h_segments. */
+                            code = add_y_line(p.prev, p.pseg, DIR_HORIZONTAL, ll);
+                            if (code < 0)
+                                return code;
+                            added = 1;
+                        }
+                    } else if (only_horizontal) {
+                        /* Add it here to avoid double processing in process_h_segments. */
+                        code = add_y_line(p.prev, p.pseg, DIR_HORIZONTAL, ll);
+                        if (code < 0)
+                            return code;
+                        added = 1;
+                    }
+                }
+            } else {
+                if (p.fi.ly0 >= fo->ymin)
+                {
+                    if (p.dir == DIR_UP && ll->main_dir == DIR_DOWN) {
+                        /* p starts within range, and heads up. q was heading down. Therefore
+                         * they meet at a local minima. Start a pair of active lines from that. */
+                        code = start_al_pair(ll, &q, &p);
+                        if (code < 0)
+                            return code;
+                        added = 1;
+                    } else if (p.fi.ly1 < fo->ymin) {
+                        /* p heading downwards */
+                        if (p.monotonic_y)
+                            code = add_y_line(p.prev, p.pseg, DIR_DOWN, ll);
+                        else
+                            code = add_y_curve_part(ll, p.prev, p.pseg, DIR_DOWN, &p.fi,
+                                                    !p.first_flattened, false, p.monotonic_x);
+                        if (code < 0)
+                            return code;
+                        added = 1;
+                    }
+                } else if (p.fi.ly1 >= fo->ymin) {
+                    /* p heading upwards */
+                    if (p.monotonic_y)
+                        code = add_y_line(p.prev, p.pseg, DIR_UP, ll);
+                    else
+                        code = add_y_curve_part(ll, p.prev, p.pseg, DIR_UP, &p.fi,
+                                                p.more_flattened, false, p.monotonic_x);
+                    if (code < 0)
+                        return code;
+                    added = 1;
+                }
+                if (p.dir == DIR_DOWN || p.dir == DIR_UP)
+                    ll->main_dir = p.dir;
             }
-            if (p.monotonic_y && p.dir == DIR_HORIZONTAL &&
-                    fo->pseudo_rasterization && only_horizontal) {
-                /* Add it here to avoid double processing in process_h_segments. */
-                code = add_y_line(p.prev, p.pseg, DIR_HORIZONTAL, ll);
-                if (code < 0)
-                    return code;
-            }
-            if (p.fi->ly0 >= fo->ymin && p.dir == DIR_UP && ll->main_dir == DIR_DOWN) {
-                code = start_al_pair(ll, q, &p);
-                if (code < 0)
-                    return code;
-            }
-            if (p.fi->ly0 < fo->ymin && p.fi->ly1 >= fo->ymin) {
-                if (p.monotonic_y)
-                    code = add_y_line(p.prev, p.pseg, DIR_UP, ll);
-                else
-                    code = add_y_curve_part(ll, p.prev, p.pseg, DIR_UP, p.fi,
-                                        p.more_flattened, false, p.monotonic_x);
-                if (code < 0)
-                    return code;
-            }
-            if (p.fi->ly0 >= fo->ymin && p.fi->ly1 < fo->ymin) {
-                if (p.monotonic_y)
-                    code = add_y_line(p.prev, p.pseg, DIR_DOWN, ll);
-                else
-                    code = add_y_curve_part(ll, p.prev, p.pseg, DIR_DOWN, p.fi,
-                                        !p.first_flattened, false, p.monotonic_x);
-                if (code < 0)
-                    return code;
-            }
-            ll->main_dir = (p.dir == DIR_DOWN ? DIR_DOWN :
-                            p.dir == DIR_UP ? DIR_UP : ll->main_dir);
             if (!p.monotonic_y && p.more_flattened) {
                 code = start_al_pair_from_min(ll, &p);
                 if (code < 0)
                     return code;
+                added = 1;
             }
             if (p.dir == DIR_DOWN || p.dir == DIR_HORIZONTAL) {
-                gx_flattened_iterator *fi1 = q->fi;
-                q->prev = p.prev;
-                q->pseg = p.pseg;
-                q->monotonic_y = p.monotonic_y;
-                q->more_flattened = p.more_flattened;
-                q->first_flattened = p.first_flattened;
-                q->fi = p.fi;
-                q->dir = p.dir;
-                p.fi = fi1;
+                q.prev = p.prev;
+                q.pseg = p.pseg;
+                q.monotonic_y = p.monotonic_y;
+                q.more_flattened = p.more_flattened;
+                q.first_flattened = p.first_flattened;
+                q.fi = p.fi;
+                q.dir = p.dir;
             }
-        }
+            /* If we haven't added lines based on this segment, and there
+             * are more flattened lines to come from it, ensure we loop
+             * to get the rest of them. */
+            /* FIXME: Do we need '!added' here? Try removing it. */
+        } while (!added && p.more_flattened);
     }
-    q->fi = NULL; /* safety. */
     return 0;
 }
 
@@ -1069,10 +1136,7 @@ add_y_list(gx_path * ppath, line_list *ll)
     subpath *psub = ppath->first_subpath;
     int close_count = 0;
     int code;
-    contour_cursor q;
-    gx_flattened_iterator fi;
 
-    q.monotonic_x = false; /* Quiet gcc warning. */
     ll->y_break = max_fixed;
 
     for (;psub; psub = (subpath *)psub->last->next) {
@@ -1080,7 +1144,6 @@ add_y_list(gx_path * ppath, line_list *ll)
         segment *pfirst = (segment *)psub;
         segment *plast = psub->last, *prev;
 
-        q.fi = &fi;
         if (plast->type != s_line_close) {
             /* Create a fake s_line_close */
             line_close_segment *lp = &psub->closer;
@@ -1103,11 +1166,7 @@ add_y_list(gx_path * ppath, line_list *ll)
             plast = prev;
             prev = prev->prev;
         }
-        q.prev = prev;
-        q.pseg = plast;
-        q.pfirst = pfirst;
-        q.plast = plast;
-        code = scan_contour(ll, &q);
+        code = scan_contour(ll, pfirst, plast, prev);
         if (code < 0)
             return code;
         ll->contour_count++;
@@ -1115,7 +1174,6 @@ add_y_list(gx_path * ppath, line_list *ll)
     return close_count;
 }
 
-
 static int
 step_al(active_line *alp, bool move_iterator)
 {
@@ -1244,7 +1302,6 @@ add_y_line_aux(const segment * prev_lp, const segment * lp,
     return 0;
 }
 
-
 /* ---------------- Filling loop utilities ---------------- */
 
 /* Insert a newly active line in the X ordering. */
@@ -1267,7 +1324,8 @@ insert_x_new(active_line * alp, line_list *ll)
     alp->prev = prev;
     if (next != 0)
         next->prev = alp;
-    prev->next = alp;
+    if (prev)
+        prev->next = alp;
 }
 
 /* Insert a newly active line in the h list. */
@@ -1301,7 +1359,7 @@ remove_al(const line_list *ll, active_line *alp)
     alp->prev->next = nlp;
     if (nlp)
         nlp->prev = alp->prev;
-    if_debug1('F', "[F]drop 0x%lx\n", (ulong) alp);
+    if_debug1m('F', ll->memory, "[F]drop 0x%lx\n", (ulong) alp);
 }
 
 /*
@@ -1355,7 +1413,7 @@ end_x_line(active_line *alp, const line_list *ll, bool update)
     }
     alp->x_current = alp->x_next = alp->start.x;
     vd_bar(alp->start.x, alp->start.y, alp->end.x, alp->end.y, 1, RGB(128, 0, 128));
-    print_al("repl", alp);
+    print_al(ll->memory, "repl", alp);
     return false;
 }
 
@@ -1603,7 +1661,7 @@ resort_x_line(active_line * alp)
         next->prev = prev;
     while (x_order(prev, alp) > 0) {
         if_debug2('F', "[F]swap 0x%lx,0x%lx\n",
-                  (ulong) alp, (ulong) prev);
+                   (ulong) alp, (ulong) prev);
         next = prev, prev = prev->prev;
     }
     alp->next = next;
@@ -1611,7 +1669,9 @@ resort_x_line(active_line * alp)
     /* next might be null, if alp was in the correct spot already. */
     if (next)
         next->prev = alp;
-    prev->next = alp;
+    /* prev can be null if the path reaches (beyond) the extent of our coordinate space */
+    if (prev)
+        prev->next = alp;
 }
 
 /* Move active lines by Y. */
@@ -1733,7 +1793,6 @@ loop_fill_trap_np(const line_list *ll, const gs_fixed_edge *le, const gs_fixed_e
 #undef TEMPLATE_slant_into_trapezoids
 #undef FILL_DIRECT
 
-
 #define COVERING_PIXEL_CENTERS(y, y1, adjust_below, adjust_above)\
     (fixed_pixround(y - adjust_below) < fixed_pixround(y1 + adjust_above))
 
@@ -1825,7 +1884,6 @@ coord_weight(const active_line *alp)
     return 1 + min(any_abs((int)((int64_t)alp->diff.y * 8 / alp->diff.x)), 256);
 }
 
-
 /* Find intersections of active lines within the band.
    Intersect and reorder them, and correct the bund top. */
 static void
@@ -1963,18 +2021,13 @@ intersect_al(line_list *ll, fixed y, fixed *y_top, int draw, bool all_bands)
     }
 #ifdef DEBUG
     if (gs_debug_c('F')) {
-        dlprintf1("[F]after loop: y1=%f\n", fixed2float(y1));
-        print_line_list(ll->x_list);
+        dmlprintf1(ll->memory, "[F]after loop: y1=%f\n", fixed2float(y1));
+        print_line_list(ll->memory, ll->x_list);
     }
 #endif
     *y_top = y1;
 }
 
-static inline int sign(int a)
-{
-    return a < 0 ? -1 : a > 0 ? 1 : 0;
-}
-
 /* ---------------- Trapezoid filling loop ---------------- */
 
 /* Generate specialized algorythms for the most important cases : */
@@ -2156,7 +2209,7 @@ static inline int sign(int a)
 static int
 spot_into_trapezoids(line_list *ll, fixed band_mask)
 {
-    /* For better porformance, choose a specialized algorithm : */
+    /* For better performance, choose a specialized algorithm : */
     const fill_options * const fo = ll->fo;
 
     if (fo->is_spotan)
