@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxfont42.h 10518 2009-12-18 05:23:14Z henrys $ */
+
 /* Type 42 font data definition */
 
 #ifndef gxfont42_INCLUDED
@@ -53,12 +55,12 @@ typedef enum gs_type42_metrics_options_s {
 #define gs_type42_metrics_options_sbw_requested(a) (~(a)&gs_type42_metrics_options_BBOX)
 #define gs_type42_metrics_options_bbox_requested(a)((a)&6)
 
-/* Export the default get_metrics procedure. 
+/* Export the default get_metrics procedure.
    The length of sbw is >=4 when bbox in not requested,
    and 8 otherwise.
  */
 int gs_type42_default_get_metrics(gs_font_type42 *pfont, uint glyph_index,
-				  gs_type42_metrics_options_t options, float *sbw);
+                                  gs_type42_metrics_options_t options, float *sbw);
 
 typedef struct gs_type42_mtx_s {
     uint numMetrics;		/* num*Metrics from [hv]hea */
@@ -75,23 +77,23 @@ struct gs_type42_data_s {
      */
     uint (*get_glyph_index)(gs_font_type42 *pfont, gs_glyph glyph);
     int (*get_outline)(gs_font_type42 *pfont, uint glyph_index,
-		       gs_glyph_data_t *pgd);
-    int (*get_metrics)(gs_font_type42 *pfont, uint glyph_index, 
-			gs_type42_metrics_options_t options,
-			float *sbw_bbox/* See comment for gs_type42_default_get_metrics */);
+                       gs_glyph_data_t *pgd);
+    int (*get_metrics)(gs_font_type42 *pfont, uint glyph_index,
+                        gs_type42_metrics_options_t options,
+                        float *sbw_bbox/* See comment for gs_type42_default_get_metrics */);
      uint (*substitute_glyph_index_vertical)(gs_font_type42 *pfont, uint glyph_index,
-					  int WMode, gs_glyph glyph);
+                                          int WMode, gs_glyph glyph);
 
     /* The following are cached values. */
     ulong cmap;			/* offset to cmap table (not used by */
-				/* renderer, only here for clients) */
+                                /* renderer, only here for clients) */
     ulong glyf;			/* offset to glyf table */
     uint unitsPerEm;		/* from head */
     uint indexToLocFormat;	/* from head */
     gs_type42_mtx_t metrics[2];	/* hhea/hmtx, vhea/vmtx (indexed by WMode) */
     ulong loca;			/* offset to loca table */
-    ulong name_offset;		/* offset to name table */		
-    ulong os2_offset;		/* offset to OS/2 table */		
+    ulong name_offset;		/* offset to name table */
+    ulong os2_offset;		/* offset to OS/2 table */
     /*
      * TrueType fonts specify the number of glyphs in two different ways:
      * the size of the loca table, and an explicit value in maxp.  Currently
@@ -106,6 +108,10 @@ struct gs_type42_data_s {
      */
     uint numGlyphs;		/* from size of loca */
     uint trueNumGlyphs;		/* from maxp */
+    uint maxPoints;         /* from maxp (used by pdfwrite) */
+    uint maxContours;       /* from maxp (used by pdfwrite) */
+    uint maxCPoints;        /* from maxp (used by pdfwrite) */
+    uint maxCContours;      /* from maxp (used by pdfwrite) */
     uint *len_glyphs;		/* built from the loca table */
     gs_glyph_cache *gdcache;
     bool warning_patented;
@@ -133,7 +139,7 @@ extern_st(st_gs_font_type42);
  * Note that this initializes the type42_data procedures other than
  * string_proc, and the font procedures as well.
  * USE_ttfReader subclasses gs_font_type42 with ttfReader or without.
- * FAPI will disable ttfReader as well. 
+ * FAPI will disable ttfReader as well.
  */
 int gs_type42_font_init(gs_font_type42 *pfont, int subfontid);
 
@@ -145,7 +151,7 @@ int gs_type42_read_data(gs_font_type42 * pfont, ulong pos, uint length, byte *bu
 #define READ_SFNTS(pfont, pos, length, buf)\
   BEGIN\
     if (length > sizeof(buf))\
-	return_error(gs_error_unregistered);/* Must not happen. */\
+        return_error(gs_error_unregistered);/* Must not happen. */\
     code = gs_type42_read_data(pfont, (ulong)(pos), length, buf);\
     if ( code < 0 ) return code;\
   END
@@ -154,15 +160,15 @@ int gs_type42_read_data(gs_font_type42 * pfont, ulong pos, uint length, byte *bu
 
 /* Append the outline of a TrueType character to a path. */
 int gs_type42_append(uint glyph_index, gs_state * pgs,
-		 gx_path * ppath, gs_text_enum_t *penum, gs_font *pfont,
-		 bool charpath_flag);
+                 gx_path * ppath, gs_text_enum_t *penum, gs_font *pfont,
+                 bool charpath_flag);
 
 /* Get the metrics of a TrueType character. */
 int gs_type42_get_metrics(gs_font_type42 * pfont, uint glyph_index,
-			  float psbw[4]);
+                          float psbw[4]);
 
 int gs_type42_get_outline_from_TT_file(gs_font_type42 * pfont, stream *s, uint glyph_index,
-		gs_glyph_data_t *pgd);
+                gs_glyph_data_t *pgd);
 
 /* Export the font procedures so they can be called from the interpreter. */
 font_proc_enumerate_glyph(gs_type42_enumerate_glyph);
@@ -171,19 +177,25 @@ font_proc_glyph_outline(gs_type42_glyph_outline);
 
 /* Get glyph info by glyph index. */
 int gs_type42_glyph_info_by_gid(gs_font *font, gs_glyph glyph, const gs_matrix *pmat,
-		     int members, gs_glyph_info_t *info, uint glyph_index);
+                     int members, gs_glyph_info_t *info, uint glyph_index);
 
 int gs_type42_font_info(gs_font *font, const gs_point *pscale, int members,
-	   gs_font_info_t *info);
+           gs_font_info_t *info);
 int gs_truetype_font_info(gs_font *font, const gs_point *pscale, int members,
-	   gs_font_info_t *info);
+           gs_font_info_t *info);
 
 uint gs_type42_substitute_glyph_index_vertical(gs_font_type42 *pfont, uint glyph_index,
-					  int WMode, gs_glyph glyph);
+                                          int WMode, gs_glyph glyph);
 
 /* Get next component in a composite TrueType glyph. */
 void gs_type42_parse_component(const byte **pdata, uint *pflags, gs_matrix_fixed *psmat,
                                int *pmp /*[2], may be null*/, const gs_font_type42 *pfont,
                                const gs_matrix_fixed *pmat);
 
+int
+gs_woff2sfnt_stream(gs_memory_t *mem, stream *s, byte *outbuf, int *outbuflen);
+
+int
+gs_woff2sfnt_buffer(gs_memory_t *mem, byte *inbuf, int inbuflen, byte *outbuf, int *outbuflen);
+
 #endif /* gxfont42_INCLUDED */
