@@ -118,6 +118,9 @@ gs_image_class_1_simple(gx_image_enum * penum)
                                          false);
                     return 0;
                 }
+#ifdef PACIFY_VALGRIND
+                memset(penum->line, 0, penum->line_size); /* For the number of scan lined < 8 */
+#endif
                 penum->xi_next = penum->line_xy = fixed2int_var_rounded(ox);
                 if_debug3m('b', penum->memory,
                            "[b]render=landscape, unpack=copy; rect.w=%d, dev_width=%ld, line_size=%ld\n",
@@ -403,8 +406,9 @@ sw:	    if ((data = psrc[1]) != 0) {
             /*
              * We've scanned the last run of 0s.
              * Prepare to fill the final run of 1s.
+             * Use int64_t to avoid overflow.
              */
-            n = fixed2int(xl0 + x_extent) - x0;
+            n = fixed2int((int64_t)xl0 + (int64_t)x_extent) - x0;
         } else {		/* Scan a run of ones. */
             /* We know the current bit is a one. */
             data ^= 0xff;	/* un-invert */
@@ -607,8 +611,6 @@ image_render_simple(gx_image_enum * penum, const byte * buffer, int data_x,
             return 1;
         /****** MAY BE UNALIGNED ******/
         line = scan_line + (line_ix >> 3);
-        if (dxx < 0)
-            ix -= line_width;
         for (dy = 1; dy < ih; dy++) {
             int code = (*copy_mono)
                 (dev, line, line_x, line_size, gx_no_bitmap_id,
