@@ -41,6 +41,7 @@
 #include "gscindex.h"
 #include "gsicc_cache.h"
 #include "gsicc_cms.h"
+#include "gxdevsop.h"
 
 /* Structure descriptors */
 private_st_gx_image_enum();
@@ -128,8 +129,7 @@ RELOC_PTRS_END
 /* Forward declarations */
 static int color_draws_b_w(gx_device * dev,
                             const gx_drawing_color * pdcolor);
-static void image_init_map(byte * map, int map_size, const float *decode);
-static void image_init_colors(gx_image_enum * penum, int bps, int spp,
+static int image_init_colors(gx_image_enum * penum, int bps, int spp,
                                gs_image_format_t format,
                                const float *decode,
                                const gs_imager_state * pis, gx_device * dev,
@@ -203,6 +203,18 @@ gx_image_enum_alloc(const gs_image_common_t * pic,
     return 0;
 }
 
+/* Convert and restrict to a valid range. */
+static inline fixed float2fixed_rounded_boxed(double src) {
+	float v = floor(src*fixed_scale + 0.5);
+
+	if (v <= min_fixed)
+		return min_fixed;
+	else if (v >= max_fixed)
+		return max_fixed;
+	else
+		return 	(fixed)v;
+}
+
 /*
  * Finish initialization for processing an ImageType 1 or 4 image.
  * Assumes the following members of *penum are set in addition to those
@@ -236,6 +248,10 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
     gs_fixed_point row_extent, col_extent, x_extent, y_extent;
     bool device_color = true;
     gs_fixed_rect obox, cbox;
+    bool gridfitimages = 0;
+    bool in_pattern_accumulator = 0;
+    int orthogonal;
+    int force_interpolation = 0;
 
     penum->clues = NULL;
     penum->icc_setup.has_transfer = false;
@@ -270,7 +286,7 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
      * covered. Bug 692666 is such a problem.
      *
      * As a workaround for this problem, the code below was introduced. The
-     * concept is that orthogonal images can be 'grid fitted' (or 'stretched')
+     * concept is that orthogonal images can be 'grid fitted' (or 'stretch')
      * to entirely cover pixels that they touch. Initially I had this working
      * for all images regardless of type, but as testing has proceeded, this
      * showed more and more regressions, so I've cut the cases back in which
@@ -278,15 +294,40 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
      * either 1 pixel high, or wide, and then not if we are rendering a
      * glyph (such as from a type3 font).
      */
+
+    /* Ask the device if we are in a pattern accumulator */
+    in_pattern_accumulator = (dev_proc(dev, dev_spec_op)(dev, gxdso_in_pattern_accumulator, NULL, 0));
+    if (in_pattern_accumulator < 0)
+        in_pattern_accumulator = 0;
+
+    /* Figure out if we are orthogonal */
+    if (mat.xy == 0 && mat.yx == 0)
+        orthogonal = 1;
+    else if (mat.xx == 0 && mat.yy == 0)
+        orthogonal = 2;
+    else
+        orthogonal = 0;
+
+    /* If we are in a pattern accumulator, we choose to always grid fit
+     * orthogonal images. We do this by asking the device whether we
+     * should grid fit. This allows us to avoid nasty blank lines around
+     * the edges of cells.
+     */
+    gridfitimages = in_pattern_accumulator && orthogonal;
+
     if (pis != NULL && pis->is_gstate && ((gs_state *)pis)->show_gstate != NULL) {
         /* If we're a graphics state, and we're in a text object, then we
          * must be in a type3 font. Don't fiddle with it. */
-    } else if (!penum->masked || penum->image_parent_type != 0) {
-        /* We only grid fit for ImageMasks, currently */
+    } else if (!gridfitimages &&
+               (!penum->masked || penum->image_parent_type != 0)) {
+        /* Other than for images we are specifically looking to grid fit (such as
+         * ones in a pattern device), we only grid fit imagemasks */
+    } else if (gridfitimages && (penum->masked && penum->image_parent_type == 0)) {
+        /* We don't gridfit imagemasks in a pattern accumulator */
     } else if (pis != NULL && pis->fill_adjust.x == 0 && pis->fill_adjust.y == 0) {
         /* If fill adjust is disabled, so is grid fitting */
-    } else if (mat.xy == 0 && mat.yx == 0) {
-        if (width == 1) {
+    } else if (orthogonal == 1) {
+        if (width == 1 || gridfitimages) {
             if (mat.xx > 0) {
                 fixed ix0 = int2fixed(fixed2int(float2fixed(mat.tx)));
                 double x1 = mat.tx + mat.xx * width;
@@ -301,7 +342,7 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
                 mat.xx = (double)(fixed2float(ix1 - ix0)/width);
             }
         }
-        if (height == 1) {
+        if (height == 1 || gridfitimages) {
             if (mat.yy > 0) {
                 fixed iy0 = int2fixed(fixed2int(float2fixed(mat.ty)));
                 double y1 = mat.ty + mat.yy * height;
@@ -316,8 +357,8 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
                 mat.yy = ((double)fixed2float(iy1 - iy0)/height);
             }
         }
-    } else if (mat.xx == 0 && mat.yy == 0) {
-        if (height == 1) {
+    } else if (orthogonal == 2) {
+        if (height == 1 || gridfitimages) {
             if (mat.yx > 0) {
                 fixed ix0 = int2fixed(fixed2int(float2fixed(mat.tx)));
                 double x1 = mat.tx + mat.yx * height;
@@ -332,7 +373,7 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
                 mat.yx = (double)(fixed2float(ix1 - ix0)/height);
             }
         }
-        if (width == 1) {
+        if (width == 1 || gridfitimages) {
             if (mat.xy > 0) {
                 fixed iy0 = int2fixed(fixed2int(float2fixed(mat.ty)));
                 double y1 = mat.ty + mat.xy * width;
@@ -349,6 +390,21 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
         }
     }
 
+    /* When rendering to a pattern accumulator, if we are downscaling
+     * then enable interpolation, as otherwise dropouts can cause
+     * serious problems. */
+    if (in_pattern_accumulator) {
+        if (orthogonal == 1) {
+            if ((mat.xx > -1 && mat.xx < 1) || (mat.yy > -1 && mat.yy < 1)) {
+                force_interpolation = true;
+            }
+        } else if (orthogonal == 2) {
+            if ((mat.xy > -1 && mat.xy < 1) || (mat.yx > -1 && mat.yx < 1)) {
+                force_interpolation = true;
+            }
+        }
+    }
+
     /* Can we restrict the amount of image we need? */
     while (pcpath) /* So we can break out of it */
     {
@@ -423,6 +479,19 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
         }
         break; /* Out of the while */
     }
+    /* Check for the intersection being null */
+    if (penum->rrect.x + penum->rrect.w <= penum->rect.x  ||
+        penum->rect.x  + penum->rect.w  <= penum->rrect.x ||
+        penum->rrect.y + penum->rrect.h <= penum->rect.y  ||
+        penum->rect.y  + penum->rect.h  <= penum->rrect.y)
+    {
+          /* Something may have gone wrong with the floating point above.
+           * set the region to something sane. */
+        penum->rrect.x = penum->rect.x;
+        penum->rrect.y = penum->rect.y;
+        penum->rrect.w = 0;
+        penum->rrect.h = 0;
+    }
 
     /*penum->matrix = mat;*/
     penum->matrix.xx = mat.xx;
@@ -431,7 +500,7 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
     penum->matrix.yy = mat.yy;
     penum->matrix.tx = mat.tx;
     penum->matrix.ty = mat.ty;
-    if_debug6m('b', pis->memory, " [%g %g %g %g %g %g]\n",
+    if_debug6m('b', mem, " [%g %g %g %g %g %g]\n",
               mat.xx, mat.xy, mat.yx, mat.yy, mat.tx, mat.ty);
     /* following works for 1, 2, 4, 8, 12, 16 */
     index_bps = (bps < 8 ? bps >> 1 : (bps >> 2) + 1);
@@ -453,14 +522,14 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
         mty = float2fixed(mat.ty + f) - int2fixed(f);
     }
 
-    row_extent.x = float2fixed_rounded(width * mat.xx);
+    row_extent.x = float2fixed_rounded_boxed(width * mat.xx);
     row_extent.y =
         (is_fzero(mat.xy) ? fixed_0 :
-         float2fixed_rounded(width * mat.xy));
+         float2fixed_rounded_boxed(width * mat.xy));
     col_extent.x =
         (is_fzero(mat.yx) ? fixed_0 :
-         float2fixed_rounded(height * mat.yx));
-    col_extent.y = float2fixed_rounded(height * mat.yy);
+         float2fixed_rounded_boxed(height * mat.yx));
+    col_extent.y = float2fixed_rounded_boxed(height * mat.yy);
     gx_image_enum_common_init((gx_image_enum_common_t *)penum,
                               (const gs_data_image_t *)pim,
                               &image1_enum_procs, dev,
@@ -472,14 +541,14 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
     } else {
         int rw = penum->rect.w, rh = penum->rect.h;
 
-        x_extent.x = float2fixed_rounded(rw * mat.xx);
+        x_extent.x = float2fixed_rounded_boxed(rw * mat.xx);
         x_extent.y =
             (is_fzero(mat.xy) ? fixed_0 :
-             float2fixed_rounded(rw * mat.xy));
+             float2fixed_rounded_boxed(rw * mat.xy));
         y_extent.x =
             (is_fzero(mat.yx) ? fixed_0 :
-             float2fixed_rounded(rh * mat.yx));
-        y_extent.y = float2fixed_rounded(rh * mat.yy);
+             float2fixed_rounded_boxed(rh * mat.yx));
+        y_extent.y = float2fixed_rounded_boxed(rh * mat.yy);
     }
     /* Set icolor0 and icolor1 to point to image clues locations if we have
        1spp or an imagemask, otherwise image clues is not used and
@@ -542,8 +611,10 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
             device_color = (*pcst->concrete_space) (pcs, pis) == pcs;
         }
 
-        image_init_colors(penum, bps, spp, format, decode, pis, dev,
+        code = image_init_colors(penum, bps, spp, format, decode, pis, dev,
                           pcs, &device_color);
+        if (code < 0) 
+            return gs_throw(code, "Image colors initialization failed");
         /* If we have a CIE based color space and the icc equivalent profile
            is not yet set, go ahead and handle that now.  It may already
            be done due to the above init_colors which may go through remap. */
@@ -656,7 +727,7 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
      * the given sub-image, or else is constructing output out of
      * overlapping pieces.
      */
-    penum->interpolate = pim->Interpolate;
+    penum->interpolate = pim->Interpolate | force_interpolation;
     penum->x_extent = x_extent;
     penum->y_extent = y_extent;
     penum->posture =
@@ -752,10 +823,10 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
                 mty = (((mty + diff) | fixed_half) & -fixed_half) - diff;
             }
         }
-        if_debug5m('b', penum->memory, "[b]Image: %sspp=%d, bps=%d, mt=(%g,%g)\n",
+        if_debug5m('b', mem, "[b]Image: %sspp=%d, bps=%d, mt=(%g,%g)\n",
                    (masked? "masked, " : ""), spp, bps,
                    fixed2float(mtx), fixed2float(mty));
-        if_debug9m('b', penum->memory,
+        if_debug9m('b', mem,
                    "[b]   cbox=(%g,%g),(%g,%g), obox=(%g,%g),(%g,%g), clip_image=0x%x\n",
                    fixed2float(cbox.p.x), fixed2float(cbox.p.y),
                    fixed2float(cbox.q.x), fixed2float(cbox.q.y),
@@ -817,18 +888,16 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
         static sample_unpack_proc_t procs[2][6] = {
         {   sample_unpack_1, sample_unpack_2,
             sample_unpack_4, sample_unpack_8,
-            0, 0
+            sample_unpack_12, sample_unpack_16
         },
         {   sample_unpack_1_interleaved, sample_unpack_2_interleaved,
             sample_unpack_4_interleaved, sample_unpack_8_interleaved,
-            0, 0
+            sample_unpack_12, sample_unpack_16
         }};
         int num_planes = penum->num_planes;
         bool interleaved = (num_planes == 1 && penum->plane_depths[0] != penum->bps);
         int i;
 
-        procs[0][4] = procs[1][4] = sample_unpack_12_proc;
-        procs[0][5] = procs[1][5] = sample_unpack_16_proc;
         if (interleaved) {
             int num_components = penum->plane_depths[0] / penum->bps;
 
@@ -840,17 +909,9 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
             if (i == num_components)
                 interleaved = false; /* Use single table. */
         }
-        if (index_bps >= 4) {
-            if ((penum->unpack = procs[interleaved][index_bps]) == 0) {         /* bps case not supported. */
-                gx_default_end_image(dev,
-                                     (gx_image_enum_common_t *) penum,
-                                     false);
-                return_error(gs_error_rangecheck);
-            }
-        } else {
-            penum->unpack = procs[interleaved][index_bps];
-        }
-        if_debug1m('b', penum->memory, "[b]unpack=%d\n", bps);
+        penum->unpack = procs[interleaved][index_bps];
+
+        if_debug1m('b', mem, "[b]unpack=%d\n", bps);
         /* Set up pixel0 for image class procedures. */
         penum->dda.pixel0 = penum->dda.strip;
         for (i = 0; i < gx_image_class_table_count; ++i)
@@ -1072,7 +1133,7 @@ image_init_color_cache(gx_image_enum * penum, int bps, int spp)
                                              "image_init_color_cache");
         if (need_decode) {
             if (is_indexed) {
-                /* Decode and un-indexed */
+                /* Decode and lookup in index */
                 for (k = 0; k < num_entries; k++) {
                     image_cache_decode(penum, k, &value, decode_scale);
                     gs_cspace_indexed_lookup_bytes(penum->pcs, value, psrc);
@@ -1087,13 +1148,18 @@ image_init_color_cache(gx_image_enum * penum, int bps, int spp)
         } else {
             /* No Decode */
             if (is_indexed) {
-                /* If index uses a table then just use its pointer */
-                if (penum->pcs->params.indexed.use_proc) {
+                /* If index uses a num_entries sized table then just use its pointer */
+                if (penum->pcs->params.indexed.use_proc ||
+                    penum->pcs->params.indexed.hival < (num_entries - 1)) {
                     /* Have to do the slow way */
-                    for (k = 0; k < num_entries; k++) {
+                    for (k = 0; k <= penum->pcs->params.indexed.hival; k++) {
                         gs_cspace_indexed_lookup_bytes(penum->pcs, (float)k, psrc);
                         memcpy(&(temp_buffer[k * num_src_comp]), psrc, num_src_comp);
                     }
+                    /* just use psrc results from converting 'hival' to fill the remaining slots */
+                    for (; k < num_entries; k++) {
+                        memcpy(&(temp_buffer[k * num_src_comp]), psrc, num_src_comp);
+                    }
                 } else {
                     /* Use the index table directly. */
                     gs_free_object(penum->memory, temp_buffer, "image_init_color_cache");
@@ -1179,13 +1245,13 @@ image_init_clues(gx_image_enum * penum, int bps, int spp)
 }
 
 /* Initialize the color mapping tables for a non-mask image. */
-static void
+static int
 image_init_colors(gx_image_enum * penum, int bps, int spp,
                   gs_image_format_t format, const float *decode /*[spp*2] */ ,
                   const gs_imager_state * pis, gx_device * dev,
                   const gs_color_space * pcs, bool * pdcb)
 {
-    int ci, decode_type;
+    int ci, decode_type, code;
     static const float default_decode[] = {
         0.0, 1.0, 0.0, 1.0, 0.0, 1.0, 0.0, 1.0, 0.0, 1.0
     };
@@ -1275,26 +1341,33 @@ image_init_colors(gx_image_enum * penum, int bps, int spp,
             for (i = 15 - step; i > 0; i -= step)
                 pmap->decode_lookup[i] = pmap->decode_base +
                     i * (255.0 / 15) * pmap->decode_factor;
-        } else
+            pmap->inverted = 0;
+        } else {
             pmap->decoding = sd_compute;
+            pmap->inverted = 0;
+        }
         if (spp == 1) {         /* and ci == 0 *//* Pre-map entries 0 and 255. */
             gs_client_color cc;
 
             /* Image clues are used in this case */
             cc.paint.values[0] = real_decode[0];
-            (*pcs->type->remap_color) (&cc, pcs, penum->icolor0,
+            code = (*pcs->type->remap_color) (&cc, pcs, penum->icolor0,
                                        pis, dev, gs_color_select_source);
+            if (code < 0) 
+                return code;
             cc.paint.values[0] = real_decode[1];
-            (*pcs->type->remap_color) (&cc, pcs, penum->icolor1,
+            code = (*pcs->type->remap_color) (&cc, pcs, penum->icolor1,
                                        pis, dev, gs_color_select_source);
+            if (code < 0) 
+                return code;
         }
     }
-
+    return 0;
 }
 /* Construct a mapping table for sample values. */
 /* map_size is 2, 4, 16, or 256.  Note that 255 % (map_size - 1) == 0, */
 /* so the division 0xffffL / (map_size - 1) is always exact. */
-static void
+void
 image_init_map(byte * map, int map_size, const float *decode)
 {
     float min_v = decode[0];
