@@ -161,7 +161,7 @@ int gx_path_new(gx_path *),
     gx_path_add_rectangle(gx_path *, fixed, fixed, fixed, fixed),
     gx_path_add_char_path(gx_path *, gx_path *, gs_char_path_mode),
     gx_path_add_curve_notes(gx_path *, fixed, fixed, fixed, fixed, fixed, fixed, segment_notes),
-    gx_path_add_partial_arc_notes(gx_path *, fixed, fixed, fixed, fixed, floatp, segment_notes),
+    gx_path_add_partial_arc_notes(gx_path *, fixed, fixed, fixed, fixed, double, segment_notes),
     gx_path_add_path(gx_path *, gx_path *),
     gx_path_close_subpath_notes(gx_path *, segment_notes),
           /* We have to remove the 'subpath' from the following name */
@@ -244,8 +244,6 @@ int gx_path_copy_reducing(const gx_path * ppath_old, gx_path * ppath_new,
 
 #define gx_path_copy(old, new)\
   gx_path_copy_reducing(old, new, max_fixed, NULL, pco_none)
-#define gx_path_add_flattened(old, new, flatness)\
-  gx_path_copy_reducing(old, new, float2fixed(flatness), NULL, pco_none)
 #define gx_path_add_flattened_accurate(old, new, flatness, accurate)\
   gx_path_copy_reducing(old, new, float2fixed(flatness), NULL,\
                         (accurate ? pco_accurate : pco_none))
@@ -253,8 +251,6 @@ int gx_path_copy_reducing(const gx_path * ppath_old, gx_path * ppath_new,
   gx_path_copy_reducing(old, new, float2fixed(flatness), pis,\
                         (pis->accurate_curves ?\
                          pco_accurate | pco_for_stroke : pco_for_stroke))
-#define gx_path_add_monotonized(old, new)\
-  gx_path_copy_reducing(old, new, max_fixed, NULL, pco_monotonize)
 int gx_path_add_dash_expansion(const gx_path * /*old*/, gx_path * /*new*/,
                                   const gs_imager_state *),
       gx_path_copy_reversed(const gx_path * /*old*/, gx_path * /*new*/),
@@ -277,7 +273,7 @@ gx_path_enum_notes(const gs_path_enum *);
 bool gx_path_enum_backup(gs_path_enum *);
 
 /* An auxiliary function to add a path point with a specified transformation. */
-int gs_moveto_aux(gs_imager_state *pis, gx_path *ppath, floatp x, floatp y);
+int gs_moveto_aux(gs_imager_state *pis, gx_path *ppath, double x, double y);
 int gx_setcurrentpoint_from_path(gs_imager_state *pis, gx_path *path);
 
 /* Path optimization for the filling algorithm. */
@@ -371,7 +367,7 @@ bool
     gx_cpath_outer_box(const gx_clip_path *, gs_fixed_rect *),
     gx_cpath_includes_rectangle(const gx_clip_path *, fixed, fixed,
                                 fixed, fixed);
-const gs_fixed_rect *cpath_is_rectangle(const gx_clip_path * pcpath);
+gx_path_rectangular_type cpath_is_rectangle(const gx_clip_path * pcpath, gs_fixed_rect *rect);
 bool gx_cpath_rect_visible(gx_clip_path * pcpath, gs_int_rect *prect);
 int gx_cpath_copy(const gx_clip_path * from, gx_clip_path * pcpath);
 
