@@ -1,24 +1,33 @@
-/* Copyright (C) 2001-2007 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Standard definitions for Ghostscript code not needing arch.h */
 
 #ifndef stdpre_INCLUDED
 #  define stdpre_INCLUDED
 
+/* if we define _LARGEFILE64_SOURCE zlib tries to include unistd.h */
+#ifndef _MSC_VER
 /* Ghostscript uses transitional LFS functions. */
 #define _LARGEFILE64_SOURCE 1
+#endif
+
+#ifndef _FILE_OFFSET_BITS
+#define _FILE_OFFSET_BITS 64
+#endif
 
 /*
  * Here we deal with the vagaries of various C compilers.  We assume that:
@@ -293,12 +302,21 @@ typedef int bool;
  * an enum in the (MacOS) Universal Interfaces. The only way around this is to escape
  * our own definitions wherever MacTypes.h is included.
  */
+#if defined(_MSC_VER) && _MSC_VER >= 1900
+/* VS 2014 defines bool already, but has it as _Bool (a 1 byte thing).
+ * We can't live with that. */
+#undef false
+#define false ((bool)0)
+#undef true
+#define true ((bool)1)
+#else
 #ifndef __MACOS__
 #undef false
 #define false ((bool)0)
 #undef true
 #define true ((bool)1)
 #endif /* __MACOS__ */
+#endif
 
 /*
  * Compilers disagree as to whether macros used in macro arguments
@@ -355,16 +373,6 @@ typedef const char *ptr_ord_t;
 #define round_up(v, m) ROUND_UP(v, m)
 #define round_down(v, m) ROUND_DOWN(v, m)
 
-/*
- * In pre-ANSI C, float parameters get converted to double.
- * However, if we pass a float to a function that has been declared
- * with a prototype, and the parameter has been declared as float,
- * the ANSI standard specifies that the parameter is left as float.
- * To avoid problems caused by missing prototypes,
- * we declare almost all float parameters as double.
- */
-typedef double floatp;
-
 /*
  * Because of C's strange insistence that ; is a terminator and not a
  * separator, compound statements {...} are not syntactically equivalent to
