@@ -415,7 +415,7 @@ OpenDevice (gx_device *pdev  /* Driver instance to open */)
       eprintf ("Error: No pDev in OpenDevice.\n");
       if (fDebugOutput) lprintf ("error was here.\n");
 
-      return gs_error_Fatal;
+      return_error(gs_error_Fatal);
    }
 
    if (!pDev->hmodOmni)
@@ -424,7 +424,7 @@ OpenDevice (gx_device *pdev  /* Driver instance to open */)
       eprintf ("Error: Could not load libomni.so\n");
       if (fDebugOutput) lprintf ("error was here.\n");
 
-      return gs_error_Fatal;
+      return_error(gs_error_Fatal);
    }
 
    if (!pDev->pcoreOmni->cDeviceName[0])
@@ -433,7 +433,7 @@ OpenDevice (gx_device *pdev  /* Driver instance to open */)
       eprintf ("Error: -sDeviceName=XXX was not defined\n");
       if (fDebugOutput) lprintf ("error was here.\n");
 
-      return gs_error_Fatal;
+      return_error(gs_error_Fatal);
    }
 
    pDev->iUseServer  = 0;
@@ -790,7 +790,7 @@ SetupDevice (gx_device *pgxdev, gs_param_list *plist)
                                             "Devicestring");
           if (pszDeviceLib)
           {
-              sprintf (pszDeviceLib, "%s%s", apszLibraryPaths[i], cOmnilib);
+              gs_sprintf (pszDeviceLib, "%s%s", apszLibraryPaths[i], cOmnilib);
 
               pDev->hmodOmni = g_module_open (pszDeviceLib, (GModuleFlags)0);
 
@@ -817,7 +817,7 @@ SetupDevice (gx_device *pgxdev, gs_param_list *plist)
                                                "Devicestring");
              if (pszDeviceLib)
              {
-                 sprintf (pszDeviceLib, "%s%s", apszLibraryPaths[i], cOmnilib);
+                 gs_sprintf (pszDeviceLib, "%s%s", apszLibraryPaths[i], cOmnilib);
 
                  pModule = g_module_open (pszDeviceLib, (GModuleFlags)0);
 
@@ -1053,7 +1053,7 @@ SetupDevice (gx_device *pgxdev, gs_param_list *plist)
 
             if (pszDeviceLib)
             {
-                sprintf (pszDeviceLib, "%s%s", apszLibraryPaths[i], cDialogName);
+                gs_sprintf (pszDeviceLib, "%s%s", apszLibraryPaths[i], cDialogName);
 
                 if (fDebugOutput) dprintf1 ("attempting to load - %s\n", pszDeviceLib);
 
@@ -1079,7 +1079,7 @@ SetupDevice (gx_device *pgxdev, gs_param_list *plist)
 
                if (pszDeviceLib)
                {
-                   sprintf (pszDeviceLib, "%s%s", apszLibraryPaths[i], cDialogName);
+                   gs_sprintf (pszDeviceLib, "%s%s", apszLibraryPaths[i], cDialogName);
 
                    pModule = g_module_open (pszDeviceLib, (GModuleFlags)0);
 
