@@ -71,6 +71,9 @@ jbig2_decode_generic_template0(Jbig2Ctx *ctx,
   printf("P4\n%d %d\n", GBW, GBH);
 #endif
 
+  if (GBW <= 0)
+    return 0;
+
   for (y = 0; y < GBH; y++)
     {
       uint32_t CONTEXT;
@@ -103,6 +106,8 @@ jbig2_decode_generic_template0(Jbig2Ctx *ctx,
 	      bool bit;
 
 	      bit = jbig2_arith_decode(as, &GB_stats[CONTEXT]);
+	      if (bit < 0)
+		return -1;
 	      result |= bit << (7 - x_minor);
 	      CONTEXT = ((CONTEXT & 0x7bf7) << 1) | bit |
 		((line_m1 >> (7 - x_minor)) & 0x10) |
@@ -159,6 +164,8 @@ jbig2_decode_generic_template0_unopt(Jbig2Ctx *ctx,
       CONTEXT |= jbig2_image_get_pixel(image, x + params->gbat[6],
 	y + params->gbat[7]) << 15;
       bit = jbig2_arith_decode(as, &GB_stats[CONTEXT]);
+      if (bit < 0)
+	return -1;
       jbig2_image_set_pixel(image, x, y, bit);
     }
   }
@@ -185,6 +192,9 @@ jbig2_decode_generic_template1(Jbig2Ctx *ctx,
   printf("P4\n%d %d\n", GBW, GBH);
 #endif
 
+  if (GBW <= 0)
+    return 0;
+
   for (y = 0; y < GBH; y++)
     {
       uint32_t CONTEXT;
@@ -217,6 +227,8 @@ jbig2_decode_generic_template1(Jbig2Ctx *ctx,
 	      bool bit;
 
 	      bit = jbig2_arith_decode(as, &GB_stats[CONTEXT]);
+	      if (bit < 0)
+		return -1;
 	      result |= bit << (7 - x_minor);
 	      CONTEXT = ((CONTEXT & 0xefb) << 1) | bit |
 		((line_m1 >> (8 - x_minor)) & 0x8) |
@@ -253,6 +265,9 @@ jbig2_decode_generic_template2(Jbig2Ctx *ctx,
   printf("P4\n%d %d\n", GBW, GBH);
 #endif
 
+  if (GBW <= 0)
+    return 0;
+
   for (y = 0; y < GBH; y++)
     {
       uint32_t CONTEXT;
@@ -285,6 +300,8 @@ jbig2_decode_generic_template2(Jbig2Ctx *ctx,
 	      bool bit;
 
 	      bit = jbig2_arith_decode(as, &GB_stats[CONTEXT]);
+	      if (bit < 0)
+		return -1;
 	      result |= bit << (7 - x_minor);
 	      CONTEXT = ((CONTEXT & 0x1bd) << 1) | bit |
 		((line_m1 >> (10 - x_minor)) & 0x4) |
@@ -321,6 +338,9 @@ jbig2_decode_generic_template2a(Jbig2Ctx *ctx,
   printf("P4\n%d %d\n", GBW, GBH);
 #endif
 
+  if (GBW <= 0)
+    return 0;
+
   for (y = 0; y < GBH; y++)
     {
       uint32_t CONTEXT;
@@ -353,6 +373,8 @@ jbig2_decode_generic_template2a(Jbig2Ctx *ctx,
 	      bool bit;
 
 	      bit = jbig2_arith_decode(as, &GB_stats[CONTEXT]);
+	      if (bit < 0)
+		return -1;
 	      result |= bit << (7 - x_minor);
 	      CONTEXT = ((CONTEXT & 0x1b9) << 1) | bit |
 		((line_m1 >> (10 - x_minor)) & 0x8) |
@@ -370,6 +392,7 @@ jbig2_decode_generic_template2a(Jbig2Ctx *ctx,
   return 0;
 }
 
+#ifdef UNUSED
 static int
 jbig2_decode_generic_template3(Jbig2Ctx *ctx,
 			       Jbig2Segment *segment,
@@ -390,6 +413,9 @@ jbig2_decode_generic_template3(Jbig2Ctx *ctx,
   printf("P4\n%d %d\n", GBW, GBH);
 #endif
 
+  if (GBW <= 0)
+    return 0;
+
   for (y = 0; y < GBH; y++)
     {
       uint32_t CONTEXT;
@@ -416,6 +442,8 @@ jbig2_decode_generic_template3(Jbig2Ctx *ctx,
 	      bool bit;
 
 	      bit = jbig2_arith_decode(as, &GB_stats[CONTEXT]);
+	      if (bit < 0)
+		return -1;
 	      result |= bit << (7 - x_minor);
 	      CONTEXT = ((CONTEXT & 0x1f7) << 1) | bit |
 		((line_m1 >> (10 - x_minor)) & 0x010);
@@ -430,6 +458,7 @@ jbig2_decode_generic_template3(Jbig2Ctx *ctx,
 
   return 0;
 }
+#endif
 
 static int
 jbig2_decode_generic_template3_unopt(Jbig2Ctx *ctx,
@@ -462,6 +491,8 @@ jbig2_decode_generic_template3_unopt(Jbig2Ctx *ctx,
       CONTEXT |= jbig2_image_get_pixel(image, x - 2, y - 1) << 8;
       CONTEXT |= jbig2_image_get_pixel(image, x - 3, y - 1) << 9;
       bit = jbig2_arith_decode(as, &GB_stats[CONTEXT]);
+      if (bit < 0)
+	return -1;
       jbig2_image_set_pixel(image, x, y, bit);
     }
   }
@@ -498,7 +529,10 @@ jbig2_decode_generic_template0_TPGDON(Jbig2Ctx *ctx,
 
   for (y = 0; y < GBH; y++)
   {
-    LTP ^= jbig2_arith_decode(as, &GB_stats[0x9B25]);
+    bit = jbig2_arith_decode(as, &GB_stats[0x9B25]);
+    if (bit < 0)
+      return -1;
+    LTP ^= bit;
     if (!LTP) {
       for (x = 0; x < GBW; x++) {
         CONTEXT  = jbig2_image_get_pixel(image, x - 1, y);
@@ -522,6 +556,8 @@ jbig2_decode_generic_template0_TPGDON(Jbig2Ctx *ctx,
         CONTEXT |= jbig2_image_get_pixel(image, x + params->gbat[6],
 					y + params->gbat[7]) << 15;
         bit = jbig2_arith_decode(as, &GB_stats[CONTEXT]);
+        if (bit < 0)
+	  return -1;
         jbig2_image_set_pixel(image, x, y, bit);
       }
     } else {
@@ -548,7 +584,10 @@ jbig2_decode_generic_template1_TPGDON(Jbig2Ctx *ctx,
   int LTP = 0;
 
   for (y = 0; y < GBH; y++) {
-    LTP ^= jbig2_arith_decode(as, &GB_stats[0x0795]);
+    bit = jbig2_arith_decode(as, &GB_stats[0x0795]);
+    if (bit < 0)
+      return -1;
+    LTP ^= bit;
     if (!LTP) {
       for (x = 0; x < GBW; x++) {
         CONTEXT  = jbig2_image_get_pixel(image, x - 1, y);
@@ -566,6 +605,8 @@ jbig2_decode_generic_template1_TPGDON(Jbig2Ctx *ctx,
         CONTEXT |= jbig2_image_get_pixel(image, x    , y - 2) << 11;
         CONTEXT |= jbig2_image_get_pixel(image, x - 1, y - 2) << 12;
         bit = jbig2_arith_decode(as, &GB_stats[CONTEXT]);
+        if (bit < 0)
+	  return -1;
         jbig2_image_set_pixel(image, x, y, bit);
       }
     } else {
@@ -592,7 +633,10 @@ jbig2_decode_generic_template2_TPGDON(Jbig2Ctx *ctx,
   int LTP = 0;
 
   for (y = 0; y < GBH; y++) {
-    LTP ^= jbig2_arith_decode(as, &GB_stats[0xE5]);
+    bit = jbig2_arith_decode(as, &GB_stats[0xE5]);
+    if (bit < 0)
+      return -1;
+    LTP ^= bit;
     if (!LTP) {
       for (x = 0; x < GBW; x++) {
         CONTEXT  = jbig2_image_get_pixel(image, x - 1, y);
@@ -607,6 +651,8 @@ jbig2_decode_generic_template2_TPGDON(Jbig2Ctx *ctx,
         CONTEXT |= jbig2_image_get_pixel(image, x    , y - 2) << 8;
         CONTEXT |= jbig2_image_get_pixel(image, x - 1, y - 2) << 9;
         bit = jbig2_arith_decode(as, &GB_stats[CONTEXT]);
+        if (bit < 0)
+	  return -1;
         jbig2_image_set_pixel(image, x, y, bit);
       }
     } else {
@@ -633,7 +679,10 @@ jbig2_decode_generic_template3_TPGDON(Jbig2Ctx *ctx,
   int LTP = 0;
 
   for (y = 0; y < GBH; y++) {
-    LTP ^= jbig2_arith_decode(as, &GB_stats[0x0195]);
+    bit = jbig2_arith_decode(as, &GB_stats[0x0195]);
+    if (bit < 0)
+      return -1;
+    LTP ^= bit;
     if (!LTP) {
       for (x = 0; x < GBW; x++) {
         CONTEXT  = jbig2_image_get_pixel(image, x - 1, y);
@@ -648,6 +697,8 @@ jbig2_decode_generic_template3_TPGDON(Jbig2Ctx *ctx,
         CONTEXT |= jbig2_image_get_pixel(image, x - 2, y - 1) << 8;
         CONTEXT |= jbig2_image_get_pixel(image, x - 3, y - 1) << 9;
         bit = jbig2_arith_decode(as, &GB_stats[CONTEXT]);
+        if (bit < 0)
+	  return -1;
         jbig2_image_set_pixel(image, x, y, bit);
       }
     } else {
@@ -711,6 +762,12 @@ jbig2_decode_generic_region(Jbig2Ctx *ctx,
 {
   const int8_t *gbat = params->gbat;
 
+  if (image->stride * image->height > (1 << 24) && segment->data_length < image->stride * image->height / 256) {
+    return jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+                       "region is far larger than data provided (%d << %d), aborting to prevent DOS",
+                       segment->data_length, image->stride * image->height);
+  }
+
   if (!params->MMR && params->TPGDON) 
      return jbig2_decode_generic_region_TPGDON(ctx, segment, params, 
 		as, image, GB_stats);
@@ -860,8 +917,12 @@ jbig2_immediate_generic_region(Jbig2Ctx *ctx, Jbig2Segment *segment,
 					 as, image, GB_stats);
     }
 
-  jbig2_page_add_result(ctx, &ctx->pages[ctx->current_page],
-			image, rsi.x, rsi.y, rsi.op);
+  if (code >= 0)
+    jbig2_page_add_result(ctx, &ctx->pages[ctx->current_page],
+			  image, rsi.x, rsi.y, rsi.op);
+  else
+    jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number,
+		"error while decoding immediate_generic_region");
 
 cleanup:
   jbig2_free(ctx->allocator, as);
