@@ -13,6 +13,8 @@
    CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
+/* prevent gp.h redefining sprintf */
+#define sprintf sprintf
 
 /* dwmainc.c */
 
@@ -74,7 +76,7 @@ gsdll_stderr(void *instance, const char *str, int len)
     return len;
 }
 
-#ifndef WINDOWS_NO_UNICODE
+#ifndef GS_NO_UTF8
 /* stdio functions - versions that translate to/from utf-8 */
 static int GSDLLCALL
 gsdll_stdin_utf8(void *instance, char *buf, int len)
@@ -85,14 +87,30 @@ gsdll_stdin_utf8(void *instance, char *buf, int len)
     int nret = 0;               /* number of bytes returned to caller */
     int i;
 
-    while (len) {
+    /* protect against caller passing invalid len */
+    while (len > 0) {
         while (len && nmore) {
             nmore--;
             *buf++ = 0x80 | ((thiswchar >> (6 * nmore)) & 0x3F), nret++;
             len--;
         }
         while (len) {
-            if (0 >= _read(fileno(stdin), buf, 1))
+            /* Previously the code here has always just checked for whether
+             * _read returns <= 0 to see whether we should exit. According
+             * to the docs -1 means error, 0 means EOF. Unfortunately,
+             * building using VS2015 there appears to be a bug in the 
+             * runtime, whereby a line with a single return on it (on an
+             * ANSI encoded Text file at least) causes a 0 return value.
+             * We hack around this by second guessing the code. We clear
+             * the buffer to start with, and (if we get a zero return
+             * value) we check to see if we have a '\n' in the buffer. If
+             * we do, then we disbelieve the return value. */
+            int num_read;
+            buf[0] = 0; /* Clear buffer */
+            num_read = _read(fileno(stdin), buf, 1);
+            if (num_read == 0 && buf[0] == 0x0a)
+                num_read = 1; /* Stupid VS2015 */
+            if (0 >= num_read)
                 return nret;
             nret++, buf++, len--;
             if (buf[-1] == '\n')
@@ -142,7 +160,8 @@ gsdll_utf8write(FILE *stdwr, const char *str, int len, WCHAR *thiswchar, int *nm
 {
     UINT consolecp = 0;
 
-    while (len) {
+    /* protect against caller passing invalid len */
+    while (len > 0) {
         const char *str0;
 
         /* write ASCII chars without translation */
@@ -487,7 +506,23 @@ display_callback display = {
 
 /*********************************************************************/
 
-#ifdef WINDOWS_NO_UNICODE
+typedef BOOL (SetProcessDPIAwareFn)(void);
+
+static void
+avoid_windows_scale(void)
+{
+    /* Fetch the function address and only call it if it is there; this keeps
+     * compatability with Windows < 8.1 */
+    HMODULE hUser32 = LoadLibrary(TEXT("user32.dll"));
+    SetProcessDPIAwareFn *ptr;
+
+    ptr = (SetProcessDPIAwareFn *)GetProcAddress(hUser32, "SetProcessDPIAware");
+    if (ptr != NULL)
+        ptr();
+    FreeLibrary(hUser32);
+}
+
+#ifdef GS_NO_UTF8
 int main(int argc, char *argv[])
 #else
 static int main_utf8(int argc, char *argv[])
@@ -502,6 +537,9 @@ static int main_utf8(int argc, char *argv[])
     char dformat[64];
     char ddpi[64];
 
+    /* Mark us as being 'system dpi aware' to avoid horrid scaling */
+    avoid_windows_scale();
+
     if (!_isatty(fileno(stdin)))
         _setmode(fileno(stdin), _O_BINARY);
     _setmode(fileno(stdout), _O_BINARY);
@@ -548,7 +586,7 @@ static int main_utf8(int argc, char *argv[])
     }
 #endif
 
-#ifdef WINDOWS_NO_UNICODE
+#ifdef GS_NO_UTF8
     gsdll.set_stdio(instance, gsdll_stdin, gsdll_stdout, gsdll_stderr);
 #else
     gsdll.set_stdio(instance,
@@ -601,16 +639,20 @@ static int main_utf8(int argc, char *argv[])
 
 #if defined(_MSC_VER) || defined(__BORLANDC__)
         __try {
+#endif
+#ifndef GS_NO_UTF8
+            code = gsdll.set_arg_encoding(instance, GS_ARG_ENCODING_UTF8);
+            if (code == 0)
 #endif
             code = gsdll.init_with_args(instance, nargc, nargv);
             if (code == 0)
                 code = gsdll.run_string(instance, start_string, 0, &exit_code);
             code1 = gsdll.exit(instance);
-            if (code == 0 || (code == e_Quit && code1 != 0))
+            if (code == 0 || (code == gs_error_Quit && code1 != 0))
                 code = code1;
 #if defined(_MSC_VER) || defined(__BORLANDC__)
         } __except(exception_code() == EXCEPTION_STACK_OVERFLOW) {
-            code = e_Fatal;
+            code = gs_error_Fatal;
             fprintf(stderr, "*** C stack overflow. Quiting...\n");
         }
 #endif
@@ -639,10 +681,10 @@ static int main_utf8(int argc, char *argv[])
     exit_status = 0;
     switch (code) {
         case 0:
-        case e_Info:
-        case e_Quit:
+        case gs_error_Info:
+        case gs_error_Quit:
             break;
-        case e_Fatal:
+        case gs_error_Fatal:
             exit_status = 1;
             break;
         default:
@@ -652,7 +694,7 @@ static int main_utf8(int argc, char *argv[])
     return exit_status;
 }
 
-#ifndef WINDOWS_NO_UNICODE
+#ifndef GS_NO_UTF8
 int wmain(int argc, wchar_t *argv[], wchar_t *envp[]) {
     /* Duplicate args as utf8 */
     char **nargv;
