@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: igstate.h 10940 2010-03-18 00:57:10Z robin $ */
+
 /* Interpreter graphics state definition */
 
 #ifndef igstate_INCLUDED
@@ -63,12 +65,12 @@ typedef struct ref_device_n_params_s {
 /* CIE transformation procedures */
 typedef struct ref_cie_procs_s {
     union {
-	ref DEFG;
-	ref DEF;
+        ref DEFG;
+        ref DEF;
     } PreDecode;
     union {
-	ref ABC;
-	ref A;
+        ref ABC;
+        ref A;
     } Decode;
     ref DecodeLMN;
 } ref_cie_procs;
@@ -90,20 +92,20 @@ typedef struct ref_separation_params_s {
 typedef struct ref_color_procs_s {
     ref_cie_procs cie;
     union {
-	ref_device_n_params device_n;
-	ref_separation_params separation;
-	ref index_proc;
+        ref_device_n_params device_n;
+        ref_separation_params separation;
+        ref index_proc;
     } special;
 } ref_color_procs;
 typedef struct ref_colorspace_s {
     ref array;			/* color space (array), only relevant if */
-				/* the current color space has parameters */
-				/* associated with it. */
+                                /* the current color space has parameters */
+                                /* associated with it. */
     ref_color_procs procs;	/* associated procedures/parameters, */
-				/* only relevant for DeviceN, CIE, */
-				/* Separation, Indexed/CIE, */
-				/* Indexed with procedure, or a Pattern */
-				/* with one of these. */
+                                /* only relevant for DeviceN, CIE, */
+                                /* Separation, Indexed/CIE, */
+                                /* Indexed with procedure, or a Pattern */
+                                /* with one of these. */
 } ref_colorspace;
 
 #ifndef int_remap_color_info_DEFINED
@@ -117,7 +119,7 @@ typedef struct int_gstate_s {
     /* executed more recently than sethalftone */
     /* (for this graphics context). */
     struct {
-	ref red, green, blue, gray;
+        ref red, green, blue, gray;
     } screen_procs,		/* halftone screen procedures */
           transfer_procs;	/* transfer procedures */
     ref black_generation;	/* (procedure) */
@@ -129,8 +131,8 @@ typedef struct int_gstate_s {
      */
     ref pattern[2];		/* pattern (dictionary) */
     struct {
-	ref dict;		/* CIE color rendering dictionary */
-	ref_cie_render_procs procs;	/* (see above) */
+        ref dict;		/* CIE color rendering dictionary */
+        ref_cie_render_procs procs;	/* (see above) */
     } colorrendering;
     /*
      * Use_cie_color tracks the UseCIEColor parameter of the page
@@ -159,7 +161,7 @@ typedef struct int_gstate_s {
     /*
      * Remap_color_info is used temporarily to communicate the need for
      * Pattern or DeviceNcolor remapping to the interpreter.  See
-     * e_RemapColor in ierrors.h.  The extra level of indirection through a
+     * gs_error_Remap_Color in gserrors.h.  The extra level of indirection through a
      * structure is needed because the gstate passed to the PaintProc is
      * different from the current gstate in the graphics state, and because
      * the DeviceN color being remapped is not necessarily the current color
