@@ -15,6 +15,8 @@
 
 
 /* Common support for interpreter front ends */
+
+
 #include "malloc_.h"
 #include "memory_.h"
 #include "string_.h"
@@ -24,7 +26,11 @@
 #include "gslib.h"
 #include "gsmatrix.h"           /* for gxdevice.h */
 #include "gsutil.h"             /* for bytes_compare */
+#include "gspaint.h"		/* for gs_erasepage */
 #include "gxdevice.h"
+#include "gxdevsop.h"		/* for gxdso_* enums */
+#include "gxclpage.h"
+#include "gdevprn.h"
 #include "gxalloc.h"
 #include "gxiodev.h"            /* for iodev struct */
 #include "gzstate.h"
@@ -53,7 +59,7 @@
 #include "iplugin.h"
 
 #ifdef PACIFY_VALGRIND
-#include <valgrind/helgrind.h>
+#include "valgrind.h"
 #endif
 
 /* ------ Exported data ------ */
@@ -131,6 +137,7 @@ gs_main_init0(gs_main_instance * minst, FILE * in, FILE * out, FILE * err,
               int max_lib_paths)
 {
     ref *paths;
+    ref *array;
 
     /* Do platform-dependent initialization. */
     /* We have to do this as the very first thing, */
@@ -149,18 +156,23 @@ gs_main_init0(gs_main_instance * minst, FILE * in, FILE * out, FILE * err,
 #   else
     /* plmain settings remain in effect */
 #   endif
-    gp_get_realtime(minst->base_time);
+    gp_get_usertime(minst->base_time);
 
     /* Initialize the file search paths. */
     paths = (ref *) gs_alloc_byte_array(minst->heap, max_lib_paths, sizeof(ref),
                                         "lib_path array");
     if (paths == 0) {
-        gs_lib_finit(1, e_VMerror, minst->heap);
-        return_error(e_VMerror);
+        gs_lib_finit(1, gs_error_VMerror, minst->heap);
+        return_error(gs_error_VMerror);
+    }
+    array = (ref *) gs_alloc_byte_array(minst->heap, max_lib_paths, sizeof(ref),
+                                        "lib_path array");
+    if (array == 0) {
+        gs_lib_finit(1, gs_error_VMerror, minst->heap);
+        return_error(gs_error_VMerror);
     }
     make_array(&minst->lib_path.container, avm_foreign, max_lib_paths,
-               (ref *) gs_alloc_byte_array(minst->heap, max_lib_paths, sizeof(ref),
-                                           "lib_path array"));
+               array);
     make_array(&minst->lib_path.list, avm_foreign | a_readonly, 0,
                minst->lib_path.container.value.refs);
     minst->lib_path.env = 0;
@@ -175,9 +187,6 @@ gs_main_init0(gs_main_instance * minst, FILE * in, FILE * out, FILE * err,
 int
 gs_main_init1(gs_main_instance * minst)
 {
-    i_ctx_t *i_ctx_p;
-    extern init_proc(gs_iodev_init);
-
     if (minst->init_done < 1) {
         gs_dual_memory_t idmem;
         int code =
@@ -196,7 +205,7 @@ gs_main_init1(gs_main_instance * minst)
                                         idmem.space_system);
 
             if (nt == 0)
-                return_error(e_VMerror);
+                return_error(gs_error_VMerror);
             mem->gs_lib_ctx->gs_name_table = nt;
             code = gs_register_struct_root(mem, NULL,
                                            (void **)&mem->gs_lib_ctx->gs_name_table,
@@ -210,8 +219,7 @@ gs_main_init1(gs_main_instance * minst)
         code = i_plugin_init(minst->i_ctx_p);
         if (code < 0)
             return code;
-        i_ctx_p = minst->i_ctx_p;
-        code = gs_iodev_init(imemory);
+        code = i_iodev_init(minst->i_ctx_p);
         if (code < 0)
             return code;
         minst->init_done = 1;
@@ -240,6 +248,8 @@ gs_main_interpret(gs_main_instance *minst, ref * pref, int user_errors,
 /* gcc wants prototypes for all external functions. */
 int gs_main_init2aux(gs_main_instance * minst);
 
+static const op_array_table empty_table = { { { 0 } } };
+
 /* This is an external function to work around      */
 /* a bug in gcc 4.5.1 optimizer. See bug 692684.    */
 int gs_main_init2aux(gs_main_instance * minst) {
@@ -249,6 +259,10 @@ int gs_main_init2aux(gs_main_instance * minst) {
         int code, exit_code;
         ref error_object, ifa;
 
+        /* Set up enough so that we can safely be garbage collected */
+        i_ctx_p->op_array_table_global = empty_table;
+        i_ctx_p->op_array_table_local = empty_table;
+
         code = zop_init(i_ctx_p);
         if (code < 0)
             return code;
@@ -303,6 +317,7 @@ gs_main_init2(gs_main_instance * minst)
 {
     i_ctx_t *i_ctx_p;
     int code = gs_main_init1(minst);
+    int initial_init_level = minst->init_done;
 
     if (code < 0)
         return code;
@@ -311,6 +326,38 @@ gs_main_init2(gs_main_instance * minst)
     if (code < 0)
        return code;
     i_ctx_p = minst->i_ctx_p; /* display_set_callback or run_string may change it */
+
+    /* Now process the initial saved-pages=... argument, if any as well as saved-pages-test */
+    if (initial_init_level < 2) {
+       gx_device *pdev = gs_currentdevice(minst->i_ctx_p->pgs);	/* get the current device */
+       gx_device_printer *ppdev = (gx_device_printer *)pdev;
+
+        if (minst->saved_pages_test_mode) {
+            if ((dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_saved_pages, NULL, 0) == 0)) {
+                /* no warning or error if saved-pages-test mode is used, just disable it */
+                minst->saved_pages_test_mode = false;  /* device doesn't support it */
+            } else {
+                if ((code = gx_saved_pages_param_process(ppdev, (byte *)"begin", 5)) < 0)
+                    return code;
+                if (code > 0)
+                    if ((code = gs_erasepage(minst->i_ctx_p->pgs)) < 0)
+                        return code;
+            }
+        } else if (minst->saved_pages_initial_arg != NULL) {
+            if (dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_saved_pages, NULL, 0) == 0) {
+                outprintf(minst->heap,
+                          "   --saved-pages not supported by the '%s' device.\n",
+                          pdev->dname);
+                return gs_error_Fatal;
+            }
+            code = gx_saved_pages_param_process(ppdev, (byte *)minst->saved_pages_initial_arg,
+                                                strlen(minst->saved_pages_initial_arg));
+            if (code > 0)
+                if ((code = gs_erasepage(minst->i_ctx_p->pgs)) < 0)
+                    return code;
+        }
+    }
+
     if (gs_debug_c(':'))
         print_resource_usage(minst, &gs_imemory, "Start");
     gp_readline_init(&minst->readline_data, imemory_system);
@@ -333,7 +380,7 @@ extend_path_list_container (gs_main_instance * minst, gs_file_path * pfp)
                                         "extend_path_list_container array");
 
     if (paths == 0) {
-        return_error(e_VMerror);
+        return_error(gs_error_VMerror);
     }
     make_array(&minst->lib_path.container, avm_foreign, len + LIB_PATH_EXTEND, paths);
     make_array(&minst->lib_path.list, avm_foreign | a_readonly, 0,
@@ -450,7 +497,12 @@ gs_main_set_lib_paths(gs_main_instance * minst)
         const char *dname = iodev->dname;
 
         if (dname && strlen(dname) == 5 && !memcmp("%rom%", dname, 5)) {
-            have_rom_device = 1;
+            struct stat pstat;
+            /* gs_error_unregistered means no usable romfs is available */
+            int code = iodev->procs.file_status((gx_io_device *)iodev, dname, &pstat);
+            if (code != gs_error_unregistered){
+                have_rom_device = 1;
+            }
             break;
         }
     }
@@ -502,7 +554,7 @@ gs_main_run_file_open(gs_main_instance * minst, const char *file_name, ref * pfr
         emprintf1(minst->heap,
                   "Can't find initialization file %s.\n",
                   file_name);
-        return_error(e_Fatal);
+        return_error(gs_error_Fatal);
     }
     r_set_attrs(pfref, a_execute + a_executable);
     return 0;
@@ -533,7 +585,7 @@ gs_run_init_file(gs_main_instance * minst, int *pexit_code, ref * perror_object)
                   "Initialization file %s does not begin with an integer.\n",
                   gs_init_file);
         *pexit_code = 255;
-        return_error(e_Fatal);
+        return_error(gs_error_Fatal);
     }
     *++osp = first_token;
     r_set_attrs(&ifile, a_executable);
@@ -562,7 +614,7 @@ gs_main_run_string_with_length(gs_main_instance * minst, const char *str,
         return code;
     code = gs_main_run_string_continue(minst, str, length, user_errors,
                                        pexit_code, perror_object);
-    if (code != e_NeedInput)
+    if (code != gs_error_NeedInput)
         return code;
     return gs_main_run_string_end(minst, user_errors,
                                   pexit_code, perror_object);
@@ -582,7 +634,7 @@ gs_main_run_string_begin(gs_main_instance * minst, int user_errors,
                       strlen(setup), (const byte *)setup);
     code = gs_main_interpret(minst, &rstr, user_errors, pexit_code,
                         perror_object);
-    return (code == e_NeedInput ? 0 : code == 0 ? e_Fatal : code);
+    return (code == gs_error_NeedInput ? 0 : code == 0 ? gs_error_Fatal : code);
 }
 /* Continue running a string with the option of suspending. */
 int
@@ -645,7 +697,7 @@ gs_push_integer(gs_main_instance * minst, long value)
 }
 
 int
-gs_push_real(gs_main_instance * minst, floatp value)
+gs_push_real(gs_main_instance * minst, double value)
 {
     ref vref;
 
@@ -668,7 +720,7 @@ static int
 pop_value(i_ctx_t *i_ctx_p, ref * pvalue)
 {
     if (!ref_stack_count(&o_stack))
-        return_error(e_stackunderflow);
+        return_error(gs_error_stackunderflow);
     *pvalue = *ref_stack_index(&o_stack, 0L);
     return 0;
 }
@@ -720,7 +772,7 @@ gs_pop_real(gs_main_instance * minst, float *result)
             *result = (float)(vref.value.intval);
             break;
         default:
-            return_error(e_typecheck);
+            return_error(gs_error_typecheck);
     }
     ref_stack_pop(&o_stack, 1);
     return 0;
@@ -746,7 +798,7 @@ gs_pop_string(gs_main_instance * minst, gs_string * result)
             result->size = r_size(&vref);
             break;
         default:
-            return_error(e_typecheck);
+            return_error(gs_error_typecheck);
     }
     ref_stack_pop(&o_stack, 1);
     return code;
@@ -821,6 +873,26 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
      * alloc_restore_all will close dynamically allocated devices.
      */
     tempnames = gs_main_tempnames(minst);
+
+#ifndef PSI_INCLUDED
+    /* We have to disable BGPrint before we call interp_reclaim() to prevent the
+     * parent rendering thread initialising for the next page, whilst we are
+     * removing objects it may want to access - for example, the I/O device table.
+     * We also have to mess with the BeginPage/EndPage procs so that we don't
+     * trigger a spurious extra page to be emitted.
+     */
+    if (minst->init_done >= 1) {
+        gs_main_run_string(minst,
+            "/systemdict .systemexec /begin .systemexec \
+            /BGPrint /GetDeviceParam .special_op \
+            {{ <</BeginPage {pop} /EndPage {pop pop //false } \
+              /BGPrint false /NumRenderingThreads 0>> setpagedevice} if} if \
+              serverdict /.jobsavelevel get 0 eq {/quit} {/stop} ifelse end \
+              .systemvar exec",
+            0 , &exit_code, &error_object);
+    }
+#endif
+
     /*
      * Close the "main" device, because it may need to write out
      * data before destruction. pdfwrite needs so.
@@ -835,7 +907,7 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
                 emprintf1(minst->heap,
                           "ERROR %d reclaiming the memory while the interpreter finalization.\n",
                           code);
-                return e_Fatal;
+                return gs_error_Fatal;
             }
             i_ctx_p = minst->i_ctx_p; /* interp_reclaim could change it. */
         }
@@ -849,8 +921,8 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
             /* deactivate the device just before we close it for the last time */
             gs_main_run_string(minst,
                 /* we need to do the 'quit' so we don't loop for input (double quit) */
-                ".uninstallpagedevice "
-                "serverdict /.jobsavelevel get 0 eq {/quit} {/stop} ifelse .systemvar exec",
+                ".uninstallpagedevice serverdict \
+                /.jobsavelevel get 0 eq {/quit} {/stop} ifelse .systemvar exec",
                 0 , &exit_code, &error_object);
             code = gs_closedevice(pdev);
             if (code < 0)
@@ -859,7 +931,7 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
                           code,
                           dname);
             rc_decrement(pdev, "gs_main_finit");                /* device might be freed */
-            if (exit_status == 0 || exit_status == e_Quit)
+            if (exit_status == 0 || exit_status == gs_error_Quit)
                 exit_status = code;
         }
 #endif
@@ -867,8 +939,10 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
     /* Flush stdout and stderr */
     if (minst->init_done >= 2)
       gs_main_run_string(minst,
-        "(%stdout) (w) file closefile (%stderr) (w) file closefile "
-        "serverdict /.jobsavelevel get 0 eq {/quit} {/stop} ifelse .systemvar exec",
+        "(%stdout) (w) file closefile (%stderr) (w) file closefile \
+        /systemdict .systemexec /begin .systemexec \
+        serverdict /.jobsavelevel get 0 eq {/quit} {/stop} ifelse .systemexec \
+        end",
         0 , &exit_code, &error_object);
     gp_readline_finit(minst->readline_data);
     i_ctx_p = minst->i_ctx_p;		/* get current interp context */
@@ -943,7 +1017,7 @@ print_resource_usage(const gs_main_instance * minst, gs_dual_memory_t * dmem,
     ulong allocated = 0, used = 0;
     long utime[2];
 
-    gp_get_realtime(utime);
+    gp_get_usertime(utime);
     {
         int i;
 
