@@ -15,6 +15,7 @@
 
 
 /* Command line parsing and dispatching */
+
 #include "ctype_.h"
 #include "memory_.h"
 #include "string_.h"
@@ -26,9 +27,13 @@
 #include "gscdefs.h"
 #include "gsmalloc.h"           /* for gs_malloc_limit */
 #include "gsmdebug.h"
+#include "gspaint.h"		/* for gs_erasepage */
 #include "gxdevice.h"
 #include "gxdevmem.h"
 #include "gsdevice.h"
+#include "gxdevsop.h"		/* for gxdso_* enums */
+#include "gxclpage.h"
+#include "gdevprn.h"
 #include "stream.h"
 #include "ierrors.h"
 #include "estack.h"
@@ -134,7 +139,9 @@ gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
     int code;
 
     arg_init(&args, (const char **)argv, argc,
-             gs_main_arg_fopen, (void *)minst);
+             gs_main_arg_fopen, (void *)minst,
+             minst->get_codepoint,
+             minst->heap);
     code = gs_main_init0(minst, 0, 0, 0, GS_MAX_LIB_DIRS);
     if (code < 0)
         return code;
@@ -167,24 +174,24 @@ gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
         bool helping = false;
 
         for (i = 1; i < argc; ++i)
-            if (!strcmp(argv[i], "--")) {
+            if (!arg_strcmp(&args, argv[i], "--")) {
                 /* A PostScript program will be interpreting all the */
                 /* remaining switches, so stop scanning. */
                 helping = false;
                 break;
-            } else if (!strcmp(argv[i], "--help")) {
+            } else if (!arg_strcmp(&args, argv[i], "--help")) {
                 print_help(minst);
                 helping = true;
-            } else if (!strcmp(argv[i], "--debug")) {
+            } else if (!arg_strcmp(&args, argv[i], "--debug")) {
                 gs_debug_flags_list(minst->heap);
                 helping = true;
-            } else if (!strcmp(argv[i], "--version")) {
+            } else if (!arg_strcmp(&args, argv[i], "--version")) {
                 print_version(minst);
                 puts(minst->heap, "");  /* \n */
                 helping = true;
             }
         if (helping)
-            return e_Info;
+            return gs_error_Info;
     }
     /* Execute files named in the command line, */
     /* processing options along the way. */
@@ -201,11 +208,16 @@ gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
             (char *)gs_alloc_bytes(minst->heap, len, "GS_OPTIONS");
 
             gp_getenv(GS_OPTIONS, opts, &len);  /* can't fail */
+#if defined(GS_NO_UTF8)
             if (arg_push_memory_string(&args, opts, false, minst->heap))
-                return e_Fatal;
+                return gs_error_Fatal;
+#else
+            if (arg_push_decoded_memory_string(&args, opts, false, true, minst->heap))
+                return gs_error_Fatal;
+#endif
         }
     }
-    while ((arg = arg_next(&args, &code, minst->heap)) != 0) {
+    while ((code = arg_next(&args, (const char **)&arg, minst->heap)) > 0) {
         switch (*arg) {
             case '-':
                 code = swproc(minst, arg, &args);
@@ -223,9 +235,22 @@ gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
                 }
                 break;
             default:
+                /* default is to treat this as a file name to be run */
                 code = argproc(minst, arg);
                 if (code < 0)
                     return code;
+                if (minst->saved_pages_test_mode) {
+                    gx_device *pdev;
+
+                    /* get the current device */
+                    pdev = gs_currentdevice(minst->i_ctx_p->pgs);
+                    if ((code = gx_saved_pages_param_process((gx_device_printer *)pdev,
+                               (byte *)"print normal flush", 18)) < 0)
+                        return code;
+                    if (code > 0)
+                        if ((code = gs_erasepage(minst->i_ctx_p->pgs)) < 0)
+                            return code;
+                }
         }
     }
     if (code < 0)
@@ -244,10 +269,24 @@ gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
         dmprintf(minst->heap, "\n");
     }
     if (!minst->run_start)
-        return e_Quit;
+        return gs_error_Quit;
     return code ;
 }
 
+/*
+ * Specify a decoding function
+ */
+void gs_main_inst_arg_decode(gs_main_instance * minst,
+                             gs_arg_get_codepoint *get_codepoint)
+{
+    minst->get_codepoint = get_codepoint;
+}
+
+gs_arg_get_codepoint *gs_main_inst_get_arg_decode(gs_main_instance * minst)
+{
+    return minst->get_codepoint;
+}
+
 /*
  * Run the 'start' procedure (after processing the command line).
  */
@@ -293,6 +332,18 @@ run_stdin:
             code = run_string(minst, ".runstdin", runFlush);
             if (code < 0)
                 return code;
+            if (minst->saved_pages_test_mode) {
+                gx_device *pdev;
+
+                /* get the current device */
+                pdev = gs_currentdevice(minst->i_ctx_p->pgs);
+                if ((code = gx_saved_pages_param_process((gx_device_printer *)pdev,
+                           (byte *)"print normal flush", 18)) < 0)
+                    return code;
+                if (code > 0)
+                    if ((code = gs_erasepage(minst->i_ctx_p->pgs)) < 0)
+                        return code;
+            }
             break;
         case '-':               /* run with command line args */
             if (strncmp(arg, "debug=", 6) == 0) {
@@ -300,6 +351,42 @@ run_stdin:
                 if (code < 0)
                     return code;
                 break;
+            } else if (strncmp(arg, "saved-pages=", 12) == 0) {
+                gx_device *pdev;
+                gx_device_printer *ppdev;
+
+                /* If init2 not yet done, just save the argument for processing then */
+                if (minst->init_done < 2) {
+                    if (minst->saved_pages_initial_arg == NULL) {
+                        /* Tuck the parameters away for later when init2 is done */
+                        minst->saved_pages_initial_arg = (char *)arg+12;
+                    } else {
+                        outprintf(minst->heap,
+                                  "   Only one --saved-pages=... command allowed before processing input\n");
+                        arg_finit(pal);
+                        return gs_error_Fatal;
+                    }
+                } else {
+                    /* get the current device */
+                    pdev = gs_currentdevice(minst->i_ctx_p->pgs);
+                    if (dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_saved_pages, NULL, 0) == 0) {
+                        outprintf(minst->heap,
+                                  "   --saved-pages not supported by the '%s' device.\n",
+                                  pdev->dname);
+                        arg_finit(pal);
+                        return gs_error_Fatal;
+                    }
+                    ppdev = (gx_device_printer *)pdev;
+                    code = gx_saved_pages_param_process(ppdev, (byte *)arg+12, strlen(arg+12));
+                    if (code > 0)
+                        if ((code = gs_erasepage(minst->i_ctx_p->pgs)) < 0)
+                            return code;
+                }
+                break;
+            /* The following code is only to allow regression testing of saved-pages */
+            } else if (strncmp(arg, "saved-pages-test", 16) == 0) {
+                minst->saved_pages_test_mode = true;
+                break;
             }
             /* FALLTHROUGH */
         case '+':
@@ -307,24 +394,26 @@ run_stdin:
             /* FALLTHROUGH */
         case '@':               /* ditto with @-expansion */
             {
-                const char *psarg = arg_next(pal, &code, minst->heap);
+                char *psarg;
+
+                code = arg_next(pal, (const char **)&psarg, minst->heap);
 
                 if (code < 0)
-                    return e_Fatal;
+                    return gs_error_Fatal;
                 if (psarg == 0) {
                     outprintf(minst->heap, "Usage: gs ... -%c file.ps arg1 ... argn\n", sw);
                     arg_finit(pal);
-                    return e_Fatal;
+                    return gs_error_Fatal;
                 }
                 psarg = arg_copy(psarg, minst->heap);
                 if (psarg == NULL)
-                    code = e_Fatal;
+                    code = gs_error_Fatal;
                 else
                     code = gs_main_init2(minst);
                 if (code >= 0)
                     code = run_string(minst, "userdict/ARGUMENTS[", 0);
                 if (code >= 0)
-                    while ((arg = arg_next(pal, &code, minst->heap)) != 0) {
+                    while ((code = arg_next(pal, (const char **)&arg, minst->heap)) > 0) {
                         code = runarg(minst, "", arg, "", runInit);
                         if (code < 0)
                             break;
@@ -333,7 +422,7 @@ run_stdin:
                     code = runarg(minst, "]put", psarg, ".runfile", runInit | runFlush);
                 arg_free((char *)psarg, minst->heap);
                 if (code >= 0)
-                    code = e_Quit;
+                    code = gs_error_Quit;
                 
                 return code;
             }
@@ -347,7 +436,7 @@ run_stdin:
                     break;
                 default:
                     puts(minst->heap, "-A may only be followed by -");
-                    return e_Fatal;
+                    return gs_error_Fatal;
             }
             break;
         case 'B':               /* set run_string buffer size */
@@ -362,7 +451,7 @@ run_stdin:
                     outprintf(minst->heap,
                               "-B must be followed by - or size between 1 and %u\n",
                               MAX_BUFFERED_SIZE);
-                    return e_Fatal;
+                    return gs_error_Fatal;
                 }
                 minst->run_buffer_size = bsize;
             }
@@ -375,7 +464,7 @@ run_stdin:
                 if (code < 0)
                     return code;
                 pal->expand_ats = false;
-                while ((arg = arg_next(pal, &code, minst->heap)) != 0) {
+                while ((code = arg_next(pal, (const char **)&arg, minst->heap)) > 0) {
                     if (arg[0] == '@' ||
                         (arg[0] == '-' && !isdigit((unsigned char)arg[1]))
                         )
@@ -385,11 +474,11 @@ run_stdin:
                         return code;
                 }
                 if (code < 0)
-                    return e_Fatal;
+                    return gs_error_Fatal;
                 if (arg != 0) {
                     char *p = arg_copy(arg, minst->heap);
                     if (p == NULL)
-                        return e_Fatal;
+                        return gs_error_Fatal;
                     arg_push_string(pal, p, true);
                 }
                 pal->expand_ats = ats;
@@ -405,7 +494,7 @@ run_stdin:
                     break;
                 default:
                     puts(minst->heap, "-E may only be followed by -");
-                    return e_Fatal;
+                    return gs_error_Fatal;
             }
             break;
         case 'f':               /* run file of arbitrary name */
@@ -413,12 +502,25 @@ run_stdin:
                 code = argproc(minst, arg);
                 if (code < 0)
                     return code;
+                if (minst->saved_pages_test_mode) {
+                    gx_device *pdev;
+
+                    /* get the current device */
+                    pdev = gs_currentdevice(minst->i_ctx_p->pgs);
+                        return code;
+                    if ((code = gx_saved_pages_param_process((gx_device_printer *)pdev,
+                               (byte *)"print normal flush", 18)) < 0)
+                        return code;
+                    if (code > 0)
+                        if ((code = gs_erasepage(minst->i_ctx_p->pgs)) < 0)
+                            return code;
+                }
             }
             break;
         case 'F':               /* run file with buffer_size = 1 */
             if (!*arg) {
                 puts(minst->heap, "-F requires a file name");
-                return e_Fatal;
+                return gs_error_Fatal;
             } {
                 uint bsize = minst->run_buffer_size;
 
@@ -427,6 +529,18 @@ run_stdin:
                 minst->run_buffer_size = bsize;
                 if (code < 0)
                     return code;
+                if (minst->saved_pages_test_mode) {
+                    gx_device *pdev;
+
+                    /* get the current device */
+                    pdev = gs_currentdevice(minst->i_ctx_p->pgs);
+                    if ((code = gx_saved_pages_param_process((gx_device_printer *)pdev,
+                               (byte *)"print normal flush", 18)) < 0)
+                        return code;
+                    if (code > 0)
+                        if ((code = gs_erasepage(minst->i_ctx_p->pgs)) < 0)
+                            return code;
+                }
             }
             break;
         case 'g':               /* define device geometry */
@@ -438,7 +552,7 @@ run_stdin:
                     return code;
                 if (sscanf((const char *)arg, "%ldx%ld", &width, &height) != 2) {
                     puts(minst->heap, "-g must be followed by <width>x<height>");
-                    return e_Fatal;
+                    return gs_error_Fatal;
                 }
                 make_int(&value, width);
                 initial_enter_name("DEVICEWIDTH", &value);
@@ -450,22 +564,22 @@ run_stdin:
         case 'h':               /* print help */
         case '?':               /* ditto */
             print_help(minst);
-            return e_Info;      /* show usage info on exit */
+            return gs_error_Info;      /* show usage info on exit */
         case 'I':               /* specify search path */
             {
                 const char *path;
 
                 if (arg[0] == 0) {
-                    path = arg_next(pal, &code, minst->heap);
+                    code = arg_next(pal, (const char **)&path, minst->heap);
                     if (code < 0)
                         return code;
                 } else
                     path = arg;
                 if (path == NULL)
-                    return e_Fatal;
+                    return gs_error_Fatal;
                 path = arg_copy(path, minst->heap);
                 if (path == NULL)
-                    return e_Fatal;
+                    return gs_error_Fatal;
                 gs_main_add_lib_path(minst, path);
             }
             break;
@@ -478,7 +592,7 @@ run_stdin:
                 if (msize <= 0 || msize > max_long >> 10) {
                     outprintf(minst->heap, "-K<numK> must have 1 <= numK <= %ld\n",
                               max_long >> 10);
-                    return e_Fatal;
+                    return gs_error_Fatal;
                 }
                 rawheap->limit = msize << 10;
             }
@@ -491,7 +605,7 @@ run_stdin:
 #if ARCH_INTS_ARE_SHORT
                 if (msize <= 0 || msize >= 64) {
                     puts(minst->heap, "-M must be between 1 and 63");
-                    return e_Fatal;
+                    return gs_error_Fatal;
                 }
 #endif
                 minst->memory_chunk_size = msize << 10;
@@ -505,7 +619,7 @@ run_stdin:
 #if ARCH_INTS_ARE_SHORT
                 if (nsize < 2 || nsize > 64) {
                     puts(minst->heap, "-N must be between 2 and 64");
-                    return e_Fatal;
+                    return gs_error_Fatal;
                 }
 #endif
                 minst->name_table_size = (ulong) nsize << 10;
@@ -519,7 +633,7 @@ run_stdin:
                 int len;
 
                 if (arg[0] == 0) {
-                    adef = arg_next(pal, &code, minst->heap);
+                    code = arg_next(pal, (const char **)&adef, minst->heap);
                     if (code < 0)
                         return code;
                 } else
@@ -528,6 +642,8 @@ run_stdin:
                     return code;
                 len = strlen(adef);
                 str = (char *)gs_alloc_bytes(minst->heap, (uint)len, "-o");
+                if (str == NULL)
+                    return gs_error_VMerror;
                 memcpy(str, adef, len);
                 make_const_string(&value, a_readonly | avm_foreign,
                                   len, (const byte *)str);
@@ -543,7 +659,7 @@ run_stdin:
                 minst->search_here_first = false;
             else {
                 puts(minst->heap, "Only -P or -P- is allowed.");
-                return e_Fatal;
+                return gs_error_Fatal;
             }
             break;
         case 'q':               /* quiet startup */
@@ -561,7 +677,7 @@ run_stdin:
                 switch (sscanf((const char *)arg, "%fx%f", &xres, &yres)) {
                     default:
                         puts(minst->heap, "-r must be followed by <res> or <xres>x<yres>");
-                        return e_Fatal;
+                        return gs_error_Fatal;
                     case 1:     /* -r<res> */
                         yres = xres;
                     case 2:     /* -r<xres>x<yres> */
@@ -584,7 +700,7 @@ run_stdin:
                 ref value;
 
                 if (adef == NULL)
-                    return e_Fatal;
+                    return gs_error_Fatal;
                 eqp = strchr(adef, '=');
 
                 if (eqp == NULL)
@@ -595,7 +711,7 @@ run_stdin:
                     return code;
                 if (eqp == adef) {
                     puts(minst->heap, "Usage: -dNAME, -dNAME=TOKEN, -sNAME=STRING");
-                    return e_Fatal;
+                    return gs_error_Fatal;
                 }
                 if (eqp == NULL) {
                     if (isd)
@@ -651,7 +767,7 @@ run_stdin:
                             code = gs_scan_token(minst->i_ctx_p, &value, &state);
                             if (code) {
                                 outprintf(minst->heap, "Invalid value for option -d%s, -dNAME= must be followed by a valid token\n", arg);
-                                return e_Fatal;
+                                return gs_error_Fatal;
                             }
                             if (r_has_type_attrs(&value, t_name,
                                                  a_executable)) {
@@ -670,7 +786,7 @@ run_stdin:
                                     make_false(&value);
                                 else {
                                     outprintf(minst->heap, "Invalid value for option -d%s, use -sNAME= to define string constants\n", arg);
-                                    return e_Fatal;
+                                    return gs_error_Fatal;
                                 }
                             }
                         }
@@ -682,7 +798,7 @@ run_stdin:
 
                         if (str == 0) {
                             lprintf("Out of memory!\n");
-                            return e_Fatal;
+                            return gs_error_Fatal;
                         }
                         memcpy(str, eqp, len);
                         make_const_string(&value,
@@ -703,7 +819,7 @@ run_stdin:
         case 'u':               /* undefine name */
             if (!*arg) {
                 puts(minst->heap, "-u requires a name to undefine.");
-                return e_Fatal;
+                return gs_error_Fatal;
             }
             if ((code = gs_main_init1(minst)) < 0)
                 return code;
@@ -711,7 +827,7 @@ run_stdin:
             break;
         case 'v':               /* print revision */
             print_revision(minst);
-            return e_Info;
+            return gs_error_Info;
 /*#ifdef DEBUG */
             /*
              * Here we provide a place for inserting debugging code that can be
@@ -736,7 +852,7 @@ run_stdin:
                 run_x("== flush\n");
                 stop_x();
             }
-            return e_Quit;
+            return gs_error_Quit;
 /*#endif */
         case 'Z':
             set_debug_flags(arg, gs_debug);
@@ -798,7 +914,7 @@ run_buffered(gs_main_instance * minst, const char *arg)
 
     if (in == 0) {
         outprintf(minst->heap, "Unable to open %s for reading", arg);
-        return_error(e_invalidfileaccess);
+        return_error(gs_error_invalidfileaccess);
     }
     code = gs_main_init2(minst);
     if (code < 0) {
@@ -811,15 +927,15 @@ run_buffered(gs_main_instance * minst, const char *arg)
         char buf[MAX_BUFFERED_SIZE];
         int count;
 
-        code = e_NeedInput;
+        code = gs_error_NeedInput;
         while ((count = fread(buf, 1, minst->run_buffer_size, in)) > 0) {
             code = gs_main_run_string_continue(minst, buf, count,
                                                minst->user_errors,
                                                &exit_code, &error_object);
-            if (code != e_NeedInput)
+            if (code != gs_error_NeedInput)
                 break;
         }
-        if (code == e_NeedInput) {
+        if (code == gs_error_NeedInput) {
             code = gs_main_run_string_end(minst, minst->user_errors,
                                           &exit_code, &error_object);
         }
@@ -846,7 +962,7 @@ runarg(gs_main_instance * minst, const char *pre, const char *arg,
     line = (char *)gs_alloc_bytes(minst->heap, len, "runarg");
     if (line == 0) {
         lprintf("Out of memory!\n");
-        return_error(e_VMerror);
+        return_error(gs_error_VMerror);
     }
     strcpy(line, pre);
     esc_strcat(line, arg);
@@ -876,10 +992,10 @@ run_finish(gs_main_instance *minst, int code, int exit_code,
            ref * perror_object)
 {
     switch (code) {
-        case e_Quit:
+        case gs_error_Quit:
         case 0:
             break;
-        case e_Fatal:
+        case gs_error_Fatal:
             emprintf1(minst->heap,
                       "Unrecoverable error, exit code %d\n",
                       exit_code);
@@ -923,7 +1039,7 @@ try_stdout_redirect(gs_main_instance * minst,
             }
             else if ((minst->heap->gs_lib_ctx->fstdout2 =
                       gp_fopen(filename, "w")) == (FILE *)NULL)
-                return_error(e_invalidfileaccess);
+                return_error(gs_error_invalidfileaccess);
             minst->heap->gs_lib_ctx->stdout_is_redirected = 1;
         }
         return 0;
@@ -959,6 +1075,11 @@ static const char help_devices[] = "Available devices:";
 static const char help_default_device[] = "Default output device:";
 static const char help_emulators[] = "Input formats:";
 static const char help_paths[] = "Search path:";
+#ifdef HAVE_FONTCONFIG
+static const char help_fontconfig[] = "Ghostscript is also using fontconfig to search for font files\n";
+#else
+static const char help_fontconfig[] = "";
+#endif
 
 extern_gx_io_device_table();
 
@@ -979,7 +1100,12 @@ print_help(gs_main_instance * minst)
         const char *dname = iodev->dname;
 
         if (dname && strlen(dname) == 5 && !memcmp("%rom%", dname, 5)) {
-            have_rom_device = 1;
+            struct stat pstat;
+            /* gs_error_unregistered means no usable romfs is available */
+            int code = iodev->procs.file_status((gx_io_device *)iodev, dname, &pstat);
+            if (code != gs_error_unregistered){
+                have_rom_device = 1;
+            }
             break;
         }
     }
@@ -1128,6 +1254,7 @@ print_paths(gs_main_instance * minst)
         }
     }
     outprintf(minst->heap, "\n");
+    outprintf(minst->heap, "%s", help_fontconfig);
 }
 
 /* Print the help trailer. */
