@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Interpreter's interface to garbage collector */
 #include "ghost.h"
 #include "ierrors.h"
@@ -31,7 +33,7 @@
 extern void ialloc_gc_prepare(gs_ref_memory_t *);
 
 /* Forward references */
-static void gs_vmreclaim(gs_dual_memory_t *, bool);
+static int gs_vmreclaim(gs_dual_memory_t *, bool);
 
 /* Initialize the GC hook in the allocator. */
 static int ireclaim(gs_dual_memory_t *, int);
@@ -49,6 +51,7 @@ ireclaim(gs_dual_memory_t * dmem, int space)
 {
     bool global;
     gs_ref_memory_t *mem;
+    int code;
 
     if (space < 0) {
         /* Determine which allocator exceeded the limit. */
@@ -67,12 +70,14 @@ ireclaim(gs_dual_memory_t * dmem, int space)
     } else {
         mem = dmem->spaces_indexed[space >> r_space_shift];
     }
-    if_debug3('0', "[0]GC called, space=%d, requestor=%d, requested=%ld\n",
-              space, mem->space, (long)mem->gc_status.requested);
+    if_debug3m('0', (gs_memory_t *)mem, "[0]GC called, space=%d, requestor=%d, requested=%ld\n",
+               space, mem->space, (long)mem->gc_status.requested);
     global = mem->space != avm_local;
     /* Since dmem may move, reset the request now. */
     ialloc_reset_requested(dmem);
-    gs_vmreclaim(dmem, global);
+    code = gs_vmreclaim(dmem, global);
+    if (code < 0)
+        return code;
     ialloc_set_limit(mem);
     if (space < 0) {
         gs_memory_status_t stats;
@@ -88,14 +93,14 @@ ireclaim(gs_dual_memory_t * dmem, int space)
         }
         if (allocated >= mem->gc_status.max_vm) {
             /* We can't satisfy this request within max_vm. */
-            return_error(e_VMerror);
+            return_error(gs_error_VMerror);
         }
     }
     return 0;
 }
 
 /* Interpreter entry to garbage collector. */
-static void
+static int
 gs_vmreclaim(gs_dual_memory_t *dmem, bool global)
 {
     /* HACK: we know the gs_dual_memory_t is embedded in a context state. */
@@ -107,6 +112,9 @@ gs_vmreclaim(gs_dual_memory_t *dmem, bool global)
     gs_ref_memory_t *mem;
     int nmem, i;
 
+    if (code < 0)
+        return code;
+
     memories[0] = dmem->space_system;
     memories[1] = mem = dmem->space_global;
     nmem = 2;
@@ -176,7 +184,7 @@ gs_vmreclaim(gs_dual_memory_t *dmem, bool global)
        we would lose those allocations when the chunks were opened */
 
     code = context_state_load(i_ctx_p);
-
+    return code;
 }
 
 /* ------ Initialization procedure ------ */
