@@ -205,10 +205,10 @@ static RELOC_PTRS_WITH(change_reloc_ptrs, alloc_change_t *ptr)
                igc_reloc_ref_ptr from RELOC_REF_PTR_VAR.
                Calling igc_reloc_ref_ptr_nocheck instead. */
             {	/* A sanity check. */
-                obj_header_t *pre = (obj_header_t *)ptr->where - 1, *pre1 = 0;
+                obj_header_t *pre = (obj_header_t *)ptr->where - 1;
 
                 if (pre->o_type != &st_refs)
-                    pre1->o_type = 0; /* issue a segfault. */
+                    gs_abort(gcst->heap);
             }
             if (ptr->where != 0 && !gcst->relocating_untraced)
                 ptr->where = igc_reloc_ref_ptr_nocheck(ptr->where, gcst);
@@ -313,9 +313,14 @@ static alloc_save_t *alloc_save_space(gs_ref_memory_t *mem,
 static void
 alloc_free_save(gs_ref_memory_t *mem, alloc_save_t *save, const char *scn)
 {
+    gs_ref_memory_t save_mem;
+    save_mem = mem->saved->state;
     gs_free_object((gs_memory_t *)mem, save, scn);
     /* Free any inner chunk structures.  This is the easiest way to do it. */
     restore_free(mem);
+    /* Restore the 'saved' state - this pulls our object off the linked
+     * list of states. Without this we hit a SEGV in the gc later. */
+    *mem = save_mem;
 }
 int
 alloc_save_state(gs_dual_memory_t * dmem, void *cdata, ulong *psid)
@@ -331,11 +336,15 @@ alloc_save_state(gs_dual_memory_t * dmem, void *cdata, ulong *psid)
     alloc_save_t *lsave = alloc_save_space(lmem, dmem, sid);
 
     if (lsave == 0 || (global && gsave == 0)) {
+        /* Only 1 of lsave or gsave will have been allocated, but
+         * nevertheless (in case things change in future), we free
+         * lsave, then gsave, so they 'pop' correctly when restoring
+         * the mem->saved states. */
         if (lsave != 0)
             alloc_free_save(lmem, lsave, "alloc_save_state(local save)");
         if (gsave != 0)
             alloc_free_save(gmem, gsave, "alloc_save_state(global save)");
-        return 0;
+        return_error(gs_error_VMerror);
     }
     if (gsave != 0) {
         gsave->client_data = 0;
@@ -435,6 +444,7 @@ alloc_save_space(gs_ref_memory_t * mem, gs_dual_memory_t * dmem, ulong sid)
         *mem = save_mem;
         return 0;
     }
+    save->client_data = NULL;
     save->state = save_mem;
     save->spaces = dmem->spaces;
     save->restore_names = (name_memory(mem) == (gs_memory_t *) mem);
@@ -515,7 +525,7 @@ alloc_save_change_alloc(gs_ref_memory_t *mem, client_name_t cname, alloc_change_
     cp = gs_alloc_struct((gs_memory_t *)mem, alloc_change_t,
                          &st_alloc_change, "alloc_save_change");
     if (cp == 0)
-        return_error(e_VMerror);
+        return_error(gs_error_VMerror);
     cp->next = mem->changes;
     cp->where = 0;
     cp->offset = AC_OFFSET_ALLOCATED;
@@ -1072,6 +1082,7 @@ combine_space(gs_ref_memory_t * mem)
             {
                 obj_header_t *hp = (obj_header_t *) outer->cbot;
 
+                hp->o_pad = 0;
                 hp->o_alone = 0;
                 hp->o_size = (char *)(cp->chead + 1)
                     - (char *)(hp + 1);
@@ -1184,7 +1195,7 @@ mark_allocated(void *obj, bool to_new, uint *psize)
     if (pre->o_type != &st_refs) {
         /* Must not happen. */
         if_debug0('u', "Wrong object type when expected a ref.\n");
-        return_error(e_Fatal);
+        return_error(gs_error_Fatal);
     }
     /* We know that every block of refs ends with */
     /* a full-size ref, so we only need the end check */
