@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Token scanner for Ghostscript interpreter */
 #include "ghost.h"
 #include "memory_.h"
@@ -86,7 +88,7 @@ dynamic_free(da_ptr pda)
 }
 
 /* Resize a dynamic string. */
-/* If the allocation fails, return e_VMerror; otherwise, return 0. */
+/* If the allocation fails, return gs_error_VMerror; otherwise, return 0. */
 static int
 dynamic_resize(da_ptr pda, uint new_size)
 {
@@ -99,11 +101,11 @@ dynamic_resize(da_ptr pda, uint new_size)
         base = gs_resize_string(mem, pda->base, old_size,
                                 new_size, "scanner");
         if (base == 0)
-            return_error(e_VMerror);
+            return_error(gs_error_VMerror);
     } else {                    /* switching from static to dynamic */
         base = gs_alloc_string(mem, new_size, "scanner");
         if (base == 0)
-            return_error(e_VMerror);
+            return_error(gs_error_VMerror);
         memcpy(base, pda->base, min(old_size, new_size));
         pda->is_dynamic = true;
     }
@@ -128,7 +130,7 @@ dynamic_grow(da_ptr pda, byte * next, uint max_size)
 
     pda->next = next;
     if (old_size >= max_size)
-        return_error(e_limitcheck);
+        return_error(gs_error_limitcheck);
     while ((code = dynamic_resize(pda, new_size)) < 0 &&
            new_size > old_size
         ) {                     /* Try trimming down the requested new size. */
@@ -295,7 +297,7 @@ gs_scan_handle_refill(i_ctx_t *i_ctx_p, scanner_state * sstate,
 
     if (s->end_status == EOFC) {
         /* More data needed, but none available, so this is a syntax error. */
-        return_error(e_syntaxerror);
+        return_error(gs_error_syntaxerror);
     }
     status = s_process_read_buf(s);
     if (sbufavailable(s) > avail)
@@ -308,7 +310,7 @@ gs_scan_handle_refill(i_ctx_t *i_ctx_p, scanner_state * sstate,
             /* Let the caller find this out. */
             return 0;
         case ERRC:
-            return_error(e_ioerror);
+            return_error(gs_error_ioerror);
         case INTC:
         case CALLC:
             {
@@ -320,7 +322,7 @@ gs_scan_handle_refill(i_ctx_t *i_ctx_p, scanner_state * sstate,
                         ialloc_struct(scanner_state_dynamic, &st_scanner_state_dynamic,
                                       "gs_scan_handle_refill");
                     if (pstate == 0)
-                        return_error(e_VMerror);
+                        return_error(gs_error_VMerror);
                     ((scanner_state_dynamic *)pstate)->mem = imemory;
                     *pstate = *sstate;
                 } else
@@ -353,9 +355,9 @@ scan_comment(i_ctx_t *i_ctx_p, ref *pref, scanner_state *pstate,
         /* Process as a DSC comment if requested. */
 #ifdef DEBUG
         if (gs_debug_c('%')) {
-            dlprintf2("[%%%%%s%c]", sstr, (len >= 3 ? '+' : '-'));
-            debug_print_string(base, len);
-            dputs("\n");
+            dmlprintf2(imemory, "[%%%%%s%c]", sstr, (len >= 3 ? '+' : '-'));
+            debug_print_string(imemory, base, len);
+            dmputs(imemory, "\n");
         }
 #endif
         if (gs_scan_dsc_proc != NULL) {
@@ -371,9 +373,9 @@ scan_comment(i_ctx_t *i_ctx_p, ref *pref, scanner_state *pstate,
 #ifdef DEBUG
     else {
         if (gs_debug_c('%')) {
-            dlprintf2("[%% %s%c]", sstr, (len >= 2 ? '+' : '-'));
-            debug_print_string(base, len);
-            dputs("\n");
+            dmlprintf2(imemory, "[%% %s%c]", sstr, (len >= 2 ? '+' : '-'));
+            debug_print_string(imemory, base, len);
+            dmputs(imemory, "\n");
         }
     }
 #endif
@@ -391,7 +393,7 @@ scan_comment(i_ctx_t *i_ctx_p, ref *pref, scanner_state *pstate,
         byte *cstr = ialloc_string(len, "scan_comment");
 
         if (cstr == 0)
-            return_error(e_VMerror);
+            return_error(gs_error_VMerror);
         memcpy(cstr, base, len);
         make_string(pref, a_all | icurrent_space, len, cstr);
     }
@@ -411,7 +413,7 @@ gs_scan_string_token_options(i_ctx_t *i_ctx_p, ref * pstr, ref * pref,
     int code;
 
     if (!r_has_attr(pstr, a_read))
-        return_error(e_invalidaccess);
+        return_error(gs_error_invalidaccess);
     s_init(s, NULL);
     sread_string(s, pstr->value.bytes, r_size(pstr));
     gs_scanner_init_stream_options(&state, s, options | SCAN_FROM_STRING);
@@ -430,7 +432,7 @@ gs_scan_string_token_options(i_ctx_t *i_ctx_p, ref * pstr, ref * pref,
             }
             break;
         case scan_Refill:       /* error */
-            code = gs_note_error(e_syntaxerror);
+            code = gs_note_error(gs_error_syntaxerror);
         case scan_EOF:
             break;
     }
@@ -444,7 +446,7 @@ gs_scan_string_token_options(i_ctx_t *i_ctx_p, ref * pstr, ref * pref,
  * >0 for special situations (see iscan.h).
  * If the token required a terminating character (i.e., was a name or
  * number) and the next character was whitespace, read and discard
- * that character.  Note that the state is relevant for e_VMerror
+ * that character.  Note that the state is relevant for gs_error_VMerror
  * as well as for scan_Refill.
  */
 int
@@ -546,7 +548,7 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
             case scanning_string:
                 goto cont_string;
             default:
-                return_error(e_Fatal);
+                return_error(gs_error_Fatal);
         }
     }
     /* Fetch any state variables that are relevant even if */
@@ -562,7 +564,7 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
      *      If pstack != 0, myref = osp, and *osp is a valid slot.
      */
   top:c = scan_getc();
-    if_debug1('S', (c >= 32 && c <= 126 ? "`%c'" : c >= 0 ? "`\\%03o'" : "`%d'"), c);
+    if_debug1m('S', imemory, (c >= 32 && c <= 126 ? "`%c'" : c >= 0 ? "`\\%03o'" : "`%d'"), c);
     switch (c) {
         case ' ':
         case '\f':
@@ -593,11 +595,12 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
                     case '~':
                         s_A85D_init_inline(&sstate.s_ss.a85d);
                         sstate.s_ss.st.templat = &s_A85D_template;
+                        sstate.s_ss.a85d.require_eod = true;
                         goto str;
                 }
                 scan_putback();
             }
-            s_AXD_init_inline(&sstate.s_ss.axd);
+            (void)s_AXD_init_inline(&sstate.s_ss.axd);
             sstate.s_ss.st.templat = &s_AXD_template;
           str:scan_end_inline();
             dynamic_init(&da, imemory);
@@ -621,7 +624,7 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
                                     scan_type = scanning_string;
                                     goto suspend;
                                 } else
-                                    sreturn(e_syntaxerror);
+                                    sreturn(gs_error_syntaxerror);
                             }
                             break;
                         }
@@ -630,7 +633,7 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
                     case 1:
                         if (!check_only) {
                             retcode = dynamic_grow(&da, da.next, max_string_size);
-                            if (retcode == e_VMerror) {
+                            if (retcode == gs_error_VMerror) {
                                 scan_type = scanning_string;
                                 goto suspend;
                             } else if (retcode < 0)
@@ -644,7 +647,7 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
             switch (status) {
                 default:
                     /*case ERRC: */
-                    sreturn(e_syntaxerror);
+                    sreturn(gs_error_syntaxerror);
                 case INTC:
                 case CALLC:
                     scan_type = scanning_string;
@@ -677,8 +680,8 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
             }
             make_int(osp, pstack);
             pstack = ref_stack_count_inline(&o_stack);
-            if_debug3('S', "[S{]d=%d, s=%d->%d\n",
-                      pdepth, (int)osp->value.intval, pstack);
+            if_debug3m('S', imemory, "[S{]d=%d, s=%d->%d\n",
+                       pdepth, (int)osp->value.intval, pstack);
             goto snext;
         case '>':
             if (scan_enable_level2) {
@@ -689,20 +692,22 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
             }
             /* falls through */
         case ')':
-            sreturn(e_syntaxerror);
+            sreturn(gs_error_syntaxerror);
         case '}':
             if (pstack == 0)
-                sreturn(e_syntaxerror);
+                sreturn(gs_error_syntaxerror);
             osp--;
             {
                 uint size = ref_stack_count_inline(&o_stack) - pstack;
                 ref arr;
 
-                if_debug4('S', "[S}]d=%d, s=%d->%d, c=%d\n",
-                          pdepth, pstack,
-                          (pstack == pdepth ? 0 :
-                          ref_stack_index(&o_stack, size)->value.intval),
-                          size + pstack);
+                if_debug4m('S', imemory, "[S}]d=%"PRIu32", s=%"PRIu32"->%"PRIpsint", c=%"PRIu32"\n",
+                           pdepth, pstack,
+                           (pstack == pdepth ? 0 :
+                           ref_stack_index(&o_stack, size)->value.intval),
+                           size + pstack);
+                if (size > max_array_size)
+                    sreturn(gs_error_limitcheck);
                 myref = (pstack == pdepth ? pref : &arr);
                 if (check_only) {
                     make_empty_array(myref, 0);
@@ -848,7 +853,7 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
                                      */
                                     goto end_comment;
                                 default:
-                                    sreturn(e_syntaxerror);
+                                    sreturn(gs_error_syntaxerror);
                             }
                         if (daptr < comment_line + max_comment_line)
                             *daptr++ = c;
@@ -870,12 +875,12 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
             if (pstack != 0) {
                 if (check_only)
                     goto pause;
-                sreturn(e_syntaxerror);
+                sreturn(gs_error_syntaxerror);
             }
             retcode = scan_EOF;
             break;
         case ERRC:
-            sreturn(e_ioerror);
+            sreturn(gs_error_ioerror);
 
             /* Check for a Level 2 funny name (<< or >>). */
             /* c is '<' or '>'.  We already did an ensure2. */
@@ -890,7 +895,7 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
                 }
                 scan_putback();
             }
-            sreturn(e_syntaxerror);
+            sreturn(gs_error_syntaxerror);
 
             /* Handle separately the names that might be a number. */
         case '0':
@@ -931,6 +936,14 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
             goto nr;
         case '-':
             sign = -1;
+            if(i_ctx_p->scanner_options & SCAN_PDF_INV_NUM) {
+                do {
+                    if (*(sptr + 1) == '-') {
+                        sptr++;
+                    } else
+                        break;
+                } while (1);
+            }
             goto nr;
 
             /* Check for a binary object */
@@ -1064,7 +1077,7 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
             retcode = dynamic_grow(&da, da.limit, name_max_string);
             if (retcode < 0) {
                 dynamic_save(&da);
-                if (retcode != e_VMerror)
+                if (retcode != gs_error_VMerror)
                     sreturn(retcode);
                 scan_type = scanning_name;
                 goto pause_ret;
@@ -1079,7 +1092,7 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
                                            name_max_string);
                     if (retcode < 0) {
                         dynamic_save(&da);
-                        if (retcode != e_VMerror)
+                        if (retcode != gs_error_VMerror)
                             sreturn(retcode);
                         scan_putback();
                         scan_type = scanning_name;
@@ -1116,7 +1129,7 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
                         case CALLC:
                             goto pause_name;
                         case ERRC:
-                            sreturn(e_ioerror);
+                            sreturn(gs_error_ioerror);
                         case EOFC:
                             break;
                     }
@@ -1130,11 +1143,11 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
                 if (retcode == 1) {
                     ref_mark_new(myref);
                     retcode = 0;
-                } else if (retcode != e_syntaxerror) {
+                } else if (retcode != gs_error_syntaxerror) {
                     dynamic_free(&da);
                     if (name_type == 2)
-                        sreturn(e_syntaxerror);
-                    break;      /* might be e_limitcheck */
+                        sreturn(gs_error_syntaxerror);
+                    break;      /* might be gs_error_limitcheck */
                 }
             }
             if (da.is_dynamic) {        /* We've already allocated the string on the heap. */
@@ -1159,7 +1172,7 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
             }
             /* Done scanning.  Check for preceding /'s. */
             if (retcode < 0) {
-                if (retcode != e_VMerror)
+                if (retcode != gs_error_VMerror)
                     sreturn(retcode);
                 if (!da.is_dynamic) {
                     da.next = daptr;
@@ -1185,12 +1198,12 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
                             ref_assign(&sstate.s_error.object, myref);
                             r_set_attrs(&sstate.s_error.object,
                                 a_executable); /* Adobe compatibility */
-                            sreturn(e_undefined);
+                            sreturn(gs_error_undefined);
                         }
                         if (pstack != 0 &&
                             r_space(pvalue) > ialloc_space(idmemory)
                             )
-                            sreturn(e_invalidaccess);
+                            sreturn(gs_error_invalidaccess);
                         ref_assign_new(myref, pvalue);
                     }
             }
@@ -1199,7 +1212,7 @@ gs_scan_token(i_ctx_t *i_ctx_p, ref * pref, scanner_state * pstate)
         scan_end_inline();
         pstate->s_error = sstate.s_error;
         if (pstack != 0) {
-            if (retcode == e_undefined)
+            if (retcode == gs_error_undefined)
                 *pref = *osp;   /* return undefined name as error token */
             ref_stack_pop(&o_stack,
                           ref_stack_count(&o_stack) - (pdepth - 1));
