@@ -1,17 +1,19 @@
-/* Copyright (C) 2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Operators for general-purpose algorithms. For now, only sorting. */
 #include "ghost.h"
 #include "gserrors.h"
@@ -57,7 +59,7 @@ zsort(i_ctx_t *i_ctx_p)
     /* Check operands for type and access */
     /* we can sort only writable [and unpacked] arrays */
     if (r_type(&op[-1]) == t_mixedarray || r_type(&op[-1]) == t_shortarray)
-        return_error(e_invalidaccess);
+        return_error(gs_error_invalidaccess);
     check_write_type(op[-1], t_array);
     /* the predicate must be an executable array/ string/ name/ [pseudo-]operator */
     if (!r_has_attr(&op[0], a_executable))
@@ -68,7 +70,7 @@ zsort(i_ctx_t *i_ctx_p)
         case t_shortarray:
         case t_string:
             if (!r_has_attr(&op[0], a_execute))
-                return_error(e_invalidaccess);
+                return_error(gs_error_invalidaccess);
             break;
         case t_name:
         case t_operator:
@@ -154,7 +156,7 @@ H4:	    i = j;
             }
         case 5:
 /*H5_cont:*/if (!r_has_type(&op[0], t_boolean))
-                return_error(e_typecheck);
+                return_error(gs_error_typecheck);
             if (op[0].value.boolval)
                 j++;
 H6:	    H = 6;
@@ -164,7 +166,7 @@ H6:	    H = 6;
             break;
         case 6:
 /*H6_cont:*/if (!r_has_type(&op[0], t_boolean))
-                return_error(e_typecheck);
+                return_error(gs_error_typecheck);
             if (op[0].value.boolval) {
 /* H7: */  	ref_assign_old(&arry, &Rn[i], &Rn[j], ".sort(H7)");
                 goto H4;
