@@ -53,7 +53,7 @@
 #include "iplugin.h"
 #include "store.h"
 #include "gzstate.h"
-#include "gdevpsf.h"
+/* #include "gdevpsf.h" */
 #include "stream.h"             /* for files.h */
 #include "gscrypt1.h"
 #include "gxfcid.h"
@@ -96,18 +96,20 @@ sfnts_next_elem(sfnts_reader *r)
 
     if (r->error)
         return;
-    r->index++;
-    code = array_get(r->memory, r->sfnts, r->index, &s);
-    if (code == e_rangecheck) {
-        r->error |= 2;
-    }
-    else if (code < 0) {
-        r->error |= 1;
-    }
-    if (r->error)
-        return;
-    r->p = s.value.const_bytes;
-    r->length = r_size(&s) & ~(uint) 1; /* See Adobe Technical Note # 5012, section 4.2. */
+    do {
+    	r->index++;
+    	code = array_get(r->memory, r->sfnts, r->index, &s);
+    	if (code == gs_error_rangecheck) {
+    		r->error |= 2;
+    	}
+    	else if (code < 0) {
+    		r->error |= 1;
+    	}
+    	if (r->error)
+    		return;
+    	r->p = s.value.const_bytes;
+    	r->length = r_size(&s) & ~(uint) 1; /* See Adobe Technical Note # 5012, section 4.2. */
+    } while (r->length == 0);
     r->offset = 0;
 }
 
@@ -583,11 +585,11 @@ FAPI_FF_get_word(gs_fapi_font *ff, gs_fapi_font_feature var_id, int index)
                             length += r_size(&string) + 1;
                             break;
                         case t_real:
-                            sprintf(Buffer, "%f", Element.value.realval);
+                            gs_sprintf(Buffer, "%f", Element.value.realval);
                             length += strlen(Buffer) + 1;
                             break;
                         case t_integer:
-                            sprintf(Buffer, "%"PRIpsint, Element.value.intval);
+                            gs_sprintf(Buffer, "%"PRIpsint, Element.value.intval);
                             length += strlen(Buffer) + 1;
                             break;
                         case t_operator:
@@ -823,12 +825,12 @@ FAPI_FF_get_proc(gs_fapi_font *ff, gs_fapi_font_feature var_id, int index,
                             ptr += r_size(&string);
                             break;
                         case t_real:
-                            sprintf(Buf, "%f", Element.value.realval);
+                            gs_sprintf(Buf, "%f", Element.value.realval);
                             strcpy(ptr, Buf);
                             ptr += strlen(Buf);
                             break;
                         case t_integer:
-                            sprintf(Buf, "%"PRIpsint, Element.value.intval);
+                            gs_sprintf(Buf, "%"PRIpsint, Element.value.intval);
                             strcpy(ptr, Buf);
                             ptr += strlen(Buf);
                             break;
@@ -976,6 +978,39 @@ FAPI_FF_get_charstring(gs_fapi_font *ff, int index, byte *buf,
     return (r_size(&eltp[1]));
 }
 
+static int
+sfnt_get_sfnt_length(ref *pdr, ulong *len)
+{
+    int code = 0;
+    ref *sfnts, sfnt_elem;
+    const gs_memory_t *mem = dict_mem(pdr->value.pdict);
+
+    *len = 0;
+    if (r_type(pdr) != t_dictionary ||
+        dict_find_string(pdr, "sfnts", &sfnts) <= 0) {
+        code = gs_error_invalidfont;
+    }
+    else {
+        if (r_type(sfnts) != t_array && r_type(sfnts) != t_string) {
+            code = gs_error_invalidfont;
+        }
+        else {
+            if (r_type(sfnts) == t_string) {
+                *len = r_size(sfnts);
+            }
+            else {
+                int i;
+                for (i = 0; i < r_size(sfnts); i++) {
+                    code = array_get(mem, sfnts, i, &sfnt_elem);
+                    if (code < 0) break;
+                    *len += r_size(&sfnt_elem);
+                }
+            }
+        }
+    }
+    return code;
+}
+
 static bool
 sfnt_get_glyph_offset(ref *pdr, gs_font_type42 *pfont42, int index,
                       ulong *offset0)
@@ -1150,9 +1185,19 @@ FAPI_FF_get_glyph(gs_fapi_font *ff, int char_code, byte *buf,
     else {                      /* type 42 */
         const byte *data_ptr;
         int l = ff->get_glyphdirectory_data(ff, char_code, &data_ptr);
+        ref *render_notdef_ref;
+        bool render_notdef = true;
+
+        if (dict_find_string(pdr, ".render_notdef", &render_notdef_ref) > 0
+            && r_has_type(render_notdef_ref, t_boolean)) {
+            render_notdef = render_notdef_ref->value.boolval;
+        }
+        else {
+            render_notdef = i_ctx_p->RenderTTNotdef;
+        }
 
         /* We should only render the TT notdef if we've been told to - logic lifted from zchar42.c */
-        if (!i_ctx_p->RenderTTNotdef
+        if (!render_notdef
             &&
             ((ff->char_data_len == 7
               && strncmp((const char *)ff->char_data, ".notdef", 7) == 0)
@@ -1172,14 +1217,38 @@ FAPI_FF_get_glyph(gs_fapi_font *ff, int char_code, byte *buf,
                            min(glyph_length, buf_length) /* safety */ );
             }
             else {
-                gs_font_type42 *pfont42 =
-                    (gs_font_type42 *) ff->client_font_data;
+                gs_font_type42 *pfont42 = (gs_font_type42 *) ff->client_font_data;
                 ulong offset0, length_read;
-                bool error =
-                    sfnt_get_glyph_offset(pdr, pfont42, char_code, &offset0);
+                bool error = sfnt_get_glyph_offset(pdr, pfont42, char_code, &offset0);
 
-                glyph_length =
-                    (error ? -1 : pfont42->data.len_glyphs[char_code]);
+                if (error != 0) {
+                    glyph_length = -1;
+                }
+                else if (pfont42->data.len_glyphs) {
+                    glyph_length = pfont42->data.len_glyphs[char_code];
+                }
+                else {
+                    ulong noffs;
+                    /* If we haven't got a len_glyphs array, try using the offset of the next glyph offset
+                     * to work out the length
+                     */
+                    error = sfnt_get_glyph_offset(pdr, pfont42, char_code + 1, &noffs);
+                    if (error == 0) {
+                        glyph_length = noffs - offset0;
+                    }
+                    else {
+                        /* And if we can't get the next glyph offset, use the end of the sfnt data
+                         * to work out the length.
+                         */
+                        int code = sfnt_get_sfnt_length(pdr, &noffs);
+                        if (code < 0) {
+                            glyph_length = -1;
+                        }
+                        else {
+                            glyph_length = noffs - offset0;
+                        }
+                    }
+                }
 
                 if (buf != 0 && !error) {
                     sfnts_reader r;
@@ -1264,6 +1333,7 @@ static const gs_fapi_font ps_ff_stub = {
     false,                      /* is_outline_font */
     false,                      /* is_mtx_skipped */
     false,                      /* is_vertical */
+    false,                      /* metrics_only */
     {{-1, -1}},                 /* ttf_cmap_req */
     0,                          /* client_ctx_p */
     0,                          /* client_font_data */
@@ -1298,7 +1368,7 @@ FAPI_get_xlatmap(i_ctx_t *i_ctx_p, char **xlatmap)
     if ((code = dict_find_string(systemdict, ".xlatmap", &pref)) < 0)
         return code;
     if (r_type(pref) != t_string)
-        return_error(e_typecheck);
+        return_error(gs_error_typecheck);
     *xlatmap = (char *)pref->value.bytes;
     /*  Note : this supposes that xlatmap doesn't move in virtual memory.
        Garbager must not be called while plugin executes get_scaled_font, get_decodingID.
@@ -1315,7 +1385,7 @@ renderer_retcode(gs_memory_t *mem, gs_fapi_server *I, gs_fapi_retcode rc)
     emprintf2(mem,
               "Error: Font Renderer Plugin ( %s ) return code = %d\n",
               I->ig.d->subtype, rc);
-    return rc < 0 ? rc : e_invalidfont;
+    return rc < 0 ? rc : gs_error_invalidfont;
 }
 
 /* <server name>/<null> object .FAPIavailable bool */
@@ -1332,7 +1402,7 @@ zFAPIavailable(i_ctx_t *i_ctx_p)
         serv_name =
             (char *) ref_to_string(&name_ref, imemory, "zFAPIavailable");
         if (!serv_name) {
-            return_error(e_VMerror);
+            return_error(gs_error_VMerror);
         }
     }
 
@@ -1397,7 +1467,7 @@ FAPI_refine_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font *pfont,
         if (dict_find_string(op, "FontBBox", &v) > 0) {
             if (!r_has_type(v, t_array) && !r_has_type(v, t_shortarray)
                 && !r_has_type(v, t_mixedarray))
-                return_error(e_invalidfont);
+                return_error(gs_error_invalidfont);
             make_real(&mat[0], pbfont->FontBBox.p.x);
             make_real(&mat[1], pbfont->FontBBox.p.y);
             make_real(&mat[2], pbfont->FontBBox.q.x);
@@ -1446,11 +1516,11 @@ FAPI_refine_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font *pfont,
 
             if (dict_find_string(pdr, "CIDSystemInfo", &CIDSystemInfo) <= 0
                 || !r_has_type(CIDSystemInfo, t_dictionary))
-                return_error(e_invalidfont);
+                return_error(gs_error_invalidfont);
 
             if (dict_find_string(CIDSystemInfo, "Ordering", &Ordering) <= 0
                 || !r_has_type(Ordering, t_string)) {
-                return_error(e_invalidfont);
+                return_error(gs_error_invalidfont);
             }
 
             ordering_length =
@@ -1520,13 +1590,13 @@ zFAPIrebuildfont(i_ctx_t *i_ctx_p)
     if (pbfont->FAPI == NULL) {
         if (dict_find_string(op - 1, "FAPI", &v) <= 0
             || !r_has_type(v, t_name))
-            return_error(e_invalidfont);
+            return_error(gs_error_invalidfont);
         obj_string_data(imemory, v, &pchars, &len);
         len = min(len, sizeof(FAPI_ID) - 1);
         strncpy((char *)FAPI_ID, (const char *)pchars, len);
         FAPI_ID[len] = 0;
 
-        gs_fapi_set_servers_client_data(imemory, NULL, i_ctx_p);
+        gs_fapi_set_servers_client_data(imemory, &ps_ff_stub, i_ctx_p);
 
         code =
             gs_fapi_find_server(imemory, FAPI_ID,
@@ -1534,7 +1604,7 @@ zFAPIrebuildfont(i_ctx_t *i_ctx_p)
                                 (gs_fapi_get_server_param_callback)
                                 ps_get_server_param);
         if (!pbfont->FAPI || code < 0) {
-            return_error(e_invalidfont);
+            return_error(gs_error_invalidfont);
         }
     }
 
@@ -1766,7 +1836,7 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
                         return code;
             }
             else {
-                return_error(e_invalidfont);
+                return_error(gs_error_invalidfont);
             }
         }
         else {
@@ -1781,6 +1851,8 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
          * it to the renderer for a disk based font. But the metrics dictionary may have
          * been constructed using the extended name....
          */
+        if (!r_has_type(&char_name, t_name))
+            return_error(gs_error_invalidfont);
         name_string_ref(imemory, &char_name, &cname_str);
         enc_char_name->data = cname_str.value.bytes;
         enc_char_name->size = r_size(&cname_str);
@@ -1792,16 +1864,16 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
         if (font_file_path != NULL) {
             ref *Decoding, *TT_cmap, *SubstNWP;
             ref src_type, dst_type;
-            uint c;
+            uint c = 0;
 
             is_glyph_index = true;
 
             if (dict_find_string(pdr, "Decoding", &Decoding) <= 0
                 || !r_has_type(Decoding, t_dictionary))
-                return_error(e_invalidfont);
+                return_error(gs_error_invalidfont);
             if (dict_find_string(pdr, "SubstNWP", &SubstNWP) <= 0
                 || !r_has_type(SubstNWP, t_array))
-                return_error(e_invalidfont);
+                return_error(gs_error_invalidfont);
             if (dict_find_string(pdr, "TT_cmap", &TT_cmap) <= 0
                 || !r_has_type(TT_cmap, t_dictionary)) {
                 ref *DecodingArray, char_code, char_code1, ih;
@@ -1812,7 +1884,7 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
                 if (dict_find(Decoding, &ih, &DecodingArray) <= 0
                     || !r_has_type(DecodingArray, t_array)
                     || array_get(imemory, DecodingArray, i, &char_code) < 0) {
-                    return_error(e_invalidfont);
+                    return_error(gs_error_invalidfont);
                 }
 
                 /* Check the Decoding entry */
@@ -1825,13 +1897,13 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
                     n = r_size(DecodingArray);
                 }
                 else {
-                    return_error(e_invalidfont);
+                    return_error(gs_error_invalidfont);
                 }
 
                 for (; n--; i++) {
                     if (array_get(imemory, DecodingArray, i, &char_code1) < 0
                         || !r_has_type(&char_code1, t_integer)) {
-                        return_error(e_invalidfont);
+                        return_error(gs_error_invalidfont);
                     }
 
                     c = char_code1.value.intval;
@@ -1843,15 +1915,33 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
             else {
                 ref *CIDSystemInfo;
                 ref *Ordering;
+                ref *fdict, *CMapDict, *CMapName, CMapNameStr;
+                char *cmapnm = NULL;
+                int cmapnmlen = 0;
+                /* leave off the -H or -V */
+                const char * const utfcmap = "Identity-UTF16";
+                int utfcmaplen = strlen(utfcmap);
 
-                /* We only have to lookup the char code if we're *not* using an identity ordering */
-                if (dict_find_string(pdr, "CIDSystemInfo", &CIDSystemInfo) >= 0
+                fdict = pfont_dict(gs_rootfont(igs));
+                code = dict_find_string(fdict, "CMap", &CMapDict);
+                if (code >= 0 && r_has_type(CMapDict, t_dictionary)) {
+                    code = dict_find_string(CMapDict, "CMapName", &CMapName);
+                    if (code >= 0 && r_has_type(CMapName, t_name)) {
+                        name_string_ref(imemory, CMapName, &CMapNameStr);
+                        cmapnm = (char *)CMapNameStr.value.bytes;
+                        cmapnmlen = r_size(&CMapNameStr);
+                    }
+                }
+                /* We only have to lookup the char code if we're *not* using an identity ordering 
+                   with the exception of Identity-UTF16 which is a different beast altogether */
+                if ((cmapnmlen > 0 && !strncmp(cmapnm, utfcmap, cmapnmlen > utfcmaplen ? utfcmaplen : cmapnmlen))
+                    || (dict_find_string(pdr, "CIDSystemInfo", &CIDSystemInfo) >= 0
                     && r_has_type(CIDSystemInfo, t_dictionary)
                     && dict_find_string(CIDSystemInfo, "Ordering",
                                         &Ordering) >= 0
                     && r_has_type(Ordering, t_string)
                     && strncmp((const char *)Ordering->value.bytes,
-                               "Identity", 8) != 0) {
+                               "Identity", 8) != 0)) {
 
                     if ((code =
                          cid_to_TT_charcode(imemory, Decoding, TT_cmap,
@@ -1901,22 +1991,33 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
                 if (r_has_type(CIDMap, t_array)) {
 
                     /* Too big for single string, so its an array of 2 strings */
-                    code =
-                        string_array_access_proc(pbfont->memory, CIDMap, 1,
+                    code = string_array_access_proc(pbfont->memory, CIDMap, 1,
                                                  client_char_code * gdb, gdb,
                                                  NULL, NULL,
                                                  (const byte **)&Map);
                 }
                 else {
-                    if (CIDMap->tas.rsize < c_code * gdb) {
+                    if (CIDMap->tas.rsize <= c_code * gdb) {
                         c_code = 0;
                     }
                     Map = &CIDMap->value.bytes[c_code * gdb];
                 }
                 cr->char_codes[0] = 0;
 
-                for (i = 0; i < gdb; i++) {
-                    cr->char_codes[0] = (cr->char_codes[0] << 8) + Map[i];
+                if (code >= 0) {
+                    for (i = 0; i < gdb; i++) {
+                        cr->char_codes[0] = (cr->char_codes[0] << 8) + Map[i];
+                    }
+                }
+                else {
+                    ref *cstr, *refcode;
+                    code = dict_find_string(pdr, "CharStrings", &cstr);
+                    if (code > 0) {
+                        code = dict_find_string(cstr, ".notdef", &refcode);
+                        if (code > 0) {
+                            cr->char_codes[0] = refcode->value.intval;
+                        }
+                    }
                 }
             }
             else
@@ -1929,7 +2030,7 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
 
         if (dict_find_string(pdr, "CharStrings", &CharStrings) <= 0
             || !r_has_type(CharStrings, t_dictionary))
-            return_error(e_invalidfont);
+            return_error(gs_error_invalidfont);
         if ((dict_find(CharStrings, &char_name, &glyph_index) < 0)
             || r_has_type(glyph_index, t_null)) {
 #ifdef DEBUG
@@ -1981,7 +2082,7 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
             ref_assign(esp, glyph_index);
             return o_push_estack;
 #else
-            return (e_invalidfont);
+            return (gs_error_invalidfont);
 #endif
         }
         cr->is_glyph_index = true;
@@ -2014,7 +2115,7 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
                     (uint) array_find(imemory, Encoding, &glyph);
             }
             else
-                return_error(e_invalidfont);
+                return_error(gs_error_invalidfont);
         }
     }
     else {                      /* a non-embedded font, i.e. a disk font */
@@ -2057,14 +2158,14 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
 
                         cr->char_codes_count = r_size(char_code);
                         if (cr->char_codes_count > count_of(cr->char_codes))
-                            code = gs_note_error(e_rangecheck);
+                            code = gs_note_error(gs_error_rangecheck);
                         if (code >= 0) {
                             for (i = 0; i < cr->char_codes_count; i++) {
                                 code = array_get(imemory, char_code, i, &v);
                                 if (code < 0)
                                     break;
                                 if (!r_has_type(char_code, t_integer)) {
-                                    code = gs_note_error(e_rangecheck);
+                                    code = gs_note_error(gs_error_rangecheck);
                                     break;
                                 }
                                 cr->char_codes[i] = v.value.intval;
@@ -2072,7 +2173,7 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
                         }
                     }
                     else {
-                        code = gs_note_error(e_rangecheck);
+                        code = gs_note_error(gs_error_rangecheck);
                     }
                     if (code < 0) {
                         char buf[16];
@@ -2086,7 +2187,7 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
                         emprintf1(imemory,
                                   "Wrong decoding entry for the character '%s'.\n",
                                   buf);
-                        return_error(e_rangecheck);
+                        return_error(gs_error_rangecheck);
                     }
                 }
             }
@@ -2141,6 +2242,8 @@ FAPI_char(i_ctx_t *i_ctx_p, bool bBuildGlyph, ref *charstring)
         /* initialise the FAPI font, this includes language specific stuff */
         I->ff = ps_ff_stub;
 
+        I->client_ctx_p = i_ctx_p;
+
         if (bBuildGlyph && !bCID) {
             if (r_type(op) != t_name) {
                 name_enter_string(imemory, ".notdef", op);
@@ -2163,11 +2266,11 @@ FAPI_char(i_ctx_t *i_ctx_p, bool bBuildGlyph, ref *charstring)
                  * here, the out of range CID is handled later
                  */
                 if ((dict_find_string(op - 1, "CharStrings", &chstrs)) <= 0) {
-                    return_error(e_undefined);
+                    return_error(gs_error_undefined);
                 }
 
                 if ((dict_find_string(chstrs, ".notdef", &chs)) <= 0) {
-                    return_error(e_undefined);
+                    return_error(gs_error_undefined);
                 }
                 ref_assign_inline(op, chs);
             }
@@ -2220,7 +2323,7 @@ FAPI_char(i_ctx_t *i_ctx_p, bool bBuildGlyph, ref *charstring)
                 return (zchar_exec_char_proc(i_ctx_p));
             }
             else {
-                return_error(e_invalidfont);
+                return_error(gs_error_invalidfont);
             }
         }
     }
@@ -2257,10 +2360,10 @@ zFAPIBuildGlyph9(i_ctx_t *i_ctx_p)
     font_index = op[0].value.intval;
     if (dict_find_string(&font9, "FDArray", &rFDArray) <= 0
         || r_type(rFDArray) != t_array)
-        return_error(e_invalidfont);
+        return_error(gs_error_invalidfont);
     if (array_get(imemory, rFDArray, font_index, &f) < 0
         || r_type(&f) != t_dictionary)
-        return_error(e_invalidfont);
+        return_error(gs_error_invalidfont);
 
     op[0] = op[-2];
     op[-2] = op[-1];            /* Keep the charstring on ostack for the garbager. */
@@ -2378,7 +2481,7 @@ zFAPIpassfont(i_ctx_t *i_ctx_p)
     if (fapi_request != NULL)
         gs_free_string(imemory, (byte *) fapi_request,
                        strlen(fapi_request) + 1, "do_FAPIpassfont");
-    if (code != 0)
+    if (code < 0 && code != gs_error_invalidaccess)
         return code;
 
     if (code >= 0 && fapi_id != NULL) {
