@@ -35,7 +35,7 @@ private_st_int_remap_color_info();
 /* ------ Utilities ------ */
 
 static int
-zset_real(i_ctx_t *i_ctx_p, int (*set_proc)(gs_state *, floatp))
+zset_real(i_ctx_t *i_ctx_p, int (*set_proc)(gs_state *, double))
 {
     os_ptr op = osp;
     double param;
@@ -117,6 +117,8 @@ int_gstate_alloc(const gs_dual_memory_t * dmem)
 
     iigs = gs_alloc_struct((gs_memory_t *)lmem, int_gstate, &st_int_gstate,
                            "int_gstate_alloc(int_gstate)");
+    if (iigs == NULL)
+        return NULL;
     int_gstate_map_refs(iigs, make_null);
     make_empty_array(&iigs->dash_pattern_array, a_all);
     gs_alloc_ref_array(lmem, &proc0, a_readonly + a_executable, 2,
@@ -134,6 +136,8 @@ int_gstate_alloc(const gs_dual_memory_t * dmem)
     prci = gs_alloc_struct((gs_memory_t *)gmem, int_remap_color_info_t,
                            &st_int_remap_color_info,
                            "int_gstate_alloc(remap color info)");
+    if (prci == NULL)
+        return NULL;
     make_struct(&iigs->remap_color_info, imemory_space(gmem), prci);
     clear_pagedevice(iigs);
     gs_state_set_client(pgs, iigs, &istate_procs, true);
@@ -307,7 +311,7 @@ zsetdash(i_ctx_t *i_ctx_p)
         (float *)gs_alloc_byte_array(mem, n, sizeof(float), "setdash");
 
     if (pattern == 0)
-        return_error(e_VMerror);
+        return_error(gs_error_VMerror);
     for (i = 0, code = 0; i < n && code >= 0; ++i) {
         ref element;
 
@@ -379,7 +383,7 @@ zsetcurvejoin(i_ctx_t *i_ctx_p)
 
     check_type(*op, t_integer);
     if (op->value.intval < -1 || op->value.intval > max_int)
-        return_error(e_rangecheck);
+        return_error(gs_error_rangecheck);
     code = gs_setcurvejoin(igs, (int)op->value.intval);
     if (code < 0)
         return code;
