@@ -32,6 +32,7 @@
 #include "igstate.h"
 #include "icie.h"
 #include "ialloc.h"
+#include "store.h"
 #include "zicc.h"
 #include "gsicc_manage.h"
 #include "gx.h"
@@ -45,10 +46,9 @@ int seticc(i_ctx_t * i_ctx_p, int ncomps, ref *ICCdict, float *range_buff)
 {
     int                     code, k;
     gs_color_space *        pcs;
-    gs_color_space *  palt_cs;
     ref *                   pstrmval;
     stream *                s = 0L;
-    cmm_profile_t           *picc_profile;
+    cmm_profile_t           *picc_profile = NULL;
     gs_imager_state *       pis = (gs_imager_state *)igs;
     int                     i, expected = 0;
     ref *                   pnameval;
@@ -59,11 +59,9 @@ int seticc(i_ctx_t * i_ctx_p, int ncomps, ref *ICCdict, float *range_buff)
             GSICC_STANDARD_PROFILES_KEYS
         };
 
-    palt_cs = gs_currentcolorspace(igs);
-
     /* verify the DataSource entry */
     if (dict_find_string(ICCdict, "DataSource", &pstrmval) <= 0)
-        return_error(e_undefined);
+        return_error(gs_error_undefined);
     check_read_file(i_ctx_p, s, pstrmval);
 
     /* build the color space object */
@@ -86,7 +84,6 @@ int seticc(i_ctx_t * i_ctx_p, int ncomps, ref *ICCdict, float *range_buff)
         str[size] = 0;
 
         /* Compare this to the standard profile names */
-        picc_profile = NULL;
         for (k = 0; k < GSICC_NUMBER_STANDARD_PROFILES; k++) {
             if ( strcmp( str, icc_std_profile_keys[k] ) == 0 ) {
                 picc_profile = gsicc_get_profile_handle_file(icc_std_profile_names[k],
@@ -96,8 +93,18 @@ int seticc(i_ctx_t * i_ctx_p, int ncomps, ref *ICCdict, float *range_buff)
         }
     } else {
         picc_profile = gsicc_profile_new(s, gs_state_memory(igs), NULL, 0);
+        if (picc_profile == NULL)
+            return gs_throw(gs_error_VMerror, "Creation of ICC profile failed");
+        /* We have to get the profile handle due to the fact that we need to know
+           if it has a data space that is CIELAB */
+        picc_profile->profile_handle =
+            gsicc_get_profile_handle_buffer(picc_profile->buffer,
+                                            picc_profile->buffer_size,
+                                            gs_state_memory(igs));
     }
-    if (picc_profile == NULL) {
+    if (picc_profile == NULL || picc_profile->profile_handle == NULL) {
+        /* Free up everything, the profile is not valid. We will end up going
+           ahead and using a default based upon the number of components */
         rc_decrement(picc_profile,"seticc");
         rc_decrement(pcs,"seticc");
         return -1;
@@ -110,19 +117,6 @@ int seticc(i_ctx_t * i_ctx_p, int ncomps, ref *ICCdict, float *range_buff)
     }
     picc_profile->num_comps = ncomps;
 
-    /* We have to get the profile handle due to the fact that we need to know
-       if it has a data space that is CIELAB */
-    picc_profile->profile_handle =
-        gsicc_get_profile_handle_buffer(picc_profile->buffer,
-                                        picc_profile->buffer_size,
-                                        gs_state_memory(igs));
-    if (picc_profile->profile_handle == NULL) {
-        /* Free up everything, the profile is not valid. We will end up going
-           ahead and using a default based upon the number of components */
-        rc_decrement(picc_profile,"seticc");
-        rc_decrement(pcs,"seticc");
-        return -1;
-    }
     picc_profile->data_cs = gscms_get_profile_data_space(picc_profile->profile_handle);
     switch( picc_profile->data_cs ) {
         case gsCIEXYZ:
@@ -137,16 +131,14 @@ int seticc(i_ctx_t * i_ctx_p, int ncomps, ref *ICCdict, float *range_buff)
             expected = 4;
             break;
         case gsNCHANNEL:
-            expected = 0;
-            break;
         case gsNAMED:            /* Silence warnings */
         case gsUNDEFINED:        /* Silence warnings */
             break;
     }
-    if (expected && ncomps != expected) {
+    if (!expected || ncomps != expected) {
         rc_decrement(picc_profile,"seticc");
         rc_decrement(pcs,"seticc");
-        return_error(e_rangecheck);
+        return_error(gs_error_rangecheck);
     }
 
     /* Lets go ahead and get the hash code and check if we match one of the default spaces */
@@ -265,10 +257,12 @@ zset_outputintent(i_ctx_t * i_ctx_p)
 
     /* verify the DataSource entry. Creat profile from stream */
     if (dict_find_string(op, "DataSource", &pstrmval) <= 0)
-        return_error(e_undefined);
+        return_error(gs_error_undefined);
     check_read_file(i_ctx_p, s, pstrmval);
 
     picc_profile = gsicc_profile_new(s, gs_state_memory(igs), NULL, 0);
+    if (picc_profile == NULL)
+        return gs_throw(gs_error_VMerror, "Creation of ICC profile failed");
     picc_profile->num_comps = ncomps;
     picc_profile->profile_handle =
         gsicc_get_profile_handle_buffer(picc_profile->buffer,
@@ -303,7 +297,7 @@ zset_outputintent(i_ctx_t * i_ctx_p)
     }
     if (expected && ncomps != expected) {
         rc_decrement(picc_profile,"zset_outputintent");
-        return_error(e_rangecheck);
+        return_error(gs_error_rangecheck);
     }
     gsicc_init_hash_cs(picc_profile, pis);
 
@@ -430,10 +424,10 @@ zseticcspace(i_ctx_t * i_ctx_p)
         return code;
     ncomps = pnval->value.intval;
     if (2*ncomps > sizeof(range_buff)/sizeof(range_buff[0]))
-        return_error(e_rangecheck);
+        return_error(gs_error_rangecheck);
     /* verify the DataSource entry */
     if (dict_find_string(op, "DataSource", &pstrmval) <= 0)
-        return_error(e_undefined);
+        return_error(gs_error_undefined);
     check_read_file(i_ctx_p, s, pstrmval);
     /*
      * Verify that the current color space can be a alternative color space.
@@ -443,7 +437,7 @@ zseticcspace(i_ctx_t * i_ctx_p)
     palt_cs = gs_currentcolorspace(igs);
     if ( !palt_cs->type->can_be_alt_space                                ||
          gs_color_space_get_index(palt_cs) == gs_color_space_index_ICC  )
-        return_error(e_rangecheck);
+        return_error(gs_error_rangecheck);
     /*
      * Fetch and verify the Range array.
      *
@@ -469,7 +463,7 @@ zseticcspace(i_ctx_t * i_ctx_p)
     for (i = 0; i < 2 * ncomps && range_buff[i + 1] >= range_buff[i]; i += 2)
         ;
     if (i != 2 * ncomps)
-        return_error(e_rangecheck);
+        return_error(gs_error_rangecheck);
     return seticc(i_ctx_p, ncomps, op, range_buff);
 }
 
@@ -479,25 +473,14 @@ seticc_lab(i_ctx_t * i_ctx_p, float *white, float *black, float *range_buff)
 {
     int                     code;
     gs_color_space *        pcs;
-    gs_color_space *        palt_cs;
     gs_imager_state *       pis = (gs_imager_state *)igs;
     int                     i;
-    static const char *const rfs = LAB_ICC;
-    gs_param_string val, *pval;
 
-    val.data = (const byte *)rfs;
-    val.size = strlen(rfs);
-    val.persistent = true;
-    pval = &val;
-
-    palt_cs = gs_currentcolorspace(igs);
     /* build the color space object */
     code = gs_cspace_build_ICC(&pcs, NULL, gs_state_memory(igs));
     if (code < 0)
         return gs_rethrow(code, "building color space object");
     /* record the current space as the alternative color space */
-    /* pcs->base_space = palt_cs;
-    rc_increment_cs(palt_cs); */
     /* Get the lab profile.  It may already be set in the icc manager.
        If not then lets populate it.  */
     if (pis->icc_manager->lab_profile == NULL ) {
@@ -508,7 +491,6 @@ seticc_lab(i_ctx_t * i_ctx_p, float *white, float *black, float *range_buff)
     }
     /* Assign the LAB to LAB profile to this color space */
     code = gsicc_set_gscs_profile(pcs, pis->icc_manager->lab_profile, gs_state_memory(igs));
-    rc_increment(pis->icc_manager->lab_profile);
     if (code < 0)
         return gs_rethrow(code, "installing the lab profile");
     pcs->cmm_icc_profile_data->Range.ranges[0].rmin = 0.0;
@@ -552,9 +534,13 @@ seticc_cal(i_ctx_t * i_ctx_p, float *white, float *black, float *gamma,
         cal_profile = gsicc_create_from_cal(white, black, gamma, matrix,
                                             mem->stable_memory, num_colorants);
         if (cal_profile == NULL)
-            return gs_rethrow(-1, "creating the cal profile");
+            return gs_rethrow(gs_error_VMerror, "creating the cal profile failed");
         /* Assign the profile to this color space */
         code = gsicc_set_gscs_profile(pcs, cal_profile, mem->stable_memory);
+        /* profile is created with ref count of 1, gsicc_set_gscs_profile()
+         * increments the ref count, so we need to decrement it here.
+         */
+        rc_decrement(cal_profile, "seticc_cal");
         if (code < 0)
             return gs_rethrow(code, "installing the cal profile");
         for (i = 0; i < num_colorants; i++) {
@@ -569,9 +555,75 @@ seticc_cal(i_ctx_t * i_ctx_p, float *white, float *black, float *gamma,
     return code;
 }
 
+static int
+znumicc_components(i_ctx_t * i_ctx_p)
+{
+    ref *                   pnval;
+    ref *                   pstrmval;
+    stream *                s;
+    int                     ncomps, expected = 0, code;
+    cmm_profile_t           *picc_profile;
+    os_ptr                  op = osp;
+
+    check_type(*op, t_dictionary);
+    check_dict_read(*op);
+
+    code = dict_find_string(op, "N", &pnval);
+    if (code < 0)
+        return code;
+    ncomps = pnval->value.intval;
+    /* verify the DataSource entry. Create profile from stream */
+    if (dict_find_string(op, "DataSource", &pstrmval) <= 0)
+        return_error(gs_error_undefined);
+    check_read_file(i_ctx_p, s, pstrmval);
+
+    picc_profile = gsicc_profile_new(s, gs_state_memory(igs), NULL, 0);
+    if (picc_profile == NULL)
+        return gs_throw(gs_error_VMerror, "Creation of ICC profile failed");
+
+    picc_profile->num_comps = ncomps;
+    picc_profile->profile_handle =
+        gsicc_get_profile_handle_buffer(picc_profile->buffer,
+                                        picc_profile->buffer_size,
+                                        gs_state_memory(igs));
+    if (picc_profile->profile_handle == NULL) {
+        rc_decrement(picc_profile,"znumicc_components");
+        make_int(op, expected);
+        return 0;
+    }
+    picc_profile->data_cs = gscms_get_profile_data_space(picc_profile->profile_handle);
+
+    switch (picc_profile->data_cs) {
+        case gsCIEXYZ:
+        case gsCIELAB:
+        case gsRGB:
+            expected = 3;
+            break;
+        case gsGRAY:
+            expected = 1;
+            break;
+        case gsCMYK:
+            expected = 4;
+            break;
+        case gsNCHANNEL:
+            expected = 0;
+            break;
+        case gsNAMED:
+        case gsUNDEFINED:
+            expected = -1;
+            break;
+    }
+
+    make_int(op, expected);
+
+    rc_decrement(picc_profile,"zset_outputintent");
+    return 0;
+}
+
 const op_def    zicc_ll3_op_defs[] = {
     op_def_begin_ll3(),
     { "1.seticcspace", zseticcspace },    
     { "1.set_outputintent", zset_outputintent },
+    { "1.numicc_components", znumicc_components },
     op_def_end(0)
 };
