@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: ztoken.c 12023 2011-01-13 14:37:33Z robin $ */
+
 /* Token reading operators */
 #include "string_.h"
 #include "stat_.h" /* get system header early to avoid name clash on Cygwin */
@@ -61,7 +63,7 @@ ztoken(i_ctx_t *i_ctx_p)
 
             /* Don't pop the operand in case of invalidaccess. */
             if (!r_has_attr(op, a_read))
-                return_error(e_invalidaccess);
+                return_error(gs_error_invalidaccess);
             code = gs_scan_string_token(i_ctx_p, op, &token);
             switch (code) {
             case scan_EOF:      /* no tokens */
@@ -113,7 +115,7 @@ again:
     switch (code) {
         default:                /* error */
             if (code > 0)       /* comment, not possible */
-                code = gs_note_error(e_syntaxerror);
+                code = gs_note_error(gs_error_syntaxerror);
             gs_scanner_error_object(i_ctx_p, pstate, &i_ctx_p->error_object);
             break;
         case scan_BOS:
@@ -253,7 +255,7 @@ ztoken_handle_comment(i_ctx_t *i_ctx_p, scanner_state *sstate,
         proc_name = "%ProcessDSCComment";
         break;
     default:
-        return_error(e_Fatal);  /* can't happen */
+        return_error(gs_error_Fatal);  /* can't happen */
     }
     /*
      * We can't use check_ostack here, because it returns on overflow.
@@ -272,7 +274,7 @@ ztoken_handle_comment(i_ctx_t *i_ctx_p, scanner_state *sstate,
         pstate = (scanner_state *)ialloc_struct(scanner_state_dynamic, &st_scanner_state_dynamic,
                                "ztoken_handle_comment");
         if (pstate == 0)
-            return_error(e_VMerror);
+            return_error(gs_error_VMerror);
         ((scanner_state_dynamic *)pstate)->mem = imemory;
         *pstate = *sstate;
     } else
