@@ -174,7 +174,7 @@ in_path(os_ptr oppath, i_ctx_t *i_ctx_p, gx_device * phdev)
         fr.q.y = fr.p.y + fixed_1;
         code = gx_clip_to_rectangle(igs, &fr);
         npop = 2;
-    } else if (code == e_stackunderflow) {
+    } else if (code == gs_error_stackunderflow) {
         /* If 0 elements, definitely a stackunderflow; otherwise, */
         /* only 1 number, also a stackunderflow. */
         npop = code;
@@ -483,7 +483,7 @@ path_length_for_upath(const gx_path *ppath)
                 size += 1;
                 continue;
             default:
-                return_error(e_unregistered);
+                return_error(gs_error_unregistered);
         }
     }
     return size;
@@ -508,7 +508,7 @@ make_upath(i_ctx_t *i_ctx_p, ref *rupath, gs_state *pgs, gx_path *ppath,
          * not in CPSI compatibility mode, we set a reasonable default
          * bbox instead.
          */
-        if (code != e_nocurrentpoint || gs_currentcpsimode(imemory))
+        if (code != gs_error_nocurrentpoint || gs_currentcpsimode(imemory))
             return code;
         bbox.p.x = bbox.p.y = bbox.q.x = bbox.q.y = 0;
     }
@@ -518,7 +518,7 @@ make_upath(i_ctx_t *i_ctx_p, ref *rupath, gs_state *pgs, gx_path *ppath,
         return code;
     size += code;
     if (size >= 65536)
-        return_error(e_limitcheck);
+        return_error(gs_error_limitcheck);
 
     code = ialloc_ref_array(rupath, a_all | a_executable, size,
                             "make_upath");
@@ -548,7 +548,7 @@ make_upath(i_ctx_t *i_ctx_p, ref *rupath, gs_state *pgs, gx_path *ppath,
         gx_path *save_path = pgs->path;
 
         pgs->path = ppath;
-        gs_path_enum_copy_init(&penum, pgs, false);
+        gs_path_enum_copy_init(pgs->memory, &penum, pgs, false);
         pgs->path = save_path;
         while ((op = gs_path_enum_next(&penum, pts)) != 0) {
             const char *opstr;
@@ -577,7 +577,7 @@ make_upath(i_ctx_t *i_ctx_p, ref *rupath, gs_state *pgs, gx_path *ppath,
                     opstr = "closepath";
                     break;
                 default:
-                    return_error(e_unregistered);
+                    return_error(gs_error_unregistered);
             }
             if ((code = name_enter_string(pgs->memory, opstr, next)) < 0)
                 return code;
@@ -610,7 +610,7 @@ zgetpath(i_ctx_t *i_ctx_p)
         dict_find_string(systemdict, "lineto", &operators[2]) <= 0 ||
         dict_find_string(systemdict, "curveto", &operators[3]) <= 0 ||
         dict_find_string(systemdict, "closepath", &operators[4]) <= 0)
-          return_error(e_undefined);
+          return_error(gs_error_undefined);
 
     main_ref = op->value.refs;
     for (i = 0; i < leaf_count; i++) {
@@ -635,7 +635,7 @@ zgetpath(i_ctx_t *i_ctx_p)
         fts[5] = &pts[2].y;
 
         main_ref = op->value.refs;
-        gs_path_enum_copy_init(&penum, igs, false);
+        gs_path_enum_copy_init(igs->memory, &penum, igs, false);
         pe = gs_path_enum_next(&penum, pts);
         if (pe < 0)
             return pe;
@@ -655,7 +655,7 @@ zgetpath(i_ctx_t *i_ctx_p)
                     if (pe <= 0)
                         return pe;
                     if (pe >= 5)
-                        return_error(e_unregistered);
+                        return_error(gs_error_unregistered);
                 }
             }
         }
@@ -673,9 +673,9 @@ upath_append_aux(os_ptr oppath, i_ctx_t *i_ctx_p, int *pnargs, bool upath_compat
     ref opcodes;
 
     if (r_has_type(oppath, t__invalid))
-        return_error(e_stackunderflow);
+        return_error(gs_error_stackunderflow);
     if (!r_is_array(oppath))
-        return_error(e_typecheck);
+        return_error(gs_error_typecheck);
     check_read(*oppath);
     gs_newpath(igs);
     /****** ROUND tx AND ty ******/
@@ -704,7 +704,7 @@ upath_append_aux(os_ptr oppath, i_ctx_t *i_ctx_p, int *pnargs, bool upath_compat
             if (opx > UPATH_REPEAT)
                 repcount = opx - UPATH_REPEAT;
             else if (opx > UPATH_MAX_OP)
-                return_error(e_rangecheck);
+                return_error(gs_error_rangecheck);
             else {		/* operator */
                 const up_data_t data = up_data[opx];
 
@@ -715,7 +715,7 @@ upath_append_aux(os_ptr oppath, i_ctx_t *i_ctx_p, int *pnargs, bool upath_compat
                     ups = ups > UPS_UCACHE ? ups : data.state_after;
                 } else {
                     if (!(ups & data.states_before))
-                        return_error(e_typecheck);
+                        return_error(gs_error_typecheck);
                     ups = data.state_after;
                 }
                 do {
@@ -734,7 +734,7 @@ upath_append_aux(os_ptr oppath, i_ctx_t *i_ctx_p, int *pnargs, bool upath_compat
                                 r_set_type_attrs(op, t_real, 0);
                                 break;
                             default:
-                                return_error(e_typecheck);
+                                return_error(gs_error_typecheck);
                         }
                     }
                     code = (*up_ops[opx])(i_ctx_p);
@@ -772,49 +772,49 @@ upath_append_aux(os_ptr oppath, i_ctx_t *i_ctx_p, int *pnargs, bool upath_compat
                     if (!r_has_attr(&rup, a_executable) ||
                         dict_find(systemdict, &rup, &defp) <= 0 ||
                         r_btype(defp) != t_operator)
-                        return_error(e_typecheck); /* all errors = typecheck */
+                        return_error(gs_error_typecheck); /* all errors = typecheck */
                     goto xop;
                 case t_operator:
                     defp = &rup;
                   xop:if (!r_has_attr(defp, a_executable))
-                        return_error(e_typecheck);
+                        return_error(gs_error_typecheck);
                     oproc = real_opproc(defp);
                     for (opx = 0; opx <= UPATH_MAX_OP; opx++)
                         if (oproc == up_ops[opx])
                             break;
                     if (opx > UPATH_MAX_OP)
-                        return_error(e_typecheck);
+                        return_error(gs_error_typecheck);
                     data = up_data[opx];
                     if (argcount != data.num_args)
-                        return_error(e_typecheck);
+                        return_error(gs_error_typecheck);
                     if (upath_compat && opx == upath_op_ucache) {
                         /* CPSI does not complain about incorrect ucache
                            placement, even though PLRM3 says it's illegal. */
                         ups = ups > UPS_UCACHE ? ups : data.state_after;
                     } else {
                         if (!(ups & data.states_before))
-                            return_error(e_typecheck);
+                            return_error(gs_error_typecheck);
                         ups = data.state_after;
                     }
                     code = (*up_ops[opx])(i_ctx_p);
                     if (code < 0) {
-                        if (code == e_nocurrentpoint)
-                            return_error(e_rangecheck); /* CET 11-22 */
+                        if (code == gs_error_nocurrentpoint)
+                            return_error(gs_error_rangecheck); /* CET 11-22 */
                         return code;
                     }
                     argcount = 0;
                     break;
                 default:
-                    return_error(e_typecheck);
+                    return_error(gs_error_typecheck);
             }
         }
         if (argcount) {
             *pnargs = argcount;
-            return_error(e_typecheck);	/* leftover args */
+            return_error(gs_error_typecheck);	/* leftover args */
         }
     }
     if (ups < UPS_SETBBOX)
-        return_error(e_typecheck);	/* no setbbox */
+        return_error(gs_error_typecheck);	/* no setbbox */
     if (ups == UPS_SETBBOX && upath_compat) {
         /*
          * In CPSI compatibility mode, an empty path with a setbbox also
