@@ -1,14 +1,16 @@
-/* Copyright (C) 2001-2010 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
 #include "stdafx.h"
@@ -23,7 +25,7 @@
 #include <math.h>
 #include "icc_create.h"
 
-#define arch_is_big_endian 0;
+#define ARCH_IS_BIG_ENDIAN 0;
 
 typedef unsigned long ulong;
 typedef unsigned short ushort;
@@ -34,6 +36,7 @@ typedef unsigned short ushort;
 static void
 add_xyzdata(unsigned char *input_ptr, icS15Fixed16Number temp_XYZ[]);
 
+#define CMYK_KONLY 0
 #define SAVEICCPROFILE 1
 #define HEADER_SIZE 128
 #define TAG_SIZE 12
@@ -132,7 +135,7 @@ static void
 write_bigendian_4bytes(unsigned char *curr_ptr,ulong input)
 {
 
-#if !arch_is_big_endian
+#if !ARCH_IS_BIG_ENDIAN
     *curr_ptr ++= ((0x000000ff) & (input >> 24));
    *curr_ptr ++= ((0x000000ff) & (input >> 16));
    *curr_ptr ++= ((0x000000ff) & (input >> 8));
@@ -148,7 +151,7 @@ write_bigendian_4bytes(unsigned char *curr_ptr,ulong input)
 static void
 write_bigendian_2bytes(unsigned char *curr_ptr,ushort input)
 {
-#if !arch_is_big_endian
+#if !ARCH_IS_BIG_ENDIAN
     *curr_ptr ++= ((0x000ff) & (input >> 8));
    *curr_ptr ++= ((0x000ff) & (input));
 #else
@@ -1026,7 +1029,7 @@ color_cmyk_to_gray(float c, float m, float y, float k)
 
 /* Convert CMYK to RGB. */
 void
-color_cmyk_to_rgb(float c, float m, float y, float k, float rgb[3])
+color_cmyk_to_rgb(float c, float m, float y, float k, float rgb[3], bool cpsi_mode)
 {
     float not_k;
 
@@ -1041,68 +1044,84 @@ color_cmyk_to_rgb(float c, float m, float y, float k, float rgb[3])
         return;
     }
     not_k = 1 - k;
-    rgb[0] = (c > not_k ? 0 : not_k - c);
-    rgb[1] = (m > not_k ? 0 : not_k - m);
-    rgb[2] = (y > not_k ? 0 : not_k - y);    
 
+    if (cpsi_mode) {
+        rgb[0] = (1.0 - c) * not_k;
+        rgb[1] = (1.0 - m) * not_k;
+        rgb[2] = (1.0 - y) * not_k;
+    } else {
+        rgb[0] = (c > not_k ? 0 : not_k - c);
+        rgb[1] = (m > not_k ? 0 : not_k - m);
+        rgb[2] = (y > not_k ? 0 : not_k - y);    
+    }
 }
 
 
 /* Convert RGB to CMYK. */
 /* Note that this involves black generation and undercolor removal.
-   Right now set up for no bg or ucr.  Just doing one minus composite.
-   Since this is for the soft mask, that should be OK. */
+   These are defined by a table ucr and bg table of data that is 
+   settable by the user.  The index into the K table is performed by 
+   */
 void
-color_rgb_to_cmyk(float r, float g, float b, float cmyk[4])
+color_rgb_to_cmyk(float r, float g, float b, float cmyk[4], ucrbg_t *ucr_data)
 {
     float c = 1 - r, m = 1 - g, y = 1 - b;
     float k = (c < m ? min(c, y) : min(m, y));
+    int r_int;
+    int g_int;
+    int b_int;
+    int k_int;
 
-    /*
-     * The default UCR and BG functions are pretty arbitrary,
-     * but they must agree with the ones in gs_init.ps.
-     */
-    float bg = 0;
-    float ucr = 0;
+    if (ucr_data == NULL) {
+        /* No ucr or bg */
+	cmyk[0] = c, cmyk[1] = m, cmyk[2] = y, cmyk[3] = 0;
+        return;
+    } else {
+        /* index and get the value */
 
-    if (ucr == 1)
-	cmyk[0] = cmyk[1] = cmyk[2] = 0;
-    else if (ucr == 0)
-	cmyk[0] = c, cmyk[1] = m, cmyk[2] = y;
-    else {
+        r_int = ROUND(r * 255.0);
+        g_int = ROUND(g * 255.0);
+        b_int = ROUND(b * 255.0);
+        k_int = ROUND(k * 255.0);
 
-#if 0
-	if (!gs_currentcpsimode(mem)) {
-	    /* C = max(0.0, min(1.0, 1 - R - UCR)), etc. */
-	    signed_frac not_ucr = (ucr < 0 ? frac_1 + ucr : frac_1);
+        if (r_int > 255) r_int = 255;
+        if (g_int > 255) g_int = 255;
+        if (b_int > 255) b_int = 255;
+        if (k_int > 255) k_int = 255;
 
-	    cmyk[0] = (c < ucr ? frac_0 : c > not_ucr ? frac_1 : c - ucr);
-	    cmyk[1] = (m < ucr ? frac_0 : m > not_ucr ? frac_1 : m - ucr);
-	    cmyk[2] = (y < ucr ? frac_0 : y > not_ucr ? frac_1 : y - ucr);
-	} else {
-	    /* Adobe CPSI method */
-	    /* C = max(0.0, min(1.0, 1 - R / (1 - UCR))), etc. */
-	    float denom = frac2float(frac_1 - ucr);		/* unscaled */
-	    float v;
-
-	    v = (float)frac_1 - r / denom;	/* unscaled */
-	    cmyk[0] =
-		(is_fneg(v) ? frac_0 : v >= (float)frac_1 ? frac_1 : (frac) v);
-	    v = (float)frac_1 - g / denom;	/* unscaled */
-	    cmyk[1] =
-		(is_fneg(v) ? frac_0 : v >= (float)frac_1 ? frac_1 : (frac) v);
-	    v = (float)frac_1 - b / denom;	/* unscaled */
-	    cmyk[2] =
-		(is_fneg(v) ? frac_0 : v >= (float)frac_1 ? frac_1 : (frac) v);
-	}
-
-#endif
+        if (r_int < 0) r_int = 0;
+        if (g_int < 0) g_int = 0;
+        if (b_int < 0) b_int = 0;
+        if (k_int < 0) k_int = 0;
 
+        cmyk[0] = ucr_data->cyan[r_int]/255.0;
+        cmyk[1] = ucr_data->magenta[g_int]/255.0;
+        cmyk[2] = ucr_data->yellow[b_int]/255.0;
+        cmyk[3] = ucr_data->black[k_int]/255.0;
     }
-    cmyk[3] = bg;
 }
 
+void 
+luminance_to_cmyk(float lum, float cmyk[4], ucrbg_t *effect_data)
+{
+    int l_int;
 
+    if (effect_data == NULL) {
+	cmyk[0] = 0, cmyk[1] = 0, cmyk[2] = 0, cmyk[3] = 0;
+        return;
+    } else {
+        /* index and get the value */
+
+        l_int = ROUND(lum * 255.0);
+        if (l_int > 255) l_int = 255;
+        if (l_int < 0) l_int = 0;
+
+        cmyk[0] = effect_data->cyan[l_int]/255.0;
+        cmyk[1] = effect_data->magenta[l_int]/255.0;
+        cmyk[2] = effect_data->yellow[l_int]/255.0;
+        cmyk[3] = effect_data->black[l_int]/255.0;
+    }
+}
 
 static void
 create_cmyk2gray(unsigned short *table_data, int mlut_size, int num_samples)
@@ -1182,7 +1201,7 @@ create_gray2rgb(unsigned short *table_data, int mlut_size, int num_samples)
 }
 
 static void
-create_cmyk2lab(unsigned short *table_data, int mlut_size, int num_samples)
+create_cmyk2lab(unsigned short *table_data, int mlut_size, int num_samples, bool cpsi_mode)
 {
     int c,m,y,k;
     float cyan,magenta,yellow,black;
@@ -1204,7 +1223,7 @@ create_cmyk2lab(unsigned short *table_data, int mlut_size, int num_samples)
                 yellow = (float) y / (float) (num_samples -1);
                 for (k = 0; k < num_samples; k++) {
                     black = (float) k / (float) (num_samples -1);
-                    color_cmyk_to_rgb(cyan, magenta, yellow, black, rgb_values);
+                    color_cmyk_to_rgb(cyan, magenta, yellow, black, rgb_values, cpsi_mode);
                     /* Now get that rgb value to CIEXYZ */
                     /* Now get that rgb value to CIEXYZ */
                     xyz[0] = rgb_values[0] * 0.60974 + rgb_values[1] * 0.20528 + rgb_values[2] * 0.14919;
@@ -1231,7 +1250,7 @@ create_cmyk2lab(unsigned short *table_data, int mlut_size, int num_samples)
 }
 
 static void
-create_cmyk2xyz(unsigned short *table_data, int mlut_size, int num_samples)
+create_cmyk2xyz(unsigned short *table_data, int mlut_size, int num_samples, bool cpsi_mode)
 {
     int c,m,y,k;
     float cyan,magenta,yellow,black;
@@ -1254,7 +1273,7 @@ create_cmyk2xyz(unsigned short *table_data, int mlut_size, int num_samples)
                 yellow = (float) y / (float) (num_samples - 1);
                 for (k = 0; k < num_samples; k++) {
                     black = (float) k / (float) (num_samples -1);
-                    color_cmyk_to_rgb(cyan, magenta, yellow, black, rgb_values);
+                    color_cmyk_to_rgb(cyan, magenta, yellow, black, rgb_values, cpsi_mode);
                     /* Now get that rgb value to CIEXYZ */
                     xyz[0] = rgb_values[0] * 0.60974 + rgb_values[1] * 0.20528 + rgb_values[2] * 0.14919;
                     xyz[1] = rgb_values[0] * 0.31111 + rgb_values[1] * 0.62567 + rgb_values[2] * 0.06322;
@@ -1277,7 +1296,7 @@ create_cmyk2xyz(unsigned short *table_data, int mlut_size, int num_samples)
 }
 
 static void
-create_xyz2cmyk(unsigned short *table_data, int mlut_size, int num_samples)
+create_xyz2cmyk(unsigned short *table_data, int mlut_size, int num_samples, ucrbg_t *ucr_data)
 {
     int x,y,z,jj;
     float xval,yval,zval;
@@ -1300,8 +1319,16 @@ create_xyz2cmyk(unsigned short *table_data, int mlut_size, int num_samples)
                 rgb_values[0] = xyz[0] * 1.96253 + xyz[1] * (-0.61068) + xyz[2] * (-0.34137);
                 rgb_values[1] = xyz[0] * (-0.97876) + xyz[1] * 1.91615 + xyz[2] * 0.03342;
                 rgb_values[2] = xyz[0] * 0.028693 + xyz[1] * (-0.14067) + xyz[2] * 1.34926;
+#if CMYK_KONLY
+                /* A strange case where the CMYK profile behaves like a monochrome device */
+                cmyk[3] = 1.0 - color_rgb_to_gray(rgb_values[0], rgb_values[1], rgb_values[2]);
+                cmyk[2] = 0.0;
+                cmyk[1] = 0.0;
+                cmyk[0] = 0.0;
+#else
                 /* Now RGB to CMYK */
-                color_rgb_to_cmyk(rgb_values[0], rgb_values[1], rgb_values[2], cmyk);
+                color_rgb_to_cmyk(rgb_values[0], rgb_values[1], rgb_values[2], cmyk, ucr_data);
+#endif
                 /* Now store the CMYK value */                
                 for (jj = 0; jj < 4; jj++) {
                     if (cmyk[jj] < 0) cmyk[jj] = 0;
@@ -1345,7 +1372,6 @@ create_lab2gray_special(unsigned short *table_data, int num_samples)
     int l,a,b;
     float lstar,astar,bstar;
     unsigned short *buffptr = table_data;
-    float rgb_values[3];
     float xyz[3], lab_value[3], gray_value;
 
     /* Step through the lab indices, convert to CIEXYZ
@@ -1372,7 +1398,7 @@ create_lab2gray_special(unsigned short *table_data, int num_samples)
 }
 
 static void
-create_lab2cmyk(unsigned short *table_data, int mlut_size, int num_samples)
+create_lab2cmyk(unsigned short *table_data, int mlut_size, int num_samples, ucrbg_t *ucr_data)
 {
     int l,a,b,jj;
     float lstar,astar,bstar;
@@ -1399,7 +1425,7 @@ create_lab2cmyk(unsigned short *table_data, int mlut_size, int num_samples)
                 rgb_values[1] = xyz[0] * (-0.97876) + xyz[1] * 1.91615 + xyz[2] * 0.03342;
                 rgb_values[2] = xyz[0] * 0.028693 + xyz[1] * (-0.14067) + xyz[2] * 1.34926;
                 /* Now RGB to CMYK */
-                color_rgb_to_cmyk(rgb_values[0], rgb_values[1], rgb_values[2], cmyk);
+                color_rgb_to_cmyk(rgb_values[0], rgb_values[1], rgb_values[2], cmyk, ucr_data);
                 /* Now store the CMYK value */                
                 for (jj = 0; jj < 4; jj++) {
                     if (cmyk[jj] < 0) cmyk[jj] = 0;
@@ -1411,7 +1437,150 @@ create_lab2cmyk(unsigned short *table_data, int mlut_size, int num_samples)
     }
 }
 
-int create_pscmyk_profile(TCHAR FileName[], bool pcs_islab)
+static void
+create_xyz2cmyk_effect(unsigned short *table_data, int mlut_size, int num_samples, ucrbg_t *effect_data)
+{
+    int x,y,z,jj;
+    float xval,yval,zval;
+    unsigned short *buffptr = table_data;
+    float rgb_values[3];
+    float xyz[3], cmyk[4];
+    float luminance_float;
+
+    /* Step through the xyz indices, convert to RGB, 
+       then to CMYK then encode into 16 bit form */
+    for (x = 0; x < num_samples; x++) {
+        xval = (float) x / (float) (num_samples - 1);
+        for (y = 0; y < num_samples; y++) {
+            yval = (float) y / (float) (num_samples - 1);
+            for (z = 0; z < num_samples; z++) {
+                zval = (float) z / (float) (num_samples - 1);
+                xyz[0] = D50WhitePoint[0]*xval;
+                xyz[1] = D50WhitePoint[1]*yval;
+                xyz[2] = D50WhitePoint[2]*zval;
+                /* To RGB.  Using Inverse of AdobeRGB Mapping */
+                rgb_values[0] = xyz[0] * 1.96253 + xyz[1] * (-0.61068) + xyz[2] * (-0.34137);
+                rgb_values[1] = xyz[0] * (-0.97876) + xyz[1] * 1.91615 + xyz[2] * 0.03342;
+                rgb_values[2] = xyz[0] * 0.028693 + xyz[1] * (-0.14067) + xyz[2] * 1.34926;
+                /* Now RGB to Luminance CMYK */
+                luminance_float = rgb_values[0]*0.3 + rgb_values[1]*0.59 + rgb_values[2]*0.11;
+                luminance_to_cmyk(luminance_float, cmyk, effect_data);
+                /* Now store the CMYK value */                
+                for (jj = 0; jj < 4; jj++) {
+                    if (cmyk[jj] < 0) cmyk[jj] = 0;
+                    if (cmyk[jj] > 1) cmyk[jj] = 1;
+                    *buffptr ++= ROUND(cmyk[jj] * 65535);
+                }
+            }
+        }
+    }
+}
+
+/* Create a special effect profile for mapping from Luminance to a set of CMYK
+   valuse.  Useful for showing object based color management working */
+int create_effect_profile(TCHAR FileName[], ucrbg_t *effect_data, char desc_ptr[])
+{
+    icProfile iccprofile;
+    icHeader  *header = &(iccprofile.header);
+    int profile_size,k;
+    int num_tags;
+    gsicc_tag *tag_list;
+    int tag_offset = 0;
+    unsigned char *curr_ptr;
+    int last_tag;
+    icS15Fixed16Number temp_XYZ[3];
+    int tag_location;
+    int debug_catch = 1;
+    unsigned char *buffer;
+    int numout = 3;              /* Number of output colorants */
+    int tag_size;
+    int num_colors = 4;
+    int num_samples = 5;
+    int mlut_size_atob, mlut_size_btoa;
+    int numinentries = 2;        /* input 1-D LUT samples */
+    int numoutentries = 2;       /* output 1-D LUT samples */
+    unsigned short *table_data_AtoB, *table_data_BtoA;
+
+    /* Fill in the common stuff */
+    setheader_common(header);
+    header->pcs = icSigXYZData;
+    profile_size = HEADER_SIZE;
+    header->deviceClass = icSigOutputClass;
+    header->colorSpace = icSigCmykData;
+    num_tags = 6;  /* common (2) + ATOB0,BTOA0,bkpt,wtpt */   
+    tag_list = (gsicc_tag*) malloc(sizeof(gsicc_tag)*num_tags);
+    /* Let us precompute the sizes of everything and all our offsets */
+    profile_size += TAG_SIZE*num_tags;
+    profile_size += 4; /* number of tags.... */
+    last_tag = -1;
+    if (desc_ptr == NULL) {
+        init_common_tags(tag_list, num_tags, &last_tag, desc_pscmyk_name);
+    } else {
+        init_common_tags(tag_list, num_tags, &last_tag, desc_ptr);
+    }
+    init_tag(tag_list, &last_tag, icSigMediaWhitePointTag, XYZPT_SIZE);
+    init_tag(tag_list, &last_tag, icSigMediaBlackPointTag, XYZPT_SIZE);
+    /* Now the ATOB0 Tag */
+    mlut_size_atob = (int) pow((float) num_samples, (int) num_colors);
+    tag_size = 52+num_colors*numinentries*2+numout*numoutentries*2+mlut_size_atob*numout*2;
+    init_tag(tag_list, &last_tag, icSigAToB0Tag, tag_size);
+    /* Now the ATOB0 Tag.  numcolors and numout switch! */
+    mlut_size_btoa = (int) pow((float) num_samples, (int) numout);
+    tag_size = 52+num_colors*numinentries*2+numout*numoutentries*2+mlut_size_btoa*num_colors*2;
+    init_tag(tag_list, &last_tag, icSigBToA0Tag, tag_size);
+    for(k = 0; k < num_tags; k++) {
+        profile_size += tag_list[k].size;
+    }
+    /* Now we can go ahead and fill our buffer with the data */
+    buffer = (unsigned char*) malloc(profile_size);
+    curr_ptr = buffer;
+
+    /* The header */
+    header->size = profile_size;
+    copy_header(curr_ptr,header);
+    curr_ptr += HEADER_SIZE;
+    /* Tag table */
+    copy_tagtable(curr_ptr,tag_list,num_tags);
+    curr_ptr += TAG_SIZE*num_tags;
+    curr_ptr += 4;
+
+    /* Now the data.  Must be in same order as we created the tag table */
+    /* First the common tags */
+    if (desc_ptr == NULL) {
+        add_common_tag_data(curr_ptr, tag_list, desc_pscmyk_name);
+    } else {
+        add_common_tag_data(curr_ptr, tag_list, desc_ptr);
+    }
+    for (k = 0; k< NUMBER_COMMON_TAGS; k++) {
+        curr_ptr += tag_list[k].size;
+    }
+    tag_location = NUMBER_COMMON_TAGS;
+    /* White and black points */
+    get_XYZ_floatptr(temp_XYZ,(float*) &(D50WhitePoint[0]));
+    add_xyzdata(curr_ptr,temp_XYZ);
+    curr_ptr += tag_list[tag_location].size;
+    tag_location++;
+    get_XYZ_floatptr(temp_XYZ,(float*) &(BlackPoint[0]));
+    add_xyzdata(curr_ptr,temp_XYZ);
+    curr_ptr += tag_list[tag_location].size;
+    tag_location++;
+    /* Create the table data that we will stuff in */
+    table_data_AtoB = (unsigned short*) malloc(mlut_size_atob*sizeof(unsigned short)*numout);
+    create_cmyk2xyz(table_data_AtoB, mlut_size_atob, num_samples, false);
+    table_data_BtoA = (unsigned short*) malloc(mlut_size_btoa*sizeof(unsigned short)*num_colors);
+    create_xyz2cmyk_effect(table_data_BtoA, mlut_size_atob, num_samples, effect_data);
+    /* Now the ATOB0 */
+    add_tabledata(curr_ptr, (void*) table_data_AtoB, num_colors, num_samples, 1, 3, false);
+    curr_ptr += tag_list[tag_location].size;
+    tag_location++;
+    /* Now the BTOA0 */
+    add_tabledata(curr_ptr, (void*) table_data_BtoA, 3, num_samples, 1, num_colors, true);
+    /* Dump the buffer to a file for testing if its a valid ICC profile */
+    save_profile(buffer,FileName,profile_size);
+    return(0);
+}
+
+int create_pscmyk_profile(TCHAR FileName[], bool pcs_islab, bool cpsi_mode, ucrbg_t *ucr_data)
 {
     icProfile iccprofile;
     icHeader  *header = &(iccprofile.header);
@@ -1496,15 +1665,15 @@ int create_pscmyk_profile(TCHAR FileName[], bool pcs_islab)
     /* Create the table data that we will stuff in */
     table_data_AtoB = (unsigned short*) malloc(mlut_size_atob*sizeof(unsigned short)*numout);
     if (pcs_islab) {
-        create_cmyk2lab(table_data_AtoB, mlut_size_atob, num_samples);
+        create_cmyk2lab(table_data_AtoB, mlut_size_atob, num_samples, cpsi_mode);
     } else {
-        create_cmyk2xyz(table_data_AtoB, mlut_size_atob, num_samples);
+        create_cmyk2xyz(table_data_AtoB, mlut_size_atob, num_samples, cpsi_mode);
     }
     table_data_BtoA = (unsigned short*) malloc(mlut_size_btoa*sizeof(unsigned short)*num_colors);
     if (pcs_islab) {
-        create_lab2cmyk(table_data_BtoA, mlut_size_btoa, num_samples); 
+        create_lab2cmyk(table_data_BtoA, mlut_size_btoa, num_samples, ucr_data); 
     } else {
-        create_xyz2cmyk(table_data_BtoA, mlut_size_atob, num_samples);
+        create_xyz2cmyk(table_data_BtoA, mlut_size_atob, num_samples, ucr_data);
     }
     /* Now the ATOB0 */
     add_tabledata(curr_ptr, (void*) table_data_AtoB, num_colors, num_samples, 1, 3, false);
@@ -1616,7 +1785,6 @@ int create_gray_threshold_profile(TCHAR FileName[], float threshhold)
     int numout = 3;     /* PCS number... */
     int num_colors = 1;  /* Gray colors */
     int num_samples = 2;    /* clut samples */
-    float encode_gamma;
     unsigned short *threshold_table;
     int midpoint;
     int mlut_size_btoa;
@@ -2282,10 +2450,6 @@ create_lutAtoBprofile(unsigned char **pp_buffer_in, icHeader *header,
     int profile_size, last_tag, tag_location, tag_size;
     unsigned char *buffer,*curr_ptr;
     icS15Fixed16Number temp_XYZ[3];
-    float d50[3];
-    float *cam;
-    float *temp_matrix[9];
-    float lmn_vector[3],d50_cieA[3];
 
     profile_size = HEADER_SIZE;
     tag_list = (gsicc_tag*) malloc(sizeof(gsicc_tag)*num_tags);
